import { IindexEntry, IndexEntry, MindexEntry, SimpleSindexEntryForWallet, SimpleUdEntryForWallet } from "../indexer";
import { ConfDTO } from "../dto/ConfDTO";
import { BlockDTO } from "../dto/BlockDTO";
import { DBHead } from "../db/DBHead";
import { DBBlock } from "../db/DBBlock";
import { FileDAL } from "../dal/fileDAL";
import { WoTBInstance } from "../wot";
export declare class DuniterBlockchain {
    static checkBlock(block: BlockDTO, withPoWAndSignature: boolean, conf: ConfDTO, dal: FileDAL): Promise<{
        index: IndexEntry[];
        HEAD: DBHead;
    }>;
    static pushTheBlock(obj: BlockDTO, index: IndexEntry[], HEAD: DBHead | null, conf: ConfDTO, dal: FileDAL, logger: any, trim?: boolean): Promise<BlockDTO>;
    static saveBlockData(current: DBBlock | null, block: BlockDTO, conf: ConfDTO, dal: FileDAL, logger: any, index: IndexEntry[], HEAD: DBHead | null, trim: boolean): Promise<DBBlock>;
    static saveParametersForRoot(block: BlockDTO, conf: ConfDTO, dal: FileDAL): Promise<void>;
    static createNewcomers(iindex: IindexEntry[], dal: FileDAL, logger: any, instance?: WoTBInstance): Promise<void>;
    static updateMembers(block: BlockDTO, dal: FileDAL, instance?: WoTBInstance): Promise<void>;
    static updateWallets(sindex: SimpleSindexEntryForWallet[], dividends: SimpleUdEntryForWallet[], aDal: any, reverse?: boolean, at?: number): Promise<void>;
    static revertBlock(number: number, hash: string, dal: FileDAL, block?: DBBlock): Promise<void>;
    static undoMembersUpdate(blockstamp: string, dal: FileDAL): Promise<void>;
    static undoDeleteTransactions(block: DBBlock, dal: FileDAL): Promise<void>;
    /**
     * Delete certifications from the sandbox since it has been written.
     *
     * @param block Block in which are contained the certifications to remove from sandbox.
     * @param dal The DAL
     */
    static removeCertificationsFromSandbox(block: BlockDTO, dal: FileDAL): Promise<void>;
    /**
     * Delete memberships from the sandbox since it has been written.
     *
     * @param block Block in which are contained the certifications to remove from sandbox.
     * @param dal The DAL
     */
    static removeMembershipsFromSandbox(block: BlockDTO, dal: FileDAL): Promise<void>;
    static computeToBeRevoked(mindex: MindexEntry[], dal: FileDAL): Promise<void>;
    static deleteTransactions(block: BlockDTO, dal: FileDAL): Promise<void>;
    static updateBlocksComputedVars(current: {
        unitbase: number;
        monetaryMass: number;
    } | null, block: {
        number: number;
        unitbase: number;
        dividend: number | null;
        membersCount: number;
        monetaryMass: number;
    }): void;
    static pushSideBlock(obj: BlockDTO, dal: FileDAL, logger: any): Promise<DBBlock>;
}
export declare function requiredBindexSizeForTail(TAIL: {
    issuersCount: number;
    issuersFrame: number;
}, conf: {
    medianTimeBlocks: number;
    dtDiffEval: number;
    forksize: number;
}): number;
