export declare enum DuniterDocument {
    ENTITY_NULL = 0,
    ENTITY_BLOCK = 1,
    ENTITY_IDENTITY = 2,
    ENTITY_CERTIFICATION = 3,
    ENTITY_MEMBERSHIP = 4,
    ENTITY_REVOCATION = 5,
    ENTITY_TRANSACTION = 6,
    ENTITY_PEER = 7
}
export declare const duniterDocument2str: (type: DuniterDocument) => "" | "block" | "identity" | "certification" | "revocation" | "membership" | "transaction" | "peer";
export declare const CommonConstants: {
    FORMATS: {
        CURRENCY: string;
        PUBKEY: string;
        INTEGER: string;
        BLOCKSTAMP: string;
        FINGERPRINT: string;
        TIMESTAMP: string;
        WS2PID: string;
        SOFTWARE: string;
        SOFT_VERSION: string;
        POW_PREFIX: string;
        ZERO_OR_POSITIVE_INT: string;
        SIGNATURE: string;
    };
    BLOCK_GENERATED_VERSION: number;
    BLOCK_NEW_GENERATED_VERSION: number;
    LAST_VERSION_FOR_TX: number;
    TRANSACTION_VERSION: number;
    DOCUMENTS_VERSION: number;
    TRANSACTION_MAX_TRIES: number;
    SWITCH_ON_BRANCH_AHEAD_BY_X_BLOCKS: number;
    BMA_REGEXP: RegExp;
    BMAS_REGEXP: RegExp;
    BMATOR_REGEXP: RegExp;
    WS2P_REGEXP: RegExp;
    WS2P_V2_REGEXP: RegExp;
    WS2PTOR_REGEXP: RegExp;
    WS2PTOR_V2_REGEXP: RegExp;
    WS_FULL_ADDRESS_ONION_REGEX: RegExp;
    IPV4_REGEXP: RegExp;
    IPV6_REGEXP: RegExp;
    HOST_ONION_REGEX: RegExp;
    PUBLIC_KEY: RegExp;
    INTEGER: RegExp;
    BASE58: RegExp;
    FINGERPRINT: RegExp;
    SIG: RegExp;
    BLOCK_UID: RegExp;
    USER_ID: RegExp;
    DOCUMENTS_VERSION_REGEXP: RegExp;
    BLOCKSTAMP_REGEXP: RegExp;
    DOCUMENTS_BLOCK_VERSION_REGEXP: RegExp;
    DOCUMENTS_TRANSACTION_VERSION_REGEXP: RegExp;
    SPECIAL_BLOCK: string;
    SPECIAL_HASH: string;
    MAXIMUM_LEN_OF_COMPACT_TX: number;
    MAXIMUM_LEN_OF_OUTPUT: number;
    MAXIMUM_LEN_OF_UNLOCK: number;
    POW_TURN_DURATION_PC: number;
    POW_TURN_DURATION_ARM: number;
    PROOF_OF_WORK: {
        UPPER_BOUND: string[];
    };
    DocumentError: string;
    ERRORS: {
        WRONG_DOCUMENT: {
            httpCode: number;
            uerr: {
                ucode: number;
                message: string;
            };
        };
        DOCUMENT_BEING_TREATED: {
            httpCode: number;
            uerr: {
                ucode: number;
                message: string;
            };
        };
        WRONG_UNLOCKER: {
            httpCode: number;
            uerr: {
                ucode: number;
                message: string;
            };
        };
        LOCKTIME_PREVENT: {
            httpCode: number;
            uerr: {
                ucode: number;
                message: string;
            };
        };
        SOURCE_ALREADY_CONSUMED: {
            httpCode: number;
            uerr: {
                ucode: number;
                message: string;
            };
        };
        WRONG_AMOUNTS: {
            httpCode: number;
            uerr: {
                ucode: number;
                message: string;
            };
        };
        WRONG_OUTPUT_BASE: {
            httpCode: number;
            uerr: {
                ucode: number;
                message: string;
            };
        };
        CANNOT_ROOT_BLOCK_NO_MEMBERS: {
            httpCode: number;
            uerr: {
                ucode: number;
                message: string;
            };
        };
        IDENTITY_WRONGLY_SIGNED: {
            httpCode: number;
            uerr: {
                ucode: number;
                message: string;
            };
        };
        TOO_OLD_IDENTITY: {
            httpCode: number;
            uerr: {
                ucode: number;
                message: string;
            };
        };
        NEWER_PEER_DOCUMENT_AVAILABLE: {
            httpCode: number;
            uerr: {
                ucode: number;
                message: string;
            };
        };
        PEER_DOCUMENT_ALREADY_KNOWN: {
            httpCode: number;
            uerr: {
                ucode: number;
                message: string;
            };
        };
        TX_INPUTS_OUTPUTS_NOT_EQUAL: {
            httpCode: number;
            uerr: {
                ucode: number;
                message: string;
            };
        };
        TX_OUTPUT_SUM_NOT_EQUALS_PREV_DELTAS: {
            httpCode: number;
            uerr: {
                ucode: number;
                message: string;
            };
        };
        BLOCKSTAMP_DOES_NOT_MATCH_A_BLOCK: {
            httpCode: number;
            uerr: {
                ucode: number;
                message: string;
            };
        };
        A_TRANSACTION_HAS_A_MAX_SIZE: {
            httpCode: number;
            uerr: {
                ucode: number;
                message: string;
            };
        };
        TOO_OLD_MEMBERSHIP: {
            httpCode: number;
            uerr: {
                ucode: number;
                message: string;
            };
        };
        MAXIMUM_LEN_OF_OUTPUT: {
            httpCode: number;
            uerr: {
                ucode: number;
                message: string;
            };
        };
        MAXIMUM_LEN_OF_UNLOCK: {
            httpCode: number;
            uerr: {
                ucode: number;
                message: string;
            };
        };
        WRONG_CURRENCY: {
            httpCode: number;
            uerr: {
                ucode: number;
                message: string;
            };
        };
        WRONG_POW: {
            httpCode: number;
            uerr: {
                ucode: number;
                message: string;
            };
        };
        OUT_OF_FORK_WINDOW: {
            httpCode: number;
            uerr: {
                ucode: number;
                message: string;
            };
        };
        WRONG_SIGNATURE_FOR_CERT: {
            httpCode: number;
            uerr: {
                ucode: number;
                message: string;
            };
        };
    };
    M_INDEX: string;
    I_INDEX: string;
    S_INDEX: string;
    C_INDEX: string;
    IDX_CREATE: string;
    IDX_UPDATE: string;
    NB_DIGITS_UD: number;
    REVOCATION_FACTOR: number;
    TX_WINDOW: number;
    POW_DIFFICULTY_RANGE_RATIO: number;
    ACCOUNT_MINIMUM_CURRENT_BASED_AMOUNT: number;
    DOCUMENTS: {
        DOC_VERSION: RegExp;
        DOC_CURRENCY: RegExp;
        DOC_ISSUER: RegExp;
        TIMESTAMP: RegExp;
    };
    IDENTITY: {
        INLINE: RegExp;
        IDTY_TYPE: RegExp;
        IDTY_UID: RegExp;
    };
    BLOCK: {
        NONCE: RegExp;
        VERSION: RegExp;
        TYPE: RegExp;
        CURRENCY: RegExp;
        BNUMBER: RegExp;
        POWMIN: RegExp;
        TIME: RegExp;
        MEDIAN_TIME: RegExp;
        UD: RegExp;
        UNIT_BASE: RegExp;
        PREV_HASH: RegExp;
        PREV_ISSUER: RegExp;
        MEMBERS_COUNT: RegExp;
        BLOCK_ISSUER: RegExp;
        BLOCK_ISSUERS_FRAME: RegExp;
        BLOCK_ISSUERS_FRAME_VAR: RegExp;
        DIFFERENT_ISSUERS_COUNT: RegExp;
        PARAMETERS: RegExp;
        JOINER: RegExp;
        ACTIVE: RegExp;
        LEAVER: RegExp;
        REVOCATION: RegExp;
        EXCLUDED: RegExp;
        INNER_HASH: RegExp;
        SPECIAL_HASH: string;
        SPECIAL_BLOCK: string;
    };
    CERT: {
        SELF: {
            UID: RegExp;
            META: RegExp;
        };
        REVOKE: RegExp;
        OTHER: {
            META: RegExp;
            INLINE: RegExp;
        };
    };
    CERTIFICATION: {
        CERT_TYPE: RegExp;
        IDTY_ISSUER: RegExp;
        IDTY_UID: RegExp;
        IDTY_TIMESTAMP: RegExp;
        IDTY_SIG: RegExp;
        CERT_TIMESTAMP: RegExp;
    };
    REVOCATION: {
        REVOC_TYPE: RegExp;
        IDTY_ISSUER: RegExp;
        IDTY_UID: RegExp;
        IDTY_TIMESTAMP: RegExp;
        IDTY_SIG: RegExp;
    };
    MEMBERSHIP: {
        BLOCK: RegExp;
        VERSION: RegExp;
        CURRENCY: RegExp;
        ISSUER: RegExp;
        MEMBERSHIP: RegExp;
        USERID: RegExp;
        CERTTS: RegExp;
    };
    TRANSACTION: {
        HEADER: RegExp;
        SENDER: RegExp;
        SOURCE_V3: RegExp;
        UNLOCK: RegExp;
        TARGET: RegExp;
        BLOCKSTAMP: RegExp;
        COMMENT: RegExp;
        LOCKTIME: RegExp;
        INLINE_COMMENT: RegExp;
        OUTPUT_CONDITION: RegExp;
    };
    PEER: {
        BLOCK: RegExp;
        SPECIAL_BLOCK: string;
    };
    BLOCK_MAX_TX_CHAINING_DEPTH: number;
    BLOCK_TX_CHAINING_ACTIVATION_MT: number;
    SYNC_BLOCKS_CHUNK: number;
    MILESTONES_PER_PAGE: number;
    CHUNK_PREFIX: string;
    BLOCKS_IN_MEMORY_MAX: number;
    MAX_AGE_OF_PEER_IN_BLOCKS: number;
    INITIAL_DOWNLOAD_SLOTS: number;
    DEFAULT_NON_WOT_PEERS_LIMIT: number;
    WAIT_P2P_CANDIDATE_HEARTBEAT: number;
    MAX_READING_SLOTS_FOR_FILE_SYNC: number;
};
