"use strict";
// Source file from duniter: Crypto-currency software to manage libre currency such as Ğ1
// Copyright (C) 2018  Cedric Moreau <cem.moreau@gmail.com>
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Affero General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Affero General Public License for more details.
Object.defineProperty(exports, "__esModule", { value: true });
const base58_1 = require("./base58");
const nacl_util_1 = require("./nacl-util");
const nacl = require('tweetnacl');
const seedrandom = require('seedrandom');
const naclBinding = require('naclb');
const crypto_sign_BYTES = 64;
class Key {
    constructor(pub, sec) {
        this.pub = pub;
        this.sec = sec;
    }
    /*****************************
    *
    *      GENERAL CRYPTO
    *
    *****************************/
    get publicKey() {
        return this.pub;
    }
    get secretKey() {
        return this.sec;
    }
    rawSec() {
        return base58_1.Base58decode(this.secretKey);
    }
    json() {
        return {
            pub: this.publicKey,
            sec: this.secretKey
        };
    }
    sign(msg) {
        return Promise.resolve(this.signSync(msg));
    }
    signSync(msg) {
        const m = nacl_util_1.decodeUTF8(msg);
        const signedMsg = naclBinding.sign(m, this.rawSec());
        const sig = new Uint8Array(crypto_sign_BYTES);
        for (let i = 0; i < sig.length; i++) {
            sig[i] = signedMsg[i];
        }
        return nacl_util_1.encodeBase64(sig);
    }
    ;
}
exports.Key = Key;
function randomKey() {
    const byteseed = new Uint8Array(32);
    for (let i = 0; i < 32; i++) {
        byteseed[i] = Math.floor(seedrandom()() * 255) + 1;
    }
    const keypair = nacl.sign.keyPair.fromSeed(byteseed);
    return new Key(base58_1.Base58encode(new Buffer(keypair.publicKey)), base58_1.Base58encode(new Buffer(keypair.secretKey)));
}
exports.randomKey = randomKey;
function KeyGen(pub, sec) {
    return new Key(pub, sec);
}
exports.KeyGen = KeyGen;
/**
 * Verify a signature against data & public key.
 * Return true of false as callback argument.
 */
function verify(rawMsg, rawSig, rawPub) {
    const msg = nacl_util_1.decodeUTF8(rawMsg);
    const sig = nacl_util_1.decodeBase64(rawSig);
    const pub = base58_1.Base58decode(rawPub);
    const m = new Uint8Array(crypto_sign_BYTES + msg.length);
    const sm = new Uint8Array(crypto_sign_BYTES + msg.length);
    let i;
    for (i = 0; i < crypto_sign_BYTES; i++)
        sm[i] = sig[i];
    for (i = 0; i < msg.length; i++)
        sm[i + crypto_sign_BYTES] = msg[i];
    // Call to verification lib...
    return naclBinding.verify(m, sm, pub);
}
exports.verify = verify;
//# sourceMappingURL=keyring.js.map