"use strict";
// Source file from duniter: Crypto-currency software to manage libre currency such as Ğ1
// Copyright (C) 2018  Cedric Moreau <cem.moreau@gmail.com>
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Affero General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Affero General Public License for more details.
Object.defineProperty(exports, "__esModule", { value: true });
const GenericParser_1 = require("./GenericParser");
const constants_1 = require("../../../lib/common-libs/constants");
const common_1 = require("../../../lib/common");
const index_1 = require("../../../lib/common-libs/index");
class IdentityParser extends GenericParser_1.GenericParser {
    constructor() {
        super([
            { prop: "version", regexp: constants_1.CommonConstants.DOCUMENTS.DOC_VERSION },
            { prop: "type", regexp: constants_1.CommonConstants.IDENTITY.IDTY_TYPE },
            { prop: "currency", regexp: constants_1.CommonConstants.DOCUMENTS.DOC_CURRENCY },
            { prop: "pubkey", regexp: constants_1.CommonConstants.DOCUMENTS.DOC_ISSUER },
            { prop: "uid", regexp: constants_1.CommonConstants.IDENTITY.IDTY_UID },
            { prop: "buid", regexp: constants_1.CommonConstants.DOCUMENTS.TIMESTAMP }
        ], index_1.rawer.getOfficialIdentity);
    }
    _clean(obj) {
        obj.sig = obj.signature;
        if (obj.uid && obj.buid && obj.pubkey) {
            obj.hash = common_1.hashf(obj.uid + obj.buid + obj.pubkey).toUpperCase();
        }
    }
    _verify(obj) {
        if (!obj.pubkey) {
            return "No pubkey found";
        }
        if (!obj.uid) {
            return "Wrong user id format";
        }
        if (!obj.buid) {
            return "Could not extract block uid";
        }
        if (!obj.sig) {
            return "No signature found for self-certification";
        }
        return "";
    }
}
exports.IdentityParser = IdentityParser;
//# sourceMappingURL=identity.js.map