"use strict";
// Source file from duniter: Crypto-currency software to manage libre currency such as Ğ1
// Copyright (C) 2018  Cedric Moreau <cem.moreau@gmail.com>
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Affero General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Affero General Public License for more details.
Object.defineProperty(exports, "__esModule", { value: true });
const dos2unix_1 = require("./dos2unix");
const PeerDTO_1 = require("../dto/PeerDTO");
const IdentityDTO_1 = require("../dto/IdentityDTO");
const MembershipDTO_1 = require("../dto/MembershipDTO");
const TransactionDTO_1 = require("../dto/TransactionDTO");
const BlockDTO_1 = require("../dto/BlockDTO");
const DOCUMENTS_VERSION = 10;
const SIGNED = false;
const UNSIGNED = true;
exports.getOfficialIdentity = (json, withSig = true) => {
    const dto = IdentityDTO_1.IdentityDTO.fromJSONObject(json);
    if (withSig !== false) {
        return dto.getRawSigned();
    }
    else {
        return dto.rawWithoutSig();
    }
};
exports.getOfficialCertification = (json) => {
    let raw = getNormalHeader('Certification', json);
    raw += "IdtyIssuer: " + json.idty_issuer + '\n';
    raw += "IdtyUniqueID: " + json.idty_uid + '\n';
    raw += "IdtyTimestamp: " + json.idty_buid + '\n';
    raw += "IdtySignature: " + json.idty_sig + '\n';
    raw += "CertTimestamp: " + json.buid + '\n';
    if (json.sig) {
        raw += json.sig + '\n';
    }
    return dos2unix_1.dos2unix(raw);
};
exports.getOfficialRevocation = (json) => {
    let raw = getNormalHeader('Revocation', json);
    raw += "IdtyUniqueID: " + json.uid + '\n';
    raw += "IdtyTimestamp: " + json.buid + '\n';
    raw += "IdtySignature: " + json.sig + '\n';
    if (json.revocation) {
        raw += json.revocation + '\n';
    }
    return dos2unix_1.dos2unix(raw);
};
exports.getPeerWithoutSignature = (json) => {
    return PeerDTO_1.PeerDTO.fromJSONObject(json).getRawUnsigned();
};
exports.getPeer = (json) => {
    return PeerDTO_1.PeerDTO.fromJSONObject(json).getRawSigned();
};
exports.getMembershipWithoutSignature = (json) => {
    return MembershipDTO_1.MembershipDTO.fromJSONObject(json).getRaw();
};
exports.getMembership = (json) => {
    return dos2unix_1.dos2unix(signed(exports.getMembershipWithoutSignature(json), json));
};
exports.getBlockInnerPart = (json) => {
    return BlockDTO_1.BlockDTO.fromJSONObject(json).getRawInnerPart();
};
exports.getBlockWithInnerHashAndNonce = (json) => {
    return BlockDTO_1.BlockDTO.fromJSONObject(json).getRawUnSigned();
};
exports.getBlockInnerHashAndNonceWithSignature = (json) => {
    return BlockDTO_1.BlockDTO.fromJSONObject(json).getSignedPartSigned();
};
exports.getBlock = (json) => {
    return dos2unix_1.dos2unix(signed(exports.getBlockWithInnerHashAndNonce(json), json));
};
exports.getTransaction = (json) => {
    return TransactionDTO_1.TransactionDTO.toRAW(json);
};
function getNormalHeader(doctype, json) {
    let raw = "";
    raw += "Version: " + (json.version || DOCUMENTS_VERSION) + "\n";
    raw += "Type: " + doctype + "\n";
    raw += "Currency: " + json.currency + "\n";
    raw += "Issuer: " + json.issuer + "\n";
    return raw;
}
function signed(raw, json) {
    raw += json.signature + '\n';
    return raw;
}
//# sourceMappingURL=rawer.js.map