import { Map } from "./crypto/map";
export interface UnderscoreClass<T> {
    filter(filterFunc: (t: T) => boolean): UnderscoreClass<T>;
    where(props: {
        [k in keyof T]?: T[k];
    }): UnderscoreClass<T>;
    sortBy(sortFunc: (element: T) => number): UnderscoreClass<T>;
    pluck<K extends keyof T>(k: K): UnderscoreClass<T>;
    uniq<K extends keyof T>(isSorted?: boolean, iteratee?: (t: T) => K): UnderscoreClass<T>;
    value(): T[];
}
export declare const Underscore: {
    filter: <T>(elements: T[], filterFunc: (t: T) => boolean) => T[];
    where: <T>(elements: T[], props: { [k in keyof T]?: T[k] | undefined; }) => T[];
    findWhere: <T>(elements: T[], props: { [k in keyof T]?: T[k] | undefined; }) => T | null;
    keys: <T>(map: T) => (keyof T)[];
    values: <T>(map: {
        [k: string]: T;
    }) => T[];
    pluck: <T, K extends keyof T>(elements: T[], k: K) => T[K][];
    pick: <T, K extends keyof T>(elements: T, ...k: K[]) => T[K][];
    omit: <T, K extends keyof T>(element: T, ...k: K[]) => T[K][];
    uniq: <T, K>(elements: T[], isSorted?: boolean, iteratee?: ((t: T) => K) | undefined) => T[];
    clone: <T>(t: T) => T;
    mapObject: <T, K extends keyof T, L extends keyof T[K]>(t: T, cb: (k: K) => T[K][L]) => Map<T[K][L]>;
    mapObjectByProp: <T, K extends keyof T, L extends keyof T[K]>(t: T, prop: L) => Map<T[K][L]>;
    sortBy: <T, K extends keyof T>(elements: T[], sortFunc: K | ((element: T) => string | number)) => T[];
    difference: <T>(array1: T[], array2: T[]) => T[];
    shuffle: <T>(elements: T[]) => T[];
    extend: <T, U>(t1: T, t2: U) => T | U;
    range: (count: number, end?: number | undefined) => number[];
    chain: <T>(element: T[]) => UnderscoreClass<T>;
};
