"use strict";
// Source file from duniter: Crypto-currency software to manage libre currency such as Ğ1
// Copyright (C) 2018  Cedric Moreau <cem.moreau@gmail.com>
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Affero General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Affero General Public License for more details.
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs = require("fs");
const path = require("path");
const ConfDAL_1 = require("./fileDALs/ConfDAL");
const ConfDTO_1 = require("../dto/ConfDTO");
const TransactionDTO_1 = require("../dto/TransactionDTO");
const MerkleDTO_1 = require("../dto/MerkleDTO");
const constants_1 = require("../common-libs/constants");
const PowDAL_1 = require("./fileDALs/PowDAL");
const errors_1 = require("../common-libs/errors");
const IdentityDTO_1 = require("../dto/IdentityDTO");
const logger_1 = require("../logger");
const CFSCore_1 = require("./fileDALs/CFSCore");
const underscore_1 = require("../common-libs/underscore");
const MonitorFlushedIndex_1 = require("../debug/MonitorFlushedIndex");
const programOptions_1 = require("../common-libs/programOptions");
const MonitorExecutionTime_1 = require("../debug/MonitorExecutionTime");
const LevelDBDividend_1 = require("./indexDAL/leveldb/LevelDBDividend");
const LevelDBBindex_1 = require("./indexDAL/leveldb/LevelDBBindex");
const LevelDBBlockchain_1 = require("./indexDAL/leveldb/LevelDBBlockchain");
const LevelDBSindex_1 = require("./indexDAL/leveldb/LevelDBSindex");
const SqliteTransactions_1 = require("./indexDAL/sqlite/SqliteTransactions");
const SqlitePeers_1 = require("./indexDAL/sqlite/SqlitePeers");
const LevelDBWallet_1 = require("./indexDAL/leveldb/LevelDBWallet");
const LevelDBCindex_1 = require("./indexDAL/leveldb/LevelDBCindex");
const LevelDBIindex_1 = require("./indexDAL/leveldb/LevelDBIindex");
const LevelDBMindex_1 = require("./indexDAL/leveldb/LevelDBMindex");
const readline = require('readline');
const indexer = require('../indexer').Indexer;
const logger = require('../logger').NewLogger('filedal');
const constants = require('../constants');
class FileDAL {
    constructor(params, getSqliteDB, getLevelDB) {
        this.params = params;
        this.getSqliteDB = getSqliteDB;
        this.getLevelDB = getLevelDB;
        this.setRevocating = (idty, revocation_sig) => {
            const dbIdentity = IdentityDTO_1.IdentityDTO.fromBasicIdentity(idty);
            dbIdentity.member = idty.member;
            dbIdentity.wasMember = idty.wasMember;
            dbIdentity.expires_on = idty.expires_on;
            dbIdentity.revocation_sig = revocation_sig;
            dbIdentity.revoked = false;
            return this.idtyDAL.saveIdentity(dbIdentity);
        };
        this.rootPath = params.home;
        this.sqliteDriver = params.dbf();
        this.profile = 'DAL';
        this.fs = params.fs;
        // DALs
        this.coreFS = new CFSCore_1.CFSCore(this.rootPath, params.fs);
        this.powDAL = new PowDAL_1.PowDAL(this.rootPath, params.fs);
        this.confDAL = new ConfDAL_1.ConfDAL(this.rootPath, params.fs);
        this.metaDAL = new (require('./sqliteDAL/MetaDAL').MetaDAL)(this.sqliteDriver);
        this.idtyDAL = new (require('./sqliteDAL/IdentityDAL').IdentityDAL)(this.sqliteDriver);
        this.certDAL = new (require('./sqliteDAL/CertDAL').CertDAL)(this.sqliteDriver);
        this.msDAL = new (require('./sqliteDAL/MembershipDAL').MembershipDAL)(this.sqliteDriver);
        this.blockDAL = new LevelDBBlockchain_1.LevelDBBlockchain(getLevelDB);
        this.txsDAL = new SqliteTransactions_1.SqliteTransactions(getSqliteDB);
        this.peerDAL = new SqlitePeers_1.SqlitePeers(getSqliteDB);
        this.walletDAL = new LevelDBWallet_1.LevelDBWallet(getLevelDB);
        this.bindexDAL = new LevelDBBindex_1.LevelDBBindex(getLevelDB);
        this.mindexDAL = new LevelDBMindex_1.LevelDBMindex(getLevelDB);
        this.iindexDAL = new LevelDBIindex_1.LevelDBIindex(getLevelDB);
        this.sindexDAL = new LevelDBSindex_1.LevelDBSindex(getLevelDB);
        this.cindexDAL = new LevelDBCindex_1.LevelDBCindex(getLevelDB);
        this.dividendDAL = new LevelDBDividend_1.LevelDBDividend(getLevelDB);
        this.newDals = {
            'powDAL': this.powDAL,
            'metaDAL': this.metaDAL,
            'blockDAL': this.blockDAL,
            'certDAL': this.certDAL,
            'msDAL': this.msDAL,
            'idtyDAL': this.idtyDAL,
            'txsDAL': this.txsDAL,
            'peerDAL': this.peerDAL,
            'confDAL': this.confDAL,
            'walletDAL': this.walletDAL,
            'bindexDAL': this.bindexDAL,
            'mindexDAL': this.mindexDAL,
            'iindexDAL': this.iindexDAL,
            'sindexDAL': this.sindexDAL,
            'cindexDAL': this.cindexDAL,
            'dividendDAL': this.dividendDAL,
        };
    }
    async init(conf) {
        this.wotb = this.params.wotbf();
        this.dals = [
            this.blockDAL,
            this.txsDAL,
            this.peerDAL,
            this.walletDAL,
            this.bindexDAL,
            this.mindexDAL,
            this.iindexDAL,
            this.sindexDAL,
            this.cindexDAL,
            this.dividendDAL,
        ];
        for (const indexDAL of this.dals) {
            indexDAL.triggerInit();
        }
        const dalNames = underscore_1.Underscore.keys(this.newDals);
        for (const dalName of dalNames) {
            const dal = this.newDals[dalName];
            await dal.init();
        }
        logger.debug("Upgrade database...");
        await this.metaDAL.upgradeDatabase(conf);
        // Update the maximum certifications count a member can issue into the C++ addon
        const currencyParams = await this.getParameters();
        if (currencyParams && currencyParams.sigStock !== undefined && currencyParams.sigStock !== null) {
            this.wotb.setMaxCert(currencyParams.sigStock);
        }
    }
    getDBVersion() {
        return this.metaDAL.getVersion();
    }
    writeFileOfBlock(block) {
        return this.blockDAL.saveBlock(block);
    }
    writeSideFileOfBlock(block) {
        return this.blockDAL.saveSideBlock(block);
    }
    listAllPeers() {
        return this.peerDAL.listAll();
    }
    async getPeer(pubkey) {
        try {
            return await this.peerDAL.getPeer(pubkey);
        }
        catch (err) {
            throw Error('Unknown peer ' + pubkey);
        }
    }
    async getWS2Peers() {
        return this.peerDAL.getPeersWithEndpointsLike('WS2P');
    }
    getAbsoluteBlockInForkWindowByBlockstamp(blockstamp) {
        if (!blockstamp)
            throw "Blockstamp is required to find the block";
        const sp = blockstamp.split('-');
        const number = parseInt(sp[0]);
        const hash = sp[1];
        return this.getAbsoluteBlockInForkWindow(number, hash);
    }
    getAbsoluteValidBlockInForkWindowByBlockstamp(blockstamp) {
        if (!blockstamp)
            throw "Blockstamp is required to find the block";
        const sp = blockstamp.split('-');
        const number = parseInt(sp[0]);
        const hash = sp[1];
        return this.getAbsoluteValidBlockInForkWindow(number, hash);
    }
    async getBlockWeHaveItForSure(number) {
        return (await this.blockDAL.getBlock(number));
    }
    // Duniter-UI dependency
    async getBlock(number) {
        return this.getFullBlockOf(number);
    }
    async getFullBlockOf(number) {
        return this.blockDAL.getBlock(number);
    }
    async getBlockstampOf(number) {
        const block = await this.getTristampOf(number);
        if (block) {
            return [block.number, block.hash].join('-');
        }
        return null;
    }
    async getTristampOf(number) {
        return this.blockDAL.getBlock(number);
    }
    async existsAbsoluteBlockInForkWindow(number, hash) {
        return !!(await this.getAbsoluteBlockByNumberAndHash(number, hash, true));
    }
    async getAbsoluteBlockInForkWindow(number, hash) {
        return this.getAbsoluteBlockByNumberAndHash(number, hash);
    }
    async getAbsoluteValidBlockInForkWindow(number, hash) {
        const block = await this.getAbsoluteBlockByNumberAndHash(number, hash);
        if (block && !block.fork) {
            return block;
        }
        return null;
    }
    async getAbsoluteBlockByNumberAndHash(number, hash, forceNumberAndHashFinding = false) {
        if (number > 0 || forceNumberAndHashFinding) {
            return await this.blockDAL.getAbsoluteBlock(number, hash);
        }
        else {
            // Block#0 is special
            return await this.blockDAL.getBlock(number);
        }
    }
    async getAbsoluteBlockByBlockstamp(blockstamp) {
        const sp = blockstamp.split('-');
        return this.getAbsoluteBlockByNumberAndHash(parseInt(sp[0]), sp[1]);
    }
    async existsNonChainableLink(from, vHEAD_1, sigStock) {
        // Cert period rule
        const medianTime = vHEAD_1 ? vHEAD_1.medianTime : 0;
        const linksFrom = await this.cindexDAL.reducablesFrom(from);
        const unchainables = underscore_1.Underscore.filter(linksFrom, (link) => link.chainable_on > medianTime);
        if (unchainables.length > 0)
            return true;
        // Max stock rule
        let activeLinks = underscore_1.Underscore.filter(linksFrom, (link) => !link.expired_on);
        return activeLinks.length >= sigStock;
    }
    async getCurrentBlockOrNull() {
        let current = null;
        try {
            current = await this.getBlockCurrent();
        }
        catch (e) {
            if (e != constants.ERROR.BLOCK.NO_CURRENT_BLOCK) {
                throw e;
            }
        }
        return current;
    }
    getPromoted(number) {
        return this.getFullBlockOf(number);
    }
    // Block
    getPotentialRootBlocks() {
        return this.blockDAL.getPotentialRoots();
    }
    lastBlockOfIssuer(issuer) {
        return this.blockDAL.lastBlockOfIssuer(issuer);
    }
    getCountOfPoW(issuer) {
        return this.blockDAL.getCountOfBlocksIssuedBy(issuer);
    }
    /**
     * Find all the blocks in the blockchain whose number is between [start ; end]
     * @param start Lower number bound (included).
     * @param end Higher number bound (included).
     */
    async getBlocksBetween(start, end) {
        start = Math.max(0, start);
        end = Math.max(0, end);
        return this.blockDAL.getBlocks(Math.max(0, start), end);
    }
    getForkBlocksFollowing(current) {
        return this.blockDAL.getNextForkBlocks(current.number, current.hash);
    }
    getPotentialForkBlocks(numberStart, medianTimeStart, maxNumber) {
        return this.blockDAL.getPotentialForkBlocks(numberStart, medianTimeStart, maxNumber);
    }
    async getBlockCurrent() {
        const current = await this.blockDAL.getCurrent();
        if (!current)
            throw 'No current block';
        return current;
    }
    getValidLinksTo(to) {
        return this.cindexDAL.getValidLinksTo(to);
    }
    async getAvailableSourcesByPubkey(pubkey) {
        const txAvailable = await this.sindexDAL.getAvailableForPubkey(pubkey);
        const sources = await this.dividendDAL.getUDSources(pubkey);
        return sources.map(d => {
            return {
                type: 'D',
                noffset: d.pos,
                identifier: pubkey,
                amount: d.amount,
                base: d.base,
                conditions: 'SIG(' + pubkey + ')'
            };
        }).concat(txAvailable.map(s => {
            return {
                type: 'T',
                noffset: s.pos,
                identifier: s.identifier,
                amount: s.amount,
                base: s.base,
                conditions: s.conditions
            };
        }));
    }
    async findByIdentifierPosAmountBase(identifier, pos, amount, base, isDividend) {
        if (isDividend) {
            return this.dividendDAL.findUdSourceByIdentifierPosAmountBase(identifier, pos, amount, base);
        }
        else {
            return this.sindexDAL.findTxSourceByIdentifierPosAmountBase(identifier, pos, amount, base);
        }
    }
    async getGlobalIdentityByHashForExistence(hash) {
        const pending = await this.idtyDAL.getByHash(hash);
        if (!pending) {
            const idty = await this.iindexDAL.getFullFromHash(hash);
            if (!idty) {
                return false;
            }
        }
        return true;
    }
    async getGlobalIdentityByHashForHashingAndSig(hash) {
        const pending = await this.idtyDAL.getByHash(hash);
        if (!pending) {
            const idty = await this.iindexDAL.getFullFromHash(hash);
            if (!idty) {
                return null;
            }
            return {
                pubkey: idty.pub,
                uid: idty.uid,
                buid: idty.created_on,
                sig: idty.sig
            };
        }
        return pending;
    }
    async getGlobalIdentityByHashForLookup(hash) {
        const pending = await this.idtyDAL.getByHash(hash);
        if (!pending) {
            const idty = await this.iindexDAL.getFullFromHash(hash);
            if (!idty) {
                return null;
            }
            return {
                pubkey: idty.pub,
                uid: idty.uid,
                buid: idty.created_on,
                sig: idty.sig,
                member: idty.member,
                wasMember: idty.wasMember
            };
        }
        return pending;
    }
    async getGlobalIdentityByHashForJoining(hash) {
        const pending = await this.idtyDAL.getByHash(hash);
        if (!pending) {
            const idty = await this.iindexDAL.getFullFromHash(hash);
            if (!idty) {
                return null;
            }
            const membership = await this.mindexDAL.getReducedMSForImplicitRevocation(idty.pub);
            return {
                pubkey: idty.pub,
                uid: idty.uid,
                buid: idty.created_on,
                sig: idty.sig,
                member: idty.member,
                wasMember: idty.wasMember,
                revoked: !!(membership.revoked_on)
            };
        }
        return pending;
    }
    async getGlobalIdentityByHashForIsMember(hash) {
        const pending = await this.idtyDAL.getByHash(hash);
        if (!pending) {
            const idty = await this.iindexDAL.getFullFromHash(hash);
            if (!idty) {
                return null;
            }
            return {
                pub: idty.pub,
                member: idty.member
            };
        }
        return {
            pub: pending.pubkey,
            member: pending.member
        };
    }
    async getGlobalIdentityByHashForRevocation(hash) {
        const pending = await this.idtyDAL.getByHash(hash);
        if (!pending) {
            const idty = await this.iindexDAL.getFullFromHash(hash);
            if (!idty) {
                return null;
            }
            const membership = await this.mindexDAL.getReducedMSForImplicitRevocation(idty.pub);
            return {
                pub: idty.pub,
                uid: idty.uid,
                sig: idty.sig,
                member: idty.member,
                wasMember: idty.wasMember,
                expires_on: membership.expires_on,
                created_on: idty.created_on,
                revoked: !!(membership.revoked_on),
                revocation_sig: membership.revocation
            };
        }
        return {
            pub: pending.pubkey,
            uid: pending.uid,
            sig: pending.sig,
            expires_on: pending.expires_on,
            created_on: pending.buid,
            member: pending.member,
            wasMember: pending.wasMember,
            revoked: pending.revoked,
            revocation_sig: pending.revocation_sig
        };
    }
    getMembers() {
        return this.iindexDAL.getMembers();
    }
    async getWrittenIdtyByPubkeyForHash(pubkey) {
        return this.getWrittenForSureIdtyByPubkey(pubkey);
    }
    async getWrittenIdtyByPubkeyForHashing(pubkey) {
        return this.getWrittenForSureIdtyByPubkey(pubkey);
    }
    async getWrittenIdtyByPubkeyForWotbID(pubkey) {
        return this.getWrittenForSureIdtyByPubkey(pubkey);
    }
    async getWrittenIdtyByPubkeyForUidAndPubkey(pubkey) {
        return this.getWrittenForSureIdtyByPubkey(pubkey);
    }
    async getWrittenIdtyByPubkeyForIsMember(pubkey) {
        return this.iindexDAL.getFromPubkey(pubkey);
    }
    async getWrittenIdtyByPubkeyForUidAndIsMemberAndWasMember(pubkey) {
        return this.iindexDAL.getFromPubkey(pubkey);
    }
    async getWrittenIdtyByPubkeyOrUidForIsMemberAndPubkey(search) {
        return this.iindexDAL.getFromPubkeyOrUid(search);
    }
    async getWrittenIdtyByPubkeyOrUIdForHashingAndIsMember(search) {
        return await this.iindexDAL.getFromPubkeyOrUid(search);
    }
    async getWrittenIdtyByPubkeyForRevocationCheck(pubkey) {
        const idty = await this.iindexDAL.getFromPubkey(pubkey);
        if (!idty) {
            return null;
        }
        const membership = await this.mindexDAL.getReducedMSForImplicitRevocation(pubkey);
        return {
            pub: idty.pub,
            uid: idty.uid,
            sig: idty.sig,
            created_on: idty.created_on,
            revoked_on: membership.revoked_on
        };
    }
    async getWrittenIdtyByPubkeyForCertificationCheck(pubkey) {
        const idty = await this.iindexDAL.getFromPubkey(pubkey);
        if (!idty) {
            return null;
        }
        return {
            pub: idty.pub,
            uid: idty.uid,
            sig: idty.sig,
            created_on: idty.created_on,
        };
    }
    async getWrittenIdtyByPubkeyForUidAndMemberAndCreatedOn(pubkey) {
        const idty = await this.iindexDAL.getFromPubkey(pubkey);
        if (!idty) {
            return null;
        }
        return {
            uid: idty.uid,
            member: idty.member,
            created_on: idty.created_on,
        };
    }
    async getWrittenForSureIdtyByPubkey(pubkey) {
        const idty = await this.iindexDAL.getFromPubkey(pubkey);
        if (!idty) {
            throw Error(errors_1.DataErrors[errors_1.DataErrors.MEMBER_NOT_FOUND]);
        }
        return idty;
    }
    async getWrittenForSureIdtyByUid(pubkey) {
        const idty = (await this.iindexDAL.getFullFromUID(pubkey));
        if (!idty) {
            throw Error(errors_1.DataErrors[errors_1.DataErrors.MEMBER_NOT_FOUND]);
        }
        return idty;
    }
    // Duniter-UI dependency
    async getWrittenIdtyByPubkey(pub) {
        return await this.iindexDAL.getFromPubkey(pub);
    }
    async getWrittenIdtyByPubkeyForExistence(uid) {
        return !!(await this.iindexDAL.getFromPubkey(uid));
    }
    async getWrittenIdtyByUIDForExistence(uid) {
        return !!(await this.iindexDAL.getFromUID(uid));
    }
    async getWrittenIdtyByUidForHashing(uid) {
        return this.getWrittenForSureIdtyByUid(uid);
    }
    async getWrittenIdtyByUIDForWotbId(uid) {
        return this.getWrittenForSureIdtyByUid(uid);
    }
    async findPeersWhoseHashIsIn(hashes) {
        const peers = await this.peerDAL.listAll();
        return underscore_1.Underscore.chain(peers).filter((p) => hashes.indexOf(p.hash) !== -1).value();
    }
    getTxByHash(hash) {
        return this.txsDAL.getTX(hash);
    }
    removeTxByHash(hash) {
        return this.txsDAL.removeTX(hash);
    }
    getTransactionsPending(versionMin = 0) {
        return this.txsDAL.getAllPending(versionMin);
    }
    async getNonWritten(pubkey) {
        const pending = await this.idtyDAL.getPendingIdentities();
        return underscore_1.Underscore.chain(pending).where({ pubkey: pubkey }).value();
    }
    async getRevocatingMembers() {
        const revoking = await this.idtyDAL.getToRevoke();
        const toRevoke = [];
        for (const pending of revoking) {
            const idty = await this.getWrittenIdtyByPubkeyForRevocationCheck(pending.pubkey);
            if (idty && !idty.revoked_on) {
                toRevoke.push(pending);
            }
        }
        return toRevoke;
    }
    getToBeKickedPubkeys() {
        return this.iindexDAL.getToBeKickedPubkeys();
    }
    getRevokedPubkeys() {
        return this.mindexDAL.getRevokedPubkeys();
    }
    async searchJustIdentities(search) {
        const pendings = await this.idtyDAL.searchThoseMatching(search);
        const writtens = await this.iindexDAL.searchThoseMatching(search);
        const nonPendings = underscore_1.Underscore.filter(writtens, (w) => {
            return underscore_1.Underscore.where(pendings, { pubkey: w.pub }).length == 0;
        });
        const found = pendings.concat(nonPendings.map((i) => {
            // Use the correct field
            i.pubkey = i.pub;
            return i;
        }));
        return await Promise.all(found.map(async (f) => {
            const ms = await this.mindexDAL.getReducedMSForImplicitRevocation(f.pubkey);
            if (ms) {
                f.revoked_on = null;
                if (ms.revoked_on) {
                    const blockOfRevocation = (await this.getAbsoluteBlockByBlockstamp(ms.revoked_on));
                    f.revoked_on = blockOfRevocation.medianTime;
                }
                f.revoked = !!f.revoked_on;
                f.revocation_sig = f.revocation_sig || ms.revocation || null;
            }
            return f;
        }));
    }
    async certsToTarget(pub, hash) {
        const certs = await this.certDAL.getToTarget(hash);
        const links = await this.cindexDAL.getValidLinksTo(pub);
        let matching = certs;
        await Promise.all(links.map(async (entry) => {
            matching.push(await this.cindexEntry2DBCert(entry));
        }));
        matching = underscore_1.Underscore.sortBy(matching, (c) => -c.block);
        matching.reverse();
        return matching;
    }
    async certsFrom(pubkey) {
        const certs = await this.certDAL.getFromPubkeyCerts(pubkey);
        const links = await this.cindexDAL.getValidLinksFrom(pubkey);
        let matching = certs;
        await Promise.all(links.map(async (entry) => {
            matching.push(await this.cindexEntry2DBCert(entry));
        }));
        matching = underscore_1.Underscore.sortBy(matching, (c) => -c.block);
        matching.reverse();
        return matching;
    }
    async cindexEntry2DBCert(entry) {
        const idty = await this.getWrittenIdtyByPubkeyForHash(entry.receiver);
        const wbt = entry.written_on.split('-');
        const block = (await this.getBlock(entry.created_on));
        return {
            issuers: [entry.issuer],
            linked: true,
            written: true,
            written_block: parseInt(wbt[0]),
            written_hash: wbt[1],
            sig: entry.sig,
            block_number: block.number,
            block_hash: block.hash,
            target: idty.hash,
            to: entry.receiver,
            from: entry.issuer,
            block: block.number,
            expired: !!entry.expired_on,
            expires_on: entry.expires_on,
        };
    }
    async isSentry(pubkey, conf) {
        const current = await this.getCurrentBlockOrNull();
        if (current) {
            const dSen = Math.ceil(Math.pow(current.membersCount, 1 / conf.stepMax));
            const linksFrom = await this.cindexDAL.getValidLinksFrom(pubkey);
            const linksTo = await this.cindexDAL.getValidLinksTo(pubkey);
            return linksFrom.length >= dSen && linksTo.length >= dSen;
        }
        return false;
    }
    async certsFindNew() {
        const certs = await this.certDAL.getNotLinked();
        return underscore_1.Underscore.chain(certs).where({ linked: false }).sortBy((c) => -c.block).value();
    }
    async certsNotLinkedToTarget(hash) {
        const certs = await this.certDAL.getNotLinkedToTarget(hash);
        return underscore_1.Underscore.chain(certs).sortBy((c) => -c.block).value();
    }
    async getMostRecentMembershipNumberForIssuer(issuer) {
        const mss = await this.msDAL.getMembershipsOfIssuer(issuer);
        const reduced = await this.mindexDAL.getReducedMSForImplicitRevocation(issuer);
        let max = reduced ? parseInt(reduced.created_on) : -1;
        for (const ms of mss) {
            max = Math.max(ms.number, max);
        }
        return max;
    }
    async lastJoinOfIdentity(target) {
        let pending = await this.msDAL.getPendingINOfTarget(target);
        return underscore_1.Underscore.sortBy(pending, (ms) => -ms.number)[0];
    }
    async findNewcomers(blockMedianTime = 0) {
        const pending = await this.msDAL.getPendingIN();
        const mss = await Promise.all(pending.map(async (p) => {
            const reduced = await this.mindexDAL.getReducedMSForImplicitRevocation(p.issuer);
            if (!reduced || !reduced.chainable_on || blockMedianTime >= reduced.chainable_on || blockMedianTime < constants.TIME_TO_TURN_ON_BRG_107) {
                return p;
            }
            return null;
        }));
        return underscore_1.Underscore.chain(underscore_1.Underscore.filter(mss, ms => !!ms))
            .sortBy((ms) => -ms.blockNumber)
            .value();
    }
    async findLeavers(blockMedianTime = 0) {
        const pending = await this.msDAL.getPendingOUT();
        const mss = await Promise.all(pending.map(async (p) => {
            const reduced = await this.mindexDAL.getReducedMSForImplicitRevocation(p.issuer);
            if (!reduced || !reduced.chainable_on || blockMedianTime >= reduced.chainable_on || blockMedianTime < constants.TIME_TO_TURN_ON_BRG_107) {
                return p;
            }
            return null;
        }));
        return underscore_1.Underscore.chain(underscore_1.Underscore.filter(mss, ms => !!ms))
            .sortBy(ms => -ms.blockNumber)
            .value();
    }
    existsNonReplayableLink(from, to, medianTime, version) {
        return this.cindexDAL.existsNonReplayableLink(from, to, medianTime, version);
    }
    async getSource(identifier, pos, isDividend) {
        if (isDividend) {
            return this.dividendDAL.getUDSource(identifier, pos);
        }
        else {
            return this.sindexDAL.getTxSource(identifier, pos);
        }
    }
    async isMember(pubkey) {
        try {
            const idty = await this.iindexDAL.getFromPubkey(pubkey);
            if (idty && idty.member) {
                return true;
            }
            return false;
        }
        catch (err) {
            return false;
        }
    }
    async isMemberAndNonLeaver(pubkey) {
        try {
            const idty = await this.iindexDAL.getFromPubkey(pubkey);
            if (idty && idty.member) {
                return !(await this.isLeaving(pubkey));
            }
            return false;
        }
        catch (err) {
            return false;
        }
    }
    async isLeaving(pubkey) {
        const ms = await this.mindexDAL.getReducedMSForImplicitRevocation(pubkey);
        return (ms && ms.leaving) || false;
    }
    async existsCert(cert, current) {
        const existing = await this.certDAL.existsGivenCert(cert);
        if (existing)
            return existing;
        if (!current) {
            return false;
        }
        const existsLink = await this.cindexDAL.existsNonReplayableLink(cert.from, cert.to, current.medianTime, current.version);
        return !!existsLink;
    }
    deleteCert(cert) {
        return this.certDAL.deleteCert(cert);
    }
    deleteMS(ms) {
        return this.msDAL.deleteMS(ms);
    }
    async setRevoked(pubkey) {
        return await this.idtyDAL.setRevoked(pubkey);
    }
    async getPeerOrNull(pubkey) {
        let peer = null;
        try {
            peer = await this.getPeer(pubkey);
        }
        catch (e) {
            if (e != constants.ERROR.BLOCK.NO_CURRENT_BLOCK) {
                throw e;
            }
        }
        return peer;
    }
    async removePeerByPubkey(pubkey) {
        return this.peerDAL.removePeerByPubkey(pubkey);
    }
    async findAllPeersBut(pubkeys) {
        const peers = await this.listAllPeers();
        return peers.filter((peer) => pubkeys.indexOf(peer.pubkey) == -1
            && ['UP'].indexOf(peer.status) !== -1);
    }
    async listAllPeersWithStatusNewUP() {
        const peers = await this.peerDAL.listAll();
        return underscore_1.Underscore.chain(peers)
            .filter((p) => ['UP']
            .indexOf(p.status) !== -1).value();
    }
    async listAllPeersWithStatusNewUPWithtout(pub) {
        const peers = await this.peerDAL.listAll();
        return underscore_1.Underscore.chain(peers).filter((p) => p.status == 'UP').filter((p) => p.pubkey !== pub).value();
    }
    async findPeers(pubkey) {
        try {
            const peer = await this.getPeer(pubkey);
            return [peer];
        }
        catch (err) {
            return [];
        }
    }
    async getRandomlyUPsWithout(pubkeys) {
        const peers = await this.listAllPeersWithStatusNewUP();
        return peers.filter(peer => pubkeys.indexOf(peer.pubkey) == -1);
    }
    async setPeerUP(pubkey) {
        try {
            const p = await this.getPeer(pubkey);
            p.status = 'UP';
            p.first_down = null;
            p.last_try = null;
            return this.peerDAL.savePeer(p);
        }
        catch (err) {
            return null;
        }
    }
    async setPeerDown(pubkey) {
        try {
            // We do not set mirror peers as down (ex. of mirror: 'M1_HnFcSms8jzwngtVomTTnzudZx7SHUQY8sVE1y8yBmULk')
            if (!pubkey.match(/_/)) {
                const p = await this.getPeer(pubkey);
                if (p) {
                    const now = (new Date()).getTime();
                    p.status = 'DOWN';
                    if (!p.first_down) {
                        p.first_down = now;
                    }
                    p.last_try = now;
                    await this.peerDAL.savePeer(p);
                }
            }
        }
        catch (err) {
            throw err;
        }
    }
    async saveBlock(dbb) {
        dbb.wrong = false;
        await Promise.all([
            this.saveBlockInFile(dbb),
            this.saveTxsInFiles(dbb.transactions, dbb.number, dbb.medianTime)
        ]);
    }
    async generateIndexes(block, conf, index, aHEAD) {
        // We need to recompute the indexes for block#0
        let HEAD;
        if (!index || !aHEAD || aHEAD.number == 0) {
            index = indexer.localIndex(block, conf);
            HEAD = await indexer.completeGlobalScope(block, conf, index, this);
        }
        else {
            HEAD = aHEAD;
        }
        let mindex = indexer.mindex(index);
        let iindex = indexer.iindex(index);
        let sindex = indexer.sindex(index);
        let cindex = indexer.cindex(index);
        const dividends = await indexer.ruleIndexGenDividend(HEAD, iindex, this); // Requires that newcomers are already in DividendDAO
        sindex = sindex.concat(await indexer.ruleIndexGarbageSmallAccounts(HEAD, sindex, dividends, this));
        cindex = cindex.concat(await indexer.ruleIndexGenCertificationExpiry(HEAD, this));
        mindex = mindex.concat(await indexer.ruleIndexGenMembershipExpiry(HEAD, this));
        iindex = iindex.concat(await indexer.ruleIndexGenExclusionByMembership(HEAD, mindex, this));
        iindex = iindex.concat(await indexer.ruleIndexGenExclusionByCertificatons(HEAD, cindex, iindex, conf, this));
        mindex = mindex.concat(await indexer.ruleIndexGenImplicitRevocation(HEAD, this));
        await indexer.ruleIndexCorrectMembershipExpiryDate(HEAD, mindex, this);
        await indexer.ruleIndexCorrectCertificationExpiryDate(HEAD, cindex, this);
        return { HEAD, mindex, iindex, sindex, cindex, dividends };
    }
    async updateWotbLinks(cindex, instance) {
        const wotb = instance || this.wotb;
        for (const entry of cindex) {
            const from = await this.getWrittenIdtyByPubkeyForWotbID(entry.issuer);
            const to = await this.getWrittenIdtyByPubkeyForWotbID(entry.receiver);
            if (entry.op == constants_1.CommonConstants.IDX_CREATE) {
                // NewLogger().trace('addLink %s -> %s', from.wotb_id, to.wotb_id)
                wotb.addLink(from.wotb_id, to.wotb_id);
            }
            else {
                // Update = removal
                logger_1.NewLogger().trace('removeLink %s -> %s', from.wotb_id, to.wotb_id);
                wotb.removeLink(from.wotb_id, to.wotb_id);
            }
        }
    }
    async trimIndexes(maxNumber) {
        if (!programOptions_1.cliprogram.notrim) {
            await this.bindexDAL.trimBlocks(maxNumber);
            await this.iindexDAL.trimRecords(maxNumber);
            await this.mindexDAL.trimRecords(maxNumber);
            if (!programOptions_1.cliprogram.notrimc) {
                await this.cindexDAL.trimExpiredCerts(maxNumber);
            }
        }
        await this.sindexDAL.trimConsumedSource(maxNumber);
        await this.dividendDAL.trimConsumedUDs(maxNumber);
    }
    async trimSandboxes(block) {
        await this.certDAL.trimExpiredCerts(block.medianTime);
        await this.msDAL.trimExpiredMemberships(block.medianTime);
        await this.idtyDAL.trimExpiredIdentities(block.medianTime);
        await this.txsDAL.trimExpiredNonWrittenTxs(block.medianTime - constants_1.CommonConstants.TX_WINDOW);
        return true;
    }
    savePendingMembership(ms) {
        return this.msDAL.savePendingMembership(ms);
    }
    async saveBlockInFile(block) {
        await this.writeFileOfBlock(block);
    }
    saveSideBlockInFile(block) {
        return this.writeSideFileOfBlock(block);
    }
    async saveTxsInFiles(txs, block_number, medianTime) {
        return Promise.all(txs.map(async (tx) => {
            const sp = tx.blockstamp.split('-');
            const basedBlock = (await this.getAbsoluteBlockByNumberAndHash(parseInt(sp[0]), sp[1]));
            tx.blockstampTime = basedBlock.medianTime;
            const txEntity = TransactionDTO_1.TransactionDTO.fromJSONObject(tx);
            txEntity.computeAllHashes();
            return this.txsDAL.addLinked(TransactionDTO_1.TransactionDTO.fromJSONObject(txEntity), block_number, medianTime);
        }));
    }
    async merkleForPeers() {
        let peers = await this.listAllPeersWithStatusNewUP();
        const leaves = peers.map((peer) => peer.hash);
        const merkle = new MerkleDTO_1.MerkleDTO();
        merkle.initialize(leaves);
        return merkle;
    }
    savePendingIdentity(idty) {
        return this.idtyDAL.saveIdentity(idty);
    }
    revokeIdentity(pubkey) {
        return this.idtyDAL.revokeIdentity(pubkey);
    }
    async removeUnWrittenWithPubkey(pubkey) {
        return await this.idtyDAL.removeUnWrittenWithPubkey(pubkey);
    }
    async removeUnWrittenWithUID(pubkey) {
        return await this.idtyDAL.removeUnWrittenWithUID(pubkey);
    }
    registerNewCertification(cert) {
        return this.certDAL.saveNewCertification(cert);
    }
    saveTransaction(tx) {
        return this.txsDAL.addPending(tx);
    }
    async getTransactionsHistory(pubkey) {
        const history = {
            sent: [],
            received: [],
            sending: [],
            receiving: [],
            pending: []
        };
        const res = await Promise.all([
            this.txsDAL.getLinkedWithIssuer(pubkey),
            this.txsDAL.getLinkedWithRecipient(pubkey),
            this.txsDAL.getPendingWithIssuer(pubkey),
            this.txsDAL.getPendingWithRecipient(pubkey)
        ]);
        history.sent = res[0] || [];
        history.received = res[1] || [];
        history.sending = res[2] || [];
        history.pending = res[3] || [];
        return history;
    }
    async getUDHistory(pubkey) {
        const sources = await this.dividendDAL.getUDSources(pubkey);
        return {
            history: (await Promise.all(sources.map(async (src) => {
                const block = await this.getBlockWeHaveItForSure(src.pos);
                return {
                    block_number: src.pos,
                    time: block.medianTime,
                    consumed: src.consumed,
                    amount: src.amount,
                    base: src.base
                };
            })))
        };
    }
    savePeer(peer) {
        return this.peerDAL.savePeer(peer);
    }
    async getUniqueIssuersBetween(start, end) {
        const current = (await this.blockDAL.getCurrent());
        const firstBlock = Math.max(0, start);
        const lastBlock = Math.max(0, Math.min(current.number, end));
        const blocks = await this.blockDAL.getBlocks(firstBlock, lastBlock);
        return underscore_1.Underscore.uniq(blocks.map(b => b.issuer));
    }
    /**
     * Gets a range of entries for the last `start`th to the last `end`th HEAD entry.
     * @param start The starting entry number (min. 1)
     * @param end The ending entry (max. BINDEX length)
     * @param property If provided, transforms the range of entries into an array of the asked property.
     */
    async range(start, end, property) {
        const range = await this.bindexDAL.range(start, end);
        if (property) {
            // Filter on a particular property
            return range.map((b) => b[property]);
        }
        else {
            return range;
        }
    }
    /**
     * Get the last `n`th entry from the BINDEX.
     * @param n The entry number (min. 1).
     */
    head(n) {
        return this.bindexDAL.head(n);
    }
    /***********************
     *    CONFIGURATION
     **********************/
    getParameters() {
        return this.confDAL.getParameters();
    }
    async loadConf(overrideConf, defaultConf = false) {
        let conf = ConfDTO_1.ConfDTO.complete(overrideConf || {});
        if (!defaultConf) {
            const savedConf = await this.confDAL.loadConf();
            conf = underscore_1.Underscore.extend(savedConf, overrideConf || {});
            if (overrideConf.proxiesConf !== undefined) { }
            else {
            }
        }
        if (this.loadConfHook) {
            await this.loadConfHook(conf);
        }
        return conf;
    }
    async saveConf(confToSave) {
        // Save the conf in file
        let theConf = confToSave;
        if (this.saveConfHook) {
            theConf = await this.saveConfHook(theConf);
        }
        return this.confDAL.saveConf(theConf);
    }
    /***********************
     *     WALLETS
     **********************/
    async getWallet(conditions) {
        let wallet = await this.walletDAL.getWallet(conditions);
        if (!wallet) {
            wallet = { conditions, balance: 0 };
        }
        return wallet;
    }
    saveWallet(wallet) {
        return this.walletDAL.saveWallet(wallet);
    }
    /***********************
     *     STATISTICS
     **********************/
    getStat(name) {
        switch (name) {
            case "newcomers":
                return this.blockDAL.findWithIdentities();
            case "certs":
                return this.blockDAL.findWithCertifications();
            case "joiners":
                return this.blockDAL.findWithJoiners();
            case "actives":
                return this.blockDAL.findWithActives();
            case "leavers":
                return this.blockDAL.findWithLeavers();
            case "excluded":
                return this.blockDAL.findWithExcluded();
            case "revoked":
                return this.blockDAL.findWithRevoked();
            case "ud":
                return this.blockDAL.findWithUD();
            case "tx":
                return this.blockDAL.findWithTXs();
            default:
                throw errors_1.DataErrors[errors_1.DataErrors.WRONG_STAT_NAME];
        }
    }
    async cleanCaches() {
        await underscore_1.Underscore.values(this.newDals).map((dal) => dal.cleanCache && dal.cleanCache());
    }
    async close() {
        await Promise.all(underscore_1.Underscore.values(this.newDals).map(async (dal) => {
            dal.cleanCache();
            await dal.close();
        }));
        await this.sqliteDriver.closeConnection();
    }
    async resetPeers() {
        await this.peerDAL.removeAll();
        return await this.close();
    }
    getLogContent(linesQuantity) {
        return new Promise((resolve, reject) => {
            try {
                let lines = [], i = 0;
                const logPath = path.join(this.rootPath, 'duniter.log');
                const readStream = fs.createReadStream(logPath);
                readStream.on('error', (err) => reject(err));
                const lineReader = readline.createInterface({
                    input: readStream
                });
                lineReader.on('line', (line) => {
                    line = "\n" + line;
                    lines.push(line);
                    i++;
                    if (i >= linesQuantity)
                        lines.shift();
                });
                lineReader.on('close', () => resolve(lines));
                lineReader.on('error', (err) => reject(err));
            }
            catch (e) {
                reject(e);
            }
        });
    }
    async findReceiversAbove(minsig) {
        const receiversAbove = await this.cindexDAL.getReceiversAbove(minsig);
        const members = [];
        for (const r of receiversAbove) {
            const i = await this.iindexDAL.getFullFromPubkey(r);
            members.push({
                hash: i.hash || "",
                member: i.member || false,
                wasMember: i.wasMember || false,
                pubkey: i.pub,
                uid: i.uid || "",
                buid: i.created_on || "",
                sig: i.sig || "",
                revocation_sig: "",
                revoked: false,
                revoked_on: 0
            });
        }
        return members;
    }
    async flushIndexes(indexes) {
        if (indexes.mindex.length) {
            await this.mindexDAL.insertBatch(indexes.mindex);
        }
        if (indexes.iindex.length) {
            await this.iindexDAL.insertBatch(indexes.iindex);
        }
        const sindex_txs = indexes.sindex.filter(s => s.srcType === 'T');
        if (sindex_txs.length) {
            await this.sindexDAL.insertBatch(sindex_txs); // We don't store dividends in SINDEX
        }
        const sindex_uds = indexes.sindex.filter(s => s.srcType === 'D');
        if (sindex_uds.length) {
            await this.dividendDAL.consume(sindex_uds);
        }
        if (indexes.cindex.length) {
            await this.cindexDAL.insertBatch(indexes.cindex);
        }
    }
    async updateDividend(blockNumber, dividend, unitbase, local_iindex) {
        if (dividend) {
            return this.dividendDAL.produceDividend(blockNumber, dividend, unitbase, local_iindex);
        }
        return [];
    }
}
__decorate([
    MonitorExecutionTime_1.MonitorExecutionTime()
], FileDAL.prototype, "trimIndexes", null);
__decorate([
    MonitorFlushedIndex_1.MonitorFlushedIndex()
], FileDAL.prototype, "flushIndexes", null);
exports.FileDAL = FileDAL;
//# sourceMappingURL=fileDAL.js.map