import { LevelUp } from 'levelup';
import { LevelDBTable } from "./LevelDBTable";
import { DBBlock } from "../../../db/DBBlock";
import { BlockchainDAO } from "../abstract/BlockchainDAO";
export declare class LevelDBBlockchain extends LevelDBTable<DBBlock> implements BlockchainDAO {
    protected getLevelDB: (dbName: string) => Promise<LevelUp>;
    private forks;
    private indexOfIdentities;
    private indexOfCertifications;
    private indexOfJoiners;
    private indexOfActives;
    private indexOfLeavers;
    private indexOfExcluded;
    private indexOfRevoked;
    private indexOfDividends;
    private indexOfTransactions;
    private indexers;
    constructor(getLevelDB: (dbName: string) => Promise<LevelUp>);
    init(): Promise<void>;
    close(): Promise<void>;
    /**
     * INSERT
     */
    insert(record: DBBlock): Promise<void>;
    insertBatch(records: DBBlock[]): Promise<void>;
    dropNonForkBlocksAbove(number: number): Promise<void>;
    findRawWithOrder(criterion: {
        pub?: string;
    }, sort: (string | (string | boolean)[])[]): Promise<DBBlock[]>;
    getAbsoluteBlock(number: number, hash: string): Promise<DBBlock | null>;
    getBlock(number: string | number): Promise<DBBlock | null>;
    getBlocks(start: number, end: number): Promise<DBBlock[]>;
    getCountOfBlocksIssuedBy(issuer: string): Promise<number>;
    getCurrent(): Promise<DBBlock | null>;
    getNextForkBlocks(number: number, hash: string): Promise<DBBlock[]>;
    getPotentialForkBlocks(numberStart: number, medianTimeStart: number, maxNumber: number): Promise<DBBlock[]>;
    getPotentialRoots(): Promise<DBBlock[]>;
    getWrittenOn(blockstamp: string): Promise<DBBlock[]>;
    lastBlockOfIssuer(issuer: string): Promise<DBBlock | null>;
    lastBlockWithDividend(): Promise<DBBlock | null>;
    removeBlock(blockstamp: string): Promise<void>;
    removeForkBlock(number: number): Promise<void>;
    removeForkBlockAboveOrEqual(number: number): Promise<void>;
    saveBlock(block: DBBlock): Promise<DBBlock>;
    saveSideBlock(block: DBBlock): Promise<DBBlock>;
    setSideBlock(number: number, previousBlock: DBBlock | null): Promise<void>;
    findBetween(db: LevelDBTable<DBBlock>, start: number, end: number): Promise<DBBlock[]>;
    findWithIdentities(): Promise<number[]>;
    findWithCertifications(): Promise<number[]>;
    findWithJoiners(): Promise<number[]>;
    findWithActives(): Promise<number[]>;
    findWithLeavers(): Promise<number[]>;
    findWithExcluded(): Promise<number[]>;
    findWithRevoked(): Promise<number[]>;
    findWithUD(): Promise<number[]>;
    findWithTXs(): Promise<number[]>;
    private findIndexed;
    private static trimKey;
    private static trimForkKey;
}
