"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
Object.defineProperty(exports, "__esModule", { value: true });
const MonitorExecutionTime_1 = require("../../../debug/MonitorExecutionTime");
const LevelDBTable_1 = require("./LevelDBTable");
const LevelIndexBlockIdentities_1 = require("./indexers/block/LevelIndexBlockIdentities");
const array_filter_1 = require("../../../common-libs/array-filter");
const LevelIndexBlockCertifications_1 = require("./indexers/block/LevelIndexBlockCertifications");
const LevelIndexBlock_1 = require("./indexers/block/LevelIndexBlock");
const logger_1 = require("../../../logger");
const LevelIndexBlockTX_1 = require("./indexers/block/LevelIndexBlockTX");
const LevelIndexBlockUD_1 = require("./indexers/block/LevelIndexBlockUD");
const LevelIndexBlockRevoked_1 = require("./indexers/block/LevelIndexBlockRevoked");
const LevelIndexBlockExcluded_1 = require("./indexers/block/LevelIndexBlockExcluded");
const LevelIndexBlockJoiners_1 = require("./indexers/block/LevelIndexBlockJoiners");
const LevelIndexBlockActives_1 = require("./indexers/block/LevelIndexBlockActives");
const LevelIndexBlockLeavers_1 = require("./indexers/block/LevelIndexBlockLeavers");
class LevelDBBlockchain extends LevelDBTable_1.LevelDBTable {
    constructor(getLevelDB) {
        super('level_blockchain', getLevelDB);
        this.getLevelDB = getLevelDB;
        this.indexers = [];
    }
    async init() {
        await super.init();
        if (this.indexers.length === 0) {
            this.forks = new LevelDBTable_1.LevelDBTable('level_blockchain/forks', this.getLevelDB);
            this.indexers.push(this.indexOfIdentities = new LevelIndexBlockIdentities_1.LevelIndexBlockIdentities('level_blockchain/idty', this.getLevelDB));
            this.indexers.push(this.indexOfCertifications = new LevelIndexBlockCertifications_1.LevelIndexBlockCertifications('level_blockchain/certs', this.getLevelDB));
            this.indexers.push(this.indexOfJoiners = new LevelIndexBlockJoiners_1.LevelIndexBlockJoiners('level_blockchain/joiners', this.getLevelDB));
            this.indexers.push(this.indexOfActives = new LevelIndexBlockActives_1.LevelIndexBlockActives('level_blockchain/actives', this.getLevelDB));
            this.indexers.push(this.indexOfLeavers = new LevelIndexBlockLeavers_1.LevelIndexBlockLeavers('level_blockchain/leavers', this.getLevelDB));
            this.indexers.push(this.indexOfExcluded = new LevelIndexBlockExcluded_1.LevelIndexBlockExcluded('level_blockchain/excluded', this.getLevelDB));
            this.indexers.push(this.indexOfRevoked = new LevelIndexBlockRevoked_1.LevelIndexBlockRevoked('level_blockchain/revoked', this.getLevelDB));
            this.indexers.push(this.indexOfDividends = new LevelIndexBlockUD_1.LevelIndexBlockUD('level_blockchain/dividends', this.getLevelDB));
            this.indexers.push(this.indexOfTransactions = new LevelIndexBlockTX_1.LevelIndexBlockTX('level_blockchain/transactions', this.getLevelDB));
        }
        await this.forks.init();
        logger_1.NewLogger().debug(`Now open indexers...`);
        await Promise.all(this.indexers.map(i => i.init()));
    }
    async close() {
        await super.close();
        await this.forks.close();
        await Promise.all(this.indexers.map(i => i.close()));
    }
    /**
     * INSERT
     */
    async insert(record) {
        await this.insertBatch([record]);
    }
    async insertBatch(records) {
        // Indexation
        await Promise.all(this.indexers.map(i => i.onInsert(records)));
        // Update the max headNumber
        await this.batchInsertWithKeyComputing(records, r => {
            return LevelDBBlockchain.trimKey(r.number);
        });
    }
    async dropNonForkBlocksAbove(number) {
        await this.applyAllKeyValue(async (kv) => {
            // console.log(`DROPPING FORK ${kv.key}`)
            return this.del(kv.key);
        }, {
            gt: LevelDBBlockchain.trimKey(number)
        });
    }
    // Never used
    async findRawWithOrder(criterion, sort) {
        return [];
    }
    async getAbsoluteBlock(number, hash) {
        const block = await this.getBlock(number);
        if (block && block.hash === hash) {
            return block;
        }
        const fork = await this.forks.getOrNull(LevelDBBlockchain.trimForkKey(number, hash));
        if (!fork) {
            return null;
        }
        fork.fork = true;
        return fork;
    }
    getBlock(number) {
        return this.getOrNull(LevelDBBlockchain.trimKey(parseInt(String(number))));
    }
    getBlocks(start, end) {
        return this.findAllValues({
            gt: LevelDBBlockchain.trimKey(start - 1),
            lt: LevelDBBlockchain.trimKey(end + 1)
        });
    }
    // Used by DuniterUI
    async getCountOfBlocksIssuedBy(issuer) {
        let nb = 0;
        await this.readAllKeyValue(kv => {
            if (kv.value.issuer === issuer) {
                nb++;
            }
        });
        return nb;
    }
    async getCurrent() {
        return (await this.findAllValues({
            limit: 1,
            reverse: true
        }))[0];
    }
    async getNextForkBlocks(number, hash) {
        const potentialForks = await this.findBetween(this.forks, number + 1, number + 1);
        return potentialForks.filter(f => f.previousHash === hash);
    }
    async getPotentialForkBlocks(numberStart, medianTimeStart, maxNumber) {
        const potentialForks = await this.findBetween(this.forks, numberStart, maxNumber);
        return potentialForks.filter(f => f.medianTime >= medianTimeStart);
    }
    getPotentialRoots() {
        return this.findBetween(this.forks, 0, 0);
    }
    // TODO: potentially never called?
    async getWrittenOn(blockstamp) {
        const number = parseInt(blockstamp);
        const blocks = await this.findBetween(this.forks, number, number);
        const block = await this.getOrNull(LevelDBBlockchain.trimKey(parseInt(blockstamp)));
        return block ? blocks.concat(block) : blocks;
    }
    // TODO: Unused? potentially costly because of full scan
    async lastBlockOfIssuer(issuer) {
        let theLast = null;
        await this.readAllKeyValue(kv => {
            if (!theLast && kv.value.issuer === issuer) {
                theLast = kv.value;
            }
        });
        return theLast;
    }
    // TODO: Unused? potentially costly because of full scan
    async lastBlockWithDividend() {
        let theLast = null;
        await this.readAllKeyValue(kv => {
            if (!theLast && kv.value.dividend) {
                theLast = kv.value;
            }
        });
        return theLast;
    }
    async removeBlock(blockstamp) {
        await this.del(LevelDBBlockchain.trimKey(parseInt(blockstamp)));
    }
    async removeForkBlock(number) {
        await this.forks.applyAllKeyValue(async (kv) => this.forks.del(kv.key), {
            gt: LevelDBBlockchain.trimKey(number - 1),
            lt: LevelDBBlockchain.trimKey(number + 1)
        });
    }
    async removeForkBlockAboveOrEqual(number) {
        await this.forks.applyAllKeyValue(async (kv) => this.forks.del(kv.key), {
            gt: LevelDBBlockchain.trimKey(number - 1)
        });
    }
    async saveBlock(block) {
        // We add the new block into legit blockchain
        await this.insert(block);
        block.fork = false;
        // We remove the eventual fork
        const forkKey = LevelDBBlockchain.trimForkKey(block.number, block.hash);
        if (this.forks.getOrNull(forkKey)) {
            await this.forks.del(forkKey);
        }
        // We return the saved block
        return this.get(LevelDBBlockchain.trimKey(block.number));
    }
    async saveSideBlock(block) {
        const k = LevelDBBlockchain.trimForkKey(block.number, block.hash);
        block.fork = true;
        await this.forks.put(k, block);
        return this.forks.get(k);
    }
    async setSideBlock(number, previousBlock) {
        const k = LevelDBBlockchain.trimKey(number);
        const block = await this.get(k);
        block.fork = true;
        // Indexation
        await Promise.all(this.indexers.map(i => i.onRemove([block])));
        await this.del(k);
        await this.forks.put(LevelDBBlockchain.trimForkKey(block.number, block.hash), block);
    }
    async findBetween(db, start, end) {
        return await db.findAllValues({
            gte: LevelDBBlockchain.trimKey(start),
            lt: LevelDBBlockchain.trimKey(end + 1)
        });
    }
    async findWithIdentities() {
        return this.findIndexed(this.indexOfIdentities);
    }
    async findWithCertifications() {
        return this.findIndexed(this.indexOfCertifications);
    }
    async findWithJoiners() {
        return this.findIndexed(this.indexOfJoiners);
    }
    async findWithActives() {
        return this.findIndexed(this.indexOfActives);
    }
    async findWithLeavers() {
        return this.findIndexed(this.indexOfLeavers);
    }
    async findWithExcluded() {
        return this.findIndexed(this.indexOfExcluded);
    }
    async findWithRevoked() {
        return this.findIndexed(this.indexOfRevoked);
    }
    async findWithUD() {
        return this.findIndexed(this.indexOfDividends);
    }
    async findWithTXs() {
        return this.findIndexed(this.indexOfTransactions);
    }
    async findIndexed(indexer) {
        const found = await indexer.getOrNull(LevelIndexBlock_1.LDBIndex_ALL);
        if (!found) {
            // When the entry does not exist (may occur for 'ALL' key)
            return [];
        }
        // Otherwise: return the records
        return Promise.all(found
            .reduce((all, some) => all.concat(some), [])
            .filter(array_filter_1.uniqFilter)
            .sort((b, a) => b - a));
    }
    static trimKey(number) {
        return String(number).padStart(10, '0');
    }
    static trimForkKey(number, hash) {
        return `${String(number).padStart(10, '0')}-${hash}`;
    }
}
__decorate([
    MonitorExecutionTime_1.MonitorExecutionTime()
], LevelDBBlockchain.prototype, "insert", null);
__decorate([
    MonitorExecutionTime_1.MonitorExecutionTime()
], LevelDBBlockchain.prototype, "insertBatch", null);
exports.LevelDBBlockchain = LevelDBBlockchain;
//# sourceMappingURL=LevelDBBlockchain.js.map