import { CindexEntry, FullCindexEntry } from "../../../indexer";
import { LevelUp } from 'levelup';
import { LevelDBTable } from "./LevelDBTable";
import { CIndexDAO } from "../abstract/CIndexDAO";
import { AbstractIteratorOptions } from "abstract-leveldown";
export interface LevelDBCindexEntry {
    received: string[];
    issued: CindexEntry[];
}
export declare class LevelDBCindex extends LevelDBTable<LevelDBCindexEntry> implements CIndexDAO {
    protected getLevelDB: (dbName: string) => Promise<LevelUp>;
    private indexForExpiresOn;
    private indexForWrittenOn;
    constructor(getLevelDB: (dbName: string) => Promise<LevelUp>);
    /**
     * TECHNICAL
     */
    init(): Promise<void>;
    close(): Promise<void>;
    /**
     * INSERT
     */
    insert(record: CindexEntry): Promise<void>;
    insertBatch(records: CindexEntry[]): Promise<void>;
    /**
     * Reduceable DAO
     */
    trimRecords(belowNumber: number): Promise<void>;
    /**
     * Generic DAO
     */
    findRawWithOrder(criterion: {
        pub?: string;
    }, sort: (string | (string | boolean)[])[]): Promise<CindexEntry[]>;
    getWrittenOn(blockstamp: string): Promise<CindexEntry[]>;
    removeBlock(blockstamp: string): Promise<void>;
    private static trimExpiredOnKey;
    private static trimWrittenOnKey;
    private static trimFullKey;
    private static trimDumpSortKey;
    private indexRecords;
    existsNonReplayableLink(issuer: string, receiver: string, medianTime: number, version: number): Promise<boolean>;
    findByIssuer(issuer: string): Promise<CindexEntry[]>;
    findByIssuerAndChainableOnGt(issuer: string, medianTime: number): Promise<CindexEntry[]>;
    findByIssuerAndReceiver(issuer: string, receiver: string): Promise<CindexEntry[]>;
    findByReceiverAndExpiredOn(pub: string, expired_on: number): Promise<CindexEntry[]>;
    findExpiresOnLteNotExpiredYet(medianTime: number): Promise<CindexEntry[]>;
    getReceiversAbove(minsig: number): Promise<string[]>;
    getValidLinksFrom(issuer: string): Promise<CindexEntry[]>;
    getValidLinksTo(receiver: string): Promise<CindexEntry[]>;
    reducablesFrom(from: string): Promise<FullCindexEntry[]>;
    trimExpiredCerts(belowNumber: number): Promise<void>;
    count(options?: AbstractIteratorOptions): Promise<number>;
}
