"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
Object.defineProperty(exports, "__esModule", { value: true });
const MonitorExecutionTime_1 = require("../../../debug/MonitorExecutionTime");
const indexer_1 = require("../../../indexer");
const LevelDBTable_1 = require("./LevelDBTable");
const underscore_1 = require("../../../common-libs/underscore");
const pint_1 = require("../../../common-libs/pint");
const reduce_1 = require("../../../common-libs/reduce");
class LevelDBCindex extends LevelDBTable_1.LevelDBTable {
    constructor(getLevelDB) {
        super('level_cindex', getLevelDB);
        this.getLevelDB = getLevelDB;
    }
    /**
     * TECHNICAL
     */
    async init() {
        await super.init();
        this.indexForExpiresOn = new LevelDBTable_1.LevelDBTable('level_cindex/expiresOn', this.getLevelDB);
        this.indexForWrittenOn = new LevelDBTable_1.LevelDBTable('level_cindex/writtenOn', this.getLevelDB);
        await this.indexForExpiresOn.init();
        await this.indexForWrittenOn.init();
    }
    async close() {
        await super.close();
        await this.indexForExpiresOn.close();
        await this.indexForWrittenOn.close();
    }
    /**
     * INSERT
     */
    async insert(record) {
        await this.insertBatch([record]);
    }
    async insertBatch(records) {
        for (const r of records) {
            const existingIssuer = await this.getOrNull(r.issuer);
            const existingReceiver = await this.getOrNull(r.receiver);
            let newValue4Issuer = existingIssuer || {
                received: [],
                issued: []
            };
            let newValue4Receiver = existingReceiver || {
                received: [],
                issued: []
            };
            newValue4Issuer.issued.push(r);
            if (!newValue4Receiver.received.includes(r.issuer) && r.op === 'CREATE') {
                newValue4Receiver.received.push(r.issuer);
            }
            await Promise.all([
                this.put(r.issuer, newValue4Issuer),
                this.put(r.receiver, newValue4Receiver)
            ]);
        }
        await this.indexRecords(records);
    }
    /**
     * Reduceable DAO
     */
    async trimRecords(belowNumber) {
        // Trim writtenOn: we remove from the index the blocks below `belowNumber`, and keep track of the deleted values
        let issuers = underscore_1.Underscore.uniq((await this.indexForWrittenOn.deleteWhere({ lt: LevelDBCindex.trimWrittenOnKey(belowNumber) }))
            .map(kv => kv.value)
            .reduce(reduce_1.reduceConcat, []));
        // Trim expired certs that won't be rolled back + we remember the max value of expired_on that was trimmed
        let maxExpired = 0;
        issuers = underscore_1.Underscore.uniq(issuers);
        await Promise.all(issuers.map(async (issuer) => {
            const entry = await this.get(issuer);
            const fullEntries = indexer_1.reduceBy(entry.issued, ['issuer', 'receiver']);
            const toRemove = [];
            // We remember the maximum value of expired_on, for efficient trimming  search
            fullEntries
                .filter(f => f.expired_on && f.writtenOn < belowNumber)
                .forEach(f => {
                maxExpired = Math.max(maxExpired, f.expired_on);
                // We must remove **all** the remaining entries for this issuer + receiver
                entry.issued
                    .filter(e => e.issuer === f.issuer && e.receiver === f.receiver)
                    .forEach(e => toRemove.push(LevelDBCindex.trimFullKey(e.issuer, e.receiver, e.created_on)));
            });
            if (toRemove.length) {
                // Trim the expired certs that won't be rolled back ever
                entry.issued = entry.issued.filter(entry => !toRemove.includes(LevelDBCindex.trimFullKey(entry.issuer, entry.receiver, entry.created_on)));
                await this.put(issuer, entry);
            }
        }));
        // Finally, we trim the expiredOn index
        await this.indexForExpiresOn.deleteWhere({ lte: LevelDBCindex.trimExpiredOnKey(maxExpired) });
    }
    /**
     * Generic DAO
     */
    async findRawWithOrder(criterion, sort) {
        const rows = (await this.findAllValues()).map(r => r.issued).reduce(reduce_1.reduceConcat, []);
        return underscore_1.Underscore.sortBy(rows, r => LevelDBCindex.trimDumpSortKey(r.written_on, r.issuer, r.receiver));
    }
    async getWrittenOn(blockstamp) {
        const ids = (await this.indexForWrittenOn.getOrNull(LevelDBCindex.trimWrittenOnKey(pint_1.pint(blockstamp)))) || [];
        return (await Promise.all(ids.map(async (id) => (await this.get(id)).issued))).reduce(reduce_1.reduceConcat, []).filter(e => e.written_on === blockstamp);
    }
    async removeBlock(blockstamp) {
        const writtenOn = pint_1.pint(blockstamp);
        const issuers = (await this.indexForWrittenOn.getOrNull(LevelDBCindex.trimWrittenOnKey(writtenOn))) || [];
        const toRemove = [];
        for (const issuer of issuers) {
            // Remove the entries
            const entry = await this.get(issuer);
            const previousLength = entry.issued.length;
            entry.issued = entry.issued.filter(e => {
                const shouldBeDeleted = e.written_on === blockstamp;
                if (shouldBeDeleted) {
                    toRemove.push(e);
                }
                return !shouldBeDeleted;
            });
            if (entry.issued.length !== previousLength) {
                // Update the entry
                await this.put(issuer, entry);
            }
        }
        // Remove the "received" arrays
        await Promise.all(toRemove.map(async (e) => {
            const entry = await this.get(e.receiver);
            // Remove the certification
            entry.received = entry.received.filter(issuer => issuer !== e.issuer);
            // Persist
            await this.put(e.receiver, entry);
        }));
        // Remove the expires_on index entries
        const expires = underscore_1.Underscore.uniq(toRemove.filter(e => e.expires_on).map(e => e.expires_on));
        await Promise.all(expires.map(async (e) => this.indexForExpiresOn.del(LevelDBCindex.trimExpiredOnKey(e))));
    }
    static trimExpiredOnKey(writtenOn) {
        return String(writtenOn).padStart(10, '0');
    }
    static trimWrittenOnKey(writtenOn) {
        return String(writtenOn).padStart(10, '0');
    }
    static trimFullKey(issuer, receiver, created_on) {
        return `${issuer}-${receiver}-${String(created_on).padStart(10, '0')}`;
    }
    static trimDumpSortKey(written_on, issuer, receiver) {
        return `${written_on.padStart(100, '0')}-${issuer}-${receiver}`;
    }
    async indexRecords(records) {
        const byExpiresOn = {};
        const byWrittenOn = {};
        records
            .filter(r => r.expires_on)
            .forEach(r => (byExpiresOn[r.expires_on] || (byExpiresOn[r.expires_on] = [])).push(r));
        records
            .forEach(r => (byWrittenOn[r.writtenOn] || (byWrittenOn[r.writtenOn] = [])).push(r));
        // Index expires_on => issuers
        for (const k of underscore_1.Underscore.keys(byExpiresOn)) {
            const issuers = ((await this.indexForExpiresOn.getOrNull(LevelDBCindex.trimExpiredOnKey(k))) || [])
                .concat(byExpiresOn[k].map(r => r.issuer));
            await this.indexForExpiresOn.put(LevelDBCindex.trimExpiredOnKey(k), issuers);
        }
        // Index writtenOn => issuers
        for (const k of underscore_1.Underscore.keys(byWrittenOn)) {
            await this.indexForWrittenOn.put(LevelDBCindex.trimWrittenOnKey(k), byWrittenOn[k].map(r => r.issuer));
        }
    }
    async existsNonReplayableLink(issuer, receiver, medianTime, version) {
        const entries = await this.findByIssuer(issuer);
        const reduced = indexer_1.Indexer.DUP_HELPERS.reduceBy(entries, ['issuer', 'receiver']);
        return reduced.filter(e => e.receiver === receiver && (version <= 10 || e.replayable_on >= medianTime)).length > 0;
    }
    async findByIssuer(issuer) {
        return (await this.getOrNull(issuer) || { issued: [], received: [] }).issued;
    }
    async findByIssuerAndChainableOnGt(issuer, medianTime) {
        return (await this.findByIssuer(issuer)).filter(e => e.chainable_on > medianTime);
    }
    async findByIssuerAndReceiver(issuer, receiver) {
        return (await this.findByIssuer(issuer)).filter(e => e.receiver === receiver);
    }
    async findByReceiverAndExpiredOn(pub, expired_on) {
        const receiver = (await this.getOrNull(pub)) || { issued: [], received: [] };
        const issuers = receiver.received;
        return (await Promise.all(issuers.map(async (issuer) => {
            return (await this.get(issuer)).issued.filter(e => e.receiver === pub && e.expired_on === 0);
        }))).reduce(reduce_1.reduceConcat, []);
    }
    async findExpiresOnLteNotExpiredYet(medianTime) {
        const issuers = underscore_1.Underscore.uniq((await this.indexForExpiresOn.findAllValues({ lte: LevelDBCindex.trimExpiredOnKey(medianTime) })).reduce(reduce_1.reduceConcat, []));
        return (await Promise.all(issuers.map(async (issuer) => {
            const fullEntries = indexer_1.Indexer.DUP_HELPERS.reduceBy((await this.get(issuer)).issued, ['issuer', 'receiver']);
            return fullEntries.filter(e => e.expires_on <= medianTime && !e.expired_on);
        }))).reduce(reduce_1.reduceConcat, []);
    }
    async getReceiversAbove(minsig) {
        return this.findWhereTransform(i => i.received.length >= minsig, i => i.key);
    }
    async getValidLinksFrom(issuer) {
        const fullEntries = indexer_1.Indexer.DUP_HELPERS.reduceBy(((await this.getOrNull(issuer)) || { issued: [] }).issued, ['issuer', 'receiver']);
        return fullEntries.filter(e => !e.expired_on);
    }
    async getValidLinksTo(receiver) {
        const issuers = ((await this.getOrNull(receiver)) || { issued: [], received: [] }).received;
        return (await Promise.all(issuers.map(async (issuer) => {
            const fullEntries = indexer_1.Indexer.DUP_HELPERS.reduceBy((await this.get(issuer)).issued, ['issuer', 'receiver']);
            return fullEntries.filter(e => e.receiver === receiver && !e.expired_on);
        }))).reduce(reduce_1.reduceConcat, []);
    }
    async reducablesFrom(from) {
        const entries = ((await this.getOrNull(from)) || { issued: [], received: [] }).issued;
        return indexer_1.Indexer.DUP_HELPERS.reduceBy(entries, ['issuer', 'receiver']);
    }
    trimExpiredCerts(belowNumber) {
        return this.trimRecords(belowNumber);
    }
    async count(options) {
        let count = 0;
        await this.readAllKeyValue(entry => {
            count += entry.value.issued.length;
        });
        return count;
    }
}
__decorate([
    MonitorExecutionTime_1.MonitorExecutionTime()
], LevelDBCindex.prototype, "insert", null);
__decorate([
    MonitorExecutionTime_1.MonitorExecutionTime()
], LevelDBCindex.prototype, "insertBatch", null);
exports.LevelDBCindex = LevelDBCindex;
//# sourceMappingURL=LevelDBCindex.js.map