"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
Object.defineProperty(exports, "__esModule", { value: true });
const MonitorExecutionTime_1 = require("../../../debug/MonitorExecutionTime");
const DividendDaoHandler_1 = require("../common/DividendDaoHandler");
const errors_1 = require("../../../common-libs/errors");
const LevelDBTable_1 = require("./LevelDBTable");
const underscore_1 = require("../../../common-libs/underscore");
class LevelDBDividend extends LevelDBTable_1.LevelDBTable {
    constructor(getLevelDB) {
        super('level_dividend', getLevelDB);
        this.getLevelDB = getLevelDB;
        this.hasIndexed = false;
    }
    /**
     * TECHNICAL
     */
    cleanCache() {
    }
    triggerInit() {
    }
    async init() {
        await super.init();
        this.indexForTrimming = new LevelDBTable_1.LevelDBTable('level_dividend/level_dividend_trim_index', this.getLevelDB);
        await this.indexForTrimming.init();
    }
    async close() {
        await super.close();
        await this.indexForTrimming.close();
    }
    /**
     * INSERT
     */
    async insert(record) {
        await this.insertBatch([record]);
    }
    async insertBatch(records) {
        await this.batchInsert(records, 'pub');
    }
    async indexConsumptions(consumptions) {
        // Index the operations by write date, for future trimming
        const consumedPerWrittenOn = {};
        consumptions.forEach(f => {
            if (!consumedPerWrittenOn[f.writtenOn]) {
                consumedPerWrittenOn[f.writtenOn] = [];
            }
            consumedPerWrittenOn[f.writtenOn].push(f.pub);
        });
        const writtenOns = underscore_1.Underscore.keys(consumedPerWrittenOn);
        await Promise.all(writtenOns.map(async (writtenOn) => {
            const existing = (await (this.indexForTrimming.getOrNull(LevelDBDividend.trimKey(writtenOn)))) || [];
            const toBeStored = underscore_1.Underscore.uniq(existing.concat(consumedPerWrittenOn[writtenOn]));
            await this.indexForTrimming.put(LevelDBDividend.trimKey(writtenOn), toBeStored);
        }));
    }
    async consume(filter) {
        for (const dividendToConsume of filter) {
            const row = await this.get(dividendToConsume.identifier);
            DividendDaoHandler_1.DividendDaoHandler.consume(row, dividendToConsume);
            await this.put(row.pub, row);
        }
        await this.indexConsumptions(filter.map(f => ({ writtenOn: f.writtenOn, pub: f.identifier })));
    }
    async createMember(pub) {
        const existing = await this.getOrNull(pub);
        if (!existing) {
            await this.insert(DividendDaoHandler_1.DividendDaoHandler.getNewDividendEntry(pub));
        }
        else {
            await this.setMember(true, pub);
        }
    }
    async deleteMember(pub) {
        await this.del(pub);
    }
    async findForDump(criterion) {
        const entries = [];
        await this.readAll(entry => entries.push(entry));
        return DividendDaoHandler_1.DividendDaoHandler.toDump(entries);
    }
    async findRawWithOrder(criterion, sort) {
        const entries = [];
        await this.readAll(entry => entries.push(entry));
        return entries;
    }
    async findUdSourceByIdentifierPosAmountBase(identifier, pos, amount, base) {
        const member = await this.get(identifier);
        return DividendDaoHandler_1.DividendDaoHandler.getUDSourceByIdPosAmountBase(member, identifier, pos, amount, base);
    }
    async getUDSource(identifier, pos) {
        const member = await this.get(identifier);
        return DividendDaoHandler_1.DividendDaoHandler.getUDSource(member, identifier, pos);
    }
    async getUDSources(pub) {
        const member = await this.getOrNull(pub);
        if (!member) {
            return [];
        }
        return DividendDaoHandler_1.DividendDaoHandler.udSources(member);
    }
    getWrittenOn(blockstamp) {
        throw Error(errors_1.DataErrors[errors_1.DataErrors.DIVIDEND_GET_WRITTEN_ON_SHOULD_NOT_BE_USED_DIVIDEND_DAO]);
    }
    async getWrittenOnUDs(number) {
        const res = [];
        await this.readAll(entry => {
            if (entry.member) {
                DividendDaoHandler_1.DividendDaoHandler.getWrittenOnUDs(entry, number, res);
            }
        });
        return res;
    }
    async produceDividend(blockNumber, dividend, unitbase, local_iindex) {
        const dividends = [];
        const updates = [];
        await this.readAll(entry => {
            if (entry.member) {
                DividendDaoHandler_1.DividendDaoHandler.produceDividend(entry, blockNumber, dividend, unitbase, dividends);
                updates.push(this.put(entry.pub, entry));
            }
        });
        await Promise.all(updates);
        return dividends;
    }
    removeBlock(blockstamp) {
        throw Error(errors_1.DataErrors[errors_1.DataErrors.DIVIDEND_REMOVE_BLOCK_SHOULD_NOT_BE_USED_BY_DIVIDEND_DAO]);
    }
    async revertUDs(number) {
        const createdUDsDestroyedByRevert = [];
        const consumedUDsRecoveredByRevert = [];
        const updates = [];
        // Remove produced dividends at this block
        await this.readAll(entry => {
            if (entry.availables.includes(number)) {
                DividendDaoHandler_1.DividendDaoHandler.removeDividendsProduced(entry, number, createdUDsDestroyedByRevert);
                updates.push(this.put(entry.pub, entry));
            }
            if (entry.consumed.includes(number)) {
                DividendDaoHandler_1.DividendDaoHandler.unconsumeDividends(entry, number, consumedUDsRecoveredByRevert);
                updates.push(this.put(entry.pub, entry));
            }
        });
        await Promise.all(updates);
        await this.indexForTrimming.del(LevelDBDividend.trimKey(number)); // TODO: test
        return {
            createdUDsDestroyedByRevert,
            consumedUDsRecoveredByRevert,
        };
    }
    async setMember(member, pub) {
        const entry = await this.get(pub);
        entry.member = member;
        await this.put(pub, entry);
    }
    async trimConsumedUDs(belowNumber) {
        const count = await this.indexForTrimming.count();
        if (count === 0 && !this.hasIndexed) {
            this.hasIndexed = true;
            await this.applyAllKeyValue(async (data) => {
                await this.indexConsumptions(data.value.consumed.map(c => ({ writtenOn: c, pub: data.value.pub })));
            });
        }
        const updates = [];
        const trimmedNumbers = [];
        // Remove produced dividends at this block
        await this.indexForTrimming.readAllKeyValue(kv => {
            updates.push((async () => {
                const pubkeys = kv.value;
                const trimNumber = kv.key;
                for (const pub of pubkeys) {
                    const entry = await this.get(pub);
                    if (DividendDaoHandler_1.DividendDaoHandler.trimConsumed(entry, belowNumber)) {
                        await this.put(entry.pub, entry);
                        trimmedNumbers.push(trimNumber);
                    }
                }
            })());
        }, {
            lt: LevelDBDividend.trimKey(belowNumber)
        });
        await Promise.all(updates);
        await Promise.all(trimmedNumbers.map(trimKey => this.indexForTrimming.del(trimKey)));
    }
    async listAll() {
        const entries = [];
        await this.readAll(entry => entries.push(entry));
        return entries;
    }
    static trimKey(writtenOn) {
        return String(writtenOn).padStart(10, '0');
    }
    async count(options) {
        let count = 0;
        await this.readAllKeyValue(entry => {
            count += entry.value.availables.length;
        });
        return count;
    }
}
__decorate([
    MonitorExecutionTime_1.MonitorExecutionTime()
], LevelDBDividend.prototype, "insert", null);
__decorate([
    MonitorExecutionTime_1.MonitorExecutionTime()
], LevelDBDividend.prototype, "insertBatch", null);
exports.LevelDBDividend = LevelDBDividend;
//# sourceMappingURL=LevelDBDividend.js.map