"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
Object.defineProperty(exports, "__esModule", { value: true });
const MonitorExecutionTime_1 = require("../../../debug/MonitorExecutionTime");
const indexer_1 = require("../../../indexer");
const LevelDBTable_1 = require("./LevelDBTable");
const underscore_1 = require("../../../common-libs/underscore");
const pint_1 = require("../../../common-libs/pint");
const LevelIIndexHashIndexer_1 = require("./indexers/LevelIIndexHashIndexer");
const reduce_1 = require("../../../common-libs/reduce");
const LevelDBWrittenOnIndexer_1 = require("./indexers/LevelDBWrittenOnIndexer");
const LevelIIndexUidIndexer_1 = require("./indexers/LevelIIndexUidIndexer");
const LevelIIndexKickIndexer_1 = require("./indexers/LevelIIndexKickIndexer");
const errors_1 = require("../../../common-libs/errors");
const OldTransformer_1 = require("../common/OldTransformer");
class LevelDBIindex extends LevelDBTable_1.LevelDBTable {
    constructor(getLevelDB) {
        super('level_iindex', getLevelDB);
        this.getLevelDB = getLevelDB;
    }
    /**
     * TECHNICAL
     */
    async init() {
        await super.init();
        this.indexForHash = new LevelIIndexHashIndexer_1.LevelIIndexHashIndexer('level_iindex/hash', this.getLevelDB);
        this.indexForUid = new LevelIIndexUidIndexer_1.LevelIIndexUidIndexer('level_iindex/uid', this.getLevelDB);
        this.indexForKick = new LevelIIndexKickIndexer_1.LevelIIndexKickIndexer('level_iindex/kick', this.getLevelDB);
        this.indexForWrittenOn = new LevelDBWrittenOnIndexer_1.LevelDBWrittenOnIndexer('level_iindex/writtenOn', this.getLevelDB, i => i.pub);
        await this.indexForHash.init();
        await this.indexForUid.init();
        await this.indexForKick.init();
        await this.indexForWrittenOn.init();
    }
    async close() {
        await super.close();
        await this.indexForHash.close();
        await this.indexForUid.close();
        await this.indexForKick.close();
        await this.indexForWrittenOn.close();
    }
    /**
     * INSERT
     */
    async insert(record) {
        await this.insertBatch([record]);
    }
    async insertBatch(records) {
        // Database insertion
        const recordsByPub = reduce_1.reduceGroupBy(records, 'pub');
        await Promise.all(underscore_1.Underscore.keys(recordsByPub).map(String).map(async (pub) => {
            const existing = (await this.getOrNull(pub)) || [];
            await this.put(pub, existing.concat(recordsByPub[pub]));
        }));
        // Indexation
        await this.indexForHash.onInsert(records);
        await this.indexForUid.onInsert(records);
        await this.indexForKick.onInsert(records);
        await this.indexForWrittenOn.onInsert(records);
    }
    /**
     * Reduceable DAO
     */
    async trimRecords(belowNumber) {
        // Trim writtenOn: we remove from the index the blocks below `belowNumber`, and keep track of the deleted values
        const pubkeys = underscore_1.Underscore.uniq((await this.indexForWrittenOn.deleteBelow(belowNumber)));
        // For each entry, we trim the records of our INDEX
        await Promise.all(pubkeys.map(async (pub) => {
            const oldEntries = await this.get(pub);
            const newEntries = indexer_1.reduceForDBTrimming(oldEntries, belowNumber);
            await this.put(pub, newEntries);
        }));
        await this.indexForHash.onTrimming(belowNumber);
        await this.indexForUid.onTrimming(belowNumber);
        await this.indexForKick.onTrimming(belowNumber);
    }
    /**
     * Generic DAO
     */
    async findRawWithOrder(criterion, sort) {
        const rows = (await this.findAllValues()).reduce(reduce_1.reduceConcat, []);
        return underscore_1.Underscore.sortBy(rows, r => `${String(r.writtenOn).padStart(10, '0')}-${String(r.wotb_id).padStart(10, '0')}`);
    }
    async getWrittenOn(blockstamp) {
        const ids = (await this.indexForWrittenOn.getWrittenOnKeys(pint_1.pint(blockstamp))) || [];
        return (await Promise.all(ids.map(id => this.get(id)))).reduce(reduce_1.reduceConcat, []).filter(e => e.written_on === blockstamp);
    }
    async removeBlock(blockstamp) {
        // Trim writtenOn: we remove from the index the blocks below `belowNumber`, and keep track of the deleted values
        const writteOn = pint_1.pint(blockstamp);
        const pubkeys = underscore_1.Underscore.uniq((await this.indexForWrittenOn.deleteAt(writteOn)));
        let removedRecords = [];
        // For each entry, we trim the records of our INDEX
        await Promise.all(pubkeys.map(async (pub) => {
            const records = await this.get(pub);
            const keptRecords = records.filter(e => e.written_on !== blockstamp);
            removedRecords = removedRecords.concat(records.filter(e => e.written_on === blockstamp));
            await this.put(pub, keptRecords);
        }));
        // Update indexes
        await this.indexForHash.onRemove(removedRecords);
        await this.indexForUid.onRemove(removedRecords);
        await this.indexForKick.onRemove(removedRecords);
    }
    async findByPub(pub) {
        if (!pub) {
            return [];
        }
        return (await this.getOrNull(pub)) || [];
    }
    async findByUid(uid) {
        const pub = await this.indexForUid.getPubByUid(uid);
        if (!pub) {
            return [];
        }
        return this.get(pub);
    }
    async getFromPubkey(pub) {
        const entries = (await this.getOrNull(pub)) || [];
        if (!entries || entries.length === 0) {
            return null;
        }
        return indexer_1.reduce(entries);
    }
    async getFromPubkeyOrUid(search) {
        const fromPub = await this.getFromPubkey(search);
        const fromUid = await this.getFromUID(search);
        return fromPub || fromUid;
    }
    async getFromUID(uid) {
        const pub = await this.indexForUid.getPubByUid(uid);
        if (!pub) {
            return null;
        }
        const entries = (await this.getOrNull(pub)) || [];
        if (!entries || entries.length === 0) {
            return null;
        }
        return indexer_1.reduce(entries);
    }
    async getFullFromHash(hash) {
        const pub = await this.indexForHash.getByHash(hash);
        if (!pub) {
            return null;
        }
        const entries = await this.get(pub);
        return OldTransformer_1.OldTransformers.iindexEntityOrNull(entries);
    }
    async getFullFromPubkey(pub) {
        const entries = await this.get(pub);
        return indexer_1.reduce(entries);
    }
    async getFullFromUID(uid) {
        const pub = await this.indexForUid.getPubByUid(uid);
        if (!pub) {
            throw Error(errors_1.DataErrors[errors_1.DataErrors.IDENTITY_UID_NOT_FOUND]);
        }
        const entries = await this.get(pub);
        return indexer_1.reduce(entries);
    }
    // Full scan
    async getMembers() {
        const members = [];
        await this.findWhere(e => {
            if (indexer_1.reduce(e).member) {
                members.push(e[0]);
            }
            return false;
        });
        return members.map(m => ({
            pubkey: m.pub,
            uid: m.uid
        }));
    }
    async getToBeKickedPubkeys() {
        return this.indexForKick.getAll();
    }
    async reducable(pub) {
        return this.findByPub(pub);
    }
    // Full scan
    async searchThoseMatching(search) {
        const uidKeys = await this.indexForUid.findAllKeys();
        const pubKeys = await this.findAllKeys();
        const uids = (uidKeys).filter(u => u.includes(search));
        const pubs = (pubKeys).filter(p => p.includes(search));
        const uidIdentities = await Promise.all(uids.map(async (uid) => OldTransformer_1.OldTransformers.toOldIindexEntry(indexer_1.reduce(await this.findByUid(uid)))));
        const pubIdentities = await Promise.all(pubs.map(async (pub) => OldTransformer_1.OldTransformers.toOldIindexEntry(indexer_1.reduce(await this.findByPub(pub)))));
        return uidIdentities
            .filter(u => u.pub)
            .concat(pubIdentities
            .filter(p => p.pub));
    }
}
__decorate([
    MonitorExecutionTime_1.MonitorExecutionTime()
], LevelDBIindex.prototype, "insert", null);
__decorate([
    MonitorExecutionTime_1.MonitorExecutionTime()
], LevelDBIindex.prototype, "insertBatch", null);
exports.LevelDBIindex = LevelDBIindex;
//# sourceMappingURL=LevelDBIindex.js.map