import { FullSindexEntry, SimpleTxEntryForWallet, SimpleTxInput, SindexEntry } from "../../../indexer";
import { LevelUp } from 'levelup';
import { LevelDBTable } from "./LevelDBTable";
import { SIndexDAO } from "../abstract/SIndexDAO";
export declare class LevelDBSindex extends LevelDBTable<SindexEntry> implements SIndexDAO {
    protected getLevelDB: (dbName: string) => Promise<LevelUp>;
    private indexForTrimming;
    private indexForConsumed;
    private indexForConditions;
    constructor(getLevelDB: (dbName: string) => Promise<LevelUp>);
    /**
     * TECHNICAL
     */
    init(): Promise<void>;
    close(): Promise<void>;
    /**
     * INSERT
     */
    insert(record: SindexEntry): Promise<void>;
    insertBatch(records: SindexEntry[]): Promise<void>;
    findByIdentifier(identifier: string): Promise<SindexEntry[]>;
    findByIdentifierAndPos(identifier: string, pos: number): Promise<SindexEntry[]>;
    findByPos(pos: number): Promise<SindexEntry[]>;
    findTxSourceByIdentifierPosAmountBase(identifier: string, pos: number, amount: number, base: number): Promise<SimpleTxInput[]>;
    getAvailableForConditions(conditionsStr: string): Promise<SindexEntry[]>;
    getAvailableForPubkey(pubkey: string): Promise<{
        amount: number;
        base: number;
        conditions: string;
        identifier: string;
        pos: number;
    }[]>;
    getTxSource(identifier: string, pos: number): Promise<FullSindexEntry | null>;
    getWrittenOnTxs(blockstamp: string): Promise<SimpleTxEntryForWallet[]>;
    trimConsumedSource(belowNumber: number): Promise<void>;
    /**
     * Reduceable DAO
     */
    trimRecords(belowNumber: number): Promise<void>;
    /**
     * Generic DAO
     */
    findRawWithOrder(criterion: {
        pub?: string;
    }, sort: (string | (string | boolean)[])[]): Promise<SindexEntry[]>;
    getWrittenOn(blockstamp: string): Promise<SindexEntry[]>;
    getForConditions(conditions: string): Promise<SindexEntry[]>;
    removeBlock(blockstamp: string): Promise<void>;
    private trimConditions;
    /**
     * Duplicate with trimConditions?!
     * @param writtenOn
     * @param id
     */
    private trimWrittenOn;
    private trimConsumed;
    private getWrittenOnSourceIds;
    private static trimKey;
    private static trimWrittenOnKey;
    private static trimPartialKey;
    static upperIdentifier(identifier: string): string;
    private indexRecords;
}
