"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
Object.defineProperty(exports, "__esModule", { value: true });
const MonitorExecutionTime_1 = require("../../../debug/MonitorExecutionTime");
const indexer_1 = require("../../../indexer");
const LevelDBTable_1 = require("./LevelDBTable");
const underscore_1 = require("../../../common-libs/underscore");
const pint_1 = require("../../../common-libs/pint");
const array_prune_1 = require("../../../common-libs/array-prune");
class LevelDBSindex extends LevelDBTable_1.LevelDBTable {
    constructor(getLevelDB) {
        super('level_sindex', getLevelDB);
        this.getLevelDB = getLevelDB;
    }
    /**
     * TECHNICAL
     */
    async init() {
        await super.init();
        this.indexForTrimming = new LevelDBTable_1.LevelDBTable('level_sindex/written_on', this.getLevelDB);
        this.indexForConsumed = new LevelDBTable_1.LevelDBTable('level_sindex/consumed_on', this.getLevelDB);
        this.indexForConditions = new LevelDBTable_1.LevelDBTable('level_sindex/conditions', this.getLevelDB);
        await this.indexForTrimming.init();
        await this.indexForConsumed.init();
        await this.indexForConditions.init();
    }
    async close() {
        await super.close();
        await this.indexForTrimming.close();
        await this.indexForConsumed.close();
        await this.indexForConditions.close();
    }
    /**
     * INSERT
     */
    async insert(record) {
        await this.insertBatch([record]);
    }
    async insertBatch(records) {
        await this.batchInsertWithKeyComputing(records, r => {
            return LevelDBSindex.trimKey(r.identifier, r.pos, r.consumed);
        });
        await this.indexRecords(records);
    }
    findByIdentifier(identifier) {
        return this.findAllValues({
            gte: identifier,
            lt: LevelDBSindex.upperIdentifier(identifier)
        });
    }
    findByIdentifierAndPos(identifier, pos) {
        return this.findAllValues({
            gte: LevelDBSindex.trimPartialKey(identifier, pos),
            lt: LevelDBSindex.upperIdentifier(LevelDBSindex.trimPartialKey(identifier, pos))
        });
    }
    // Not used by the protocol: we can accept a full scan
    async findByPos(pos) {
        return (await this.findAllValues()).filter(r => r.pos === pos);
    }
    async findTxSourceByIdentifierPosAmountBase(identifier, pos, amount, base) {
        return (await this.findByIdentifier(identifier))
            .filter(r => r.pos === pos && r.amount === amount && r.base === base);
    }
    async getAvailableForConditions(conditionsStr) {
        const forConditions = await this.getForConditions(conditionsStr);
        const reduced = indexer_1.Indexer.DUP_HELPERS.reduceBy(forConditions, ['identifier', 'pos']);
        return reduced.filter(r => !r.consumed);
    }
    async getAvailableForPubkey(pubkey) {
        // TODO: very costly: needs a full scan, would be better to change this implementatio
        const entries = await this.findWhere(e => e.conditions.includes(`SIG(${pubkey})`));
        const reduced = indexer_1.Indexer.DUP_HELPERS.reduceBy(entries, ['identifier', 'pos']);
        return reduced.filter(r => !r.consumed);
    }
    async getTxSource(identifier, pos) {
        const entries = (await this.findByIdentifierAndPos(identifier, pos));
        return indexer_1.Indexer.DUP_HELPERS.reduceOrNull(entries);
    }
    async getWrittenOnTxs(blockstamp) {
        const writtenOn = await this.getWrittenOn(blockstamp);
        const entries = [];
        writtenOn.forEach(w => {
            entries.push({
                srcType: 'T',
                op: w.op,
                conditions: w.conditions,
                amount: w.amount,
                base: w.base,
                identifier: w.identifier,
                pos: w.pos,
            });
        });
        return entries;
    }
    async trimConsumedSource(belowNumber) {
        let belowNumberIds = [];
        const mapIds = {};
        const mapIds2WrittenOn = {};
        // First: we look at what was written before `belowNumber`
        await this.indexForConsumed.readAllKeyValue(async (kv) => {
            belowNumberIds = belowNumberIds.concat(kv.value);
            for (const id of kv.value) {
                mapIds2WrittenOn[id] = pint_1.pint(kv.key);
            }
        }, {
            lt: LevelDBSindex.trimWrittenOnKey(belowNumber)
        });
        // Second: we identify the corresponding **consumed** sources and remove them.
        for (const id of belowNumberIds) {
            // Remove consumed sources
            const identifier = id.split('-')[0];
            const pos = pint_1.pint(id.split('-')[1]);
            const entry = await this.getOrNull(LevelDBSindex.trimKey(identifier, pos, true));
            if (entry && entry.writtenOn < belowNumber) {
                // We remember the trimmed source id to remove it from the writtenOn and conditions index
                mapIds[id] = {
                    writtenOn: mapIds2WrittenOn[id],
                    conditions: entry.conditions
                };
                await this.del(LevelDBSindex.trimKey(identifier, pos, false));
                await this.del(LevelDBSindex.trimKey(identifier, pos, true));
            }
        }
        // We update indexes
        for (const id of underscore_1.Underscore.keys(mapIds).map(String)) {
            const map = mapIds[id];
            await this.trimConditions(map.conditions, id);
            await this.trimConsumed(map.writtenOn, id);
            await this.trimWrittenOn(map.writtenOn, id);
        }
    }
    /**
     * Reduceable DAO
     */
    trimRecords(belowNumber) {
        return this.trimConsumedSource(belowNumber);
    }
    /**
     * Generic DAO
     */
    async findRawWithOrder(criterion, sort) {
        const rows = await this.findAllValues();
        return underscore_1.Underscore.sortBy(rows, r => 1000 * r.writtenOn + (r.consumed ? 1 : 0));
    }
    async getWrittenOn(blockstamp) {
        const ids = underscore_1.Underscore.uniq((await this.indexForTrimming.getOrNull(LevelDBSindex.trimWrittenOnKey(pint_1.pint(blockstamp)))) || []);
        const found = [];
        for (const id of ids) {
            const entries = await this.findByIdentifierAndPos(id.split('-')[0], pint_1.pint(id.split('-')[1]));
            entries
                .filter(e => e.written_on === blockstamp)
                .forEach(e => found.push(e));
        }
        return found;
    }
    async getForConditions(conditions) {
        const ids = (await this.indexForConditions.getOrNull(conditions)) || [];
        const found = [];
        for (const id of ids) {
            const entries = await this.findByIdentifierAndPos(id.split('-')[0], pint_1.pint(id.split('-')[1]));
            entries.forEach(e => found.push(e));
        }
        return found;
    }
    async removeBlock(blockstamp) {
        const writtenOn = pint_1.pint(blockstamp);
        // We look at records written on this blockstamp: `indexForTrimming` allows to get them
        const ids = (await this.indexForTrimming.getOrNull(LevelDBSindex.trimWrittenOnKey(writtenOn))) || [];
        // `ids` contains both CREATE and UPDATE sources
        for (const id of ids) {
            // Remove sources
            const identifier = id.split('-')[0];
            const pos = parseInt(id.split('-')[1]);
            const conditions = [];
            const createKey = LevelDBSindex.trimKey(identifier, pos, false);
            const updateKey = LevelDBSindex.trimKey(identifier, pos, true);
            const createRecord = await this.getOrNull(createKey);
            const updateRecord = await this.getOrNull(updateKey);
            // Undo consumption
            if (updateRecord && updateRecord.writtenOn === writtenOn) {
                conditions.push(updateRecord.conditions);
                await this.del(updateKey);
            }
            // Undo creation?
            if (createRecord && createRecord.writtenOn === writtenOn) {
                conditions.push(createRecord.conditions);
                await this.del(createKey);
            }
            // Update balance
            // 1. Conditions
            const uniqConditions = underscore_1.Underscore.uniq(conditions);
            for (const condition of uniqConditions) {
                // Remove this source from the balance
                await this.trimConditions(condition, id);
            }
        }
        if (ids.length) {
            // 2. WrittenOn
            await this.indexForTrimming.del(LevelDBSindex.trimWrittenOnKey(writtenOn));
            await this.indexForConsumed.del(LevelDBSindex.trimWrittenOnKey(writtenOn));
        }
    }
    async trimConditions(condition, id) {
        // Get all the account's TX sources
        const existing = (await this.indexForConditions.getOrNull(condition)) || [];
        // Prune the source from the account
        const trimmed = array_prune_1.arrayPruneAllCopy(existing, id);
        if (trimmed.length) {
            // If some sources are left for this "account", persist what remains
            await this.indexForConditions.put(condition, trimmed);
        }
        else {
            // Otherwise just delete the "account"
            await this.indexForConditions.del(condition);
        }
    }
    /**
     * Duplicate with trimConditions?!
     * @param writtenOn
     * @param id
     */
    async trimWrittenOn(writtenOn, id) {
        const k = LevelDBSindex.trimWrittenOnKey(writtenOn);
        const existing = await this.getWrittenOnSourceIds(writtenOn);
        const trimmed = array_prune_1.arrayPruneAllCopy(existing, id);
        if (trimmed.length) {
            await this.indexForConditions.put(k, trimmed);
        }
        else {
            await this.indexForConditions.del(k);
        }
    }
    async trimConsumed(writtenOn, id) {
        const k = LevelDBSindex.trimWrittenOnKey(writtenOn);
        const existing = (await this.indexForConsumed.getOrNull(k)) || [];
        const trimmed = array_prune_1.arrayPruneAllCopy(existing, id);
        if (trimmed.length) {
            await this.indexForConsumed.put(k, trimmed);
        }
        else {
            await this.indexForConsumed.del(k);
        }
    }
    async getWrittenOnSourceIds(writtenOn) {
        const indexForTrimmingId = LevelDBSindex.trimWrittenOnKey(writtenOn);
        return (await this.indexForTrimming.getOrNull(indexForTrimmingId)) || [];
    }
    static trimKey(identifier, pos, consumed) {
        return `${identifier}-${String(pos).padStart(10, '0')}-${consumed ? 1 : 0}`;
    }
    static trimWrittenOnKey(writtenOn) {
        return String(writtenOn).padStart(10, '0');
    }
    static trimPartialKey(identifier, pos) {
        return `${identifier}-${String(pos).padStart(10, '0')}`;
    }
    static upperIdentifier(identifier) {
        let indexOfLastNonFletter = identifier.length - 1;
        let nextLastLetter = String.fromCharCode(identifier.charCodeAt(indexOfLastNonFletter) + 1);
        // We only use 0-9A-G notation
        if (nextLastLetter === ':') {
            nextLastLetter = 'A';
        }
        return identifier.substr(0, indexOfLastNonFletter)
            + nextLastLetter
            + identifier.substr(indexOfLastNonFletter + 1);
    }
    async indexRecords(records) {
        const byConsumed = {};
        const byWrittenOn = {};
        const byConditions = {};
        records
            .filter(r => r.consumed)
            .forEach(r => {
            // WrittenOn consumed
            let arrConsumed = byConsumed[r.writtenOn];
            if (!arrConsumed) {
                arrConsumed = byConsumed[r.writtenOn] = [];
            }
            arrConsumed.push(r);
        });
        records.forEach(r => {
            // WrittenOn
            let arrWO = byWrittenOn[r.writtenOn];
            if (!arrWO) {
                arrWO = byWrittenOn[r.writtenOn] = [];
            }
            arrWO.push(r);
            // Conditiosn
            let arrCN = byConditions[r.conditions];
            if (!arrCN) {
                arrCN = byConditions[r.conditions] = [];
            }
            arrCN.push(r);
        });
        // Index consumed => (identifier + pos)[]
        for (const k of underscore_1.Underscore.keys(byConsumed)) {
            await this.indexForConsumed.put(LevelDBSindex.trimWrittenOnKey(k), byConsumed[k].map(r => LevelDBSindex.trimPartialKey(r.identifier, r.pos)));
        }
        // Index writtenOn => (identifier + pos)[]
        for (const k of underscore_1.Underscore.keys(byWrittenOn)) {
            await this.indexForTrimming.put(LevelDBSindex.trimWrittenOnKey(k), byWrittenOn[k].map(r => LevelDBSindex.trimPartialKey(r.identifier, r.pos)));
        }
        // Index conditions => (identifier + pos)[]
        for (const k of underscore_1.Underscore.keys(byConditions).map(String)) {
            const existing = (await this.indexForConditions.getOrNull(k)) || [];
            const newSources = byConditions[k].map(r => LevelDBSindex.trimPartialKey(r.identifier, r.pos));
            await this.indexForConditions.put(k, underscore_1.Underscore.uniq(existing.concat(newSources)));
        }
    }
}
__decorate([
    MonitorExecutionTime_1.MonitorExecutionTime()
], LevelDBSindex.prototype, "insert", null);
__decorate([
    MonitorExecutionTime_1.MonitorExecutionTime()
], LevelDBSindex.prototype, "insertBatch", null);
exports.LevelDBSindex = LevelDBSindex;
//# sourceMappingURL=LevelDBSindex.js.map