"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const reduce_1 = require("../../../../common-libs/reduce");
const underscore_1 = require("../../../../common-libs/underscore");
const LevelDBTable_1 = require("../LevelDBTable");
const pint_1 = require("../../../../common-libs/pint");
class LevelDBWrittenOnIndexer extends LevelDBTable_1.LevelDBTable {
    constructor(name, getLevelDB, toKey) {
        super(name, getLevelDB);
        this.toKey = toKey;
    }
    async onInsert(records) {
        const byWrittenOn = reduce_1.reduceGroupBy(records, 'writtenOn');
        await Promise.all(underscore_1.Underscore.keys(byWrittenOn).map(async (writtenOn) => {
            await this.put(LevelDBWrittenOnIndexer.trimWrittenOnKey(pint_1.pint(writtenOn)), byWrittenOn[writtenOn].map(e => this.toKey(e)));
        }));
    }
    getWrittenOnKeys(writtenOn) {
        return this.getOrNull(LevelDBWrittenOnIndexer.trimWrittenOnKey(writtenOn));
    }
    trim(writtenOn) {
        return this.del(LevelDBWrittenOnIndexer.trimWrittenOnKey(writtenOn));
    }
    static trimWrittenOnKey(writtenOn) {
        return String(writtenOn).padStart(10, '0');
    }
    async deleteBelow(writtenOn) {
        return (await this.deleteWhere({ lt: LevelDBWrittenOnIndexer.trimWrittenOnKey(writtenOn) }))
            .map(kv => kv.value)
            .reduce(reduce_1.reduceConcat, []);
    }
    async deleteAt(writtenOn) {
        const k = LevelDBWrittenOnIndexer.trimWrittenOnKey(writtenOn);
        const value = await this.getOrNull(k);
        if (!value) {
            // Nothing to delete, nothing to return
            return [];
        }
        await this.del(k);
        return value;
    }
}
exports.LevelDBWrittenOnIndexer = LevelDBWrittenOnIndexer;
//# sourceMappingURL=LevelDBWrittenOnIndexer.js.map