"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const LevelDBDataIndex_1 = require("../generic/LevelDBDataIndex");
const errors_1 = require("../../../../common-libs/errors");
class LevelIIndexKickIndexer extends LevelDBDataIndex_1.LevelDBDataIndex {
    async onInsert(records) {
        // Case 1: to be kicked
        await Promise.all(records
            .filter(e => e.kick)
            .map(async (e) => {
            let entry = await this.getOrNull(e.pub);
            if (!entry) {
                entry = {
                    on: e.writtenOn,
                    done: []
                };
            }
            entry.on = e.writtenOn;
            await this.put(e.pub, entry);
        }));
        // Case 2: just kicked
        await Promise.all(records
            .filter(e => e.member === false)
            .map(async (e) => {
            const entry = await this.getOrNull(e.pub);
            if (entry && entry.on === e.writtenOn - 1) { // Members are excluded at B# +1
                entry.done.push(entry.on);
                entry.on = undefined;
                await this.put(e.pub, entry);
            }
            // Otherwise it is not a kicking
        }));
    }
    async onRemove(records) {
        // Case 1: to be kicked => unkicked
        await Promise.all(records
            .filter(e => e.kick)
            .map(async (e) => {
            const entry = await this.get(e.pub);
            if (entry.on === e.writtenOn) {
                entry.on = entry.done.pop();
                if (entry.on === undefined) {
                    // No more kicking left
                    await this.del(e.pub);
                }
                // Some kicks left
                await this.put(e.pub, entry); // TODO: test this, can occur, probably not covered
            }
            else {
                throw Error(errors_1.DataErrors[errors_1.DataErrors.INVALID_LEVELDB_IINDEX_DATA_TO_BE_KICKED]);
            }
        }));
        // Case 2: just kicked => to be kicked
        await Promise.all(records
            .filter(e => e.member === false)
            .map(async (e) => {
            const entry = await this.getOrNull(e.pub);
            if (entry && entry.done.includes(e.writtenOn - 1)) {
                // It was a kicking
                entry.on = entry.done.pop();
                if (!entry.on) {
                    throw Error(errors_1.DataErrors[errors_1.DataErrors.INVALID_LEVELDB_IINDEX_DATA_WAS_KICKED]);
                }
                await this.put(e.pub, entry);
            }
        }));
    }
    async onTrimming(belowNumber) {
        await this.applyAllKeyValue(async (kv) => {
            const initialLength = kv.value.done.length;
            kv.value.done = kv.value.done.filter(e => e >= belowNumber);
            if (kv.value.done.length !== initialLength && kv.value.done.length > 0) {
                // We simply update the entry which was pruned
                await this.put(kv.key, kv.value);
            }
            else if (kv.value.done.length !== initialLength && kv.value.done.length === 0 && !kv.value.on) {
                // We remove the entry, no more necessary
                await this.del(kv.key);
            }
        });
    }
    async getAll() {
        return this.findWhereTransform(t => !!t.on, kv => kv.key);
    }
}
exports.LevelIIndexKickIndexer = LevelIIndexKickIndexer;
//# sourceMappingURL=LevelIIndexKickIndexer.js.map