"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const LevelDBDataIndex_1 = require("../../generic/LevelDBDataIndex");
const errors_1 = require("../../../../../common-libs/errors");
class LevelIndexBlock extends LevelDBDataIndex_1.LevelDBDataIndex {
    async onInsert(records) {
        const recordsByBlock = records
            .filter(this.matches)
            .map(b => ({
            keys: this.keys(b),
            b
        }));
        const map = {};
        recordsByBlock.forEach(m => {
            m.keys.forEach(k => {
                if (!map[k]) {
                    map[k] = [];
                }
                map[k].push(m.b.number);
            });
        });
        await Promise.all(Object.keys(map)
            .map(k => this.indexIt(k, map[k])));
    }
    async onRemove(records) {
        await Promise.all(records
            .filter(this.matches)
            .map(async (b) => Promise.all(this
            .keys(b)
            .map(i => this.unindexIt(i, b.number)))));
    }
    async indexIt(pub, newNumbers) {
        const blockNumbers = (await this.getOrNull(pub)) || [];
        newNumbers.forEach(n => blockNumbers.push(n));
        await this.put(pub, blockNumbers);
    }
    async unindexIt(pub, blockNumber) {
        const blockNumbers = (await this.getOrNull(pub));
        if (!blockNumbers) {
            throw errors_1.DataErrors[errors_1.DataErrors.DB_INCORRECT_INDEX];
        }
        const index = blockNumbers.indexOf(blockNumber);
        if (index === -1) {
            throw errors_1.DataErrors[errors_1.DataErrors.DB_INDEXED_BLOCK_NOT_FOUND];
        }
        blockNumbers.splice(index, 1);
        if (blockNumbers.length) {
            await this.put(pub, blockNumbers);
        }
        else {
            await this.del(pub);
        }
    }
}
exports.LevelIndexBlock = LevelIndexBlock;
exports.LDBIndex_ALL = 'ALL';
//# sourceMappingURL=LevelIndexBlock.js.map