"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
Object.defineProperty(exports, "__esModule", { value: true });
const MonitorExecutionTime_1 = require("../../../debug/MonitorExecutionTime");
const SqliteTable_1 = require("./SqliteTable");
const SqlFieldDefinition_1 = require("./SqlFieldDefinition");
const DividendDaoHandler_1 = require("../common/DividendDaoHandler");
const errors_1 = require("../../../common-libs/errors");
class SqliteDividend extends SqliteTable_1.SqliteTable {
    constructor(getSqliteDB) {
        super('dividend', {
            'pub': new SqlFieldDefinition_1.SqlNotNullableFieldDefinition('VARCHAR', true, 50),
            'member': new SqlFieldDefinition_1.SqlNotNullableFieldDefinition('BOOLEAN', true),
            'availables': new SqlFieldDefinition_1.SqlNotNullableFieldDefinition('JSON', false),
            'consumed': new SqlFieldDefinition_1.SqlNotNullableFieldDefinition('JSON', false),
            'consumedUDs': new SqlFieldDefinition_1.SqlNotNullableFieldDefinition('JSON', false),
            'dividends': new SqlFieldDefinition_1.SqlNotNullableFieldDefinition('JSON', false),
        }, getSqliteDB);
    }
    /**
     * TECHNICAL
     */
    cleanCache() {
    }
    triggerInit() {
    }
    /**
     * INSERT
     */
    async insert(record) {
        await this.insertInTable(this.driver, record);
    }
    async insertBatch(records) {
        if (records.length) {
            return this.insertBatchInTable(this.driver, records);
        }
    }
    async find(sql, params) {
        return (await this.driver.sqlRead(sql, params)).map(r => {
            return {
                pub: r.pub,
                member: r.member,
                availables: r.availables == null ? null : JSON.parse(r.availables),
                consumed: r.consumed == null ? null : JSON.parse(r.consumed),
                consumedUDs: r.consumedUDs == null ? null : JSON.parse(r.consumedUDs),
                dividends: r.dividends == null ? null : JSON.parse(r.dividends),
            };
        });
    }
    async consume(filter) {
        for (const dividendToConsume of filter) {
            const row = (await this.find('SELECT * FROM dividend WHERE pub = ?', [dividendToConsume.identifier]))[0];
            DividendDaoHandler_1.DividendDaoHandler.consume(row, dividendToConsume);
            await this.update(this.driver, row, ['consumed', 'consumedUDs', 'availables', 'dividends'], ['pub']);
        }
    }
    async createMember(pub) {
        const existing = (await this.find('SELECT * FROM dividend WHERE pub = ?', [pub]))[0];
        if (!existing) {
            await this.insert(DividendDaoHandler_1.DividendDaoHandler.getNewDividendEntry(pub));
        }
        else {
            await this.setMember(true, pub);
        }
    }
    deleteMember(pub) {
        return this.driver.sqlWrite('DELETE FROM dividend WHERE pub = ?', [pub]);
    }
    async findForDump(criterion) {
        return DividendDaoHandler_1.DividendDaoHandler.toDump(await this.find('SELECT * FROM dividend', []));
    }
    findRawWithOrder(criterion, sort) {
        let sql = `SELECT * FROM dividend ${criterion.pub ? 'WHERE pub = ?' : ''}`;
        if (sort.length) {
            sql += ` ORDER BY ${sort.map(s => `${s[0]} ${s[1] ? 'DESC' : 'ASC'}`).join(', ')}`;
        }
        return this.find(sql, criterion.pub ? [criterion.pub] : []);
    }
    async findUdSourceByIdentifierPosAmountBase(identifier, pos, amount, base) {
        const member = (await this.find('SELECT * FROM dividend WHERE pub = ?', [identifier]))[0];
        return DividendDaoHandler_1.DividendDaoHandler.getUDSourceByIdPosAmountBase(member, identifier, pos, amount, base);
    }
    async getUDSource(identifier, pos) {
        const member = (await this.find('SELECT * FROM dividend WHERE pub = ?', [identifier]))[0];
        return DividendDaoHandler_1.DividendDaoHandler.getUDSource(member, identifier, pos);
    }
    async getUDSources(pub) {
        const member = (await this.find('SELECT * FROM dividend WHERE pub = ?', [pub]))[0];
        if (!member) {
            return [];
        }
        return DividendDaoHandler_1.DividendDaoHandler.udSources(member);
    }
    getWrittenOn(blockstamp) {
        throw Error(errors_1.DataErrors[errors_1.DataErrors.DIVIDEND_GET_WRITTEN_ON_SHOULD_NOT_BE_USED_DIVIDEND_DAO]);
    }
    async getWrittenOnUDs(number) {
        const res = [];
        const rows = await this.find('SELECT * FROM dividend WHERE member', []);
        for (const row of rows) {
            DividendDaoHandler_1.DividendDaoHandler.getWrittenOnUDs(row, number, res);
        }
        return res;
    }
    async produceDividend(blockNumber, dividend, unitbase, local_iindex) {
        const dividends = [];
        const rows = await this.find('SELECT * FROM dividend WHERE member', []);
        for (const row of rows) {
            DividendDaoHandler_1.DividendDaoHandler.produceDividend(row, blockNumber, dividend, unitbase, dividends);
            await this.update(this.driver, row, ['availables', 'dividends'], ['pub']);
        }
        return dividends;
    }
    removeBlock(blockstamp) {
        throw Error(errors_1.DataErrors[errors_1.DataErrors.DIVIDEND_REMOVE_BLOCK_SHOULD_NOT_BE_USED_BY_DIVIDEND_DAO]);
    }
    async revertUDs(number) {
        const createdUDsDestroyedByRevert = [];
        const consumedUDsRecoveredByRevert = [];
        // Remove produced dividends at this block
        const rows = await this.find('SELECT * FROM dividend WHERE availables like ? or dividends like ?', ['%' + number + '%', '%' + number + '%']);
        for (const row of rows.filter(row => row.availables.includes(number))) {
            DividendDaoHandler_1.DividendDaoHandler.removeDividendsProduced(row, number, createdUDsDestroyedByRevert);
            await this.update(this.driver, row, ['availables', 'dividends'], ['pub']);
        }
        // Unconsumed dividends consumed at this block
        for (const row of rows.filter(row => row.consumed.includes(number))) {
            DividendDaoHandler_1.DividendDaoHandler.unconsumeDividends(row, number, consumedUDsRecoveredByRevert);
            await this.update(this.driver, row, ['availables', 'dividends'], ['pub']);
        }
        return {
            createdUDsDestroyedByRevert,
            consumedUDsRecoveredByRevert,
        };
    }
    async setMember(member, pub) {
        await this.driver.sqlWrite('UPDATE dividend SET member = ? WHERE pub = ?', [true, pub]);
    }
    async trimConsumedUDs(belowNumber) {
        const rows = await this.find('SELECT * FROM dividend', []);
        for (const row of rows) {
            if (DividendDaoHandler_1.DividendDaoHandler.trimConsumed(row, belowNumber)) {
                await this.update(this.driver, row, ['consumed', 'consumedUDs'], ['pub']);
            }
        }
    }
    listAll() {
        return this.find('SELECT * FROM dividend', []);
    }
}
__decorate([
    MonitorExecutionTime_1.MonitorExecutionTime()
], SqliteDividend.prototype, "insert", null);
__decorate([
    MonitorExecutionTime_1.MonitorExecutionTime()
], SqliteDividend.prototype, "insertBatch", null);
exports.SqliteDividend = SqliteDividend;
//# sourceMappingURL=SqliteDividend.js.map