"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
Object.defineProperty(exports, "__esModule", { value: true });
const indexer_1 = require("../../../indexer");
const MonitorExecutionTime_1 = require("../../../debug/MonitorExecutionTime");
const OldTransformer_1 = require("../common/OldTransformer");
const SqliteTable_1 = require("./SqliteTable");
const SqlFieldDefinition_1 = require("./SqlFieldDefinition");
class SqliteIIndex extends SqliteTable_1.SqliteTable {
    constructor(getSqliteDB) {
        super('iindex', {
            'op': new SqlFieldDefinition_1.SqlNotNullableFieldDefinition('CHAR', false, 6),
            'pub': new SqlFieldDefinition_1.SqlNotNullableFieldDefinition('VARCHAR', true, 50),
            'written_on': new SqlFieldDefinition_1.SqlNotNullableFieldDefinition('VARCHAR', false, 80),
            'writtenOn': new SqlFieldDefinition_1.SqlNotNullableFieldDefinition('INT', true),
            'created_on': new SqlFieldDefinition_1.SqlNullableFieldDefinition('VARCHAR', false, 80),
            'uid': new SqlFieldDefinition_1.SqlNullableFieldDefinition('VARCHAR', true, 100),
            'hash': new SqlFieldDefinition_1.SqlNullableFieldDefinition('VARCHAR', false, 70),
            'sig': new SqlFieldDefinition_1.SqlNullableFieldDefinition('VARCHAR', false, 100),
            'member': new SqlFieldDefinition_1.SqlNullableFieldDefinition('BOOLEAN', true),
            'wasMember': new SqlFieldDefinition_1.SqlNullableFieldDefinition('BOOLEAN', true),
            'kick': new SqlFieldDefinition_1.SqlNullableFieldDefinition('BOOLEAN', true),
            'wotb_id': new SqlFieldDefinition_1.SqlNullableFieldDefinition('INT', true),
        }, getSqliteDB);
    }
    /**
     * TECHNICAL
     */
    cleanCache() {
    }
    triggerInit() {
    }
    /**
     * INSERT
     */
    async insert(record) {
        await this.insertInTable(this.driver, record);
    }
    async insertBatch(records) {
        if (records.length) {
            return this.insertBatchInTable(this.driver, records);
        }
    }
    /**
     * DELETE
     */
    async removeBlock(blockstamp) {
        await this.driver.sqlWrite(`DELETE FROM iindex WHERE written_on = ?`, [blockstamp]);
    }
    async trimRecords(belowNumber) {
        const belowRecords = await this.driver.sqlRead('SELECT COUNT(*) as nbRecords, pub FROM iindex ' +
            'WHERE writtenOn < ? ' +
            'GROUP BY pub ' +
            'HAVING nbRecords > 1', [belowNumber]);
        const reducedByPub = indexer_1.Indexer.DUP_HELPERS.reduceBy(belowRecords, ['pub']);
        for (const record of reducedByPub) {
            const recordsOfPub = await this.reducable(record.pub);
            const toReduce = recordsOfPub.filter(rec => parseInt(rec.written_on) < belowNumber);
            if (toReduce.length && recordsOfPub.length > 1) {
                // Clean the records in the DB
                await this.driver.sqlExec('DELETE FROM iindex WHERE pub = \'' + record.pub + '\'');
                const nonReduced = recordsOfPub.filter(rec => parseInt(rec.written_on) >= belowNumber);
                const reduced = indexer_1.Indexer.DUP_HELPERS.reduce(toReduce);
                // Persist
                await this.insertBatch([reduced].concat(nonReduced));
            }
        }
    }
    /**
     * FIND
     */
    async getWrittenOn(blockstamp) {
        return this.find('SELECT * FROM iindex WHERE written_on = ?', [blockstamp]);
    }
    async findRawWithOrder(criterion, sort) {
        let sql = `SELECT * FROM iindex ${criterion.pub ? 'WHERE pub = ?' : ''}`;
        if (sort.length) {
            sql += ` ORDER BY ${sort.map(s => `${s[0]} ${s[1] ? 'DESC' : 'ASC'}`).join(', ')}`;
        }
        return this.find(sql, criterion.pub ? [criterion.pub] : []);
    }
    async find(sql, params) {
        return this.findEntities(sql, params);
    }
    /**
     * OTHER
     */
    async reducable(pub) {
        return this.find('SELECT * FROM iindex WHERE pub = ? order by writtenOn ASC', [pub]);
    }
    //-----------------
    async findByPub(pub) {
        return this.find('SELECT * FROM iindex WHERE pub = ? order by writtenOn ASC', [pub]);
    }
    async findByUid(uid) {
        return this.find('SELECT * FROM iindex WHERE uid = ? order by writtenOn ASC', [uid]);
    }
    async getFromPubkey(pub) {
        const entries = await this.find('SELECT * FROM iindex WHERE pub = ? order by writtenOn ASC', [pub]);
        if (!entries.length) {
            return null;
        }
        return OldTransformer_1.OldTransformers.iindexEntityOrNull(entries);
    }
    // Non-protocol
    async getFromPubkeyOrUid(search) {
        return indexer_1.Indexer.DUP_HELPERS.reduceOrNull((await this.find('SELECT * FROM iindex WHERE pub = ? OR uid = ?', [search, search])));
    }
    async getFromUID(uid) {
        const entries = await this.find('SELECT * FROM iindex WHERE uid = ? order by writtenOn ASC', [uid]);
        if (!entries.length) {
            return null;
        }
        return this.getFromPubkey(entries[0].pub);
    }
    async getFullFromHash(hash) {
        const entries = await this.find('SELECT * FROM iindex WHERE hash = ? order by writtenOn ASC', [hash]);
        if (!entries.length) {
            return null;
        }
        return this.getFromPubkey(entries[0].pub);
    }
    async getFullFromPubkey(pub) {
        return (await this.getFromPubkey(pub));
    }
    async getFullFromUID(uid) {
        return (await this.getFromUID(uid));
    }
    async getMembers() {
        const members = await this.find('SELECT * FROM iindex i1 ' +
            'WHERE member AND NOT EXISTS (' +
            '  SELECT * FROM iindex i2 ' +
            '  WHERE i2.pub = i1.pub' +
            '  AND i2.writtenOn > i1.writtenOn' +
            '  AND NOT i2.member)', []);
        await Promise.all(members.map(async (m) => {
            if (!m.uid) {
                const withUID = await this.find('SELECT * FROM iindex WHERE pub = ? AND uid IS NOT NULL', [m.pub]);
                m.uid = withUID[0].uid;
            }
        }));
        return members.map(m => ({
            pubkey: m.pub,
            uid: m.uid
        }));
    }
    async getToBeKickedPubkeys() {
        return (await this.find('SELECT * FROM iindex i1 ' +
            'WHERE kick AND NOT EXISTS (' +
            '  SELECT * FROM iindex i2 ' +
            '  WHERE i2.pub = i1.pub' +
            '  AND i2.writtenOn > i1.writtenOn)', [])).map(r => r.pub);
    }
    async searchThoseMatching(search) {
        return (await this.find('SELECT * FROM iindex WHERE pub = ? OR uid = ?', [search, search]))
            .map(OldTransformer_1.OldTransformers.toOldIindexEntry);
    }
}
__decorate([
    MonitorExecutionTime_1.MonitorExecutionTime()
], SqliteIIndex.prototype, "insert", null);
__decorate([
    MonitorExecutionTime_1.MonitorExecutionTime()
], SqliteIIndex.prototype, "insertBatch", null);
__decorate([
    MonitorExecutionTime_1.MonitorExecutionTime()
], SqliteIIndex.prototype, "removeBlock", null);
__decorate([
    MonitorExecutionTime_1.MonitorExecutionTime()
], SqliteIIndex.prototype, "trimRecords", null);
__decorate([
    MonitorExecutionTime_1.MonitorExecutionTime()
], SqliteIIndex.prototype, "getWrittenOn", null);
__decorate([
    MonitorExecutionTime_1.MonitorExecutionTime()
], SqliteIIndex.prototype, "findRawWithOrder", null);
__decorate([
    MonitorExecutionTime_1.MonitorExecutionTime()
], SqliteIIndex.prototype, "reducable", null);
__decorate([
    MonitorExecutionTime_1.MonitorExecutionTime()
], SqliteIIndex.prototype, "findByPub", null);
__decorate([
    MonitorExecutionTime_1.MonitorExecutionTime()
], SqliteIIndex.prototype, "findByUid", null);
__decorate([
    MonitorExecutionTime_1.MonitorExecutionTime()
], SqliteIIndex.prototype, "getFromPubkey", null);
__decorate([
    MonitorExecutionTime_1.MonitorExecutionTime()
], SqliteIIndex.prototype, "getFromPubkeyOrUid", null);
__decorate([
    MonitorExecutionTime_1.MonitorExecutionTime()
], SqliteIIndex.prototype, "getFromUID", null);
__decorate([
    MonitorExecutionTime_1.MonitorExecutionTime()
], SqliteIIndex.prototype, "getFullFromHash", null);
__decorate([
    MonitorExecutionTime_1.MonitorExecutionTime()
], SqliteIIndex.prototype, "getFullFromPubkey", null);
__decorate([
    MonitorExecutionTime_1.MonitorExecutionTime()
], SqliteIIndex.prototype, "getFullFromUID", null);
__decorate([
    MonitorExecutionTime_1.MonitorExecutionTime()
], SqliteIIndex.prototype, "getMembers", null);
__decorate([
    MonitorExecutionTime_1.MonitorExecutionTime()
], SqliteIIndex.prototype, "getToBeKickedPubkeys", null);
__decorate([
    MonitorExecutionTime_1.MonitorExecutionTime()
], SqliteIIndex.prototype, "searchThoseMatching", null);
exports.SqliteIIndex = SqliteIIndex;
//# sourceMappingURL=SqliteIIndex.js.map