"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
Object.defineProperty(exports, "__esModule", { value: true });
const indexer_1 = require("../../../indexer");
const MonitorExecutionTime_1 = require("../../../debug/MonitorExecutionTime");
const SqliteNodeIOManager_1 = require("./SqliteNodeIOManager");
const constants_1 = require("../../../common-libs/constants");
const SqliteTable_1 = require("./SqliteTable");
const SqlFieldDefinition_1 = require("./SqlFieldDefinition");
class SqliteMIndex extends SqliteTable_1.SqliteTable {
    constructor(getSqliteDB) {
        super('mindex', {
            'op': new SqlFieldDefinition_1.SqlNotNullableFieldDefinition('CHAR', false, 6),
            'pub': new SqlFieldDefinition_1.SqlNotNullableFieldDefinition('VARCHAR', true, 50),
            'written_on': new SqlFieldDefinition_1.SqlNotNullableFieldDefinition('VARCHAR', true, 80),
            'writtenOn': new SqlFieldDefinition_1.SqlNotNullableFieldDefinition('INT', true),
            'created_on': new SqlFieldDefinition_1.SqlNotNullableFieldDefinition('VARCHAR', true, 80),
            'expires_on': new SqlFieldDefinition_1.SqlNullableFieldDefinition('INT', true),
            'expired_on': new SqlFieldDefinition_1.SqlNullableFieldDefinition('INT', false),
            'revocation': new SqlFieldDefinition_1.SqlNullableFieldDefinition('VARCHAR', false, 100),
            'revokes_on': new SqlFieldDefinition_1.SqlNullableFieldDefinition('INT', true),
            'chainable_on': new SqlFieldDefinition_1.SqlNullableFieldDefinition('INT', true),
            'revoked_on': new SqlFieldDefinition_1.SqlNullableFieldDefinition('VARCHAR', true, 80),
            'leaving': new SqlFieldDefinition_1.SqlNullableFieldDefinition('BOOLEAN', false),
        }, getSqliteDB);
        this.p2 = getSqliteDB('c_mindex.db');
    }
    /**
     * TECHNICAL
     */
    cleanCache() {
    }
    async init() {
        await super.init();
        this.d2 = new SqliteNodeIOManager_1.SqliteNodeIOManager(await this.p2, 'c_mindex');
        // COMPUTED
        await this.d2.sqlExec(`
    BEGIN;
    CREATE TABLE IF NOT EXISTS c_mindex (
      pub VARCHAR(50) NOT NULL,
      created_on VARCHAR(80) NOT NULL,
      expires_on INT NULL,
      expired_on INT NULL,
      revokes_on INT NULL,
      writtenOn INT NOT NULL
    );

    CREATE INDEX IF NOT EXISTS idx_c_mindex_pub ON c_mindex (pub);
    CREATE INDEX IF NOT EXISTS idx_c_mindex_expires_on ON c_mindex (expires_on);
    CREATE INDEX IF NOT EXISTS idx_c_mindex_expired_on ON c_mindex (expired_on);
    CREATE INDEX IF NOT EXISTS idx_c_mindex_revokes_on ON c_mindex (revokes_on);
    CREATE INDEX IF NOT EXISTS idx_c_mindex_writtenOn ON c_mindex (writtenOn);
    COMMIT;
    `);
    }
    triggerInit() {
    }
    /**
     * INSERT
     */
    async insert(record) {
        await this.insertInTable(this.driver, record);
    }
    async insertBatch(records) {
        if (records.length) {
            await this.insertBatchInTable(this.driver, records);
            // Computed
            const cCreates = records.filter(r => r.op === constants_1.CommonConstants.IDX_CREATE).map(r => `(
        '${r.pub}',
        '${r.created_on}',
        ${r.expires_on || null},
        ${r.expired_on},
        ${r.revokes_on || null},
        ${r.writtenOn}
      )`).join(',');
            if (cCreates) {
                await this.insertD2(cCreates);
            }
            records
                .filter(r => r.op === constants_1.CommonConstants.IDX_UPDATE)
                .forEach(async (r) => {
                if (r.expires_on || r.expired_on || r.revokes_on) {
                    await this.updateD2(r);
                }
            });
        }
    }
    async insertD2(cCreates) {
        const req = `INSERT INTO c_mindex (
        pub,
        created_on,
        expires_on,
        expired_on,
        revokes_on,
        writtenOn
        ) VALUES ${cCreates}`;
        await this.d2.sqlWrite(req, []);
    }
    async updateD2(r) {
        const req = `UPDATE c_mindex SET
          ${r.created_on ? `created_on = '${r.created_on}',` : ''}
          ${r.expires_on ? `expires_on = ${r.expires_on},` : ''}
          ${r.expired_on ? `expired_on = ${r.expired_on},` : ''}
          ${r.revokes_on ? `revokes_on = ${r.revokes_on},` : ''}
          writtenOn = ${r.writtenOn}
          WHERE pub = ?`;
        await this.d2.sqlWrite(req, [r.pub]);
    }
    /**
     * DELETE
     */
    async removeBlock(blockstamp) {
        await this.driver.sqlWrite(`DELETE FROM mindex WHERE written_on = ?`, [blockstamp]);
    }
    async trimRecords(belowNumber) {
        const belowRecords = await this.driver.sqlRead('SELECT COUNT(*) as nbRecords, pub FROM mindex ' +
            'WHERE writtenOn < ? ' +
            'GROUP BY pub ' +
            'HAVING nbRecords > 1', [belowNumber]);
        const reducedByPub = indexer_1.Indexer.DUP_HELPERS.reduceBy(belowRecords, ['pub']);
        for (const record of reducedByPub) {
            const recordsOfPub = await this.reducable(record.pub);
            const toReduce = recordsOfPub.filter(rec => parseInt(rec.written_on) < belowNumber);
            if (toReduce.length && recordsOfPub.length > 1) {
                // Clean the records in the DB
                await this.driver.sqlExec('DELETE FROM mindex WHERE pub = \'' + record.pub + '\'');
                const nonReduced = recordsOfPub.filter(rec => parseInt(rec.written_on) >= belowNumber);
                const reduced = indexer_1.Indexer.DUP_HELPERS.reduce(toReduce);
                // Persist
                await this.insertBatch([reduced].concat(nonReduced));
            }
        }
    }
    /**
     * FIND
     */
    async findByPubAndChainableOnGt(pub, medianTime) {
        return this.find('SELECT * FROM mindex WHERE pub = ? AND chainable_on > ?', [pub, medianTime]);
    }
    async findPubkeysThatShouldExpire(medianTime) {
        return this.find('SELECT *, (' +
            // Le dernier renouvellement
            '  SELECT m2.expires_on ' +
            '  FROM mindex m2 ' +
            '  WHERE m2.pub = m1.pub ' +
            '  AND m2.writtenOn = (' +
            '    SELECT MAX(m4.writtenOn)' +
            '    FROM mindex m4' +
            '    WHERE pub = m2.pub' +
            '  )' +
            ') as renewal, (' +
            // La dernière expiration
            '  SELECT m2.expired_on ' +
            '  FROM mindex m2 ' +
            '  WHERE m2.pub = m1.pub ' +
            '  AND m2.writtenOn = (' +
            '    SELECT MAX(m4.writtenOn)' +
            '    FROM mindex m4' +
            '    WHERE pub = m2.pub' +
            '  )' +
            ') as expiry ' +
            'FROM mindex m1 ' +
            'WHERE m1.expires_on <= ? ' +
            'AND m1.revokes_on > ? ' +
            'AND (renewal IS NULL OR renewal <= ?) ' +
            'AND (expiry IS NULL)', [medianTime, medianTime, medianTime]);
    }
    async findRevokesOnLteAndRevokedOnIsNull(medianTime) {
        return (await this.find('SELECT * FROM mindex WHERE revokes_on <= ? AND revoked_on IS NULL', [medianTime]))
            .map(e => e.pub);
    }
    async getWrittenOn(blockstamp) {
        return this.find('SELECT * FROM mindex WHERE written_on = ?', [blockstamp]);
    }
    async findRawWithOrder(criterion, sort) {
        let sql = `SELECT * FROM mindex ${criterion.pub ? 'WHERE pub = ?' : ''}`;
        if (sort.length) {
            sql += ` ORDER BY ${sort.map(s => `${s[0]} ${s[1] ? 'DESC' : 'ASC'}`).join(', ')}`;
        }
        return this.find(sql, criterion.pub ? [criterion.pub] : []);
    }
    async find(sql, params) {
        return (await this.driver.sqlRead(sql, params)).map(r => {
            return {
                index: 'MINDEX',
                op: r.op,
                pub: r.pub,
                written_on: r.written_on,
                writtenOn: r.writtenOn,
                created_on: r.created_on,
                type: r.type,
                expires_on: r.expires_on !== null ? r.expires_on : null,
                expired_on: r.expired_on !== null ? r.expired_on : null,
                revocation: r.revocation,
                revokes_on: r.revokes_on !== null ? r.revokes_on : null,
                chainable_on: r.chainable_on !== null ? r.chainable_on : null,
                revoked_on: r.revoked_on,
                leaving: r.leaving !== null ? r.leaving : null,
                age: 0,
                unchainables: 0,
            };
        });
    }
    async getReducedMSForImplicitRevocation(pub) {
        return indexer_1.Indexer.DUP_HELPERS.reduceOrNull((await this.reducable(pub)));
    }
    async getReducedMSForMembershipExpiry(pub) {
        return indexer_1.Indexer.DUP_HELPERS.reduceOrNull((await this.reducable(pub)));
    }
    async getRevokedPubkeys() {
        return (await this.driver.sqlRead('SELECT DISTINCT(pub) FROM mindex WHERE revoked_on IS NOT NULL', [])).map(r => r.pub);
    }
    /**
     * OTHER
     */
    async reducable(pub) {
        // await this.dump()
        return this.findEntities('SELECT * FROM mindex WHERE pub = ? order by writtenOn ASC', [pub]);
    }
    async findExpiresOnLteAndRevokesOnGt(medianTime) {
        return [];
    }
    async getReducedMS(pub) {
        return null;
    }
}
__decorate([
    MonitorExecutionTime_1.MonitorExecutionTime()
], SqliteMIndex.prototype, "insert", null);
__decorate([
    MonitorExecutionTime_1.MonitorExecutionTime()
], SqliteMIndex.prototype, "insertBatch", null);
__decorate([
    MonitorExecutionTime_1.MonitorExecutionTime()
], SqliteMIndex.prototype, "insertD2", null);
__decorate([
    MonitorExecutionTime_1.MonitorExecutionTime()
], SqliteMIndex.prototype, "updateD2", null);
__decorate([
    MonitorExecutionTime_1.MonitorExecutionTime()
], SqliteMIndex.prototype, "removeBlock", null);
__decorate([
    MonitorExecutionTime_1.MonitorExecutionTime()
], SqliteMIndex.prototype, "trimRecords", null);
__decorate([
    MonitorExecutionTime_1.MonitorExecutionTime()
], SqliteMIndex.prototype, "findByPubAndChainableOnGt", null);
__decorate([
    MonitorExecutionTime_1.MonitorExecutionTime()
], SqliteMIndex.prototype, "findPubkeysThatShouldExpire", null);
__decorate([
    MonitorExecutionTime_1.MonitorExecutionTime()
], SqliteMIndex.prototype, "findRevokesOnLteAndRevokedOnIsNull", null);
__decorate([
    MonitorExecutionTime_1.MonitorExecutionTime()
], SqliteMIndex.prototype, "getWrittenOn", null);
__decorate([
    MonitorExecutionTime_1.MonitorExecutionTime()
], SqliteMIndex.prototype, "findRawWithOrder", null);
__decorate([
    MonitorExecutionTime_1.MonitorExecutionTime()
], SqliteMIndex.prototype, "getReducedMSForImplicitRevocation", null);
__decorate([
    MonitorExecutionTime_1.MonitorExecutionTime()
], SqliteMIndex.prototype, "getReducedMSForMembershipExpiry", null);
__decorate([
    MonitorExecutionTime_1.MonitorExecutionTime()
], SqliteMIndex.prototype, "getRevokedPubkeys", null);
__decorate([
    MonitorExecutionTime_1.MonitorExecutionTime()
], SqliteMIndex.prototype, "reducable", null);
exports.SqliteMIndex = SqliteMIndex;
//# sourceMappingURL=SqliteMIndex.js.map