"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
Object.defineProperty(exports, "__esModule", { value: true });
const MonitorExecutionTime_1 = require("../../../debug/MonitorExecutionTime");
const logger_1 = require("../../../logger");
const exit_codes_1 = require("../../../common-libs/exit-codes");
class SqliteNodeIOManager {
    constructor(driver, id) {
        this.driver = driver;
        this.id = id;
        this.writePromise = null;
    }
    async wait4writing() {
        await this.writePromise;
        // We no more need to wait
        this.writePromise = null;
    }
    async sqlWrite(sql, params) {
        // // Just promise that the writing will be done
        this.writePromise = (this.writePromise || Promise.resolve())
            .then(() => this.driver.executeAll(sql, params))
            .catch(e => {
            logger_1.NewLogger().error(e);
            process.exit(exit_codes_1.ExitCodes.MINDEX_WRITING_ERROR);
        });
    }
    async sqlExec(sql) {
        if (this.writePromise) {
            // Wait for current writings to be done
            await this.wait4writing();
        }
        return this.driver.executeSql(sql);
    }
    async sqlRead(sql, params) {
        if (this.writePromise) {
            // Wait for current writings to be done
            await this.wait4writing();
        }
        return this.driver.executeAll(sql, params);
    }
    async close() {
        await this.driver.closeConnection();
    }
}
__decorate([
    MonitorExecutionTime_1.MonitorExecutionTime('id')
], SqliteNodeIOManager.prototype, "wait4writing", null);
exports.SqliteNodeIOManager = SqliteNodeIOManager;
//# sourceMappingURL=SqliteNodeIOManager.js.map