"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
Object.defineProperty(exports, "__esModule", { value: true });
const indexer_1 = require("../../../indexer");
const MonitorExecutionTime_1 = require("../../../debug/MonitorExecutionTime");
const SqliteTable_1 = require("./SqliteTable");
const SqlFieldDefinition_1 = require("./SqlFieldDefinition");
class SqliteSIndex extends SqliteTable_1.SqliteTable {
    constructor(getSqliteDB) {
        super('sindex', {
            'op': new SqlFieldDefinition_1.SqlNotNullableFieldDefinition('CHAR', false, 6),
            'written_on': new SqlFieldDefinition_1.SqlNotNullableFieldDefinition('VARCHAR', false, 80),
            'writtenOn': new SqlFieldDefinition_1.SqlNotNullableFieldDefinition('INT', true),
            'srcType': new SqlFieldDefinition_1.SqlNotNullableFieldDefinition('CHAR', true, 1),
            'tx': new SqlFieldDefinition_1.SqlNullableFieldDefinition('VARCHAR', true, 70),
            'identifier': new SqlFieldDefinition_1.SqlNotNullableFieldDefinition('VARCHAR', true, 70),
            'pos': new SqlFieldDefinition_1.SqlNotNullableFieldDefinition('INT', true),
            'created_on': new SqlFieldDefinition_1.SqlNullableFieldDefinition('VARCHAR', false, 100),
            'written_time': new SqlFieldDefinition_1.SqlNotNullableFieldDefinition('INT', true),
            'locktime': new SqlFieldDefinition_1.SqlNullableFieldDefinition('INT', false),
            'unlock': new SqlFieldDefinition_1.SqlNullableFieldDefinition('VARCHAR', false, 255),
            'amount': new SqlFieldDefinition_1.SqlNotNullableFieldDefinition('INT', false),
            'base': new SqlFieldDefinition_1.SqlNotNullableFieldDefinition('INT', false),
            'conditions': new SqlFieldDefinition_1.SqlNotNullableFieldDefinition('VARCHAR', true, 1000),
            'consumed': new SqlFieldDefinition_1.SqlNullableFieldDefinition('BOOLEAN', true),
        }, getSqliteDB);
    }
    /**
     * TECHNICAL
     */
    cleanCache() {
    }
    triggerInit() {
    }
    /**
     * INSERT
     */
    async insert(record) {
        await this.insertInTable(this.driver, record);
    }
    async insertBatch(records) {
        if (records.length) {
            return this.insertBatchInTable(this.driver, records);
        }
    }
    /**
     * DELETE
     */
    async removeBlock(blockstamp) {
        await this.driver.sqlWrite(`DELETE FROM sindex WHERE written_on = ?`, [blockstamp]);
    }
    async trimRecords(belowNumber) {
        await this.trimConsumedSource(belowNumber);
    }
    /**
     * FIND
     */
    async getWrittenOn(blockstamp) {
        return this.find('SELECT * FROM sindex WHERE written_on = ?', [blockstamp]);
    }
    async findRawWithOrder(criterion, sort) {
        let sql = `SELECT * FROM sindex ${criterion.pub ? 'WHERE pub = ?' : ''}`;
        if (sort.length) {
            sql += ` ORDER BY ${sort.map(s => `${s[0]} ${s[1] ? 'DESC' : 'ASC'}`).join(', ')}`;
        }
        return this.find(sql, criterion.pub ? [criterion.pub] : []);
    }
    async find(sql, params) {
        return (await this.driver.sqlRead(sql, params)).map(r => {
            return {
                index: 'CINDEX',
                op: r.op,
                written_on: r.written_on,
                writtenOn: r.writtenOn,
                srcType: r.srcType,
                tx: r.tx,
                identifier: r.identifier,
                pos: r.pos,
                created_on: r.created_on,
                written_time: r.written_time,
                locktime: r.locktime,
                unlock: r.unlock,
                amount: r.amount,
                base: r.base,
                conditions: r.conditions,
                consumed: r.consumed,
                txObj: null,
                age: 0,
            };
        });
    }
    /**
     * OTHER
     */
    findByIdentifier(identifier) {
        return this.find('SELECT * FROM sindex WHERE identifier = ?', [identifier]);
    }
    findByPos(pos) {
        return this.find('SELECT * FROM sindex WHERE pos = ?', [pos]);
    }
    findTxSourceByIdentifierPosAmountBase(identifier, pos, amount, base) {
        return this.find('SELECT * FROM sindex ' +
            'WHERE identifier = ? ' +
            'AND pos = ? ' +
            'AND amount = ? ' +
            'AND base = ?', [identifier, pos, amount, base]);
    }
    getAvailableForConditions(conditionsStr) {
        return this.find('SELECT * FROM sindex s1 ' +
            'WHERE s1.conditions LIKE ? ' +
            'AND NOT s1.consumed ' +
            'AND NOT EXISTS (' +
            '  SELECT * FROM sindex s2' +
            '  WHERE s1.identifier = s2.identifier' +
            '  AND s1.pos = s2.pos' +
            '  AND s2.consumed' +
            ')', [conditionsStr]);
    }
    async getAvailableForPubkey(pubkey) {
        return this.getAvailableForConditions(`SIG(${pubkey})`); // TODO: maybe %SIG(...)%
    }
    async getTxSource(identifier, pos) {
        const entries = await this.find('SELECT * FROM sindex WHERE identifier = ? AND pos = ? ORDER BY writtenOn', [identifier, pos]);
        return indexer_1.Indexer.DUP_HELPERS.reduceOrNull(entries);
    }
    async getWrittenOnTxs(blockstamp) {
        const entries = await this.find('SELECT * FROM sindex WHERE written_on = ?', [blockstamp]);
        const res = [];
        entries.forEach(s => {
            res.push({
                srcType: 'T',
                op: s.op,
                conditions: s.conditions,
                amount: s.amount,
                base: s.base,
                identifier: s.identifier,
                pos: s.pos
            });
        });
        return res;
    }
    async trimConsumedSource(belowNumber) {
        const sources = await this.find('SELECT * FROM sindex WHERE consumed AND writtenOn < ?', [belowNumber]);
        await Promise.all(sources.map(async (s) => this.driver.sqlWrite('DELETE FROM sindex ' +
            'WHERE identifier = ? ' +
            'AND pos = ?', [
            s.identifier,
            s.pos,
        ])));
    }
}
__decorate([
    MonitorExecutionTime_1.MonitorExecutionTime()
], SqliteSIndex.prototype, "insert", null);
__decorate([
    MonitorExecutionTime_1.MonitorExecutionTime()
], SqliteSIndex.prototype, "insertBatch", null);
__decorate([
    MonitorExecutionTime_1.MonitorExecutionTime()
], SqliteSIndex.prototype, "removeBlock", null);
__decorate([
    MonitorExecutionTime_1.MonitorExecutionTime()
], SqliteSIndex.prototype, "trimRecords", null);
__decorate([
    MonitorExecutionTime_1.MonitorExecutionTime()
], SqliteSIndex.prototype, "getWrittenOn", null);
__decorate([
    MonitorExecutionTime_1.MonitorExecutionTime()
], SqliteSIndex.prototype, "findRawWithOrder", null);
exports.SqliteSIndex = SqliteSIndex;
//# sourceMappingURL=SqliteSIndex.js.map