import { SqlFieldDefinition } from "./SqlFieldDefinition";
import { SqliteNodeIOManager } from "./SqliteNodeIOManager";
import { SQLiteDriver } from "../../drivers/SQLiteDriver";
export declare class SqliteTable<T> {
    protected name: string;
    protected fields: {
        [k in keyof T]?: SqlFieldDefinition;
    };
    private readonly pdriver;
    protected driver: SqliteNodeIOManager<T>;
    protected constructor(name: string, fields: {
        [k in keyof T]?: SqlFieldDefinition;
    }, getSqliteDB: (dbName: string) => Promise<SQLiteDriver>);
    init(): Promise<void>;
    close(): Promise<void>;
    generateCreateTable(): string;
    generateCreateIndexes(): string;
    keys(): (keyof T)[];
    insertInTable(driver: SqliteNodeIOManager<T>, record: T): Promise<void>;
    update<K extends keyof T>(driver: SqliteNodeIOManager<T>, record: T, fieldsToUpdate: K[], whereFields: K[]): Promise<void>;
    insertBatchInTable(driver: SqliteNodeIOManager<T>, records: T[]): Promise<void>;
    findEntities(sql: string, params: any[]): Promise<T[]>;
    /**
     * Extract an SQL value of a field into its Object value
     * @param {keyof T} fieldName Name of the field in the record.
     * @param {T} record The record from which extracting a column's value.
     * @returns {any} The translated value.
     */
    protected sqlValue2Object<K extends keyof T>(fieldName: K, record: T): any;
    private getFieldValue;
    dump(): Promise<void>;
    count(): Promise<any>;
    /**
     * Debugging function: allows to make a hot copy of an SQLite database to a new file, even if the source is in-memory.
     * @param {string} path The path where to write the copy.
     * @returns {Promise<void>} Promise of done.
     */
    copy2file(path: string): Promise<void>;
}
