"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
Object.defineProperty(exports, "__esModule", { value: true });
const MonitorExecutionTime_1 = require("../../../debug/MonitorExecutionTime");
const SqliteTable_1 = require("./SqliteTable");
const SqlFieldDefinition_1 = require("./SqlFieldDefinition");
const DBTx_1 = require("../../../db/DBTx");
const SandBox_1 = require("../../sqliteDAL/SandBox");
const constants = require('../../../constants');
class SqliteTransactions extends SqliteTable_1.SqliteTable {
    constructor(getSqliteDB) {
        super('txs', {
            'hash': new SqlFieldDefinition_1.SqlNotNullableFieldDefinition('VARCHAR', true, 70),
            'block_number': new SqlFieldDefinition_1.SqlNullableFieldDefinition('INT', true),
            'locktime': new SqlFieldDefinition_1.SqlNullableFieldDefinition('INT', false),
            'version': new SqlFieldDefinition_1.SqlNullableFieldDefinition('INT', false),
            'currency': new SqlFieldDefinition_1.SqlNullableFieldDefinition('VARCHAR', false, 10),
            'comment': new SqlFieldDefinition_1.SqlNullableFieldDefinition('TEXT', false),
            'blockstamp': new SqlFieldDefinition_1.SqlNullableFieldDefinition('VARCHAR', false, 100),
            'blockstampTime': new SqlFieldDefinition_1.SqlNullableFieldDefinition('INT', false),
            'time': new SqlFieldDefinition_1.SqlNullableFieldDefinition('INT', false),
            'inputs': new SqlFieldDefinition_1.SqlNullableFieldDefinition('JSON', false),
            'unlocks': new SqlFieldDefinition_1.SqlNullableFieldDefinition('JSON', false),
            'outputs': new SqlFieldDefinition_1.SqlNullableFieldDefinition('JSON', false),
            'issuers': new SqlFieldDefinition_1.SqlNullableFieldDefinition('JSON', false),
            'signatures': new SqlFieldDefinition_1.SqlNullableFieldDefinition('JSON', false),
            'recipients': new SqlFieldDefinition_1.SqlNullableFieldDefinition('JSON', false),
            'written': new SqlFieldDefinition_1.SqlNotNullableFieldDefinition('BOOLEAN', true),
            'removed': new SqlFieldDefinition_1.SqlNotNullableFieldDefinition('BOOLEAN', true),
            'received': new SqlFieldDefinition_1.SqlNullableFieldDefinition('BOOLEAN', false),
            'output_base': new SqlFieldDefinition_1.SqlNullableFieldDefinition('INT', false),
            'output_amount': new SqlFieldDefinition_1.SqlNullableFieldDefinition('INT', false),
            'written_on': new SqlFieldDefinition_1.SqlNullableFieldDefinition('VARCHAR', false, 100),
            'writtenOn': new SqlFieldDefinition_1.SqlNullableFieldDefinition('INT', false),
        }, getSqliteDB);
        this.sandbox = new SandBox_1.SandBox(constants.SANDBOX_SIZE_TRANSACTIONS, () => this.getSandboxTxs(), (compared, reference) => {
            if (compared.output_base < reference.output_base) {
                return -1;
            }
            else if (compared.output_base > reference.output_base) {
                return 1;
            }
            else if (compared.output_amount > reference.output_amount) {
                return -1;
            }
            else if (compared.output_amount < reference.output_amount) {
                return 1;
            }
            else {
                return 0;
            }
        });
    }
    /**
     * TECHNICAL
     */
    async insert(record) {
        await this.insertInTable(this.driver, record);
    }
    async insertBatch(records) {
        if (records.length) {
            return this.insertBatchInTable(this.driver, records);
        }
    }
    async addLinked(tx, block_number, time) {
        const dbTx = await this.getTX(tx.hash);
        const theDBTx = DBTx_1.DBTx.fromTransactionDTO(tx);
        theDBTx.written = true;
        theDBTx.block_number = block_number;
        theDBTx.time = time;
        if (!dbTx) {
            await this.insert(theDBTx);
        }
        else {
            await this.update(this.driver, theDBTx, ['block_number', 'time', 'received', 'written', 'removed', 'hash'], ['hash']);
        }
        return dbTx;
    }
    async addPending(dbTx) {
        const existing = (await this.findEntities('SELECT * FROM txs WHERE hash = ?', [dbTx.hash]))[0];
        if (existing) {
            await this.driver.sqlWrite('UPDATE txs SET written = ? WHERE hash = ?', [false, dbTx.hash]);
            return existing;
        }
        await this.insert(dbTx);
        return dbTx;
    }
    cleanCache() {
    }
    findRawWithOrder(criterion, sort) {
        throw Error('Should not be used method findRawWithOrder() on SqliteTransactions');
    }
    getAllPending(versionMin) {
        return this.findEntities('SELECT * FROM txs WHERE NOT written', []);
    }
    getLinkedWithIssuer(pubkey) {
        return this.findEntities('SELECT * FROM txs WHERE written AND issuers LIKE ?', [`%${pubkey}%`]);
    }
    getLinkedWithRecipient(pubkey) {
        return this.findEntities('SELECT * FROM txs WHERE written AND recipients LIKE ?', [`%${pubkey}%`]);
    }
    getPendingWithIssuer(pubkey) {
        return this.findEntities('SELECT * FROM txs WHERE NOT written AND issuers LIKE ?', [`%${pubkey}%`]);
    }
    getPendingWithRecipient(pubkey) {
        return this.findEntities('SELECT * FROM txs WHERE NOT written AND recipients LIKE ?', [`%${pubkey}%`]);
    }
    async getTX(hash) {
        return (await this.findEntities('SELECT * FROM txs WHERE hash = ?', [hash]))[0];
    }
    getWrittenOn(blockstamp) {
        return this.findEntities('SELECT * FROM txs WHERE blockstamp = ?', [blockstamp]);
    }
    async removeAll() {
        await this.driver.sqlWrite('DELETE FROM txs', []);
    }
    removeBlock(blockstamp) {
        throw Error('Should not be used method removeBlock() on SqliteTransactions');
    }
    removeTX(hash) {
        return this.driver.sqlWrite('DELETE FROM txs WHERE hash = ?', [hash]);
    }
    triggerInit() {
    }
    trimExpiredNonWrittenTxs(limitTime) {
        return this.driver.sqlWrite('DELETE FROM txs WHERE NOT written AND blockstampTime <= ?', [limitTime]);
    }
    /**************************
     * SANDBOX STUFF
     */
    async getSandboxTxs() {
        return this.findEntities('SELECT * FROM txs WHERE NOT written AND NOT removed ORDER BY output_base DESC, output_amount DESC', []);
    }
    getSandboxRoom() {
        return this.sandbox.getSandboxRoom();
    }
    setSandboxSize(maxSize) {
        this.sandbox.maxSize = maxSize;
    }
}
__decorate([
    MonitorExecutionTime_1.MonitorExecutionTime()
], SqliteTransactions.prototype, "insert", null);
__decorate([
    MonitorExecutionTime_1.MonitorExecutionTime()
], SqliteTransactions.prototype, "insertBatch", null);
__decorate([
    MonitorExecutionTime_1.MonitorExecutionTime()
], SqliteTransactions.prototype, "getSandboxTxs", null);
exports.SqliteTransactions = SqliteTransactions;
//# sourceMappingURL=SqliteTransactions.js.map