import { SQLiteDriver } from "../drivers/SQLiteDriver";
import { Initiable } from "./Initiable";
export interface BeforeSaveHook<T> {
    (t: T): void;
}
export declare abstract class AbstractSQLite<T> extends Initiable {
    private driver;
    readonly table: string;
    private pkFields;
    protected fields: string[];
    private arrays;
    private booleans;
    private bigintegers;
    private transientFields;
    private beforeSaveHook;
    constructor(driver: SQLiteDriver, table: string, pkFields?: string[], fields?: string[], arrays?: string[], booleans?: string[], bigintegers?: string[], transientFields?: string[], beforeSaveHook?: BeforeSaveHook<T> | null);
    query(sql: string, params?: any[]): Promise<T[]>;
    cleanData(): Promise<void>;
    sqlListAll(): Promise<T[]>;
    sqlDeleteAll(): Promise<void>;
    sqlFind(obj: any, sortObj?: any): Promise<T[]>;
    sqlFindOne(obj: any, sortObj?: any): Promise<T>;
    sqlFindLikeAny(obj: any): Promise<T[]>;
    sqlRemoveWhere(obj: any): Promise<void>;
    sqlExisting(entity: T): Promise<T>;
    saveEntity(entity: any): Promise<void>;
    insert(entity: T): Promise<void>;
    getEntity(entity: any): Promise<T>;
    deleteEntity(entity: any): Promise<void>;
    exec(sql: string): Promise<void>;
    getInsertQuery(): string;
    getInsertHead(): string;
    getInsertValue(toSave: T): string;
    toInsertValues(entity: T): string;
    /**
     * Make a batch insert.
     * @param records The records to insert as a batch.
     */
    insertBatch(records: T[]): Promise<void>;
    /**
     * To redefine if necessary in subclasses.
     */
    cleanCache(): void;
    close(): Promise<void>;
    private toConditionsArray;
    private toParams;
    private escapeToSQLite;
    private getPKFields;
    private toEntity;
    private toRow;
}
