import { SQLiteDriver } from "../drivers/SQLiteDriver";
import { AbstractSQLite } from "./AbstractSQLite";
import { SandBox } from './SandBox';
import { DBDocument } from './DocumentDAL';
export interface DBCert extends DBDocument {
    linked: boolean;
    written: boolean;
    written_block: number | null;
    written_hash: string | null;
    sig: string;
    block_number: number;
    block_hash: string;
    target: string;
    to: string;
    from: string;
    block: number;
    expired: boolean | null;
    expires_on: number;
}
export declare class CertDAL extends AbstractSQLite<DBCert> {
    constructor(driver: SQLiteDriver);
    init(): Promise<void>;
    getToTarget(hash: string): Promise<DBCert[]>;
    getFromPubkeyCerts(pubkey: string): Promise<DBCert[]>;
    getNotLinked(): Promise<DBCert[]>;
    getNotLinkedToTarget(hash: string): Promise<DBCert[]>;
    saveNewCertification(cert: DBCert): Promise<void>;
    existsGivenCert(cert: DBCert): Promise<DBCert>;
    deleteCert(cert: {
        from: string;
        target: string;
        sig: string;
    }): Promise<void>;
    trimExpiredCerts(medianTime: number): Promise<void>;
    /**************************
     * SANDBOX STUFF
     */
    getSandboxForKey: (pub: string) => SandBox<DBCert>;
}
