"use strict";
// Source file from duniter: Crypto-currency software to manage libre currency such as Ğ1
// Copyright (C) 2018  Cedric Moreau <cem.moreau@gmail.com>
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Affero General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Affero General Public License for more details.
Object.defineProperty(exports, "__esModule", { value: true });
const AbstractSQLite_1 = require("./AbstractSQLite");
const SandBox_1 = require("./SandBox");
const constants = require('../../constants');
class CertDAL extends AbstractSQLite_1.AbstractSQLite {
    constructor(driver) {
        super(driver, 'cert', 
        // PK fields
        ['from', 'target', 'sig'], 
        // Fields
        [
            'linked',
            'written',
            'written_block',
            'written_hash',
            'sig',
            'block_number',
            'block_hash',
            'target',
            'to',
            'from',
            'block',
            'expired',
            'expires_on'
        ], 
        // Arrays
        [], 
        // Booleans
        ['linked', 'written'], 
        // BigIntegers
        [], 
        // Transient
        [], (entity) => {
            entity.written = entity.written || !!(entity.written_hash);
        });
        /**************************
         * SANDBOX STUFF
         */
        this.getSandboxForKey = (pub) => {
            const getRecorded = () => this.query('SELECT * FROM cert WHERE `from` = ? ORDER BY block_number ASC LIMIT ' + constants.SANDBOX_SIZE_CERTIFICATIONS, [pub]);
            const compare = (compared, reference) => {
                if (compared.block_number < reference.block_number) {
                    return -1;
                }
                else if (compared.block_number > reference.block_number) {
                    return 1;
                }
                else {
                    return 0;
                }
            };
            return new SandBox_1.SandBox(constants.SANDBOX_SIZE_CERTIFICATIONS, getRecorded, compare);
        };
    }
    async init() {
        await this.exec('BEGIN;' +
            'CREATE TABLE IF NOT EXISTS ' + this.table + ' (' +
            '`from` VARCHAR(50) NOT NULL,' +
            '`to` VARCHAR(50) NOT NULL,' +
            'target CHAR(64) NOT NULL,' +
            'sig VARCHAR(100) NOT NULL,' +
            'block_number INTEGER NOT NULL,' +
            'block_hash VARCHAR(64),' +
            'block INTEGER NOT NULL,' +
            'linked BOOLEAN NOT NULL,' +
            'written BOOLEAN NOT NULL,' +
            'written_block INTEGER,' +
            'written_hash VARCHAR(64),' +
            'expires_on INTEGER NULL,' +
            'PRIMARY KEY (`from`, target, sig, written_block)' +
            ');' +
            'CREATE INDEX IF NOT EXISTS idx_cert_from ON cert (`from`);' +
            'CREATE INDEX IF NOT EXISTS idx_cert_target ON cert (target);' +
            'CREATE INDEX IF NOT EXISTS idx_cert_linked ON cert (linked);' +
            'COMMIT;');
    }
    getToTarget(hash) {
        return this.sqlFind({
            target: hash
        });
    }
    getFromPubkeyCerts(pubkey) {
        return this.sqlFind({
            from: pubkey
        });
    }
    getNotLinked() {
        return this.sqlFind({
            linked: false
        });
    }
    getNotLinkedToTarget(hash) {
        return this.sqlFind({
            target: hash,
            linked: false
        });
    }
    saveNewCertification(cert) {
        return this.saveEntity(cert);
    }
    existsGivenCert(cert) {
        return this.sqlExisting(cert);
    }
    deleteCert(cert) {
        return this.deleteEntity(cert);
    }
    async trimExpiredCerts(medianTime) {
        await this.exec('DELETE FROM ' + this.table + ' WHERE expires_on IS NULL OR expires_on < ' + medianTime);
    }
}
exports.CertDAL = CertDAL;
//# sourceMappingURL=CertDAL.js.map