"use strict";
// Source file from duniter: Crypto-currency software to manage libre currency such as Ğ1
// Copyright (C) 2018  Cedric Moreau <cem.moreau@gmail.com>
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Affero General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Affero General Public License for more details.
Object.defineProperty(exports, "__esModule", { value: true });
const AbstractSQLite_1 = require("./AbstractSQLite");
const SandBox_1 = require("./SandBox");
const underscore_1 = require("../../common-libs/underscore");
const constants = require('../../constants');
class MembershipDAL extends AbstractSQLite_1.AbstractSQLite {
    constructor(driver) {
        super(driver, 'membership', 
        // PK fields
        ['issuer', 'signature'], 
        // Fields
        [
            'membership',
            'issuer',
            'number',
            'blockNumber',
            'blockHash',
            'userid',
            'certts',
            'block',
            'fpr',
            'idtyHash',
            'written',
            'written_number',
            'expires_on',
            'signature',
            'expired'
        ], 
        // Arrays
        [], 
        // Booleans
        ['written'], 
        // BigIntegers
        [], 
        // Transient
        []);
        this.sandbox = new SandBox_1.SandBox(constants.SANDBOX_SIZE_MEMBERSHIPS, this.getSandboxMemberships.bind(this), (compared, reference) => {
            if (compared.block_number < reference.block_number) {
                return -1;
            }
            else if (compared.block_number > reference.block_number) {
                return 1;
            }
            else {
                return 0;
            }
        });
    }
    async init() {
        await this.exec('BEGIN;' +
            'CREATE TABLE IF NOT EXISTS membership (' +
            'membership CHAR(2) NOT NULL,' +
            'issuer VARCHAR(50) NOT NULL,' +
            'number INTEGER NOT NULL,' +
            'blockNumber INTEGER,' +
            'blockHash VARCHAR(64) NOT NULL,' +
            'userid VARCHAR(255) NOT NULL,' +
            'certts VARCHAR(100) NOT NULL,' +
            'block INTEGER,' +
            'fpr VARCHAR(64),' +
            'idtyHash VARCHAR(64),' +
            'written BOOLEAN NOT NULL,' +
            'written_number INTEGER,' +
            'expires_on INTEGER NULL,' +
            'signature VARCHAR(50),' +
            'PRIMARY KEY (issuer,signature)' +
            ');' +
            'CREATE INDEX IF NOT EXISTS idx_mmembership_idtyHash ON membership (idtyHash);' +
            'CREATE INDEX IF NOT EXISTS idx_mmembership_membership ON membership (membership);' +
            'CREATE INDEX IF NOT EXISTS idx_mmembership_written ON membership (written);' +
            'COMMIT;');
    }
    getMembershipsOfIssuer(issuer) {
        return this.sqlFind({
            issuer: issuer
        });
    }
    getPendingINOfTarget(hash) {
        return this.sqlFind({
            idtyHash: hash,
            membership: 'IN'
        });
    }
    getPendingIN() {
        return this.sqlFind({
            membership: 'IN'
        });
    }
    getPendingOUT() {
        return this.sqlFind({
            membership: 'OUT'
        });
    }
    savePendingMembership(ms) {
        ms.membership = ms.membership.toUpperCase();
        ms.written = false;
        return this.saveEntity(underscore_1.Underscore.pick(ms, 'membership', 'issuer', 'number', 'blockNumber', 'blockHash', 'userid', 'certts', 'block', 'fpr', 'idtyHash', 'expires_on', 'written', 'written_number', 'signature'));
    }
    async deleteMS(ms) {
        await this.deleteEntity(ms);
    }
    async trimExpiredMemberships(medianTime) {
        await this.exec('DELETE FROM ' + this.table + ' WHERE expires_on IS NULL OR expires_on < ' + medianTime);
    }
    /**************************
     * SANDBOX STUFF
     */
    getSandboxMemberships() {
        return this.query('SELECT * FROM sandbox_memberships LIMIT ' + (this.sandbox.maxSize), []);
    }
    getSandboxRoom() {
        return this.sandbox.getSandboxRoom();
    }
    setSandboxSize(maxSize) {
        this.sandbox.maxSize = maxSize;
    }
}
exports.MembershipDAL = MembershipDAL;
//# sourceMappingURL=MembershipDAL.js.map