"use strict";
// Source file from duniter: Crypto-currency software to manage libre currency such as Ğ1
// Copyright (C) 2018  Cedric Moreau <cem.moreau@gmail.com>
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Affero General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Affero General Public License for more details.
Object.defineProperty(exports, "__esModule", { value: true });
const ProcessCpuProfiler_1 = require("../../ProcessCpuProfiler");
const other_constants_1 = require("../other_constants");
const underscore_1 = require("../common-libs/underscore");
const monitorings = {};
process.on('exit', () => {
    let traces = [];
    Object
        .keys(monitorings)
        .forEach(k => {
        const m = monitorings[k];
        const total = m.times.reduce((s, t) => s + t.time / 1000, 0);
        const avg = m.times.length ? total / m.times.length : 0;
        traces.push({
            name: k,
            times: m.times.length,
            avg,
            total
        });
    });
    traces = underscore_1.Underscore.sortBy(traces, t => t.total);
    traces
        .forEach(t => {
        console.log('%s %s times %sms (average) %sms (total time)', (t.name + ':').padEnd(50, ' '), String(t.times).padStart(10, ' '), t.avg.toFixed(3).padStart(10, ' '), t.total.toFixed(0).padStart(10, ' '));
    });
});
exports.MonitorExecutionTime = function (idProperty) {
    return function (target, propertyKey, descriptor) {
        if (other_constants_1.OtherConstants.ENABLE_MONITORING) {
            const original = descriptor.value;
            if (original.__proto__.constructor.name === "AsyncFunction") {
                descriptor.value = async function (...args) {
                    const start = ProcessCpuProfiler_1.getMicrosecondsTime();
                    const entities = await original.apply(this, args);
                    const duration = ProcessCpuProfiler_1.getDurationInMicroSeconds(start);
                    const k = target.constructor.name + '.' + propertyKey + (idProperty ? `[${this[idProperty]}]` : '');
                    if (!monitorings[k]) {
                        monitorings[k] = {
                            times: []
                        };
                    }
                    monitorings[k].times.push({
                        time: duration
                    });
                    return entities;
                };
            }
            else {
                descriptor.value = function (...args) {
                    const start = ProcessCpuProfiler_1.getMicrosecondsTime();
                    const entities = original.apply(this, args);
                    const duration = ProcessCpuProfiler_1.getDurationInMicroSeconds(start);
                    const k = target.constructor.name + '.' + propertyKey + (idProperty ? `[${this[idProperty]}]` : '');
                    if (!monitorings[k]) {
                        monitorings[k] = {
                            times: []
                        };
                    }
                    monitorings[k].times.push({
                        time: duration
                    });
                    return entities;
                };
            }
        }
    };
};
//# sourceMappingURL=MonitorExecutionTime.js.map