"use strict";
// Source file from duniter: Crypto-currency software to manage libre currency such as Ğ1
// Copyright (C) 2018  Cedric Moreau <cem.moreau@gmail.com>
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Affero General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Affero General Public License for more details.
Object.defineProperty(exports, "__esModule", { value: true });
const ProcessCpuProfiler_1 = require("../../ProcessCpuProfiler");
const logger_1 = require("../logger");
const other_constants_1 = require("../other_constants");
const theLogger = logger_1.NewLogger();
exports.MonitorSQLExecutionTime = function () {
    return function (target, propertyKey, descriptor) {
        if (other_constants_1.OtherConstants.ENABLE_SQL_MONITORING) {
            const original = descriptor.value;
            if (original.__proto__.constructor.name === "AsyncFunction") {
                descriptor.value = async function (...args) {
                    const start = ProcessCpuProfiler_1.getMicrosecondsTime();
                    const sql = args[0];
                    const params = args[1];
                    const entities = await original.apply(this, args);
                    const duration = ProcessCpuProfiler_1.getDurationInMicroSeconds(start);
                    theLogger.trace('[sqlite][query] %s %s %sµs', sql, JSON.stringify(params || []), duration);
                    return entities;
                };
            }
            else {
                throw Error("Monitoring an SQL synchronous function is not allowed.");
            }
        }
    };
};
//# sourceMappingURL=MonitorSQLExecutionTime.js.map