"use strict";
// Source file from duniter: Crypto-currency software to manage libre currency such as Ğ1
// Copyright (C) 2018  Cedric Moreau <cem.moreau@gmail.com>
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Affero General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Affero General Public License for more details.
Object.defineProperty(exports, "__esModule", { value: true });
const IdentityDTO_1 = require("./IdentityDTO");
const buid_1 = require("../common-libs/buid");
const common_1 = require("../common");
const DEFAULT_DOCUMENT_VERSION = 10;
class ShortCertificationDTO {
    constructor(pubkey, block_number, sig, idty_issuer) {
        this.pubkey = pubkey;
        this.block_number = block_number;
        this.sig = sig;
        this.idty_issuer = idty_issuer;
    }
    get issuer() {
        return this.pubkey;
    }
    get from() {
        return this.pubkey;
    }
    get to() {
        return this.idty_issuer;
    }
}
exports.ShortCertificationDTO = ShortCertificationDTO;
class CertificationDTO extends ShortCertificationDTO {
    constructor(version, currency, pubkey, buid, sig, idty_issuer, idty_uid, idty_buid, idty_sig) {
        super(pubkey, parseInt(buid.split(':')[0]), sig, idty_issuer);
        this.version = version;
        this.currency = currency;
        this.pubkey = pubkey;
        this.buid = buid;
        this.sig = sig;
        this.idty_issuer = idty_issuer;
        this.idty_uid = idty_uid;
        this.idty_buid = idty_buid;
        this.idty_sig = idty_sig;
    }
    clone() {
        return CertificationDTO.fromJSONObject(this);
    }
    getTargetHash() {
        return IdentityDTO_1.IdentityDTO.getTargetHash({
            uid: this.idty_uid,
            created_on: this.idty_buid,
            pub: this.idty_issuer
        });
    }
    getRawUnSigned() {
        let raw = "";
        raw += "Version: " + this.version + "\n";
        raw += "Type: Certification\n";
        raw += "Currency: " + this.currency + "\n";
        raw += "Issuer: " + this.pubkey + "\n";
        raw += "IdtyIssuer: " + this.idty_issuer + '\n';
        raw += "IdtyUniqueID: " + this.idty_uid + '\n';
        raw += "IdtyTimestamp: " + this.idty_buid + '\n';
        raw += "IdtySignature: " + this.idty_sig + '\n';
        raw += "CertTimestamp: " + this.buid + '\n';
        return raw;
    }
    getRawSigned() {
        return this.getRawUnSigned() + this.sig + '\n';
    }
    json() {
        return {
            "issuer": this.pubkey,
            "timestamp": this.buid,
            "sig": this.sig,
            "target": {
                "issuer": this.idty_issuer,
                "uid": this.idty_uid,
                "timestamp": this.idty_buid,
                "sig": this.idty_sig
            }
        };
    }
    inline() {
        return [this.pubkey, this.to, this.block_number, this.sig].join(':');
    }
    static fromInline(inline) {
        const [pubkey, to, block_number, sig] = inline.split(':');
        return new ShortCertificationDTO(pubkey, parseInt(block_number), sig, to);
    }
    static fromJSONObject(obj) {
        return new CertificationDTO(obj.version || DEFAULT_DOCUMENT_VERSION, obj.currency, obj.pubkey || obj.issuer || obj.from, obj.buid || buid_1.Buid.format.buid(obj.block_number, obj.block_hash), obj.sig, obj.idty_issuer || obj.to, obj.idty_uid, obj.idty_buid, obj.idty_sig);
    }
    getHash() {
        return common_1.hashf(this.getRawSigned());
    }
}
exports.CertificationDTO = CertificationDTO;
//# sourceMappingURL=CertificationDTO.js.map