"use strict";
// Source file from duniter: Crypto-currency software to manage libre currency such as Ğ1
// Copyright (C) 2018  Cedric Moreau <cem.moreau@gmail.com>
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Affero General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Affero General Public License for more details.
Object.defineProperty(exports, "__esModule", { value: true });
const moment = require("moment");
const IdentityDTO_1 = require("./IdentityDTO");
const common_1 = require("../common");
const DEFAULT_DOCUMENT_VERSION = 10;
class MembershipDTO {
    constructor(version, currency, issuer, type, blockstamp, userid, certts, signature) {
        this.version = version;
        this.currency = currency;
        this.issuer = issuer;
        this.type = type;
        this.blockstamp = blockstamp;
        this.userid = userid;
        this.certts = certts;
        this.signature = signature;
    }
    clone() {
        return MembershipDTO.fromJSONObject(this);
    }
    get pubkey() {
        return this.issuer;
    }
    get pub() {
        return this.issuer;
    }
    get membership() {
        return this.type;
    }
    get fpr() {
        return this.blockstamp.split('-')[1];
    }
    get number() {
        return parseInt(this.blockstamp);
    }
    get block_number() {
        return parseInt(this.blockstamp);
    }
    get block_hash() {
        return this.blockstamp.split('-')[1];
    }
    inline() {
        return [
            this.issuer,
            this.signature,
            this.blockstamp,
            this.certts,
            this.userid
        ].join(':');
    }
    getIdtyHash() {
        return IdentityDTO_1.IdentityDTO.getTargetHash({
            created_on: this.certts,
            uid: this.userid,
            pub: this.issuer
        });
    }
    getRaw() {
        let raw = "";
        raw += "Version: " + this.version + "\n";
        raw += "Type: Membership\n";
        raw += "Currency: " + this.currency + "\n";
        raw += "Issuer: " + this.issuer + "\n";
        raw += "Block: " + this.blockstamp + "\n";
        raw += "Membership: " + this.type + "\n";
        raw += "UserID: " + this.userid + "\n";
        raw += "CertTS: " + this.certts + "\n";
        return raw;
    }
    getRawSigned() {
        return this.getRaw() + this.signature + "\n";
    }
    json() {
        return {
            signature: this.signature,
            membership: {
                version: this.version,
                currency: this.currency,
                issuer: this.issuer,
                membership: this.type,
                date: this.date && moment(this.date).unix(),
                sigDate: this.sigDate && moment(this.sigDate).unix(),
                raw: this.getRaw()
            }
        };
    }
    static fromInline(inlineMS, type = "", currency = "") {
        const [issuer, sig, blockstamp, certts, userid] = inlineMS.split(':');
        return new MembershipDTO(DEFAULT_DOCUMENT_VERSION, currency, issuer, type, blockstamp, userid, certts, sig);
    }
    static fromJSONObject(obj) {
        return new MembershipDTO(obj.version || DEFAULT_DOCUMENT_VERSION, obj.currency, obj.issuer || obj.pubkey, obj.type || obj.membership, obj.blockstamp || obj.block, obj.userid, obj.certts, obj.signature);
    }
    getHash() {
        return common_1.hashf(this.getRawSigned());
    }
}
exports.MembershipDTO = MembershipDTO;
//# sourceMappingURL=MembershipDTO.js.map