"use strict";
// Source file from duniter: Crypto-currency software to manage libre currency such as Ğ1
// Copyright (C) 2018  Cedric Moreau <cem.moreau@gmail.com>
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Affero General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Affero General Public License for more details.
Object.defineProperty(exports, "__esModule", { value: true });
const common_1 = require("../common");
const constants_1 = require("../common-libs/constants");
const DBPeer_1 = require("../db/DBPeer");
class PeerDTO {
    constructor(version, currency, pubkey, blockstamp, endpoints, signature, status, statusTS, member = false) {
        this.version = version;
        this.currency = currency;
        this.pubkey = pubkey;
        this.blockstamp = blockstamp;
        this.endpoints = endpoints;
        this.signature = signature;
        this.status = status;
        this.statusTS = statusTS;
        this.member = false;
        this.member = member;
    }
    clone() {
        return PeerDTO.fromJSONObject(this);
    }
    get block() {
        return this.blockstamp;
    }
    blockNumber() {
        return parseInt(this.blockstamp);
    }
    keyID() {
        return this.pubkey && this.pubkey.length > 10 ? this.pubkey.substring(0, 10) : "Unknown";
    }
    getRawUnsigned() {
        return this.getRaw();
    }
    getRaw() {
        let raw = "";
        raw += "Version: " + this.version + "\n";
        raw += "Type: Peer\n";
        raw += "Currency: " + this.currency + "\n";
        raw += "PublicKey: " + this.pubkey + "\n";
        raw += "Block: " + this.blockstamp + "\n";
        raw += "Endpoints:" + "\n";
        for (const ep of this.endpoints) {
            raw += ep + "\n";
        }
        return raw;
    }
    getRawSigned() {
        return this.getRaw() + this.signature + "\n";
    }
    json() {
        return {
            version: this.version,
            currency: this.currency,
            endpoints: this.endpoints,
            status: this.status,
            block: this.block,
            signature: this.signature,
            raw: this.getRawSigned(),
            pubkey: this.pubkey
        };
    }
    getBMA() {
        let bma = {};
        let notFound = true;
        this.endpoints.forEach((ep) => {
            const matchesBMA = notFound && ep.match(constants_1.CommonConstants.BMA_REGEXP);
            const matchesBMAS = notFound && ep.match(constants_1.CommonConstants.BMAS_REGEXP);
            if (matchesBMA) {
                notFound = false;
                bma = {
                    "dns": matchesBMA[2] || '',
                    "ipv4": matchesBMA[4] || '',
                    "ipv6": matchesBMA[6] || '',
                    "port": parseInt(matchesBMA[8]) || 9101
                };
            }
            else if (matchesBMAS) {
                notFound = false;
                bma = {
                    "dns": matchesBMAS[2] || '',
                    "ipv4": matchesBMAS[4] || '',
                    "ipv6": matchesBMAS[6] || '',
                    "port": parseInt(matchesBMAS[8]) || 9101,
                    "path": matchesBMAS[10] || ''
                };
            }
        });
        return bma;
    }
    getOnceWS2PEndpoint(canReachTorEp, canReachClearEp, uuidExcluded = []) {
        let api = null;
        let bestWS2PVersionAvailable = 0;
        let bestWS2PTORVersionAvailable = 0;
        for (const ep of this.endpoints) {
            if (canReachTorEp) {
                let matches = ep.match(constants_1.CommonConstants.WS2PTOR_V2_REGEXP);
                if (matches && parseInt(matches[1]) > bestWS2PTORVersionAvailable && (uuidExcluded.indexOf(matches[2]) === -1)) {
                    bestWS2PTORVersionAvailable = parseInt(matches[1]);
                    api = {
                        version: parseInt(matches[1]),
                        uuid: matches[2],
                        host: matches[3] || '',
                        port: parseInt(matches[4]) || 0,
                        path: matches[5]
                    };
                }
                else {
                    matches = ep.match(constants_1.CommonConstants.WS2PTOR_REGEXP);
                    if (matches && bestWS2PTORVersionAvailable == 0 && (uuidExcluded.indexOf(matches[1]) === -1)) {
                        bestWS2PTORVersionAvailable = 1;
                        api = {
                            version: 1,
                            uuid: matches[1],
                            host: matches[2] || '',
                            port: parseInt(matches[3]) || 0,
                            path: matches[4]
                        };
                    }
                }
            }
            // If can reach clear endpoint and not found tor endpoint
            if (canReachClearEp && bestWS2PTORVersionAvailable == 0) {
                let matches = ep.match(constants_1.CommonConstants.WS2P_V2_REGEXP);
                if (matches && parseInt(matches[1]) > bestWS2PVersionAvailable && (uuidExcluded.indexOf(matches[2]) === -1)) {
                    bestWS2PVersionAvailable = parseInt(matches[1]);
                    api = {
                        version: parseInt(matches[1]),
                        uuid: matches[2],
                        host: matches[3] || '',
                        port: parseInt(matches[4]) || 0,
                        path: matches[5]
                    };
                }
                else {
                    matches = ep.match(constants_1.CommonConstants.WS2P_REGEXP);
                    if (matches && bestWS2PVersionAvailable == 0 && (uuidExcluded.indexOf(matches[1]) === -1)) {
                        bestWS2PVersionAvailable = 1;
                        api = {
                            version: 1,
                            uuid: matches[1],
                            host: matches[2] || '',
                            port: parseInt(matches[3]) || 0,
                            path: matches[4]
                        };
                    }
                }
            }
        }
        return api || null;
    }
    getAllWS2PEndpoints(canReachTorEp, canReachClearEp, myUUID) {
        let apis = [];
        let uuidExcluded = [myUUID];
        let api = this.getOnceWS2PEndpoint(canReachTorEp, canReachClearEp, uuidExcluded);
        while (api !== null) {
            uuidExcluded.push(api.uuid);
            apis.push(api);
            api = this.getOnceWS2PEndpoint(canReachTorEp, canReachClearEp, uuidExcluded);
        }
        return apis;
    }
    getFirstNonTorWS2P() {
        return this.getOnceWS2PEndpoint(false, true);
    }
    getDns() {
        const bma = this.getBMA();
        return bma.dns ? bma.dns : null;
    }
    getIPv4() {
        const bma = this.getBMA();
        return bma.ipv4 ? bma.ipv4 : null;
    }
    getIPv6() {
        const bma = this.getBMA();
        return bma.ipv6 ? bma.ipv6 : null;
    }
    getPort() {
        const bma = this.getBMA();
        return bma.port ? bma.port : null;
    }
    getPath() {
        const bma = this.getBMA();
        return bma.path ? bma.path : null;
    }
    getHostPreferDNS() {
        const bma = this.getBMA();
        return (bma.dns ? bma.dns :
            (bma.ipv4 ? bma.ipv4 :
                (bma.ipv6 ? bma.ipv6 : '')));
    }
    getURL() {
        const bma = this.getBMA();
        let base = this.getHostPreferDNS();
        if (bma.port)
            base += ':' + bma.port;
        return base;
    }
    hasValid4(bma) {
        return !!(bma.ipv4 && !bma.ipv4.match(/^127.0/) && !bma.ipv4.match(/^192.168/));
    }
    getNamedURL() {
        return this.getURL();
    }
    isReachable() {
        return !!(this.getURL());
    }
    containsEndpoint(ep) {
        return this.endpoints.reduce((found, endpoint) => found || endpoint == ep, false);
    }
    containsAllEndpoints(endpoints) {
        for (const ep of endpoints) {
            if (!this.containsEndpoint(ep)) {
                return false;
            }
        }
        return true;
    }
    endpointSum() {
        return this.endpoints.join('_');
    }
    getHash() {
        return common_1.hashf(this.getRawSigned());
    }
    toDBPeer() {
        const p = new DBPeer_1.DBPeer();
        p.version = this.version;
        p.currency = this.currency;
        p.status = this.status || "DOWN";
        p.statusTS = this.statusTS || 0;
        p.hash = this.getHash();
        p.first_down = 0;
        p.last_try = 0;
        p.pubkey = this.pubkey;
        p.block = this.block;
        p.signature = this.signature;
        p.endpoints = this.endpoints;
        p.raw = this.getRawSigned();
        return p;
    }
    static blockNumber(blockstamp) {
        return parseInt(blockstamp);
    }
    static fromDBPeer(p) {
        return new PeerDTO(p.version, p.currency, p.pubkey, p.block, p.endpoints, p.signature, p.status, p.statusTS, false);
    }
    static fromJSONObject(obj) {
        return new PeerDTO(parseInt(obj.version), obj.currency || "", obj.pubkey || obj.pub || obj.issuer || "", obj.blockstamp || obj.block, obj.endpoints || [], obj.signature || obj.sig, obj.status || "DOWN", obj.statusTS || 0, obj.member);
    }
    static endpoint2host(endpoint) {
        return PeerDTO.fromJSONObject({ endpoints: [endpoint] }).getURL();
    }
    static indexOfFirst(endpoints, intoEndpoints) {
        for (let i = 0; i < intoEndpoints.length; i++) {
            const index = endpoints.indexOf(intoEndpoints[i]);
            if (index !== -1) {
                return index;
            }
        }
        return 0;
    }
}
exports.PeerDTO = PeerDTO;
//# sourceMappingURL=PeerDTO.js.map