import { Cloneable } from "./Cloneable";
export interface BaseDTO {
    base: number;
}
export declare class InputDTO implements BaseDTO {
    amount: number;
    base: number;
    type: 'T' | 'D';
    identifier: string;
    pos: number;
    raw: string;
    constructor(amount: number, base: number, type: 'T' | 'D', identifier: string, pos: number, raw: string);
}
export declare class OutputDTO implements BaseDTO {
    amount: number;
    base: number;
    conditions: string;
    raw: string;
    constructor(amount: number, base: number, conditions: string, raw: string);
}
export interface TxSignatureResult {
    sigs: {
        k: string;
        ok: boolean;
    }[];
}
export declare class TxSignatureResultImpl implements TxSignatureResult {
    sigs: {
        k: string;
        ok: boolean;
    }[];
    constructor(issuers: string[]);
    readonly allMatching: boolean;
}
export declare class TransactionDTO implements Cloneable {
    version: number;
    currency: string;
    locktime: number;
    hash: string;
    blockstamp: string;
    blockstampTime: number;
    issuers: string[];
    inputs: string[];
    outputs: string[];
    unlocks: string[];
    signatures: string[];
    comment: string;
    clone(): any;
    constructor(version: number, currency: string, locktime: number, hash: string, blockstamp: string, blockstampTime: number, issuers: string[], inputs: string[], outputs: string[], unlocks: string[], signatures: string[], comment: string);
    readonly signature: string;
    readonly output_amount: number;
    readonly output_base: number;
    readonly blockNumber: number;
    readonly block_hash: string;
    getLen(): number;
    getHash(): string;
    getRawTxNoSig(): string;
    inputsAsObjects(): InputDTO[];
    outputsAsObjects(): OutputDTO[];
    outputsAsRecipients(): string[];
    getRaw(): string;
    getCompactVersion(): string;
    computeAllHashes(): void;
    json(): {
        'version': number;
        'currency': string;
        'issuers': string[];
        'inputs': string[];
        'unlocks': string[];
        'outputs': string[];
        'comment': string;
        'locktime': number;
        'blockstamp': string;
        'blockstampTime': number;
        'signatures': string[];
        'raw': string;
        'hash': string;
    };
    getTransactionSigResult(): TxSignatureResultImpl;
    checkSignatures(): boolean;
    static fromJSONObject(obj: any, currency?: string): TransactionDTO;
    static toRAW(json: TransactionDTO, noSig?: boolean): string;
    static outputObj2Str(o: OutputDTO): string;
    static inputObj2Str(i: InputDTO): string;
    static outputStr2Obj(outputStr: string): {
        amount: number;
        base: number;
        conditions: string;
        raw: string;
    };
    static inputStr2Obj(inputStr: string): {
        amount: string;
        base: string;
        type: string;
        identifier: string;
        pos: number;
        raw: string;
    };
    static unlock2params(unlock: string): string[];
    static mock(): TransactionDTO;
}
