// Source file from duniter: Crypto-currency software to manage libre currency such as Ğ1
// Copyright (C) 2018  Cedric Moreau <cem.moreau@gmail.com>
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Affero General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Affero General Public License for more details.
"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const BlockDTO_1 = require("../dto/BlockDTO");
const local_rules_1 = require("./local_rules");
exports.ALIAS = {
    ALL_LOCAL: async (block, conf, index) => {
        await local_rules_1.LOCAL_RULES_FUNCTIONS.checkParameters(block);
        await local_rules_1.LOCAL_RULES_FUNCTIONS.checkProofOfWork(block);
        await local_rules_1.LOCAL_RULES_FUNCTIONS.checkInnerHash(block);
        await local_rules_1.LOCAL_RULES_FUNCTIONS.checkPreviousHash(block);
        await local_rules_1.LOCAL_RULES_FUNCTIONS.checkPreviousIssuer(block);
        await local_rules_1.LOCAL_RULES_FUNCTIONS.checkUnitBase(block);
        await local_rules_1.LOCAL_RULES_FUNCTIONS.checkBlockSignature(block);
        await local_rules_1.LOCAL_RULES_FUNCTIONS.checkBlockTimes(block, conf);
        await local_rules_1.LOCAL_RULES_FUNCTIONS.checkIdentitiesSignature(block);
        await local_rules_1.LOCAL_RULES_FUNCTIONS.checkIdentitiesUserIDConflict(block, conf, index);
        await local_rules_1.LOCAL_RULES_FUNCTIONS.checkIdentitiesPubkeyConflict(block, conf, index);
        await local_rules_1.LOCAL_RULES_FUNCTIONS.checkIdentitiesMatchJoin(block, conf, index);
        await local_rules_1.LOCAL_RULES_FUNCTIONS.checkMembershipUnicity(block, conf, index);
        await local_rules_1.LOCAL_RULES_FUNCTIONS.checkRevokedUnicity(block, conf, index);
        await local_rules_1.LOCAL_RULES_FUNCTIONS.checkRevokedAreExcluded(block, conf, index);
        await local_rules_1.LOCAL_RULES_FUNCTIONS.checkMembershipsSignature(block);
        await local_rules_1.LOCAL_RULES_FUNCTIONS.checkPubkeyUnicity(block);
        await local_rules_1.LOCAL_RULES_FUNCTIONS.checkCertificationOneByIssuer(block, conf, index);
        await local_rules_1.LOCAL_RULES_FUNCTIONS.checkCertificationUnicity(block, conf, index);
        await local_rules_1.LOCAL_RULES_FUNCTIONS.checkCertificationIsntForLeaverOrExcluded(block, conf, index);
        await local_rules_1.LOCAL_RULES_FUNCTIONS.checkTxVersion(block);
        await local_rules_1.LOCAL_RULES_FUNCTIONS.checkTxIssuers(block);
        await local_rules_1.LOCAL_RULES_FUNCTIONS.checkTxSources(block);
        await local_rules_1.LOCAL_RULES_FUNCTIONS.checkTxRecipients(block);
        await local_rules_1.LOCAL_RULES_FUNCTIONS.checkTxAmounts(block);
        await local_rules_1.LOCAL_RULES_FUNCTIONS.checkTxSignature(block);
        await local_rules_1.LOCAL_RULES_FUNCTIONS.checkMaxTransactionChainingDepth(block, conf, index);
    },
    ALL_LOCAL_BUT_POW_AND_SIGNATURE: async (block, conf, index) => {
        await local_rules_1.LOCAL_RULES_FUNCTIONS.checkParameters(block);
        await local_rules_1.LOCAL_RULES_FUNCTIONS.checkInnerHash(block);
        await local_rules_1.LOCAL_RULES_FUNCTIONS.checkPreviousHash(block);
        await local_rules_1.LOCAL_RULES_FUNCTIONS.checkPreviousIssuer(block);
        await local_rules_1.LOCAL_RULES_FUNCTIONS.checkUnitBase(block);
        await local_rules_1.LOCAL_RULES_FUNCTIONS.checkBlockTimes(block, conf);
        await local_rules_1.LOCAL_RULES_FUNCTIONS.checkIdentitiesSignature(block);
        await local_rules_1.LOCAL_RULES_FUNCTIONS.checkIdentitiesUserIDConflict(block, conf, index);
        await local_rules_1.LOCAL_RULES_FUNCTIONS.checkIdentitiesPubkeyConflict(block, conf, index);
        await local_rules_1.LOCAL_RULES_FUNCTIONS.checkIdentitiesMatchJoin(block, conf, index);
        await local_rules_1.LOCAL_RULES_FUNCTIONS.checkMembershipUnicity(block, conf, index);
        await local_rules_1.LOCAL_RULES_FUNCTIONS.checkRevokedUnicity(block, conf, index);
        await local_rules_1.LOCAL_RULES_FUNCTIONS.checkRevokedAreExcluded(block, conf, index);
        await local_rules_1.LOCAL_RULES_FUNCTIONS.checkMembershipsSignature(block);
        await local_rules_1.LOCAL_RULES_FUNCTIONS.checkPubkeyUnicity(block);
        await local_rules_1.LOCAL_RULES_FUNCTIONS.checkCertificationOneByIssuer(block, conf, index);
        await local_rules_1.LOCAL_RULES_FUNCTIONS.checkCertificationUnicity(block, conf, index);
        await local_rules_1.LOCAL_RULES_FUNCTIONS.checkCertificationIsntForLeaverOrExcluded(block, conf, index);
        await local_rules_1.LOCAL_RULES_FUNCTIONS.checkTxVersion(block);
        await local_rules_1.LOCAL_RULES_FUNCTIONS.checkTxIssuers(block);
        await local_rules_1.LOCAL_RULES_FUNCTIONS.checkTxSources(block);
        await local_rules_1.LOCAL_RULES_FUNCTIONS.checkTxRecipients(block);
        await local_rules_1.LOCAL_RULES_FUNCTIONS.checkTxAmounts(block);
        await local_rules_1.LOCAL_RULES_FUNCTIONS.checkTxSignature(block);
        await local_rules_1.LOCAL_RULES_FUNCTIONS.checkMaxTransactionChainingDepth(block, conf, index);
    }
};
exports.CHECK = {
    ASYNC: {
        ALL_LOCAL: checkLocal(exports.ALIAS.ALL_LOCAL),
        ALL_LOCAL_BUT_POW: checkLocal(exports.ALIAS.ALL_LOCAL_BUT_POW_AND_SIGNATURE)
    }
};
function checkLocal(contract) {
    return async (b, conf, index, done = undefined) => {
        try {
            const block = BlockDTO_1.BlockDTO.fromJSONObject(b);
            await contract(block, conf, index);
            done && done();
        }
        catch (err) {
            if (done)
                return done(err);
            throw err;
        }
    };
}
//# sourceMappingURL=index.js.map