import { BlockDTO } from "../dto/BlockDTO";
import { ConfDTO } from "../dto/ConfDTO";
import { IndexEntry } from "../indexer";
import { TransactionDTO } from "../dto/TransactionDTO";
import { DBBlock } from "../db/DBBlock";
import { FileDAL } from "../dal/fileDAL";
export declare const LOCAL_RULES_FUNCTIONS: {
    checkParameters: (block: BlockDTO) => Promise<boolean>;
    isProofOfWorkCorrect: (block: BlockDTO) => boolean;
    checkProofOfWork: (block: BlockDTO) => Promise<boolean>;
    checkInnerHash: (block: BlockDTO) => Promise<boolean>;
    checkPreviousHash: (block: BlockDTO) => Promise<boolean>;
    checkPreviousIssuer: (block: BlockDTO) => Promise<boolean>;
    checkUnitBase: (block: BlockDTO) => Promise<boolean>;
    checkBlockSignature: (block: BlockDTO) => Promise<boolean>;
    checkBlockTimes: (block: BlockDTO, conf: ConfDTO) => Promise<boolean>;
    checkIdentitiesSignature: (block: BlockDTO) => Promise<boolean>;
    checkIdentitiesUserIDConflict: (block: BlockDTO, conf: ConfDTO, index: IndexEntry[]) => Promise<boolean>;
    checkIdentitiesPubkeyConflict: (block: BlockDTO, conf: ConfDTO, index: IndexEntry[]) => Promise<boolean>;
    checkIdentitiesMatchJoin: (block: BlockDTO, conf: ConfDTO, index: IndexEntry[]) => Promise<boolean>;
    checkRevokedAreExcluded: (block: BlockDTO, conf: ConfDTO, index: IndexEntry[]) => Promise<boolean>;
    checkRevokedUnicity: (block: BlockDTO, conf: ConfDTO, index: IndexEntry[]) => Promise<boolean>;
    checkMembershipUnicity: (block: BlockDTO, conf: ConfDTO, index: IndexEntry[]) => Promise<boolean>;
    checkMembershipsSignature: (block: BlockDTO) => Promise<boolean>;
    checkPubkeyUnicity: (block: BlockDTO) => Promise<boolean>;
    checkCertificationOneByIssuer: (block: BlockDTO, conf: ConfDTO, index: IndexEntry[]) => Promise<boolean>;
    checkCertificationUnicity: (block: BlockDTO, conf: ConfDTO, index: IndexEntry[]) => Promise<boolean>;
    checkCertificationIsntForLeaverOrExcluded: (block: BlockDTO, conf: ConfDTO, index: IndexEntry[]) => Promise<boolean>;
    checkTxVersion: (block: BlockDTO) => Promise<boolean>;
    checkTxLen: (block: BlockDTO) => Promise<boolean>;
    checkTxIssuers: (block: BlockDTO) => Promise<boolean>;
    checkTxSources: (block: BlockDTO) => Promise<boolean>;
    checkTxAmounts: (block: BlockDTO) => Promise<void>;
    checkTxRecipients: (block: BlockDTO) => Promise<boolean>;
    checkTxSignature: (block: BlockDTO) => Promise<boolean>;
    checkMaxTransactionChainingDepth: (block: BlockDTO, conf: ConfDTO, index: IndexEntry[]) => Promise<boolean>;
};
export interface SindexShortEntry {
    op: string;
    identifier: string;
    pos: number;
    tx: string | null;
}
declare function getMaxTransactionDepth(sindex: SindexShortEntry[]): number;
declare function getTransactionDepth(txHash: string, sindex: SindexShortEntry[], localDepth?: number): number;
declare function checkSingleMembershipSignature(ms: any): any;
declare function checkBunchOfTransactions(transactions: TransactionDTO[], conf: ConfDTO, medianTime: number, options?: {
    dontCareAboutChaining?: boolean;
}): Promise<void>;
export declare const LOCAL_RULES_HELPERS: {
    maxAcceleration: (conf: ConfDTO) => any;
    checkSingleMembershipSignature: typeof checkSingleMembershipSignature;
    checkBunchOfTransactions: typeof checkBunchOfTransactions;
    getTransactionDepth: typeof getTransactionDepth;
    getMaxTransactionDepth: typeof getMaxTransactionDepth;
    checkSingleTransactionLocally: (tx: any, conf: ConfDTO) => Promise<void>;
    checkTxAmountsValidity: (tx: TransactionDTO) => void;
    getMaxPossibleVersionNumber: (current: DBBlock | null, dal: FileDAL) => Promise<number>;
};
export {};
