import { SQLiteDriver } from "../dal/drivers/SQLiteDriver";
import { FileDALParams } from "../dal/fileDAL";
import { LevelUp } from 'levelup';
import { AbstractLevelDOWN } from 'abstract-leveldown';
export interface FileSystem {
    fsExists(file: string): Promise<boolean>;
    fsReadFile(file: string): Promise<string>;
    fsUnlink(file: string): Promise<boolean>;
    fsList(dir: string): Promise<string[]>;
    fsWrite(file: string, content: string): Promise<void>;
    fsMakeDirectory(dir: string): Promise<void>;
    fsRemoveTree(dir: string): Promise<void>;
    fsStreamTo(file: string, iterator: IterableIterator<string>): Promise<void>;
}
export declare const RealFS: () => FileSystem;
export declare const MemFS: (initialTree?: {
    [folder: string]: {
        [file: string]: string;
    };
}) => FileSystem;
export declare const Directory: {
    DATA_FILES: string[];
    WW_FILES: string[];
    DATA_DIRS: string[];
    INSTANCE_NAME: string;
    INSTANCE_HOME: string;
    GET_FILE_PATH: (fileSubPath: string, home?: string) => string;
    INSTANCE_HOMELOG_FILE: string;
    DUNITER_DB_NAME: string;
    LOKI_DB_DIR: string;
    WOTB_FILE: string;
    getHome: (profile?: string | null, directory?: string | null) => string;
    getHomeDB: (isMemory: boolean, dbName: string, home?: string) => Promise<SQLiteDriver>;
    getHomeLevelDB: (isMemory: boolean, dbName: string, home?: string) => Promise<LevelUp<AbstractLevelDOWN<any, any>>>;
    getHomeFS: (isMemory: boolean, theHome: string, makeTree?: boolean) => Promise<{
        home: string;
        fs: FileSystem;
    }>;
    getHomeParams: (isMemory: boolean, theHome: string) => Promise<FileDALParams>;
    createHomeIfNotExists: (fileSystem: any, theHome: string) => Promise<void>;
};
