"use strict";
// Source file from duniter: Crypto-currency software to manage libre currency such as Ğ1
// Copyright (C) 2018  Cedric Moreau <cem.moreau@gmail.com>
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Affero General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Affero General Public License for more details.
Object.defineProperty(exports, "__esModule", { value: true });
const network_1 = require("./network");
const dtos_1 = require("./dtos");
const limiter_1 = require("./limiter");
const blockchain_1 = require("./controllers/blockchain");
const node_1 = require("./controllers/node");
const network_2 = require("./controllers/network");
const wot_1 = require("./controllers/wot");
const transactions_1 = require("./controllers/transactions");
const uds_1 = require("./controllers/uds");
const PeerDTO_1 = require("../../../lib/dto/PeerDTO");
const BlockDTO_1 = require("../../../lib/dto/BlockDTO");
const other_constants_1 = require("../../../lib/other_constants");
const websocket_1 = require("../../../lib/common-libs/websocket");
const es = require('event-stream');
exports.bma = function (server, interfaces, httpLogs, logger) {
    if (!interfaces) {
        interfaces = [];
        if (server.conf) {
            if (server.conf.ipv4) {
                interfaces = [{
                        ip: server.conf.ipv4,
                        port: server.conf.port
                    }];
            }
            if (server.conf.ipv6) {
                interfaces.push({
                    ip: server.conf.ipv6,
                    port: (server.conf.remoteport || server.conf.port) // We try to get the best one
                });
            }
        }
    }
    return network_1.Network.createServersAndListen('BMA server', server, interfaces, httpLogs, logger, null, (app, httpMethods) => {
        const node = new node_1.NodeBinding(server);
        const blockchain = new blockchain_1.BlockchainBinding(server);
        const net = new network_2.NetworkBinding(server);
        const wot = new wot_1.WOTBinding(server);
        const transactions = new transactions_1.TransactionBinding(server);
        const dividend = new uds_1.UDBinding(server);
        httpMethods.httpGET('/', (req) => node.summary(), limiter_1.BMALimitation.limitAsHighUsage());
        httpMethods.httpGET('/node/summary', (req) => node.summary(), limiter_1.BMALimitation.limitAsHighUsage());
        httpMethods.httpGET('/node/sandboxes', (req) => node.sandboxes(), limiter_1.BMALimitation.limitAsHighUsage());
        httpMethods.httpGET('/blockchain/parameters', (req) => blockchain.parameters(), limiter_1.BMALimitation.limitAsHighUsage());
        httpMethods.httpPOST('/blockchain/membership', (req) => blockchain.parseMembership(req), limiter_1.BMALimitation.limitAsHighUsage());
        httpMethods.httpGET('/blockchain/memberships/:search', (req) => blockchain.memberships(req), limiter_1.BMALimitation.limitAsHighUsage());
        httpMethods.httpPOST('/blockchain/block', (req) => blockchain.parseBlock(req), limiter_1.BMALimitation.limitAsHighUsage());
        httpMethods.httpGET('/blockchain/block/:number', (req) => blockchain.promoted(req), limiter_1.BMALimitation.limitAsHighUsage());
        httpMethods.httpGET('/blockchain/blocks/:count/:from', (req) => blockchain.blocks(req), limiter_1.BMALimitation.limitAsHighUsage());
        httpMethods.httpGET('/blockchain/milestones', (req) => blockchain.milestones(req), limiter_1.BMALimitation.limitAsHighUsage());
        httpMethods.httpGET('/blockchain/milestones/:page', (req) => blockchain.milestones(req), limiter_1.BMALimitation.limitAsHighUsage());
        httpMethods.httpGET('/blockchain/current', (req) => blockchain.current(), limiter_1.BMALimitation.limitAsHighUsage());
        httpMethods.httpGET('/blockchain/hardship/:search', (req) => blockchain.hardship(req), limiter_1.BMALimitation.limitAsHighUsage());
        httpMethods.httpGET('/blockchain/difficulties', (req) => blockchain.difficulties(), limiter_1.BMALimitation.limitAsHighUsage());
        httpMethods.httpGET('/blockchain/with/newcomers', (req) => blockchain.with.newcomers(req), limiter_1.BMALimitation.limitAsHighUsage());
        httpMethods.httpGET('/blockchain/with/certs', (req) => blockchain.with.certs(req), limiter_1.BMALimitation.limitAsHighUsage());
        httpMethods.httpGET('/blockchain/with/joiners', (req) => blockchain.with.joiners(req), limiter_1.BMALimitation.limitAsHighUsage());
        httpMethods.httpGET('/blockchain/with/actives', (req) => blockchain.with.actives(req), limiter_1.BMALimitation.limitAsHighUsage());
        httpMethods.httpGET('/blockchain/with/leavers', (req) => blockchain.with.leavers(req), limiter_1.BMALimitation.limitAsHighUsage());
        httpMethods.httpGET('/blockchain/with/excluded', (req) => blockchain.with.excluded(req), limiter_1.BMALimitation.limitAsHighUsage());
        httpMethods.httpGET('/blockchain/with/revoked', (req) => blockchain.with.revoked(req), limiter_1.BMALimitation.limitAsHighUsage());
        httpMethods.httpGET('/blockchain/with/ud', (req) => blockchain.with.ud(req), limiter_1.BMALimitation.limitAsHighUsage());
        httpMethods.httpGET('/blockchain/with/tx', (req) => blockchain.with.tx(req), limiter_1.BMALimitation.limitAsHighUsage());
        httpMethods.httpGET('/blockchain/branches', (req) => blockchain.branches(), limiter_1.BMALimitation.limitAsHighUsage());
        httpMethods.httpGET('/network/peering', (req) => net.peer(), limiter_1.BMALimitation.limitAsHighUsage());
        httpMethods.httpGET('/network/peering/peers', (req) => net.peersGet(req), limiter_1.BMALimitation.limitAsVeryHighUsage());
        httpMethods.httpPOST('/network/peering/peers', (req) => net.peersPost(req), limiter_1.BMALimitation.limitAsHighUsage());
        httpMethods.httpGET('/network/peers', (req) => net.peers(), limiter_1.BMALimitation.limitAsHighUsage());
        httpMethods.httpGET('/network/ws2p/info', (req) => net.ws2pInfo(), limiter_1.BMALimitation.limitAsHighUsage());
        httpMethods.httpGET('/network/ws2p/heads', (req) => net.ws2pHeads(), limiter_1.BMALimitation.limitAsHighUsage());
        httpMethods.httpPOST('/wot/add', (req) => wot.add(req), limiter_1.BMALimitation.limitAsHighUsage());
        httpMethods.httpPOST('/wot/certify', (req) => wot.certify(req), limiter_1.BMALimitation.limitAsHighUsage());
        httpMethods.httpPOST('/wot/revoke', (req) => wot.revoke(req), limiter_1.BMALimitation.limitAsHighUsage());
        httpMethods.httpGET('/wot/lookup/:search', (req) => wot.lookup(req), limiter_1.BMALimitation.limitAsHighUsage());
        httpMethods.httpGET('/wot/members', (req) => wot.members(), limiter_1.BMALimitation.limitAsHighUsage());
        httpMethods.httpGET('/wot/pending', (req) => wot.pendingMemberships(), limiter_1.BMALimitation.limitAsHighUsage());
        httpMethods.httpGET('/wot/requirements/:search', (req) => wot.requirements(req), limiter_1.BMALimitation.limitAsHighUsage());
        httpMethods.httpGET('/wot/requirements-of-pending/:minsig', (req) => wot.requirementsOfPending(req), limiter_1.BMALimitation.limitAsLowUsage());
        httpMethods.httpGET('/wot/certifiers-of/:search', (req) => wot.certifiersOf(req), limiter_1.BMALimitation.limitAsHighUsage());
        httpMethods.httpGET('/wot/certified-by/:search', (req) => wot.certifiedBy(req), limiter_1.BMALimitation.limitAsHighUsage());
        httpMethods.httpGET('/wot/identity-of/:search', (req) => wot.identityOf(req), limiter_1.BMALimitation.limitAsHighUsage());
        httpMethods.httpPOST('/tx/process', (req) => transactions.parseTransaction(req), limiter_1.BMALimitation.limitAsHighUsage());
        httpMethods.httpGET('/tx/hash/:hash', (req) => transactions.getByHash(req), limiter_1.BMALimitation.limitAsHighUsage());
        httpMethods.httpGET('/tx/sources/:pubkey', (req) => transactions.getSources(req), limiter_1.BMALimitation.limitAsHighUsage());
        httpMethods.httpGET('/tx/history/:pubkey', (req) => transactions.getHistory(req), limiter_1.BMALimitation.limitAsHighUsage());
        httpMethods.httpGET('/tx/history/:pubkey/blocks/:from/:to', (req) => transactions.getHistoryBetweenBlocks(req), limiter_1.BMALimitation.limitAsHighUsage());
        httpMethods.httpGET('/tx/history/:pubkey/times/:from/:to', (req) => transactions.getHistoryBetweenTimes(req), limiter_1.BMALimitation.limitAsHighUsage());
        httpMethods.httpGET('/tx/history/:pubkey/pending', (req) => transactions.getPendingForPubkey(req), limiter_1.BMALimitation.limitAsHighUsage());
        httpMethods.httpGET('/tx/pending', (req) => transactions.getPending(), limiter_1.BMALimitation.limitAsHighUsage());
        httpMethods.httpGET('/ud/history/:pubkey', (req) => dividend.getHistory(req), limiter_1.BMALimitation.limitAsHighUsage());
        httpMethods.httpGET('/ud/history/:pubkey/blocks/:from/:to', (req) => dividend.getHistoryBetweenBlocks(req), limiter_1.BMALimitation.limitAsHighUsage());
        httpMethods.httpGET('/ud/history/:pubkey/times/:from/:to', (req) => dividend.getHistoryBetweenTimes(req), limiter_1.BMALimitation.limitAsHighUsage());
    }, (httpServer) => {
        let currentBlock = {};
        let wssBlock = new websocket_1.WebSocketServer({
            server: httpServer,
            path: '/ws/block'
        });
        let wssPeer = new websocket_1.WebSocketServer({
            server: httpServer,
            path: '/ws/peer'
        });
        let wssHeads = new websocket_1.WebSocketServer({
            server: httpServer,
            path: '/ws/heads'
        });
        const errorHandler = function (error) {
            logger && logger.error('Error on WS Server');
            logger && logger.error(error);
        };
        wssBlock.on('error', errorHandler);
        wssPeer.on('error', errorHandler);
        wssHeads.on('error', errorHandler);
        wssBlock.on('connection', async function connection(ws) {
            try {
                currentBlock = await server.dal.getCurrentBlockOrNull();
                if (currentBlock) {
                    const blockDTO = BlockDTO_1.BlockDTO.fromJSONObject(currentBlock);
                    ws.send(JSON.stringify(dtos_1.block2HttpBlock(blockDTO)));
                }
            }
            catch (e) {
                logger.error(e);
            }
        });
        wssHeads.on('connection', async (ws) => {
            if (server.ws2pCluster) {
                try {
                    ws.send(JSON.stringify(await server.ws2pCluster.getKnownHeads()));
                }
                catch (e) {
                    logger.error(e);
                }
            }
        });
        const wssHeadsBroadcast = (data) => wssHeads.clients.forEach((client) => client.send(data));
        const wssBlockBroadcast = (data) => wssBlock.clients.forEach((client) => {
            try {
                client.send(data);
            }
            catch (e) {
                logger && logger.error('error on ws: %s', e);
            }
        });
        const wssPeerBroadcast = (data) => wssPeer.clients.forEach((client) => client.send(data));
        // Forward current HEAD change
        server
            .on('bcEvent', (e) => {
            if (e.bcEvent === other_constants_1.OtherConstants.BC_EVENT.HEAD_CHANGED || e.bcEvent === other_constants_1.OtherConstants.BC_EVENT.SWITCHED) {
                try {
                    // Broadcast block
                    currentBlock = e.block;
                    const blockDTO = BlockDTO_1.BlockDTO.fromJSONObject(currentBlock);
                    wssBlockBroadcast(JSON.stringify(dtos_1.block2HttpBlock(blockDTO)));
                }
                catch (e) {
                    logger && logger.error('error on ws mapSync:', e);
                }
            }
        });
        // Forward peers documents
        server
            .pipe(es.mapSync(function (data) {
            try {
                // Broadcast peer
                if (data.endpoints) {
                    const peerDTO = PeerDTO_1.PeerDTO.fromJSONObject(data);
                    const peerResult = {
                        version: peerDTO.version,
                        currency: peerDTO.currency,
                        pubkey: peerDTO.pubkey,
                        block: peerDTO.blockstamp,
                        endpoints: peerDTO.endpoints,
                        signature: peerDTO.signature,
                        raw: peerDTO.getRaw()
                    };
                    wssPeerBroadcast(JSON.stringify(peerResult));
                }
                // Broadcast heads
                else if (data.ws2p === 'heads' && data.added.length) {
                    wssHeadsBroadcast(JSON.stringify(data.added));
                }
            }
            catch (e) {
                logger && logger.error('error on ws mapSync:', e);
            }
        }));
    });
};
//# sourceMappingURL=bma.js.map