"use strict";
// Source file from duniter: Crypto-currency software to manage libre currency such as Ğ1
// Copyright (C) 2018  Cedric Moreau <cem.moreau@gmail.com>
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Affero General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Affero General Public License for more details.
Object.defineProperty(exports, "__esModule", { value: true });
const dos2unix_1 = require("../../../../lib/common-libs/dos2unix");
const constants_1 = require("../../../../lib/common-libs/constants");
class AbstractController {
    constructor(server) {
        this.server = server;
    }
    get conf() {
        return this.server.conf;
    }
    get logger() {
        return this.server.logger;
    }
    get BlockchainService() {
        return this.server.BlockchainService;
    }
    get IdentityService() {
        return this.server.IdentityService;
    }
    get PeeringService() {
        return this.server.PeeringService;
    }
    get MerkleService() {
        return this.server.MerkleService;
    }
    async pushEntity(req, rawer, task) {
        let rawDocument = rawer(req);
        rawDocument = dos2unix_1.dos2unix(rawDocument);
        try {
            return await task(rawDocument);
        }
        catch (e) {
            const event = constants_1.CommonConstants.DocumentError;
            this.server.emit(event, e);
            if (e !== "Block already known" && (!e || !e.uerr || (e.uerr.ucode !== constants_1.CommonConstants.ERRORS.PEER_DOCUMENT_ALREADY_KNOWN.uerr.ucode
                && e.uerr.ucode !== constants_1.CommonConstants.ERRORS.DOCUMENT_BEING_TREATED.uerr.ucode))) {
                this.logger.error(e);
            }
            throw e;
        }
    }
}
exports.AbstractController = AbstractController;
//# sourceMappingURL=AbstractController.js.map