"use strict";
// Source file from duniter: Crypto-currency software to manage libre currency such as Ğ1
// Copyright (C) 2018  Cedric Moreau <cem.moreau@gmail.com>
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Affero General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Affero General Public License for more details.
Object.defineProperty(exports, "__esModule", { value: true });
const AbstractController_1 = require("./AbstractController");
const parameters_1 = require("../parameters");
const source_1 = require("../entity/source");
const underscore_1 = require("../../../../lib/common-libs/underscore");
class UDBinding extends AbstractController_1.AbstractController {
    async getHistory(req) {
        const pubkey = await parameters_1.ParametersService.getPubkeyP(req);
        return this.getUDSources(pubkey, (results) => results);
    }
    async getHistoryBetweenBlocks(req) {
        const pubkey = await parameters_1.ParametersService.getPubkeyP(req);
        const from = await parameters_1.ParametersService.getFromP(req);
        const to = await parameters_1.ParametersService.getToP(req);
        return this.getUDSources(pubkey, (results) => {
            results.history.history = underscore_1.Underscore.filter(results.history.history, function (ud) { return ud.block_number >= from && ud.block_number <= to; });
            return results;
        });
    }
    async getHistoryBetweenTimes(req) {
        const pubkey = await parameters_1.ParametersService.getPubkeyP(req);
        const from = await parameters_1.ParametersService.getFromP(req);
        const to = await parameters_1.ParametersService.getToP(req);
        return this.getUDSources(pubkey, (results) => {
            results.history.history = underscore_1.Underscore.filter(results.history.history, function (ud) { return ud.time >= from && ud.time <= to; });
            return results;
        });
    }
    async getUDSources(pubkey, filter) {
        const history = await this.server.dal.getUDHistory(pubkey);
        const result = {
            "currency": this.conf.currency,
            "pubkey": pubkey,
            "history": history
        };
        underscore_1.Underscore.keys(history).map((key) => {
            history[key].map((src, index) => {
                history[key][index] = new source_1.Source(src).UDjson();
                underscore_1.Underscore.extend(history[key][index], { block_number: src && src.block_number, time: src && src.time });
            });
        });
        return filter(result);
    }
}
exports.UDBinding = UDBinding;
//# sourceMappingURL=uds.js.map