import { BlockDTO } from "../../../lib/dto/BlockDTO";
import { WS2PHead } from "../../ws2p/lib/WS2PCluster";
import { JSONDBPeer } from "../../../lib/db/DBPeer";
export declare const Summary: {
    duniter: {
        "software": StringConstructor;
        "version": StringConstructor;
        "forkWindowSize": NumberConstructor;
    };
};
export interface HttpSummary {
    duniter: {
        software: string;
        version: string;
        forkWindowSize: number;
    };
}
export declare const Parameters: {
    currency: StringConstructor;
    c: NumberConstructor;
    dt: NumberConstructor;
    ud0: NumberConstructor;
    sigPeriod: NumberConstructor;
    sigStock: NumberConstructor;
    sigWindow: NumberConstructor;
    sigValidity: NumberConstructor;
    sigReplay: NumberConstructor;
    sigQty: NumberConstructor;
    idtyWindow: NumberConstructor;
    msWindow: NumberConstructor;
    msPeriod: NumberConstructor;
    xpercent: NumberConstructor;
    msValidity: NumberConstructor;
    stepMax: NumberConstructor;
    medianTimeBlocks: NumberConstructor;
    avgGenTime: NumberConstructor;
    dtDiffEval: NumberConstructor;
    percentRot: NumberConstructor;
    udTime0: NumberConstructor;
    udReevalTime0: NumberConstructor;
    dtReeval: NumberConstructor;
};
export interface HttpParameters {
    currency: string;
    c: number;
    dt: number;
    ud0: number;
    sigPeriod: number;
    sigStock: number;
    sigWindow: number;
    sigValidity: number;
    sigReplay: number;
    sigQty: number;
    idtyWindow: number;
    msWindow: number;
    msPeriod: number;
    xpercent: number;
    msValidity: number;
    stepMax: number;
    medianTimeBlocks: number;
    avgGenTime: number;
    dtDiffEval: number;
    percentRot: number;
    udTime0: number;
    udReevalTime0: number;
    dtReeval: number;
}
export declare const Membership: {
    "signature": StringConstructor;
    "membership": {
        "version": NumberConstructor;
        "currency": StringConstructor;
        "issuer": StringConstructor;
        "membership": StringConstructor;
        "date": NumberConstructor;
        "sigDate": NumberConstructor;
        "raw": StringConstructor;
    };
};
export interface HttpMembership {
    signature: string;
    membership: {
        version: number;
        currency: string;
        issuer: string;
        membership: string;
        date: number;
        sigDate: number;
        raw: string;
    };
}
export declare const Memberships: {
    "pubkey": StringConstructor;
    "uid": StringConstructor;
    "sigDate": StringConstructor;
    "memberships": {
        "version": NumberConstructor;
        "currency": StringConstructor;
        "membership": StringConstructor;
        "blockNumber": NumberConstructor;
        "blockHash": StringConstructor;
        "written": NumberConstructor;
    }[];
};
export interface HttpMemberships {
    pubkey: string;
    uid: string;
    sigDate: string;
    memberships: {
        version: number;
        currency: string;
        membership: string;
        blockNumber: number;
        blockHash: string;
        written: number;
    }[];
}
export declare const MembershipList: {
    "memberships": {
        "pubkey": StringConstructor;
        "uid": StringConstructor;
        "version": NumberConstructor;
        "currency": StringConstructor;
        "membership": StringConstructor;
        "blockNumber": NumberConstructor;
        "blockHash": StringConstructor;
        "written": NumberConstructor;
    }[];
};
export interface HttpMembershipList {
    memberships: {
        pubkey: string;
        uid: string;
        version: number;
        currency: string;
        membership: string;
        blockNumber: number;
        blockHash: string;
        written: number | null;
    }[];
}
export declare const TransactionOfBlock: {
    "version": NumberConstructor;
    "currency": StringConstructor;
    "comment": StringConstructor;
    "locktime": NumberConstructor;
    "signatures": StringConstructor[];
    "outputs": StringConstructor[];
    "inputs": StringConstructor[];
    "unlocks": StringConstructor[];
    "block_number": NumberConstructor;
    "blockstamp": StringConstructor;
    "blockstampTime": NumberConstructor;
    "time": NumberConstructor;
    "issuers": StringConstructor[];
};
export interface HttpTransactionOfBlock {
    version: number;
    currency: string;
    locktime: number;
    hash: string;
    blockstamp: string;
    blockstampTime: number;
    issuers: string[];
    inputs: string[];
    outputs: string[];
    unlocks: string[];
    signatures: string[];
    comment: string;
}
export declare const Block: {
    "version": NumberConstructor;
    "currency": StringConstructor;
    "number": NumberConstructor;
    "issuer": StringConstructor;
    "issuersFrame": NumberConstructor;
    "issuersFrameVar": NumberConstructor;
    "issuersCount": NumberConstructor;
    "parameters": StringConstructor;
    "membersCount": NumberConstructor;
    "monetaryMass": NumberConstructor;
    "powMin": NumberConstructor;
    "time": NumberConstructor;
    "medianTime": NumberConstructor;
    "dividend": NumberConstructor;
    "unitbase": NumberConstructor;
    "hash": StringConstructor;
    "previousHash": StringConstructor;
    "previousIssuer": StringConstructor;
    "identities": StringConstructor[];
    "certifications": StringConstructor[];
    "joiners": StringConstructor[];
    "actives": StringConstructor[];
    "leavers": StringConstructor[];
    "revoked": StringConstructor[];
    "excluded": StringConstructor[];
    "transactions": {
        "version": NumberConstructor;
        "currency": StringConstructor;
        "comment": StringConstructor;
        "locktime": NumberConstructor;
        "signatures": StringConstructor[];
        "outputs": StringConstructor[];
        "inputs": StringConstructor[];
        "unlocks": StringConstructor[];
        "block_number": NumberConstructor;
        "blockstamp": StringConstructor;
        "blockstampTime": NumberConstructor;
        "time": NumberConstructor;
        "issuers": StringConstructor[];
    }[];
    "nonce": NumberConstructor;
    "inner_hash": StringConstructor;
    "signature": StringConstructor;
    "raw": StringConstructor;
};
export interface HttpBlock {
    version: number;
    currency: string;
    number: number;
    issuer: string;
    issuersFrame: number;
    issuersFrameVar: number;
    issuersCount: number;
    parameters: string;
    membersCount: number;
    monetaryMass: number;
    powMin: number;
    time: number;
    medianTime: number;
    dividend: number | null;
    unitbase: number;
    hash: string;
    previousHash: string;
    previousIssuer: string;
    identities: string[];
    certifications: string[];
    joiners: string[];
    actives: string[];
    leavers: string[];
    revoked: string[];
    excluded: string[];
    transactions: HttpTransactionOfBlock[];
    nonce: number;
    inner_hash: string;
    signature: string;
    raw: string;
}
export declare function block2HttpBlock(blockDTO: BlockDTO): HttpBlock;
export declare const Hardship: {
    "block": NumberConstructor;
    "level": NumberConstructor;
};
export interface HttpHardship {
    block: number;
    level: number;
}
export declare const Difficulty: {
    "uid": StringConstructor;
    "level": NumberConstructor;
};
export interface HttpDifficulty {
    uid: string;
    level: number;
}
export declare const Difficulties: {
    "block": NumberConstructor;
    "levels": {
        "uid": StringConstructor;
        "level": NumberConstructor;
    }[];
};
export interface HttpDifficulties {
    block: number;
    levels: HttpDifficulty[];
}
export declare const Blocks: {
    "version": NumberConstructor;
    "currency": StringConstructor;
    "number": NumberConstructor;
    "issuer": StringConstructor;
    "issuersFrame": NumberConstructor;
    "issuersFrameVar": NumberConstructor;
    "issuersCount": NumberConstructor;
    "parameters": StringConstructor;
    "membersCount": NumberConstructor;
    "monetaryMass": NumberConstructor;
    "powMin": NumberConstructor;
    "time": NumberConstructor;
    "medianTime": NumberConstructor;
    "dividend": NumberConstructor;
    "unitbase": NumberConstructor;
    "hash": StringConstructor;
    "previousHash": StringConstructor;
    "previousIssuer": StringConstructor;
    "identities": StringConstructor[];
    "certifications": StringConstructor[];
    "joiners": StringConstructor[];
    "actives": StringConstructor[];
    "leavers": StringConstructor[];
    "revoked": StringConstructor[];
    "excluded": StringConstructor[];
    "transactions": {
        "version": NumberConstructor;
        "currency": StringConstructor;
        "comment": StringConstructor;
        "locktime": NumberConstructor;
        "signatures": StringConstructor[];
        "outputs": StringConstructor[];
        "inputs": StringConstructor[];
        "unlocks": StringConstructor[];
        "block_number": NumberConstructor;
        "blockstamp": StringConstructor;
        "blockstampTime": NumberConstructor;
        "time": NumberConstructor;
        "issuers": StringConstructor[];
    }[];
    "nonce": NumberConstructor;
    "inner_hash": StringConstructor;
    "signature": StringConstructor;
    "raw": StringConstructor;
}[];
export declare const Stat: {
    "result": {
        "blocks": NumberConstructor[];
    };
};
export interface HttpStat {
    result: {
        blocks: number[];
    };
}
export declare const Branches: {
    "blocks": {
        "version": NumberConstructor;
        "currency": StringConstructor;
        "number": NumberConstructor;
        "issuer": StringConstructor;
        "issuersFrame": NumberConstructor;
        "issuersFrameVar": NumberConstructor;
        "issuersCount": NumberConstructor;
        "parameters": StringConstructor;
        "membersCount": NumberConstructor;
        "monetaryMass": NumberConstructor;
        "powMin": NumberConstructor;
        "time": NumberConstructor;
        "medianTime": NumberConstructor;
        "dividend": NumberConstructor;
        "unitbase": NumberConstructor;
        "hash": StringConstructor;
        "previousHash": StringConstructor;
        "previousIssuer": StringConstructor;
        "identities": StringConstructor[];
        "certifications": StringConstructor[];
        "joiners": StringConstructor[];
        "actives": StringConstructor[];
        "leavers": StringConstructor[];
        "revoked": StringConstructor[];
        "excluded": StringConstructor[];
        "transactions": {
            "version": NumberConstructor;
            "currency": StringConstructor;
            "comment": StringConstructor;
            "locktime": NumberConstructor;
            "signatures": StringConstructor[];
            "outputs": StringConstructor[];
            "inputs": StringConstructor[];
            "unlocks": StringConstructor[];
            "block_number": NumberConstructor;
            "blockstamp": StringConstructor;
            "blockstampTime": NumberConstructor;
            "time": NumberConstructor;
            "issuers": StringConstructor[];
        }[];
        "nonce": NumberConstructor;
        "inner_hash": StringConstructor;
        "signature": StringConstructor;
        "raw": StringConstructor;
    }[];
};
export interface HttpBranches {
    blocks: HttpBlock[];
}
export declare const Peer: {
    "version": NumberConstructor;
    "currency": StringConstructor;
    "pubkey": StringConstructor;
    "block": StringConstructor;
    "endpoints": StringConstructor[];
    "signature": StringConstructor;
    "raw": StringConstructor;
};
export interface HttpPeer {
    version: number;
    currency: string;
    pubkey: string;
    block: string;
    endpoints: string[];
    signature: string;
    raw: string;
}
export declare const DBPeer: {
    "version": NumberConstructor;
    "currency": StringConstructor;
    "pubkey": StringConstructor;
    "block": StringConstructor;
    "status": StringConstructor;
    "first_down": NumberConstructor;
    "last_try": NumberConstructor;
    "endpoints": StringConstructor[];
    "signature": StringConstructor;
    "raw": StringConstructor;
};
export declare const Peers: {
    "peers": {
        "version": NumberConstructor;
        "currency": StringConstructor;
        "pubkey": StringConstructor;
        "block": StringConstructor;
        "status": StringConstructor;
        "first_down": NumberConstructor;
        "last_try": NumberConstructor;
        "endpoints": StringConstructor[];
        "signature": StringConstructor;
        "raw": StringConstructor;
    }[];
};
export interface HttpPeers {
    peers: JSONDBPeer[];
}
export interface HttpWS2PInfo {
    peers: {
        level1: number;
        level2: number;
    };
}
export interface HttpWS2PHeads {
    heads: WS2PHead[];
}
export declare const MerkleOfPeers: {
    "depth": NumberConstructor;
    "nodesCount": NumberConstructor;
    "leavesCount": NumberConstructor;
    "root": StringConstructor;
    "leaves": StringConstructor[];
    "leaf": {
        "hash": StringConstructor;
        "value": {
            "version": NumberConstructor;
            "currency": StringConstructor;
            "pubkey": StringConstructor;
            "block": StringConstructor;
            "status": StringConstructor;
            "first_down": NumberConstructor;
            "last_try": NumberConstructor;
            "endpoints": StringConstructor[];
            "signature": StringConstructor;
            "raw": StringConstructor;
        };
    };
};
export interface HttpMerkleOfPeers {
    depth: number;
    nodesCount: number;
    leavesCount: number;
    root: string;
    leaves: string[];
    leaf: {
        hash: string;
        value: JSONDBPeer;
    };
}
export declare const Other: {
    "pubkey": StringConstructor;
    "meta": {
        "block_number": NumberConstructor;
        "block_hash": StringConstructor;
    };
    "uids": StringConstructor[];
    "isMember": BooleanConstructor;
    "wasMember": BooleanConstructor;
    "signature": StringConstructor;
};
export interface HttpOther {
    pubkey: string;
    meta: {
        block_number: number;
        block_hash: string;
    };
    uids: string[];
    isMember: boolean;
    wasMember: boolean;
    signature: string;
}
export declare const UID: {
    "uid": StringConstructor;
    "meta": {
        "timestamp": StringConstructor;
    };
    "self": StringConstructor;
    "revocation_sig": StringConstructor;
    "revoked": BooleanConstructor;
    "revoked_on": NumberConstructor;
    "others": {
        "pubkey": StringConstructor;
        "meta": {
            "block_number": NumberConstructor;
            "block_hash": StringConstructor;
        };
        "uids": StringConstructor[];
        "isMember": BooleanConstructor;
        "wasMember": BooleanConstructor;
        "signature": StringConstructor;
    }[];
};
export interface HttpUID {
    uid: string;
    meta: {
        timestamp: string;
    };
    self: string;
    revocation_sig: string | null;
    revoked: boolean;
    revoked_on: number | null;
    others: HttpOther[];
}
export declare const Signed: {
    "uid": StringConstructor;
    "pubkey": StringConstructor;
    "meta": {
        "timestamp": StringConstructor;
    };
    "cert_time": {
        "block": NumberConstructor;
        "block_hash": StringConstructor;
    };
    "isMember": BooleanConstructor;
    "wasMember": BooleanConstructor;
    "signature": StringConstructor;
};
export interface HttpSigned {
    uid: string;
    pubkey: string;
    meta: {
        timestamp: string;
    };
    cert_time: {
        block: number;
        block_hash: string;
    };
    isMember: boolean;
    wasMember: boolean;
    signature: string;
}
export declare const CertIdentity: {
    "issuer": StringConstructor;
    "uid": StringConstructor;
    "timestamp": StringConstructor;
    "sig": StringConstructor;
};
export interface HttpCertIdentity {
    issuer: string;
    uid: string;
    timestamp: string;
    sig: string;
}
export declare const Cert: {
    "issuer": StringConstructor;
    "timestamp": StringConstructor;
    "sig": StringConstructor;
    "target": {
        "issuer": StringConstructor;
        "uid": StringConstructor;
        "timestamp": StringConstructor;
        "sig": StringConstructor;
    };
};
export interface HttpCert {
    issuer: string;
    timestamp: string;
    sig: string;
    target: HttpCertIdentity;
}
export declare const Identity: {
    "pubkey": StringConstructor;
    "uids": {
        "uid": StringConstructor;
        "meta": {
            "timestamp": StringConstructor;
        };
        "self": StringConstructor;
        "revocation_sig": StringConstructor;
        "revoked": BooleanConstructor;
        "revoked_on": NumberConstructor;
        "others": {
            "pubkey": StringConstructor;
            "meta": {
                "block_number": NumberConstructor;
                "block_hash": StringConstructor;
            };
            "uids": StringConstructor[];
            "isMember": BooleanConstructor;
            "wasMember": BooleanConstructor;
            "signature": StringConstructor;
        }[];
    }[];
    "signed": {
        "uid": StringConstructor;
        "pubkey": StringConstructor;
        "meta": {
            "timestamp": StringConstructor;
        };
        "cert_time": {
            "block": NumberConstructor;
            "block_hash": StringConstructor;
        };
        "isMember": BooleanConstructor;
        "wasMember": BooleanConstructor;
        "signature": StringConstructor;
    }[];
};
export interface HttpIdentity {
    pubkey: string;
    uids: HttpUID[];
    signed: HttpSigned[];
}
export declare const Result: {
    "result": BooleanConstructor;
};
export interface HttpResult {
    result: boolean;
}
export declare const Lookup: {
    "partial": BooleanConstructor;
    "results": {
        "pubkey": StringConstructor;
        "uids": {
            "uid": StringConstructor;
            "meta": {
                "timestamp": StringConstructor;
            };
            "self": StringConstructor;
            "revocation_sig": StringConstructor;
            "revoked": BooleanConstructor;
            "revoked_on": NumberConstructor;
            "others": {
                "pubkey": StringConstructor;
                "meta": {
                    "block_number": NumberConstructor;
                    "block_hash": StringConstructor;
                };
                "uids": StringConstructor[];
                "isMember": BooleanConstructor;
                "wasMember": BooleanConstructor;
                "signature": StringConstructor;
            }[];
        }[];
        "signed": {
            "uid": StringConstructor;
            "pubkey": StringConstructor;
            "meta": {
                "timestamp": StringConstructor;
            };
            "cert_time": {
                "block": NumberConstructor;
                "block_hash": StringConstructor;
            };
            "isMember": BooleanConstructor;
            "wasMember": BooleanConstructor;
            "signature": StringConstructor;
        }[];
    }[];
};
export interface HttpLookup {
    partial: boolean;
    results: HttpIdentity[];
}
export declare const Members: {
    "results": {
        pubkey: StringConstructor;
        uid: StringConstructor;
    }[];
};
export interface HttpMembers {
    results: {
        pubkey: string;
        uid: string;
    }[];
}
export declare const RequirementsCert: {
    from: StringConstructor;
    to: StringConstructor;
    expiresIn: NumberConstructor;
    sig: StringConstructor;
};
export interface HttpRequirementsCert {
    from: string;
    to: string;
    expiresIn: number;
    sig: string;
}
export declare const RequirementsPendingCert: {
    from: StringConstructor;
    to: StringConstructor;
    blockstamp: StringConstructor;
    sig: StringConstructor;
};
export interface HttpRequirementsPendingCert {
    from: string;
    to: string;
    blockstamp: string;
    sig: string;
}
export declare const RequirementsPendingMembership: {
    type: StringConstructor;
    blockstamp: StringConstructor;
    sig: StringConstructor;
};
export interface HttpRequirementsPendingMembership {
    type: string;
    blockstamp: string;
    sig: string;
}
export declare const Requirements: {
    "identities": {
        pubkey: StringConstructor;
        uid: StringConstructor;
        meta: {
            timestamp: StringConstructor;
        };
        sig: StringConstructor;
        revocation_sig: StringConstructor;
        revoked: BooleanConstructor;
        revoked_on: NumberConstructor;
        expired: BooleanConstructor;
        outdistanced: BooleanConstructor;
        isSentry: BooleanConstructor;
        wasMember: BooleanConstructor;
        certifications: {
            from: StringConstructor;
            to: StringConstructor;
            expiresIn: NumberConstructor;
            sig: StringConstructor;
        }[];
        pendingCerts: {
            from: StringConstructor;
            to: StringConstructor;
            blockstamp: StringConstructor;
            sig: StringConstructor;
        }[];
        pendingMemberships: {
            type: StringConstructor;
            blockstamp: StringConstructor;
            sig: StringConstructor;
        }[];
        membershipPendingExpiresIn: NumberConstructor;
        membershipExpiresIn: NumberConstructor;
    }[];
};
export interface HttpRequirements {
    identities: HttpIdentityRequirement[];
}
export interface HttpIdentityRequirement {
    pubkey: string;
    uid: string;
    meta: {
        timestamp: string;
    };
    sig: string;
    revocation_sig: string | null;
    revoked: boolean;
    revoked_on: number | null;
    expired: boolean;
    outdistanced: boolean;
    isSentry: boolean;
    wasMember: boolean;
    certifications: HttpRequirementsCert[];
    pendingCerts: HttpRequirementsPendingCert[];
    pendingMemberships: HttpRequirementsPendingMembership[];
    membershipPendingExpiresIn: number;
    membershipExpiresIn: number;
}
export declare const Certification: {
    "pubkey": StringConstructor;
    "uid": StringConstructor;
    "isMember": BooleanConstructor;
    "wasMember": BooleanConstructor;
    "cert_time": {
        "block": NumberConstructor;
        "medianTime": NumberConstructor;
    };
    "sigDate": StringConstructor;
    "written": {
        "number": NumberConstructor;
        "hash": StringConstructor;
    };
    "signature": StringConstructor;
};
export interface HttpCertification {
    pubkey: string;
    uid: string;
    isMember: boolean;
    wasMember: boolean;
    cert_time: {
        block: number;
        medianTime: number;
    };
    sigDate: string;
    written: {
        number: number;
        hash: string;
    } | null;
    signature: string;
}
export declare const Certifications: {
    "pubkey": StringConstructor;
    "uid": StringConstructor;
    "sigDate": StringConstructor;
    "isMember": BooleanConstructor;
    "certifications": {
        "pubkey": StringConstructor;
        "uid": StringConstructor;
        "isMember": BooleanConstructor;
        "wasMember": BooleanConstructor;
        "cert_time": {
            "block": NumberConstructor;
            "medianTime": NumberConstructor;
        };
        "sigDate": StringConstructor;
        "written": {
            "number": NumberConstructor;
            "hash": StringConstructor;
        };
        "signature": StringConstructor;
    }[];
};
export interface HttpCertifications {
    pubkey: string;
    uid: string;
    sigDate: string;
    isMember: boolean;
    certifications: HttpCertification[];
}
export declare const SimpleIdentity: {
    "pubkey": StringConstructor;
    "uid": StringConstructor;
    "sigDate": StringConstructor;
};
export interface HttpSimpleIdentity {
    pubkey: string;
    uid: string;
    sigDate: string;
}
export declare const Transaction: {
    "version": NumberConstructor;
    "currency": StringConstructor;
    "issuers": StringConstructor[];
    "inputs": StringConstructor[];
    "unlocks": StringConstructor[];
    "outputs": StringConstructor[];
    "comment": StringConstructor;
    "locktime": NumberConstructor;
    "signatures": StringConstructor[];
    "raw": StringConstructor;
    "written_block": NumberConstructor;
    "hash": StringConstructor;
};
export interface HttpTransaction {
    version: number;
    currency: string;
    issuers: string[];
    inputs: string[];
    unlocks: string[];
    outputs: string[];
    comment: string;
    locktime: number;
    signatures: string[];
    raw: string;
    written_block: number | null;
    hash: string;
}
export interface HttpTransactionPending {
    version: number;
    issuers: string[];
    inputs: string[];
    unlocks: string[];
    outputs: string[];
    comment: string;
    locktime: number;
    signatures: string[];
    hash: string;
}
export declare const Source: {
    "type": StringConstructor;
    "noffset": NumberConstructor;
    "identifier": StringConstructor;
    "amount": NumberConstructor;
    "base": NumberConstructor;
    "conditions": StringConstructor;
};
export interface HttpSource {
    type: string;
    noffset: number;
    identifier: string;
    amount: number;
    base: number;
    conditions: string;
}
export declare const Sources: {
    "currency": StringConstructor;
    "pubkey": StringConstructor;
    "sources": {
        "type": StringConstructor;
        "noffset": NumberConstructor;
        "identifier": StringConstructor;
        "amount": NumberConstructor;
        "base": NumberConstructor;
        "conditions": StringConstructor;
    }[];
};
export interface HttpSources {
    currency: string;
    pubkey: string;
    sources: HttpSource[];
}
export declare const TxOfHistory: {
    "version": NumberConstructor;
    "issuers": StringConstructor[];
    "inputs": StringConstructor[];
    "unlocks": StringConstructor[];
    "outputs": StringConstructor[];
    "comment": StringConstructor;
    "locktime": NumberConstructor;
    "received": NumberConstructor;
    "signatures": StringConstructor[];
    "hash": StringConstructor;
    "block_number": NumberConstructor;
    "time": NumberConstructor;
    "blockstamp": StringConstructor;
    "blockstampTime": NumberConstructor;
};
export interface HttpTxOfHistory {
    version: number;
    issuers: string[];
    inputs: string[];
    unlocks: string[];
    outputs: string[];
    comment: string;
    locktime: number;
    received: number;
    signatures: string[];
    hash: string;
    block_number: number | null;
    time: number | null;
    blockstamp: string;
    blockstampTime: number | null;
}
export declare const TxHistory: {
    "currency": StringConstructor;
    "pubkey": StringConstructor;
    "history": {
        "sent": {
            "version": NumberConstructor;
            "issuers": StringConstructor[];
            "inputs": StringConstructor[];
            "unlocks": StringConstructor[];
            "outputs": StringConstructor[];
            "comment": StringConstructor;
            "locktime": NumberConstructor;
            "received": NumberConstructor;
            "signatures": StringConstructor[];
            "hash": StringConstructor;
            "block_number": NumberConstructor;
            "time": NumberConstructor;
            "blockstamp": StringConstructor;
            "blockstampTime": NumberConstructor;
        }[];
        "received": {
            "version": NumberConstructor;
            "issuers": StringConstructor[];
            "inputs": StringConstructor[];
            "unlocks": StringConstructor[];
            "outputs": StringConstructor[];
            "comment": StringConstructor;
            "locktime": NumberConstructor;
            "received": NumberConstructor;
            "signatures": StringConstructor[];
            "hash": StringConstructor;
            "block_number": NumberConstructor;
            "time": NumberConstructor;
            "blockstamp": StringConstructor;
            "blockstampTime": NumberConstructor;
        }[];
        "sending": {
            "version": NumberConstructor;
            "issuers": StringConstructor[];
            "inputs": StringConstructor[];
            "unlocks": StringConstructor[];
            "outputs": StringConstructor[];
            "comment": StringConstructor;
            "locktime": NumberConstructor;
            "received": NumberConstructor;
            "signatures": StringConstructor[];
            "hash": StringConstructor;
            "block_number": NumberConstructor;
            "time": NumberConstructor;
            "blockstamp": StringConstructor;
            "blockstampTime": NumberConstructor;
        }[];
        "receiving": {
            "version": NumberConstructor;
            "issuers": StringConstructor[];
            "inputs": StringConstructor[];
            "unlocks": StringConstructor[];
            "outputs": StringConstructor[];
            "comment": StringConstructor;
            "locktime": NumberConstructor;
            "received": NumberConstructor;
            "signatures": StringConstructor[];
            "hash": StringConstructor;
            "block_number": NumberConstructor;
            "time": NumberConstructor;
            "blockstamp": StringConstructor;
            "blockstampTime": NumberConstructor;
        }[];
        "pending": {
            "version": NumberConstructor;
            "issuers": StringConstructor[];
            "inputs": StringConstructor[];
            "unlocks": StringConstructor[];
            "outputs": StringConstructor[];
            "comment": StringConstructor;
            "locktime": NumberConstructor;
            "received": NumberConstructor;
            "signatures": StringConstructor[];
            "hash": StringConstructor;
            "block_number": NumberConstructor;
            "time": NumberConstructor;
            "blockstamp": StringConstructor;
            "blockstampTime": NumberConstructor;
        }[];
    };
};
export interface HttpTxHistory {
    currency: string;
    pubkey: string;
    history: {
        sent: HttpTxOfHistory[];
        received: HttpTxOfHistory[];
        sending: HttpTxOfHistory[];
        receiving: HttpTxOfHistory[];
        pending: HttpTxOfHistory[];
    };
}
export declare const TxPending: {
    "currency": StringConstructor;
    "pending": {
        "version": NumberConstructor;
        "currency": StringConstructor;
        "issuers": StringConstructor[];
        "inputs": StringConstructor[];
        "unlocks": StringConstructor[];
        "outputs": StringConstructor[];
        "comment": StringConstructor;
        "locktime": NumberConstructor;
        "signatures": StringConstructor[];
        "raw": StringConstructor;
        "written_block": NumberConstructor;
        "hash": StringConstructor;
    }[];
};
export interface HttpTxPending {
    currency: string;
    pending: HttpTransactionPending[];
}
export declare const UD: {
    "block_number": NumberConstructor;
    "consumed": BooleanConstructor;
    "time": NumberConstructor;
    "amount": NumberConstructor;
    "base": NumberConstructor;
};
export interface HttpUD {
    block_number: number;
    consumed: boolean;
    time: number;
    amount: number;
    base: number;
}
export declare const UDHistory: {
    "currency": StringConstructor;
    "pubkey": StringConstructor;
    "history": {
        "history": {
            "block_number": NumberConstructor;
            "consumed": BooleanConstructor;
            "time": NumberConstructor;
            "amount": NumberConstructor;
            "base": NumberConstructor;
        }[];
    };
};
export interface HttpUDHistory {
    currency: string;
    pubkey: string;
    history: {
        history: HttpUD[];
    };
}
export declare const BooleanDTO: {
    "success": BooleanConstructor;
};
export declare const SummaryConf: {
    "cpu": NumberConstructor;
};
export declare const AdminSummary: {
    "version": StringConstructor;
    "host": StringConstructor;
    "current": {
        "version": NumberConstructor;
        "currency": StringConstructor;
        "number": NumberConstructor;
        "issuer": StringConstructor;
        "issuersFrame": NumberConstructor;
        "issuersFrameVar": NumberConstructor;
        "issuersCount": NumberConstructor;
        "parameters": StringConstructor;
        "membersCount": NumberConstructor;
        "monetaryMass": NumberConstructor;
        "powMin": NumberConstructor;
        "time": NumberConstructor;
        "medianTime": NumberConstructor;
        "dividend": NumberConstructor;
        "unitbase": NumberConstructor;
        "hash": StringConstructor;
        "previousHash": StringConstructor;
        "previousIssuer": StringConstructor;
        "identities": StringConstructor[];
        "certifications": StringConstructor[];
        "joiners": StringConstructor[];
        "actives": StringConstructor[];
        "leavers": StringConstructor[];
        "revoked": StringConstructor[];
        "excluded": StringConstructor[];
        "transactions": {
            "version": NumberConstructor;
            "currency": StringConstructor;
            "comment": StringConstructor;
            "locktime": NumberConstructor;
            "signatures": StringConstructor[];
            "outputs": StringConstructor[];
            "inputs": StringConstructor[];
            "unlocks": StringConstructor[];
            "block_number": NumberConstructor;
            "blockstamp": StringConstructor;
            "blockstampTime": NumberConstructor;
            "time": NumberConstructor;
            "issuers": StringConstructor[];
        }[];
        "nonce": NumberConstructor;
        "inner_hash": StringConstructor;
        "signature": StringConstructor;
        "raw": StringConstructor;
    };
    "rootBlock": {
        "version": NumberConstructor;
        "currency": StringConstructor;
        "number": NumberConstructor;
        "issuer": StringConstructor;
        "issuersFrame": NumberConstructor;
        "issuersFrameVar": NumberConstructor;
        "issuersCount": NumberConstructor;
        "parameters": StringConstructor;
        "membersCount": NumberConstructor;
        "monetaryMass": NumberConstructor;
        "powMin": NumberConstructor;
        "time": NumberConstructor;
        "medianTime": NumberConstructor;
        "dividend": NumberConstructor;
        "unitbase": NumberConstructor;
        "hash": StringConstructor;
        "previousHash": StringConstructor;
        "previousIssuer": StringConstructor;
        "identities": StringConstructor[];
        "certifications": StringConstructor[];
        "joiners": StringConstructor[];
        "actives": StringConstructor[];
        "leavers": StringConstructor[];
        "revoked": StringConstructor[];
        "excluded": StringConstructor[];
        "transactions": {
            "version": NumberConstructor;
            "currency": StringConstructor;
            "comment": StringConstructor;
            "locktime": NumberConstructor;
            "signatures": StringConstructor[];
            "outputs": StringConstructor[];
            "inputs": StringConstructor[];
            "unlocks": StringConstructor[];
            "block_number": NumberConstructor;
            "blockstamp": StringConstructor;
            "blockstampTime": NumberConstructor;
            "time": NumberConstructor;
            "issuers": StringConstructor[];
        }[];
        "nonce": NumberConstructor;
        "inner_hash": StringConstructor;
        "signature": StringConstructor;
        "raw": StringConstructor;
    };
    "pubkey": StringConstructor;
    "seckey": StringConstructor;
    "conf": {
        "cpu": NumberConstructor;
    };
    "parameters": {
        currency: StringConstructor;
        c: NumberConstructor;
        dt: NumberConstructor;
        ud0: NumberConstructor;
        sigPeriod: NumberConstructor;
        sigStock: NumberConstructor;
        sigWindow: NumberConstructor;
        sigValidity: NumberConstructor;
        sigReplay: NumberConstructor;
        sigQty: NumberConstructor;
        idtyWindow: NumberConstructor;
        msWindow: NumberConstructor;
        msPeriod: NumberConstructor;
        xpercent: NumberConstructor;
        msValidity: NumberConstructor;
        stepMax: NumberConstructor;
        medianTimeBlocks: NumberConstructor;
        avgGenTime: NumberConstructor;
        dtDiffEval: NumberConstructor;
        percentRot: NumberConstructor;
        udTime0: NumberConstructor;
        udReevalTime0: NumberConstructor;
        dtReeval: NumberConstructor;
    };
    "lastUDBlock": {
        "version": NumberConstructor;
        "currency": StringConstructor;
        "number": NumberConstructor;
        "issuer": StringConstructor;
        "issuersFrame": NumberConstructor;
        "issuersFrameVar": NumberConstructor;
        "issuersCount": NumberConstructor;
        "parameters": StringConstructor;
        "membersCount": NumberConstructor;
        "monetaryMass": NumberConstructor;
        "powMin": NumberConstructor;
        "time": NumberConstructor;
        "medianTime": NumberConstructor;
        "dividend": NumberConstructor;
        "unitbase": NumberConstructor;
        "hash": StringConstructor;
        "previousHash": StringConstructor;
        "previousIssuer": StringConstructor;
        "identities": StringConstructor[];
        "certifications": StringConstructor[];
        "joiners": StringConstructor[];
        "actives": StringConstructor[];
        "leavers": StringConstructor[];
        "revoked": StringConstructor[];
        "excluded": StringConstructor[];
        "transactions": {
            "version": NumberConstructor;
            "currency": StringConstructor;
            "comment": StringConstructor;
            "locktime": NumberConstructor;
            "signatures": StringConstructor[];
            "outputs": StringConstructor[];
            "inputs": StringConstructor[];
            "unlocks": StringConstructor[];
            "block_number": NumberConstructor;
            "blockstamp": StringConstructor;
            "blockstampTime": NumberConstructor;
            "time": NumberConstructor;
            "issuers": StringConstructor[];
        }[];
        "nonce": NumberConstructor;
        "inner_hash": StringConstructor;
        "signature": StringConstructor;
        "raw": StringConstructor;
    };
};
export declare const PoWSummary: {
    "total": NumberConstructor;
    "mirror": BooleanConstructor;
    "waiting": BooleanConstructor;
};
export declare const PreviewPubkey: {
    "pubkey": StringConstructor;
};
export declare const Sandbox: {
    size: NumberConstructor;
    free: NumberConstructor;
};
export interface HttpSandbox {
    size: number;
    free: number;
}
export declare const IdentitySandbox: {
    size: NumberConstructor;
    free: NumberConstructor;
};
export declare const MembershipSandbox: {
    size: NumberConstructor;
    free: NumberConstructor;
};
export declare const TransactionSandbox: {
    size: NumberConstructor;
    free: NumberConstructor;
};
export declare const Sandboxes: {
    identities: {
        size: NumberConstructor;
        free: NumberConstructor;
    };
    memberships: {
        size: NumberConstructor;
        free: NumberConstructor;
    };
    transactions: {
        size: NumberConstructor;
        free: NumberConstructor;
    };
};
export interface HttpSandboxes {
    identities: HttpSandbox;
    memberships: HttpSandbox;
    transactions: HttpSandbox;
}
export declare const LogLink: {
    link: StringConstructor;
};
export interface HttpMilestonePage {
    totalPages: number;
    chunkSize: number;
    milestonesPerPage: number;
    currentPage?: number;
    blocks?: HttpBlock[];
}
export declare const Milestones: {
    totalPages: NumberConstructor;
    chunkSize: NumberConstructor;
    milestonesPerPage: NumberConstructor;
    currentPage: NumberConstructor;
    "blocks": {
        "version": NumberConstructor;
        "currency": StringConstructor;
        "number": NumberConstructor;
        "issuer": StringConstructor;
        "issuersFrame": NumberConstructor;
        "issuersFrameVar": NumberConstructor;
        "issuersCount": NumberConstructor;
        "parameters": StringConstructor;
        "membersCount": NumberConstructor;
        "monetaryMass": NumberConstructor;
        "powMin": NumberConstructor;
        "time": NumberConstructor;
        "medianTime": NumberConstructor;
        "dividend": NumberConstructor;
        "unitbase": NumberConstructor;
        "hash": StringConstructor;
        "previousHash": StringConstructor;
        "previousIssuer": StringConstructor;
        "identities": StringConstructor[];
        "certifications": StringConstructor[];
        "joiners": StringConstructor[];
        "actives": StringConstructor[];
        "leavers": StringConstructor[];
        "revoked": StringConstructor[];
        "excluded": StringConstructor[];
        "transactions": {
            "version": NumberConstructor;
            "currency": StringConstructor;
            "comment": StringConstructor;
            "locktime": NumberConstructor;
            "signatures": StringConstructor[];
            "outputs": StringConstructor[];
            "inputs": StringConstructor[];
            "unlocks": StringConstructor[];
            "block_number": NumberConstructor;
            "blockstamp": StringConstructor;
            "blockstampTime": NumberConstructor;
            "time": NumberConstructor;
            "issuers": StringConstructor[];
        }[];
        "nonce": NumberConstructor;
        "inner_hash": StringConstructor;
        "signature": StringConstructor;
        "raw": StringConstructor;
    }[];
};
export declare const MilestonesPage: {
    totalPages: NumberConstructor;
    chunkSize: NumberConstructor;
    milestonesPerPage: NumberConstructor;
};
