"use strict";
// Source file from duniter: Crypto-currency software to manage libre currency such as Ğ1
// Copyright (C) 2018  Cedric Moreau <cem.moreau@gmail.com>
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Affero General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Affero General Public License for more details.
Object.defineProperty(exports, "__esModule", { value: true });
exports.Summary = {
    duniter: {
        "software": String,
        "version": String,
        "forkWindowSize": Number
    }
};
exports.Parameters = {
    currency: String,
    c: Number,
    dt: Number,
    ud0: Number,
    sigPeriod: Number,
    sigStock: Number,
    sigWindow: Number,
    sigValidity: Number,
    sigReplay: Number,
    sigQty: Number,
    idtyWindow: Number,
    msWindow: Number,
    msPeriod: Number,
    xpercent: Number,
    msValidity: Number,
    stepMax: Number,
    medianTimeBlocks: Number,
    avgGenTime: Number,
    dtDiffEval: Number,
    percentRot: Number,
    udTime0: Number,
    udReevalTime0: Number,
    dtReeval: Number
};
exports.Membership = {
    "signature": String,
    "membership": {
        "version": Number,
        "currency": String,
        "issuer": String,
        "membership": String,
        "date": Number,
        "sigDate": Number,
        "raw": String
    }
};
exports.Memberships = {
    "pubkey": String,
    "uid": String,
    "sigDate": String,
    "memberships": [
        {
            "version": Number,
            "currency": String,
            "membership": String,
            "blockNumber": Number,
            "blockHash": String,
            "written": Number
        }
    ]
};
exports.MembershipList = {
    "memberships": [
        {
            "pubkey": String,
            "uid": String,
            "version": Number,
            "currency": String,
            "membership": String,
            "blockNumber": Number,
            "blockHash": String,
            "written": Number
        }
    ]
};
exports.TransactionOfBlock = {
    "version": Number,
    "currency": String,
    "comment": String,
    "locktime": Number,
    "signatures": [String],
    "outputs": [String],
    "inputs": [String],
    "unlocks": [String],
    "block_number": Number,
    "blockstamp": String,
    "blockstampTime": Number,
    "time": Number,
    "issuers": [String]
};
exports.Block = {
    "version": Number,
    "currency": String,
    "number": Number,
    "issuer": String,
    "issuersFrame": Number,
    "issuersFrameVar": Number,
    "issuersCount": Number,
    "parameters": String,
    "membersCount": Number,
    "monetaryMass": Number,
    "powMin": Number,
    "time": Number,
    "medianTime": Number,
    "dividend": Number,
    "unitbase": Number,
    "hash": String,
    "previousHash": String,
    "previousIssuer": String,
    "identities": [String],
    "certifications": [String],
    "joiners": [String],
    "actives": [String],
    "leavers": [String],
    "revoked": [String],
    "excluded": [String],
    "transactions": [exports.TransactionOfBlock],
    "nonce": Number,
    "inner_hash": String,
    "signature": String,
    "raw": String
};
function block2HttpBlock(blockDTO) {
    return {
        version: blockDTO.version,
        currency: blockDTO.currency,
        number: blockDTO.number,
        issuer: blockDTO.issuer,
        issuersFrame: blockDTO.issuersFrame,
        issuersFrameVar: blockDTO.issuersFrameVar,
        issuersCount: blockDTO.issuersCount,
        parameters: blockDTO.parameters,
        membersCount: blockDTO.membersCount,
        monetaryMass: blockDTO.monetaryMass,
        powMin: blockDTO.powMin,
        time: blockDTO.time,
        medianTime: blockDTO.medianTime,
        dividend: blockDTO.dividend,
        unitbase: blockDTO.unitbase,
        hash: blockDTO.hash,
        previousHash: blockDTO.previousHash,
        previousIssuer: blockDTO.previousIssuer,
        identities: blockDTO.identities,
        certifications: blockDTO.certifications,
        joiners: blockDTO.joiners,
        actives: blockDTO.actives,
        leavers: blockDTO.leavers,
        revoked: blockDTO.revoked,
        excluded: blockDTO.excluded,
        transactions: blockDTO.transactions.map((tx) => {
            return {
                version: tx.version,
                currency: tx.currency,
                comment: tx.comment,
                locktime: tx.locktime,
                issuers: tx.issuers,
                signatures: tx.signatures,
                outputs: tx.outputs,
                inputs: tx.inputs,
                unlocks: tx.unlocks,
                hash: tx.hash,
                blockstamp: tx.blockstamp,
                blockstampTime: tx.blockstampTime,
            };
        }),
        nonce: blockDTO.nonce,
        inner_hash: blockDTO.inner_hash,
        signature: blockDTO.signature,
        raw: blockDTO.getRawSigned()
    };
}
exports.block2HttpBlock = block2HttpBlock;
exports.Hardship = {
    "block": Number,
    "level": Number
};
exports.Difficulty = {
    "uid": String,
    "level": Number
};
exports.Difficulties = {
    "block": Number,
    "levels": [exports.Difficulty]
};
exports.Blocks = [exports.Block];
exports.Stat = {
    "result": {
        "blocks": [Number]
    }
};
exports.Branches = {
    "blocks": [exports.Block]
};
exports.Peer = {
    "version": Number,
    "currency": String,
    "pubkey": String,
    "block": String,
    "endpoints": [String],
    "signature": String,
    "raw": String
};
exports.DBPeer = {
    "version": Number,
    "currency": String,
    "pubkey": String,
    "block": String,
    "status": String,
    "first_down": Number,
    "last_try": Number,
    "endpoints": [String],
    "signature": String,
    "raw": String
};
exports.Peers = {
    "peers": [exports.DBPeer]
};
exports.MerkleOfPeers = {
    "depth": Number,
    "nodesCount": Number,
    "leavesCount": Number,
    "root": String,
    "leaves": [String],
    "leaf": {
        "hash": String,
        "value": exports.DBPeer
    }
};
exports.Other = {
    "pubkey": String,
    "meta": {
        "block_number": Number,
        "block_hash": String
    },
    "uids": [String],
    "isMember": Boolean,
    "wasMember": Boolean,
    "signature": String
};
exports.UID = {
    "uid": String,
    "meta": {
        "timestamp": String
    },
    "self": String,
    "revocation_sig": String,
    "revoked": Boolean,
    "revoked_on": Number,
    "others": [exports.Other]
};
exports.Signed = {
    "uid": String,
    "pubkey": String,
    "meta": {
        "timestamp": String
    },
    "cert_time": {
        "block": Number,
        "block_hash": String
    },
    "isMember": Boolean,
    "wasMember": Boolean,
    "signature": String
};
exports.CertIdentity = {
    "issuer": String,
    "uid": String,
    "timestamp": String,
    "sig": String
};
exports.Cert = {
    "issuer": String,
    "timestamp": String,
    "sig": String,
    "target": exports.CertIdentity
};
exports.Identity = {
    "pubkey": String,
    "uids": [exports.UID],
    "signed": [exports.Signed]
};
exports.Result = {
    "result": Boolean
};
exports.Lookup = {
    "partial": Boolean,
    "results": [exports.Identity]
};
exports.Members = {
    "results": [{
            pubkey: String,
            uid: String
        }]
};
exports.RequirementsCert = {
    from: String,
    to: String,
    expiresIn: Number,
    sig: String
};
exports.RequirementsPendingCert = {
    from: String,
    to: String,
    blockstamp: String,
    sig: String
};
exports.RequirementsPendingMembership = {
    type: String,
    blockstamp: String,
    sig: String
};
exports.Requirements = {
    "identities": [{
            pubkey: String,
            uid: String,
            meta: {
                timestamp: String
            },
            sig: String,
            revocation_sig: String,
            revoked: Boolean,
            revoked_on: Number,
            expired: Boolean,
            outdistanced: Boolean,
            isSentry: Boolean,
            wasMember: Boolean,
            certifications: [exports.RequirementsCert],
            pendingCerts: [exports.RequirementsPendingCert],
            pendingMemberships: [exports.RequirementsPendingMembership],
            membershipPendingExpiresIn: Number,
            membershipExpiresIn: Number
        }]
};
exports.Certification = {
    "pubkey": String,
    "uid": String,
    "isMember": Boolean,
    "wasMember": Boolean,
    "cert_time": {
        "block": Number,
        "medianTime": Number
    },
    "sigDate": String,
    "written": {
        "number": Number,
        "hash": String
    },
    "signature": String
};
exports.Certifications = {
    "pubkey": String,
    "uid": String,
    "sigDate": String,
    "isMember": Boolean,
    "certifications": [exports.Certification]
};
exports.SimpleIdentity = {
    "pubkey": String,
    "uid": String,
    "sigDate": String
};
exports.Transaction = {
    "version": Number,
    "currency": String,
    "issuers": [String],
    "inputs": [String],
    "unlocks": [String],
    "outputs": [String],
    "comment": String,
    "locktime": Number,
    "signatures": [String],
    "raw": String,
    "written_block": Number,
    "hash": String
};
exports.Source = {
    "type": String,
    "noffset": Number,
    "identifier": String,
    "amount": Number,
    "base": Number,
    "conditions": String
};
exports.Sources = {
    "currency": String,
    "pubkey": String,
    "sources": [exports.Source]
};
exports.TxOfHistory = {
    "version": Number,
    "issuers": [String],
    "inputs": [String],
    "unlocks": [String],
    "outputs": [String],
    "comment": String,
    "locktime": Number,
    "received": Number,
    "signatures": [String],
    "hash": String,
    "block_number": Number,
    "time": Number,
    "blockstamp": String,
    "blockstampTime": Number
};
exports.TxHistory = {
    "currency": String,
    "pubkey": String,
    "history": {
        "sent": [exports.TxOfHistory],
        "received": [exports.TxOfHistory],
        "sending": [exports.TxOfHistory],
        "receiving": [exports.TxOfHistory],
        "pending": [exports.TxOfHistory]
    }
};
exports.TxPending = {
    "currency": String,
    "pending": [exports.Transaction]
};
exports.UD = {
    "block_number": Number,
    "consumed": Boolean,
    "time": Number,
    "amount": Number,
    "base": Number
};
exports.UDHistory = {
    "currency": String,
    "pubkey": String,
    "history": {
        "history": [exports.UD]
    }
};
exports.BooleanDTO = {
    "success": Boolean
};
exports.SummaryConf = {
    "cpu": Number
};
exports.AdminSummary = {
    "version": String,
    "host": String,
    "current": exports.Block,
    "rootBlock": exports.Block,
    "pubkey": String,
    "seckey": String,
    "conf": exports.SummaryConf,
    "parameters": exports.Parameters,
    "lastUDBlock": exports.Block
};
exports.PoWSummary = {
    "total": Number,
    "mirror": Boolean,
    "waiting": Boolean
};
exports.PreviewPubkey = {
    "pubkey": String
};
exports.Sandbox = {
    size: Number,
    free: Number
};
exports.IdentitySandbox = exports.Sandbox;
exports.MembershipSandbox = exports.Sandbox;
exports.TransactionSandbox = exports.Sandbox;
exports.Sandboxes = {
    identities: exports.IdentitySandbox,
    memberships: exports.MembershipSandbox,
    transactions: exports.TransactionSandbox
};
exports.LogLink = {
    link: String
};
exports.Milestones = {
    totalPages: Number,
    chunkSize: Number,
    milestonesPerPage: Number,
    currentPage: Number,
    "blocks": [exports.Block]
};
exports.MilestonesPage = {
    totalPages: Number,
    chunkSize: Number,
    milestonesPerPage: Number,
};
//# sourceMappingURL=dtos.js.map