"use strict";
// Source file from duniter: Crypto-currency software to manage libre currency such as Ğ1
// Copyright (C) 2018  Cedric Moreau <cem.moreau@gmail.com>
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Affero General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Affero General Public License for more details.
Object.defineProperty(exports, "__esModule", { value: true });
const constants_1 = require("./constants");
const limiter_1 = require("./limiter");
const underscore_1 = require("../../../lib/common-libs/underscore");
const constants_2 = require("../../../lib/common-libs/constants");
const logger_1 = require("../../../lib/logger");
const os = require('os');
const Q = require('q');
const ddos = require('ddos');
const http = require('http');
const express = require('express');
const morgan = require('morgan');
const errorhandler = require('errorhandler');
const bodyParser = require('body-parser');
const cors = require('cors');
const fileUpload = require('express-fileupload');
const logger = logger_1.NewLogger();
exports.Network = {
    getBestLocalIPv4,
    getBestLocalIPv6: getBestLocalIPv6,
    listInterfaces: listInterfaces,
    upnpConf,
    getRandomPort: getRandomPort,
    createServersAndListen: async (name, server, interfaces, httpLogs, logger, staticPath, routingCallback, listenWebSocket, enableFileUpload = false) => {
        const app = express();
        // all environments
        if (httpLogs) {
            app.use(morgan('\x1b[90m:remote-addr - :method :url HTTP/:http-version :status :res[content-length] - :response-time ms\x1b[0m', {
                stream: {
                    write: function (message) {
                        message && logger && logger.trace(message.replace(/\n$/, ''));
                    }
                }
            }));
        }
        // DDOS protection
        const whitelist = interfaces.map(i => i.ip);
        if (whitelist.indexOf('127.0.0.1') === -1) {
            whitelist.push('127.0.0.1');
        }
        const ddosConf = server.conf.dos || {};
        ddosConf.silentStart = true;
        ddosConf.whitelist = underscore_1.Underscore.uniq((ddosConf.whitelist || []).concat(whitelist));
        const ddosInstance = new ddos(ddosConf);
        app.use(ddosInstance.express);
        // CORS for **any** HTTP request
        app.use(cors());
        if (enableFileUpload) {
            // File upload for backup API
            app.use(fileUpload());
        }
        app.use(bodyParser.urlencoded({
            extended: true
        }));
        app.use(bodyParser.json({ limit: '10mb' }));
        // development only
        if (app.get('env') == 'development') {
            app.use(errorhandler());
        }
        const handleRequest = (method, uri, promiseFunc, theLimiter) => {
            const limiter = theLimiter || limiter_1.BMALimitation.limitAsUnlimited();
            method(uri, async function (req, res) {
                res.set('Access-Control-Allow-Origin', '*');
                res.type('application/json');
                try {
                    if (!limiter.canAnswerNow()) {
                        throw constants_1.BMAConstants.ERRORS.HTTP_LIMITATION;
                    }
                    limiter.processRequest();
                    let result = await promiseFunc(req);
                    // HTTP answer
                    res.status(200).send(JSON.stringify(result, null, "  "));
                }
                catch (e) {
                    let error = getResultingError(e, logger);
                    // HTTP error
                    res.status(error.httpCode).send(JSON.stringify(error.uerr, null, "  "));
                }
            });
        };
        const handleFileRequest = (method, uri, promiseFunc, theLimiter) => {
            const limiter = theLimiter || limiter_1.BMALimitation.limitAsUnlimited();
            method(uri, async function (req, res) {
                res.set('Access-Control-Allow-Origin', '*');
                try {
                    if (!limiter.canAnswerNow()) {
                        throw constants_1.BMAConstants.ERRORS.HTTP_LIMITATION;
                    }
                    limiter.processRequest();
                    let fileStream = await promiseFunc(req);
                    // HTTP answer
                    fileStream.pipe(res);
                }
                catch (e) {
                    let error = getResultingError(e, logger);
                    // HTTP error
                    res.status(error.httpCode).send(JSON.stringify(error.uerr, null, "  "));
                    throw e;
                }
            });
        };
        routingCallback(app, {
            httpGET: (uri, promiseFunc, limiter) => handleRequest(app.get.bind(app), uri, promiseFunc, limiter),
            httpPOST: (uri, promiseFunc, limiter) => handleRequest(app.post.bind(app), uri, promiseFunc, limiter),
            httpGETFile: (uri, promiseFunc, limiter) => handleFileRequest(app.get.bind(app), uri, promiseFunc, limiter)
        });
        if (staticPath) {
            app.use(express.static(staticPath));
        }
        const httpServers = interfaces.map(() => {
            const httpServer = http.createServer(app);
            const sockets = {};
            let nextSocketId = 0;
            httpServer.on('connection', (socket) => {
                const socketId = nextSocketId++;
                sockets[socketId] = socket;
                //logger && logger.debug('socket %s opened', socketId);
                socket.on('close', () => {
                    //logger && logger.debug('socket %s closed', socketId);
                    delete sockets[socketId];
                });
            });
            httpServer.on('error', (err) => {
                httpServer.errorPropagates(err);
            });
            listenWebSocket && listenWebSocket(httpServer);
            return {
                http: httpServer,
                closeSockets: () => {
                    underscore_1.Underscore.keys(sockets).map((socketId) => {
                        sockets[socketId].destroy();
                    });
                }
            };
        });
        if (httpServers.length == 0) {
            throw 'Duniter does not have any interface to listen to.';
        }
        // Return API
        return new BmaApi(name, interfaces, ddosInstance, httpServers, logger);
    }
};
class BmaApi {
    constructor(name, interfaces, ddosInstance, httpServers, logger) {
        this.name = name;
        this.interfaces = interfaces;
        this.ddosInstance = ddosInstance;
        this.httpServers = httpServers;
        this.logger = logger;
        // May be removed when using Node 5.x where httpServer.listening boolean exists
        this.listenings = interfaces.map(() => false);
    }
    getDDOS() {
        return this.ddosInstance;
    }
    async closeConnections() {
        for (let i = 0, len = this.httpServers.length; i < len; i++) {
            const httpServer = this.httpServers[i].http;
            const isListening = this.listenings[i];
            if (isListening) {
                this.listenings[i] = false;
                this.logger && this.logger.info(this.name + ' stop listening');
                await Q.Promise((resolve, reject) => {
                    httpServer.errorPropagates((err) => {
                        reject(err);
                    });
                    this.httpServers[i].closeSockets();
                    httpServer.close((err) => {
                        err && this.logger && this.logger.error(err.stack || err);
                        resolve();
                    });
                });
            }
        }
        return [];
    }
    async openConnections() {
        for (let i = 0, len = this.httpServers.length; i < len; i++) {
            const httpServer = this.httpServers[i].http;
            const isListening = this.listenings[i];
            if (!isListening) {
                const netInterface = this.interfaces[i].ip;
                const port = this.interfaces[i].port;
                try {
                    await Q.Promise((resolve, reject) => {
                        // Weird the need of such a hack to catch an exception...
                        httpServer.errorPropagates = function (err) {
                            reject(err);
                        };
                        //httpServer.on('listening', resolve.bind(this, httpServer));
                        httpServer.listen(port, netInterface, (err) => {
                            if (err)
                                return reject(err);
                            this.listenings[i] = true;
                            resolve(httpServer);
                        });
                    });
                    logger.info(this.name + ' listening on http://' + (netInterface.match(/:/) ? '[' + netInterface + ']' : netInterface) + ':' + port);
                }
                catch (e) {
                    logger.warn('Could NOT listen to http://' + netInterface + ':' + port);
                    logger.warn(e);
                }
            }
        }
        return [];
    }
}
exports.BmaApi = BmaApi;
function getResultingError(e, logger) {
    // Default is 500 unknown error
    let error = constants_1.BMAConstants.ERRORS.UNKNOWN;
    if (e) {
        // Print eventual stack trace
        typeof e == 'string' && e !== "Block already known" && logger && logger.error(e);
        e.stack && logger && logger.error(e.stack);
        e.message && logger && logger.warn(e.message);
        // BusinessException
        if (e.uerr) {
            error = e;
        }
        else {
            const cp = constants_1.BMAConstants.ERRORS.UNHANDLED;
            error = {
                httpCode: cp.httpCode,
                uerr: {
                    ucode: cp.uerr.ucode,
                    message: e.message || e || error.uerr.message
                }
            };
        }
    }
    return error;
}
function getBestLocalIPv4() {
    return getBestLocal('IPv4');
}
function getBestLocalIPv6() {
    const osInterfaces = listInterfaces();
    for (let netInterface of osInterfaces) {
        const addresses = netInterface.addresses;
        const filtered = underscore_1.Underscore.where(addresses, { family: 'IPv6', scopeid: 0, internal: false });
        const filtered2 = underscore_1.Underscore.filter(filtered, (address) => !address.address.match(/^fe80/) && !address.address.match(/^::1/));
        if (filtered2[0]) {
            return filtered2[0].address;
        }
    }
    return null;
}
function getBestLocal(family) {
    let netInterfaces = os.networkInterfaces();
    let keys = underscore_1.Underscore.keys(netInterfaces);
    let res = [];
    for (const name of keys) {
        let addresses = netInterfaces[name];
        for (const addr of addresses) {
            if (!family || addr.family == family) {
                res.push({
                    name: name,
                    value: addr.address
                });
            }
        }
    }
    const interfacePriorityRegCatcher = [
        /^tun\d+/,
        /^enp\d+s\d+/,
        /^enp\d+s\d+f\d+/,
        /^eth\d+/,
        /^Ethernet/,
        /^wlp\d+s\d+/,
        /^wlan\d+/,
        /^Wi-Fi/,
        /^lo/,
        /^Loopback/,
        /^None/
    ];
    const best = underscore_1.Underscore.sortBy(res, function (entry) {
        for (let i = 0; i < interfacePriorityRegCatcher.length; i++) {
            // `i` is the priority (0 is the better, 1 is the second, ...)
            if (entry.name.match(interfacePriorityRegCatcher[i]))
                return i;
        }
        return interfacePriorityRegCatcher.length;
    })[0];
    return (best && best.value) || "";
}
function listInterfaces() {
    const netInterfaces = os.networkInterfaces();
    const keys = underscore_1.Underscore.keys(netInterfaces);
    const res = [];
    for (const name of keys) {
        res.push({
            name: name,
            addresses: netInterfaces[name]
        });
    }
    return res;
}
async function upnpConf(noupnp, logger) {
    const client = require('nat-upnp').createClient();
    // Look for 2 random ports
    const publicPort = await getAvailablePort(client);
    const privatePort = publicPort;
    const conf = {
        proxiesConf: undefined,
        nobma: true,
        bmaWithCrawler: false,
        port: privatePort,
        ipv4: '127.0.0.1',
        ipv6: '::1',
        dos: null,
        upnp: false,
        httplogs: false,
        remoteport: publicPort,
        remotehost: null,
        remoteipv4: null,
        remoteipv6: null,
        nonWoTPeersLimit: constants_2.CommonConstants.DEFAULT_NON_WOT_PEERS_LIMIT
    };
    logger && logger.info('Checking UPnP features...');
    if (noupnp) {
        throw Error('No UPnP');
    }
    const publicIP = await Q.nbind(client.externalIp, client)();
    await Q.nbind(client.portMapping, client)({
        public: publicPort,
        private: privatePort,
        ttl: constants_1.BMAConstants.UPNP_TTL
    });
    const privateIP = await Q.Promise((resolve, reject) => {
        client.findGateway((err, res, localIP) => {
            if (err)
                return reject(err);
            resolve(localIP);
        });
    });
    conf.remoteipv4 = publicIP.match(constants_1.BMAConstants.IPV4_REGEXP) ? publicIP : null;
    conf.remoteport = publicPort;
    conf.port = privatePort;
    conf.ipv4 = privateIP.match(constants_1.BMAConstants.IPV4_REGEXP) ? privateIP : null;
    return conf;
}
async function getAvailablePort(client) {
    const mappings = await Q.nbind(client.getMappings, client)();
    const externalPortsUsed = mappings.map(m => m.public.port);
    let availablePort = constants_1.BMAConstants.BMA_PORTS_START;
    while (externalPortsUsed.indexOf(availablePort) !== -1 && availablePort <= constants_1.BMAConstants.BMA_PORTS_END) {
        availablePort++;
    }
    if (availablePort > constants_1.BMAConstants.BMA_PORTS_END) {
        throw "No port available for UPnP";
    }
    return availablePort;
}
function getRandomPort(conf) {
    if (conf && conf.remoteport) {
        return conf.remoteport;
    }
    else {
        return ~~(Math.random() * (65536 - constants_1.BMAConstants.PORT_START)) + constants_1.BMAConstants.PORT_START;
    }
}
//# sourceMappingURL=network.js.map