import { BlockDTO } from "../../../lib/dto/BlockDTO";
import { DBBlock } from "../../../lib/db/DBBlock";
import { PeerDTO } from "../../../lib/dto/PeerDTO";
import { BranchingDTO } from "../../../lib/dto/ConfDTO";
export declare abstract class PullingDao {
    abstract applyBranch(blocks: BlockDTO[]): Promise<boolean>;
    abstract localCurrent(): Promise<DBBlock | null>;
    abstract remoteCurrent(source?: any): Promise<BlockDTO | null>;
    abstract remotePeers(source?: any): Promise<PeerDTO[]>;
    abstract getLocalBlock(number: number): Promise<DBBlock>;
    abstract getRemoteBlock(thePeer: PeerDTO, number: number): Promise<BlockDTO>;
    abstract applyMainBranch(block: BlockDTO): Promise<boolean>;
    abstract removeForks(): Promise<boolean>;
    abstract isMemberPeer(thePeer: PeerDTO): Promise<boolean>;
    abstract downloadBlocks(thePeer: PeerDTO, fromNumber: number, count?: number): Promise<BlockDTO[]>;
}
export declare abstract class AbstractDAO extends PullingDao {
    /**
     * Sugar function. Apply a bunch of blocks instead of one.
     * @param blocks
     */
    applyBranch(blocks: BlockDTO[]): Promise<boolean>;
    /**
     * Binary search algorithm to find the common root block between a local and a remote blockchain.
     * @param fork An object containing a peer, its current block and top fork block
     * @param forksize The maximum length we can look at to find common root block.
     * @returns {*|Promise}
     */
    findCommonRoot(fork: any, forksize: number): Promise<BlockDTO | null>;
    static defaultLocalBlock(): DBBlock;
    /**
     * Pull algorithm. Look at given peers' blockchain and try to pull blocks from it.
     * May lead local blockchain to fork.
     * @param conf The local node configuration
     * @param dao An abstract layer to retrieve peers data (blocks).
     * @param logger Logger of the main application.
     */
    pull(conf: BranchingDTO, logger: any): Promise<DBBlock | null>;
}
