"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const path = require("path");
const constants_1 = require("../../../../lib/common-libs/constants");
const ASyncDownloader_1 = require("./ASyncDownloader");
class FsSyncDownloader extends ASyncDownloader_1.ASyncDownloader {
    constructor(fs, basePath, getChunkName, chunkSize) {
        super(chunkSize);
        this.fs = fs;
        this.basePath = basePath;
        this.getChunkName = getChunkName;
        this.chunkSize = chunkSize;
        this.ttas = [];
        this.ls = this.fs.fsList(basePath);
    }
    async getChunk(i) {
        const start = Date.now();
        const files = await this.ls;
        const filepath = path.join(this.basePath, this.getChunkName(i));
        const basename = path.basename(filepath);
        let existsOnDAL = files.filter(f => f === basename).length === 1;
        if (!existsOnDAL) {
            // We make another try in case the file was created after the initial `ls` test
            existsOnDAL = await this.fs.fsExists(filepath);
        }
        if (existsOnDAL) {
            const content = JSON.parse(await this.fs.fsReadFile(filepath));
            // Record the reading duration
            this.ttas.push(Date.now() - start);
            // Returns a promise of file content
            return content.blocks;
        }
        return [];
    }
    get maxSlots() {
        return constants_1.CommonConstants.MAX_READING_SLOTS_FOR_FILE_SYNC;
    }
}
exports.FsSyncDownloader = FsSyncDownloader;
//# sourceMappingURL=FsSyncDownloader.js.map