"use strict";
// Source file from duniter: Crypto-currency software to manage libre currency such as Ğ1
// Copyright (C) 2018  Cedric Moreau <cem.moreau@gmail.com>
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Affero General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Affero General Public License for more details.
Object.defineProperty(exports, "__esModule", { value: true });
const FsSyncDownloader_1 = require("./FsSyncDownloader");
const AbstractSynchronizer_1 = require("./AbstractSynchronizer");
const constants_1 = require("../../../../lib/common-libs/constants");
const directory_1 = require("../../../../lib/system/directory");
class LocalPathSynchronizer extends AbstractSynchronizer_1.AbstractSynchronizer {
    constructor(path, server, chunkSize) {
        super(chunkSize);
        this.path = path;
        this.server = server;
        const fs = directory_1.RealFS();
        this.ls = fs.fsList(path);
        // We read from the real file system here, directly.
        this.theFsDownloader = new FsSyncDownloader_1.FsSyncDownloader(fs, this.path, this.getChunkName.bind(this), chunkSize);
        this.theP2pDownloader = new FsSyncDownloader_1.FsSyncDownloader(fs, this.path, this.getChunkName.bind(this), chunkSize);
    }
    get dal() {
        return this.server.dal;
    }
    get readDAL() {
        return this.dal;
    }
    get PeeringService() {
        return this.server.PeeringService;
    }
    getCurrency() {
        return this.currency;
    }
    getPeer() {
        return this;
    }
    getChunksPath() {
        return this.path;
    }
    setWatcher(watcher) {
        this.watcher = watcher;
    }
    async init() {
        // TODO: check that path exists and that files seem consistent
    }
    async initWithKnownLocalAndToAndCurrency(to, localNumber, currency) {
        this.currency = currency;
    }
    p2pDownloader() {
        return this.theP2pDownloader;
    }
    fsDownloader() {
        return this.theFsDownloader;
    }
    async getCurrent() {
        const chunkNumbers = (await this.ls).map(s => parseInt(s.replace(constants_1.CommonConstants.CHUNK_PREFIX, '')));
        const topChunk = chunkNumbers.reduce((number, max) => Math.max(number, max), -1);
        if (topChunk === -1) {
            return null;
        }
        const chunk = await this.theFsDownloader.getChunk(topChunk);
        return chunk[chunk.length - 1]; // This is the top block of the top chunk = the current block
    }
    async getBlock(number) {
        const chunkNumber = parseInt(String(number / this.chunkSize));
        const position = number % this.chunkSize;
        const chunk = await this.theFsDownloader.getChunk(chunkNumber);
        return chunk[position];
    }
    getMilestone(number) {
        return this.getBlock(number);
    }
    async syncPeers(fullSync, to) {
        // Does nothing on LocalPathSynchronizer
    }
    async syncSandbox() {
        // Does nothing on LocalPathSynchronizer
    }
}
exports.LocalPathSynchronizer = LocalPathSynchronizer;
//# sourceMappingURL=LocalPathSynchronizer.js.map