"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const events = require("events");
const programOptions_1 = require("../../../../lib/common-libs/programOptions");
const multimeter = require('multimeter');
class EventWatcher extends events.EventEmitter {
    constructor(innerWatcher) {
        super();
        this.innerWatcher = innerWatcher;
    }
    writeStatus(str) {
        this.innerWatcher.writeStatus(str);
    }
    downloadPercent(pct) {
        return this.change('downloadChange', (pct) => this.innerWatcher.downloadPercent(pct), pct);
    }
    storagePercent(pct) {
        return this.change('storageChange', (pct) => this.innerWatcher.storagePercent(pct), pct);
    }
    appliedPercent(pct) {
        return this.change('appliedChange', (pct) => this.innerWatcher.appliedPercent(pct), pct);
    }
    sbxPercent(pct) {
        return this.change('sbxChange', (pct) => this.innerWatcher.sbxPercent(pct), pct);
    }
    peersPercent(pct) {
        return this.change('peersChange', (pct) => this.innerWatcher.peersPercent(pct), pct);
    }
    change(changeName, method, pct) {
        if (pct !== undefined && method() < pct) {
            this.emit(changeName, pct || 0);
        }
        return method(pct);
    }
    end() {
        this.innerWatcher.end();
    }
    onEvent(e, cb) {
        this.on(e, cb);
    }
    getStats() {
        return {
            download: this.downloadPercent(),
            saved: this.storagePercent(),
            applied: this.appliedPercent(),
            sandbox: this.sbxPercent(),
            peersSync: this.peersPercent(),
        };
    }
    /************* P2P DOWNLOAD EVENTS ****************/
    addWrongChunkFailure(chunkIndex, lastSupplier) {
        this.emit('addWrongChunkFailure', { chunkIndex, node: lastSupplier });
    }
    failToGetChunk(chunkIndex, node) {
        this.emit('failToGetChunk', { chunkIndex, node });
    }
    gettingChunk(chunkIndex, candidates) {
        this.emit('gettingChunk', { chunkIndex, nodes: candidates });
    }
    gotChunk(chunkIndex, node) {
        this.emit('gotChunk', { chunkIndex, node });
    }
    reserveNodes(nodesAvailable) {
        this.emit('reserveNodes', { nodes: nodesAvailable });
    }
    unableToDownloadChunk(chunkIndex) {
        this.emit('unableToDownloadChunk', { chunkIndex });
    }
    wantToDownload(chunkIndex) {
        this.emit('wantToDownload', { chunkIndex });
    }
    wantToLoad(chunkIndex) {
        this.emit('wantToLoad', { chunkIndex });
    }
    beforeReadyNodes(p2pCandidates) {
        this.emit('beforeReadyNodes', { nodes: p2pCandidates });
    }
    syncFailNoNodeFound() {
        this.emit('syncFailNoNodeFound', {});
    }
    syncFailCannotConnectToRemote() {
        this.emit('syncFailCannotConnectToRemote', {});
    }
}
exports.EventWatcher = EventWatcher;
class MultimeterWatcher {
    constructor() {
        this.writtens = [];
        this.multi = multimeter(process);
        this.charm = this.multi.charm;
        this.charm.on('^C', process.exit);
        this.charm.reset();
        this.multi.write('Progress:\n\n');
        let line = 3;
        this.savedBar = this.createBar('Milestones', line++);
        this.downloadBar = this.createBar('Download', line++);
        this.appliedBar = this.createBar('Apply', line++);
        if (!programOptions_1.cliprogram.nosbx) {
            this.sbxBar = this.createBar('Sandbox', line++);
        }
        if (!programOptions_1.cliprogram.nopeers) {
            this.peersBar = this.createBar('Peers', line++);
        }
        this.multi.write('\nStatus: ');
        this.charm.position((x, y) => {
            this.xPos = x;
            this.yPos = y;
        });
        this.writtens = [];
        this.downloadBar.percent(0);
        this.savedBar.percent(0);
        this.appliedBar.percent(0);
        if (!programOptions_1.cliprogram.nosbx) {
            this.sbxBar.percent(0);
        }
        if (!programOptions_1.cliprogram.nopeers) {
            this.peersBar.percent(0);
        }
    }
    writeStatus(str) {
        this.writtens.push(str);
        this.charm
            .position(this.xPos, this.yPos)
            .erase('end')
            .write(str);
    }
    ;
    downloadPercent(pct) {
        return this.downloadBar.percent(pct);
    }
    storagePercent(pct) {
        return this.savedBar.percent(pct);
    }
    appliedPercent(pct) {
        return this.appliedBar.percent(pct);
    }
    sbxPercent(pct) {
        if (!programOptions_1.cliprogram.nosbx) {
            return this.sbxBar.percent(pct);
        }
        return 0;
    }
    peersPercent(pct) {
        if (!programOptions_1.cliprogram.nopeers) {
            return this.peersBar.percent(pct);
        }
        return 0;
    }
    end() {
        this.multi.write('\nAll done.\n');
        this.multi.destroy();
    }
    createBar(title, line) {
        const header = (title + ':').padEnd(14, ' ') + '\n';
        this.multi.write(header);
        return this.multi(header.length, line, {
            width: 20,
            solid: {
                text: '|',
                foreground: 'white',
                background: 'blue'
            },
            empty: { text: ' ' }
        });
    }
    /************* NOT IMPLEMENTED ****************/
    addWrongChunkFailure(chunkIndex, lastSupplier) {
    }
    failToGetChunk(chunkIndex, node) {
    }
    gettingChunk(chunkIndex, candidates) {
    }
    gotChunk(chunkIndex, node) {
    }
    reserveNodes(nodesAvailable) {
    }
    unableToDownloadChunk(chunkIndex) {
    }
    wantToDownload(chunkIndex) {
    }
    wantToLoad(chunkIndex) {
    }
    beforeReadyNodes(p2pCandidates) {
    }
    syncFailNoNodeFound() {
    }
    syncFailCannotConnectToRemote() {
    }
}
exports.MultimeterWatcher = MultimeterWatcher;
class LoggerWatcher {
    constructor(logger) {
        this.logger = logger;
        this.downPct = 0;
        this.savedPct = 0;
        this.appliedPct = 0;
        this.sbxPct = 0;
        this.peersPct = 0;
        this.lastMsg = "";
    }
    showProgress() {
        return this.logger.info('Milestones %s%, Downloaded %s%, Applied %s%', this.savedPct, this.downPct, this.appliedPct);
    }
    writeStatus(str) {
        if (str != this.lastMsg) {
            this.lastMsg = str;
            this.logger.info(str);
        }
    }
    downloadPercent(pct) {
        return this.change('downPct', pct);
    }
    storagePercent(pct) {
        return this.change('savedPct', pct);
    }
    appliedPercent(pct) {
        return this.change('appliedPct', pct);
    }
    sbxPercent(pct) {
        if (pct > this.sbxPct) {
            this.sbxPct = pct;
        }
        return this.sbxPct;
    }
    peersPercent(pct) {
        if (pct > this.peersPct) {
            this.peersPct = pct;
        }
        return this.peersPct;
    }
    change(prop, pct) {
        if (pct !== undefined) {
            let changed = pct > this[prop];
            this[prop] = pct;
            if (changed)
                this.showProgress();
        }
        return this[prop];
    }
    end() {
    }
    /************* NOT IMPLEMENTED ****************/
    addWrongChunkFailure(chunkIndex, lastSupplier) {
    }
    failToGetChunk(chunkIndex, node) {
    }
    gettingChunk(chunkIndex, candidates) {
    }
    gotChunk(chunkIndex, node) {
    }
    reserveNodes(nodesAvailable) {
    }
    unableToDownloadChunk(chunkIndex) {
    }
    wantToDownload(chunkIndex) {
    }
    wantToLoad(chunkIndex) {
    }
    beforeReadyNodes(p2pCandidates) {
    }
    syncFailNoNodeFound() {
    }
    syncFailCannotConnectToRemote() {
    }
}
exports.LoggerWatcher = LoggerWatcher;
//# sourceMappingURL=Watcher.js.map