"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
Object.defineProperty(exports, "__esModule", { value: true });
const stream_1 = require("stream");
const indexer_1 = require("../../../../../lib/indexer");
const DuniterBlockchain_1 = require("../../../../../lib/blockchain/DuniterBlockchain");
const BlockDTO_1 = require("../../../../../lib/dto/BlockDTO");
const underscore_1 = require("../../../../../lib/common-libs/underscore");
const MonitorExecutionTime_1 = require("../../../../../lib/debug/MonitorExecutionTime");
const wot_1 = require("../../../../../lib/wot");
const logger_1 = require("../../../../../lib/logger");
const DBBlock_1 = require("../../../../../lib/db/DBBlock");
const programOptions_1 = require("../../../../../lib/common-libs/programOptions");
const constants = require('../../constants');
let sync_expires = [];
let sync_bindex = [];
let sync_iindex = [];
let sync_mindex = [];
let sync_cindex = [];
let sync_nextExpiring = 0;
let sync_bindexSize = 0;
let txCount = 0;
let logger = logger_1.NewLogger();
const sync_memoryWallets = {};
const sync_memoryDAL = {
    getWallet: (conditions) => Promise.resolve(sync_memoryWallets[conditions] || { conditions, balance: 0 }),
    saveWallet: async (wallet) => {
        // Make a copy
        sync_memoryWallets[wallet.conditions] = {
            conditions: wallet.conditions,
            balance: wallet.balance
        };
    },
    sindexDAL: {
        getAvailableForConditions: (conditions) => Promise.resolve([])
    }
};
class GlobalIndexStream extends stream_1.Duplex {
    constructor(conf, dal, to, localNumber, cautious, syncStrategy, watcher) {
        super({ objectMode: true });
        this.conf = conf;
        this.dal = dal;
        this.to = to;
        this.localNumber = localNumber;
        this.cautious = cautious;
        this.syncStrategy = syncStrategy;
        this.watcher = watcher;
        this.wotbMem = wot_1.WoTBObject.memoryInstance();
        this.currentChunkNumber = 0;
        this.memToCopyDone = false;
        this.mapInjection = {};
        this.wotbMem = dal.wotb;
        const nbBlocksToDownload = Math.max(0, to - localNumber);
        this.numberOfChunksToDownload = Math.ceil(nbBlocksToDownload / syncStrategy.chunkSize);
        this.readChunk(this.currentChunkNumber);
        sync_memoryDAL.sindexDAL = {
            getAvailableForConditions: (conditions) => this.dal.sindexDAL.getAvailableForConditions(conditions)
        };
        this.memSyncInjection = (async () => {
            // Disabled function
        })();
    }
    async injectLoki(dal, f, obj) {
        // this.mapInjection[f] = dal[f]
        // dal[f] = obj
        // await (obj as any).triggerInit()
    }
    readChunk(i) {
    }
    _read(size) {
        this.push(null);
    }
    _write(dataArray, encoding, callback) {
        (async () => {
            await this.memSyncInjection;
            if (!dataArray) {
                return callback(null);
            }
            await this.transform(dataArray);
            this.watcher.appliedPercent(Math.round(dataArray[0].block.number / this.syncStrategy.chunkSize / this.numberOfChunksToDownload * 100));
            callback(null);
        })();
    }
    /**
     * Interpretes a chunk of blocks, and return the generated INDEX entries for eventual backup
     * @param {ProtocolIndexesStream[]} dataArray
     * @returns {Promise<GindexData[]>}
     */
    async transform(dataArray) {
        await this.beforeBlocks(dataArray.map(d => d.block));
        const gindex = [];
        for (const data of dataArray) {
            const block = data.block;
            const gData = {
                lindex: {
                    mindex: data.mindex.slice(),
                    iindex: data.iindex.slice(),
                    sindex: data.sindex.slice(),
                    cindex: data.cindex.slice(),
                },
                gindex: {
                    mindex: [],
                    iindex: [],
                    sindex: [],
                    cindex: [],
                },
                block,
                head: null,
            };
            // VERY FIRST: parameters, otherwise we compute wrong variables such as UDTime
            if (block.number == 0) {
                this.sync_currConf = BlockDTO_1.BlockDTO.getConf(block);
                await DuniterBlockchain_1.DuniterBlockchain.saveParametersForRoot(block, this.conf, this.dal);
            }
            const bindexSize = DuniterBlockchain_1.requiredBindexSizeForTail(block, this.conf);
            if ((block.number <= this.to - bindexSize - 1 || programOptions_1.cliprogram.noSources) && !this.cautious) { // If we require nosources option, this blockchain can't be valid so we don't make checks
                const HEAD = await indexer_1.Indexer.quickCompleteGlobalScope(block, this.sync_currConf, sync_bindex, data.iindex, data.mindex, data.cindex, this.dal);
                sync_bindex.push(HEAD);
                // GINDEX
                gData.head = HEAD;
                // Remember expiration dates
                for (const entry of data.cindex) {
                    if (entry.expires_on) {
                        sync_expires.push(entry.expires_on);
                    }
                }
                for (const entry of data.mindex) {
                    if (entry.expires_on) {
                        sync_expires.push(entry.expires_on);
                    }
                }
                for (const entry of data.mindex) {
                    if (entry.revokes_on) {
                        sync_expires.push(entry.revokes_on);
                    }
                }
                if (data.iindex.length) {
                    await DuniterBlockchain_1.DuniterBlockchain.createNewcomers(data.iindex, this.dal, logger_1.NewLogger(), this.wotbMem);
                }
                if ((block.dividend && !programOptions_1.cliprogram.noSources)
                    || block.joiners.length
                    || block.actives.length
                    || block.revoked.length
                    || block.excluded.length
                    || block.certifications.length
                    || (block.transactions.length && !programOptions_1.cliprogram.noSources)
                    || block.medianTime >= sync_nextExpiring) {
                    const nextExpiringChanged = block.medianTime >= sync_nextExpiring;
                    for (let i = 0; i < sync_expires.length; i++) {
                        let expire = sync_expires[i];
                        if (block.medianTime >= expire) {
                            sync_expires.splice(i, 1);
                            i--;
                        }
                    }
                    sync_nextExpiring = sync_expires.reduce((max, value) => max ? Math.min(max, value) : value, 9007199254740991); // Far far away date
                    if (!programOptions_1.cliprogram.noSources) {
                        if (data.sindex.length) {
                            await this.blockFillTxSourcesConditions(data.sindex);
                        }
                        // Dividends and account garbaging
                        let dividends = [];
                        if (HEAD.new_dividend) {
                            dividends = await indexer_1.Indexer.ruleIndexGenDividend(HEAD, data.iindex, this.dal);
                        }
                        else {
                            for (const newcomer of data.iindex) {
                                await this.dal.dividendDAL.createMember(newcomer.pub);
                            }
                        }
                        if (block.transactions.length) {
                            data.sindex = data.sindex.concat(await indexer_1.Indexer.ruleIndexGarbageSmallAccounts(HEAD, data.sindex, dividends, sync_memoryDAL));
                        }
                        if (data.sindex.length) {
                            gData.gindex.sindex = data.sindex;
                            await this.flushSindex(data.sindex);
                        }
                        if (data.sindex.length || dividends.length) {
                            await DuniterBlockchain_1.DuniterBlockchain.updateWallets(data.sindex, dividends, sync_memoryDAL, false, block.number);
                        }
                    }
                    if (data.mindex.length || data.iindex.length || data.cindex.length) {
                        await this.flushMicIndexes(data.mindex, data.iindex, data.cindex);
                    }
                    if (nextExpiringChanged) {
                        sync_cindex = sync_cindex.concat(await indexer_1.Indexer.ruleIndexGenCertificationExpiry(HEAD, this.dal));
                        sync_mindex = sync_mindex.concat(await indexer_1.Indexer.ruleIndexGenMembershipExpiry(HEAD, this.dal));
                        sync_iindex = sync_iindex.concat(await indexer_1.Indexer.ruleIndexGenExclusionByMembership(HEAD, sync_mindex, this.dal));
                        sync_iindex = sync_iindex.concat(await indexer_1.Indexer.ruleIndexGenExclusionByCertificatons(HEAD, sync_cindex, data.iindex, this.conf, this.dal));
                        sync_mindex = sync_mindex.concat(await indexer_1.Indexer.ruleIndexGenImplicitRevocation(HEAD, this.dal));
                    }
                    if (sync_mindex.length || sync_iindex.length || sync_cindex.length) {
                        // Flush the INDEX again (needs to be done *before* the update of wotb links because of block#0)
                        await this.dal.flushIndexes({
                            mindex: sync_mindex,
                            iindex: sync_iindex,
                            sindex: [],
                            cindex: sync_cindex,
                        });
                    }
                    if (sync_cindex.length) {
                        await this.updateWotbLinks(sync_cindex);
                    }
                    gData.gindex.iindex = sync_iindex;
                    gData.gindex.mindex = sync_mindex;
                    gData.gindex.cindex = sync_cindex;
                    sync_iindex = [];
                    sync_mindex = [];
                    sync_cindex = [];
                    // TODO GINDEX
                    if (block.joiners.length || block.revoked.length || block.excluded.length) {
                        await this.updateMembers(block);
                    }
                }
                else {
                    // Concat the results to the pending data
                    sync_iindex = sync_iindex.concat(data.iindex);
                    sync_cindex = sync_cindex.concat(data.cindex);
                    sync_mindex = sync_mindex.concat(data.mindex);
                    gData.gindex.iindex = data.iindex;
                    gData.gindex.cindex = data.cindex;
                    gData.gindex.mindex = data.mindex;
                }
                // Trim the bindex
                sync_bindexSize = this.conf.forksize + [
                    block.issuersCount,
                    block.issuersFrame,
                    this.conf.medianTimeBlocks,
                    this.conf.dtDiffEval,
                    dataArray.length
                ].reduce((max, value) => {
                    return Math.max(max, value);
                }, 0);
                if (sync_bindexSize && sync_bindex.length >= 2 * sync_bindexSize) {
                    // We trim it, not necessary to store it all (we already store the full blocks)
                    sync_bindex.splice(0, sync_bindexSize);
                    // TODO GINDEX
                    await this.doTrimming();
                }
            }
            else if (block.number <= this.to) {
                const dto = BlockDTO_1.BlockDTO.fromJSONObject(block);
                await this.finalizeSync(block, dto);
            }
            gindex.push(gData);
        }
        return gindex;
    }
    async beforeBlocks(blocks) {
        await this.dal.blockDAL.insertBatch(blocks.map(b => {
            txCount += b.transactions.length;
            const block = DBBlock_1.DBBlock.fromBlockDTO(b);
            block.fork = false;
            return block;
        }));
        if (this.conf.storage && this.conf.storage.transactions) {
            await Promise.all(blocks.map(block => this.dal.saveTxsInFiles(block.transactions, block.number, block.medianTime)));
        }
        logger.debug('Total tx count: %s', txCount);
    }
    async flushSindex(local_sindex) {
        await this.dal.flushIndexes({
            mindex: [],
            iindex: [],
            cindex: [],
            sindex: local_sindex,
        });
    }
    async flushMicIndexes(local_mindex, local_iindex, local_cindex) {
        // Flush the INDEX (not bindex, which is particular)
        await this.dal.flushIndexes({
            mindex: sync_mindex,
            iindex: sync_iindex,
            sindex: [],
            cindex: sync_cindex,
        });
        sync_iindex = local_iindex;
        sync_cindex = local_cindex;
        sync_mindex = local_mindex;
    }
    async blockFillTxSourcesConditions(local_sindex) {
        // Fills in correctly the SINDEX
        await Promise.all(underscore_1.Underscore.where(local_sindex, { op: 'UPDATE' }).map(async (entry) => {
            if (!entry.conditions) {
                if (entry.srcType === 'D') {
                    entry.conditions = 'SIG(' + entry.identifier + ')';
                }
                else {
                    // First: have a look locally, but only chained transactions would have `localSrc` matching (giving conditions)
                    const localSrc = local_sindex.filter(s => s.identifier === entry.identifier && s.pos === entry.pos && s.conditions)[0];
                    const src = localSrc || (await this.dal.getSource(entry.identifier, entry.pos, false));
                    entry.conditions = src.conditions;
                }
            }
        }));
    }
    async updateWotbLinks(links) {
        // --> Update links
        await this.dal.updateWotbLinks(links, this.wotbMem);
    }
    async updateMembers(block) {
        // Create/Update nodes in wotb
        await DuniterBlockchain_1.DuniterBlockchain.updateMembers(block, this.dal, this.wotbMem);
    }
    async doTrimming() {
        // Process triming & archiving continuously to avoid super long ending of sync
        await this.dal.trimIndexes(sync_bindex[0].number);
    }
    async finalizeSync(block, dto) {
        // Save the INDEX
        await this.dal.bindexDAL.insertBatch(sync_bindex);
        await this.dal.flushIndexes({
            mindex: sync_mindex,
            iindex: sync_iindex,
            sindex: [],
            cindex: sync_cindex,
        });
        if (!this.memToCopyDone) {
            // Save the intermediary table of wallets
            const conditions = underscore_1.Underscore.keys(sync_memoryWallets);
            const nonEmptyKeys = underscore_1.Underscore.filter(conditions, (k) => sync_memoryWallets[k] && sync_memoryWallets[k].balance > 0);
            const walletsToRecord = nonEmptyKeys.map((k) => sync_memoryWallets[k]);
            await this.dal.walletDAL.insertBatch(walletsToRecord);
            for (const cond of conditions) {
                delete sync_memoryWallets[cond];
            }
            logger_1.NewLogger().info('Mem2File [wotb]...');
            // Persist the memory wotb
            this.wotbMem.fileCopy(this.dal.wotb.filePath);
            const that = this;
            // Disabled for now
            async function inject(fileDal, field, getRows) {
                // const dao = that.mapInjection[field]
                // if (dao) {
                //   NewLogger().info(`Mem2File [${field}]...`)
                //   const rows = await getRows()
                //   await (dao as any).insertBatch(rows) // TODO : "any" complicated to remove
                //   fileDal[field] = dao
                // }
                // else {
                //   throw Error(DataErrors[DataErrors.SYNC_FAST_MEM_ERROR_DURING_INJECTION])
                // }
            }
            this.memToCopyDone = true;
        }
        if (block.number === 0) {
            await DuniterBlockchain_1.DuniterBlockchain.saveParametersForRoot(block, this.conf, this.dal);
        }
        // Last block: cautious mode to trigger all the INDEX expiry mechanisms
        const { index, HEAD } = await DuniterBlockchain_1.DuniterBlockchain.checkBlock(dto, constants.WITH_SIGNATURES_AND_POW, this.conf, this.dal);
        await DuniterBlockchain_1.DuniterBlockchain.pushTheBlock(dto, index, HEAD, this.conf, this.dal, logger_1.NewLogger());
        // Clean temporary variables
        sync_bindex = [];
        sync_iindex = [];
        sync_mindex = [];
        sync_cindex = [];
        sync_bindexSize = 0;
        sync_expires = [];
        sync_nextExpiring = 0;
    }
}
__decorate([
    MonitorExecutionTime_1.MonitorExecutionTime()
], GlobalIndexStream.prototype, "transform", null);
__decorate([
    MonitorExecutionTime_1.MonitorExecutionTime()
], GlobalIndexStream.prototype, "beforeBlocks", null);
__decorate([
    MonitorExecutionTime_1.MonitorExecutionTime()
], GlobalIndexStream.prototype, "flushSindex", null);
__decorate([
    MonitorExecutionTime_1.MonitorExecutionTime()
], GlobalIndexStream.prototype, "flushMicIndexes", null);
__decorate([
    MonitorExecutionTime_1.MonitorExecutionTime()
], GlobalIndexStream.prototype, "blockFillTxSourcesConditions", null);
__decorate([
    MonitorExecutionTime_1.MonitorExecutionTime()
], GlobalIndexStream.prototype, "updateWotbLinks", null);
__decorate([
    MonitorExecutionTime_1.MonitorExecutionTime()
], GlobalIndexStream.prototype, "updateMembers", null);
__decorate([
    MonitorExecutionTime_1.MonitorExecutionTime()
], GlobalIndexStream.prototype, "doTrimming", null);
__decorate([
    MonitorExecutionTime_1.MonitorExecutionTime()
], GlobalIndexStream.prototype, "finalizeSync", null);
exports.GlobalIndexStream = GlobalIndexStream;
//# sourceMappingURL=GlobalIndexStream.js.map