"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const stream_1 = require("stream");
const indexer_1 = require("../../../../../lib/indexer");
const programOptions_1 = require("../../../../../lib/common-libs/programOptions");
const BlockDTO_1 = require("../../../../../lib/dto/BlockDTO");
class LocalIndexStream extends stream_1.Transform {
    constructor() {
        super({ objectMode: true });
        this.currentChunkNumber = 0;
    }
    _transform(blocks, encoding, callback) {
        (async () => {
            if (!blocks) {
                return setTimeout(() => callback(null, undefined), 1);
            }
            const result = [];
            for (const block of blocks) {
                // The new kind of object stored
                const dto = BlockDTO_1.BlockDTO.fromJSONObject(block);
                if (block.number == 0) {
                    this.sync_currConf = BlockDTO_1.BlockDTO.getConf(block);
                }
                const index = indexer_1.Indexer.localIndex(dto, this.sync_currConf);
                result.push({
                    block,
                    iindex: indexer_1.Indexer.iindex(index),
                    cindex: indexer_1.Indexer.cindex(index),
                    sindex: programOptions_1.cliprogram.noSources ? [] : indexer_1.Indexer.sindex(index),
                    mindex: indexer_1.Indexer.mindex(index),
                });
            }
            this.currentChunkNumber++;
            // Done for this chunk
            callback(null, result);
        })();
    }
}
exports.LocalIndexStream = LocalIndexStream;
//# sourceMappingURL=LocalIndexStream.js.map