"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const underscore_1 = require("../../../lib/common-libs/underscore");
const logger_1 = require("../../../lib/logger");
async function copyMemPool(server, wwDAL) {
    const logger = logger_1.NewLogger();
    const identities = await server.dal.idtyDAL.sqlListAll();
    // Blocks on which are based identities
    const blocks = await Promise.all(identities.map(async (idty) => returnBlockIfPresentInServerButNotInWW(idty.buid, server, wwDAL)));
    const toPersist = underscore_1.Underscore.uniq(blocks.filter(b => b), false, b => [b.number, b.hash].join('-'));
    logger.debug('Persisting %s blocks for identities...', toPersist.length);
    await wwDAL.blockDao.insertBatch(toPersist.map(b => { b.legacy = true; return b; }));
    await wwDAL.idtyDao.insertBatch(identities);
    await wwDAL.certDao.insertBatch(await server.dal.certDAL.sqlListAll());
    await wwDAL.msDao.insertBatch(await server.dal.msDAL.sqlListAll());
}
exports.copyMemPool = copyMemPool;
/**
 * Returns the server's block of given blockstamp if found in server and not found in WW
 * @param blockstamp
 * @param server
 * @param wwDAL
 */
async function returnBlockIfPresentInServerButNotInWW(blockstamp, server, wwDAL) {
    let b = await server.dal.getAbsoluteBlockByBlockstamp(blockstamp);
    if (b) {
        if (!(await wwHasBlock(wwDAL, b))) {
            return b;
        }
    }
    return null;
}
/**
 * Tells wether given block is found in WW database or not.
 * @param wwDAL
 * @param b
 */
async function wwHasBlock(wwDAL, b) {
    const wwBlock = await wwDAL.blockDao.getAbsoluteBlock(b.number, b.hash);
    return !!wwBlock;
}
exports.wwHasBlock = wwHasBlock;
//# sourceMappingURL=wotwizard.copy.mempool.js.map