"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const fs = require("fs");
const wotwizard_init_structure_1 = require("./wotwizard.init.structure");
const wotwizard_constants_1 = require("./wotwizard.constants");
const wotwizard_legacy_blocks_1 = require("./wotwizard.legacy.blocks");
const wotwizard_new_blocks_1 = require("./wotwizard.new.blocks");
const wotwizard_delete_1 = require("./wotwizard.delete");
const wotwizard_copy_mempool_1 = require("./wotwizard.copy.mempool");
const directory_1 = require("../../../lib/system/directory");
async function dumpWotWizard(server) {
    // 1. Create dump structure if it does not exist
    const wwDAL = await wotwizard_init_structure_1.createExportStructure(wotwizard_constants_1.WotWizardConstants.DB_NAME_0);
    // 2. Integrate legacy blocks (= non-forkable)
    await wotwizard_legacy_blocks_1.addLegacyBlocks(server, wwDAL);
    // 3. Delete non-legacy data
    await wotwizard_delete_1.deleteNonLegacy(wwDAL);
    // 4. Integrate new blocks (= forkable)
    await wotwizard_new_blocks_1.addNewBlocks(server, wwDAL);
    // 5. Copy mempool
    await wotwizard_copy_mempool_1.copyMemPool(server, wwDAL);
    // 6. Close SQL connections
    await Promise.all([
        wwDAL.blockDao,
        wwDAL.iindexDao,
        wwDAL.idtyDao,
        wwDAL.certDao,
        wwDAL.msDao,
    ].map(dao => dao.close()));
    // 7. Copy
    let lastCopyIsOldEnough = false;
    const updatingFile = directory_1.Directory.GET_FILE_PATH(wotwizard_constants_1.WotWizardConstants.FILE_UPDATING);
    if (fs.existsSync(updatingFile)) {
        const content = parseInt(fs.readFileSync(updatingFile, 'utf8'));
        lastCopyIsOldEnough = Date.now() - content > wotwizard_constants_1.WotWizardConstants.DELAY_FOR_UPDATING;
    }
    else {
        // Never done
        lastCopyIsOldEnough = true;
    }
    if (lastCopyIsOldEnough) {
        fs.copyFileSync(directory_1.Directory.GET_FILE_PATH(wotwizard_constants_1.WotWizardConstants.DB_NAME_0), directory_1.Directory.GET_FILE_PATH(wotwizard_constants_1.WotWizardConstants.DB_NAME));
        fs.writeFileSync(updatingFile, Date.now());
    }
}
exports.dumpWotWizard = dumpWotWizard;
//# sourceMappingURL=wotwizard.dump.js.map