"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const directory_1 = require("../../../lib/system/directory");
const IdentityDAL_1 = require("../../../lib/dal/sqliteDAL/IdentityDAL");
const MembershipDAL_1 = require("../../../lib/dal/sqliteDAL/MembershipDAL");
const CertDAL_1 = require("../../../lib/dal/sqliteDAL/CertDAL");
const BlockDAL_1 = require("../../../lib/dal/sqliteDAL/BlockDAL");
const IIndexDAL_1 = require("../../../lib/dal/sqliteDAL/index/IIndexDAL");
async function createExportStructure(dbName) {
    const driver = await directory_1.Directory.getHomeDB(false, dbName);
    // DAOs
    const idtyDao = new IdentityDAL_1.IdentityDAL(driver);
    const certDao = new CertDAL_1.CertDAL(driver);
    const msDao = new MembershipDAL_1.MembershipDAL(driver);
    const blockDao = new BlockDAL_1.BlockDAL(driver);
    const iindexDao = new IIndexDAL_1.IIndexDAL(driver);
    // Create tables
    await idtyDao.init();
    await certDao.init();
    await msDao.init();
    await blockDao.init();
    await iindexDao.init();
    const data = await blockDao.query('SELECT COUNT(*) as count FROM block');
    const blocksCount = parseInt(String(data[0].count));
    // If first DB initialization
    if (blocksCount === 0) {
        // Manual updates (which are normally present in MetaDAL)
        await idtyDao.exec('ALTER TABLE idty ADD COLUMN expired INTEGER NULL');
        await idtyDao.exec('ALTER TABLE idty ADD COLUMN revoked_on INTEGER NULL');
        await idtyDao.exec('ALTER TABLE idty ADD COLUMN removed BOOLEAN NULL DEFAULT 0');
        await certDao.exec('ALTER TABLE cert ADD COLUMN expired INTEGER NULL');
        await msDao.exec('ALTER TABLE membership ADD COLUMN expired INTEGER NULL');
    }
    return {
        idtyDao,
        certDao,
        msDao,
        blockDao,
        iindexDao,
    };
}
exports.createExportStructure = createExportStructure;
//# sourceMappingURL=wotwizard.init.structure.js.map