"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const constants_1 = require("../../../lib/common-libs/constants");
const logger_1 = require("../../../lib/logger");
async function addNewBlocks(server, wwDAL) {
    const logger = logger_1.NewLogger();
    wwDAL.blockDao.cleanCache();
    const currentWW = await wwDAL.blockDao.getCurrent();
    const current = await server.dal.blockDAL.getCurrent();
    const start = currentWW && currentWW.number + 1 || 0;
    const end = current && current.number || -1;
    const blocksSaved = [];
    // We loop to work in flow mode (avoid big memory consumption)
    for (let i = start; i <= end; i += constants_1.CommonConstants.BLOCKS_IN_MEMORY_MAX) {
        const beginAt = i;
        const endAt = Math.min(end, i + constants_1.CommonConstants.BLOCKS_IN_MEMORY_MAX) - 1;
        const blocks = await server.dal.getBlocksBetween(beginAt, endAt);
        const forks = await server.dal.getPotentialForkBlocks(beginAt, 0, endAt);
        const all = blocks.concat(forks).map(f => { f.legacy = false; return f; });
        logger.debug('Saving %s pending blocks...', all.length);
        blocksSaved.push(wwDAL.blockDao.insertBatch(all));
    }
    await Promise.all(blocksSaved);
    const iindexRows = (await server.dal.iindexDAL.findRawWithOrder({}, [['writtenOn', false], ['wotb_id', false]]))
        .filter(r => r.writtenOn >= start && r.uid);
    logger.debug('Saving %s iindex rows...', iindexRows.length);
    const legacies = iindexRows.map(f => { f.legacy = false; return f; });
    await wwDAL.iindexDao.insertBatch(legacies);
}
exports.addNewBlocks = addNewBlocks;
//# sourceMappingURL=wotwizard.new.blocks.js.map