"use strict";
// Source file from duniter: Crypto-currency software to manage libre currency such as Ğ1
// Copyright (C) 2018  Cedric Moreau <cem.moreau@gmail.com>
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Affero General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Affero General Public License for more details.
Object.defineProperty(exports, "__esModule", { value: true });
const base58_1 = require("../../../lib/common-libs/crypto/base58");
const nacl_util_1 = require("../../../lib/common-libs/crypto/nacl-util");
const crypto = require("crypto");
const nacl = require('tweetnacl');
const SEED_LENGTH = 32; // Length of the key
/**
 * Generates a new keypair object from salt + password strings.
 * @param salt
 * @param key
 * @param N Scrypt parameter N. Defaults to 4096.
 * @param r Scrypt parameter r. Defaults to 16.
 * @param p Scrypt parameter p. Defaults to 1.
 * @return keyPair An object containing the public and private keys, base58 encoded.
 */
exports.Scrypt = async (salt, key, N = 4096, r = 16, p = 1) => {
    const keyBytes = await getScryptKey(key, salt, N, r, p);
    const pair = nacl.sign.keyPair.fromSeed(keyBytes);
    return {
        pub: base58_1.Base58encode(new Buffer(pair.publicKey, 'hex')),
        sec: base58_1.Base58encode(new Buffer(pair.secretKey, 'hex'))
    };
};
const getScryptKey = async (key, salt, N, r, p) => {
    const res = await new Promise((resolve, reject) => {
        crypto.scrypt(key, salt, SEED_LENGTH, { N, r, p }, (err, res) => {
            if (err)
                return reject(err);
            resolve(res);
        });
    });
    return nacl_util_1.decodeBase64(res.toString("base64"));
};
//# sourceMappingURL=scrypt.js.map