"use strict";
// Source file from duniter: Crypto-currency software to manage libre currency such as Ğ1
// Copyright (C) 2018  Cedric Moreau <cem.moreau@gmail.com>
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Affero General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Affero General Public License for more details.
Object.defineProperty(exports, "__esModule", { value: true });
const powCluster_1 = require("./powCluster");
const os = require('os');
// Super important for Node.js debugging
const debug = process.execArgv.toString().indexOf('--debug') !== -1;
if (debug) {
    //Set an unused port number.
    process.execArgv = [];
}
class PowEngine {
    constructor(conf, logger, dal) {
        this.conf = conf;
        this.dal = dal;
        // We use as much cores as available, but not more than CORES_MAXIMUM_USE_IN_PARALLEL
        this.nbWorkers = conf.nbCores;
        this.cluster = new powCluster_1.Master(this.nbWorkers, logger, dal);
        this.id = this.cluster.clusterId;
    }
    getNbWorkers() {
        return this.cluster.nbWorkers;
    }
    forceInit() {
        return this.cluster.initCluster();
    }
    async prove(stuff) {
        await this.cluster.cancelWork();
        return await this.cluster.proveByWorkers(stuff);
    }
    cancel() {
        return this.cluster.cancelWork();
    }
    setConf(value) {
        return this.cluster.changeConf(value);
    }
    setOnInfoMessage(callback) {
        return this.cluster.onInfoMessage = callback;
    }
    async shutDown() {
        return this.cluster.shutDownWorkers();
    }
}
exports.PowEngine = PowEngine;
//# sourceMappingURL=engine.js.map