// Source file from duniter: Crypto-currency software to manage libre currency such as Ğ1
// Copyright (C) 2018  Cedric Moreau <cem.moreau@gmail.com>
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Affero General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Affero General Public License for more details.
"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const stream = require("stream");
const multicaster_1 = require("../lib/streams/multicaster");
const router_1 = require("../lib/streams/router");
exports.RouterDependency = {
    duniter: {
        service: {
            output: (server, conf, logger) => new Router(server)
        },
        methods: {
            routeToNetwork: (server) => {
                const theRouter = new Router(server);
                theRouter.startService();
                server.pipe(theRouter);
            }
        }
    }
};
/**
 * Service which triggers the server's peering generation (actualization of the Peer document).
 * @constructor
 */
class Router extends stream.Transform {
    constructor(server) {
        super({ objectMode: true });
        this.server = server;
        this.theMulticaster = new multicaster_1.Multicaster();
    }
    _write(obj, enc, done) {
        // Never close the stream
        if (obj) {
            this.push(obj);
        }
        done && done();
    }
    ;
    async startService() {
        if (this.server.conf.nobma || !this.server.conf.bmaWithCrawler) {
            // Disable BMA
            return Promise.resolve();
        }
        if (!this.theRouter) {
            this.theRouter = new router_1.RouterStream(this.server.PeeringService, this.server.dal);
        }
        this.theRouter.setActive(true);
        this.theRouter.setConfDAL(this.server.dal);
        /**
         * Enable routing features:
         *   - The server will try to send documents to the network
         *   - The server will eventually be notified of network failures
         */
        // The router asks for multicasting of documents
        this
            .pipe(this.theRouter)
            // The documents get sent to peers
            .pipe(this.theMulticaster)
            // The multicaster may answer 'unreachable peer'
            .pipe(this.theRouter);
    }
    async stopService() {
        if (this.server.conf.nobma || !this.server.conf.bmaWithCrawler) {
            // Disable BMA
            return Promise.resolve();
        }
        this.unpipe();
        this.theRouter && this.theRouter.unpipe();
        this.theMulticaster && this.theMulticaster.unpipe();
    }
}
exports.Router = Router;
//# sourceMappingURL=router.js.map