"use strict";
// Source file from duniter: Crypto-currency software to manage libre currency such as Ğ1
// Copyright (C) 2018  Cedric Moreau <cem.moreau@gmail.com>
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Affero General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Affero General Public License for more details.
Object.defineProperty(exports, "__esModule", { value: true });
const wizard_1 = require("../lib/wizard");
const underscore_1 = require("../lib/common-libs/underscore");
const logger = require('../lib/logger').NewLogger('wizard');
module.exports = {
    duniter: {
        wizard: {
            // The wizard itself also defines its personal tasks
            'currency': (conf) => wizard_1.Wizard.configCurrency(conf),
            'pow': (conf) => wizard_1.Wizard.configPoW(conf),
            'parameters': (conf) => wizard_1.Wizard.configUCP(conf)
        },
        cli: [{
                name: 'wizard [key|network|network-reconfigure|currency|pow|parameters]',
                desc: 'Launch the configuration wizard.',
                onConfiguredExecute: async (server, conf, program, params, wizardTasks) => {
                    const step = params[0];
                    const tasks = step ? [wizardTasks[step]] : underscore_1.Underscore.values(wizardTasks);
                    for (const task of tasks) {
                        if (!task) {
                            throw 'Unknown task';
                        }
                        await task(conf, program);
                    }
                    // Check config
                    await server.checkConfig();
                    await server.dal.saveConf(conf);
                    logger.debug("Configuration saved.");
                }
            }]
    }
};
//# sourceMappingURL=wizard.js.map