"use strict";
// Source file from duniter: Crypto-currency software to manage libre currency such as Ğ1
// Copyright (C) 2018  Cedric Moreau <cem.moreau@gmail.com>
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Affero General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Affero General Public License for more details.
Object.defineProperty(exports, "__esModule", { value: true });
const pulling_1 = require("../../crawler/lib/pulling");
const constants_1 = require("../../crawler/lib/constants");
const tx_cleaner_1 = require("../../crawler/lib/tx_cleaner");
const WS2PRequester_1 = require("./WS2PRequester");
class WS2PBlockPuller {
    constructor(server, connection) {
        this.server = server;
        this.connection = connection;
    }
    async pull() {
        const requester = WS2PRequester_1.WS2PRequester.fromConnection(this.connection);
        // node.pubkey = p.pubkey;
        let dao = new WS2PDao(this.server, requester);
        await dao.pull(this.server.conf, this.server.logger);
    }
}
exports.WS2PBlockPuller = WS2PBlockPuller;
class WS2PDao extends pulling_1.AbstractDAO {
    constructor(server, requester) {
        super();
        this.server = server;
        this.requester = requester;
        this.nodeCurrent = null;
        this.newCurrent = null;
        this.node = {
            getCurrent: async () => {
                return this.requester.getCurrent();
            },
            getBlock: async (number) => {
                return this.requester.getBlock(number);
            },
            getBlocks: async (count, fromNumber) => {
                return this.requester.getBlocks(count, fromNumber);
            },
            pubkey: this.requester.getPubkey()
        };
    }
    async localCurrent() {
        return this.server.dal.getCurrentBlockOrNull();
    }
    async remoteCurrent(source) {
        this.nodeCurrent = await source.getCurrent();
        return this.nodeCurrent;
    }
    async remotePeers(source) {
        const peer = this.node;
        return Promise.resolve([peer]);
    }
    async getLocalBlock(number) {
        return this.server.dal.getBlockWeHaveItForSure(number);
    }
    async getRemoteBlock(thePeer, number) {
        let block = null;
        try {
            block = await thePeer.getBlock(number);
            tx_cleaner_1.tx_cleaner(block.transactions);
        }
        catch (e) {
            if (e.httpCode != 404) {
                throw e;
            }
        }
        return block;
    }
    async applyMainBranch(block) {
        const existing = await this.server.dal.getAbsoluteBlockByNumberAndHash(block.number, block.hash);
        if (!existing) {
            let addedBlock = await this.server.writeBlock(block, false, true);
            if (!this.lastDownloaded) {
                this.lastDownloaded = await this.remoteCurrent(this.node);
            }
            this.server.pullingEvent('applying', { number: block.number, last: this.lastDownloaded && this.lastDownloaded.number });
            if (addedBlock) {
                this.newCurrent = addedBlock;
                // Emit block events (for sharing with the network) only in forkWindowSize
                if (this.nodeCurrent && this.nodeCurrent.number - addedBlock.number < this.server.conf.forksize) {
                    this.server.streamPush(addedBlock);
                }
            }
        }
        return true;
    }
    async removeForks() {
        return true;
    }
    async isMemberPeer(thePeer) {
        return true;
    }
    async downloadBlocks(thePeer, fromNumber, count) {
        if (!count) {
            count = constants_1.CrawlerConstants.CRAWL_BLOCK_CHUNK;
        }
        let blocks = await thePeer.getBlocks(count, fromNumber);
        // Fix for #734
        for (const block of blocks) {
            for (const tx of block.transactions) {
                tx.version = constants_1.CrawlerConstants.TRANSACTION_VERSION;
            }
        }
        return blocks;
    }
}
//# sourceMappingURL=WS2PBlockPuller.js.map