"use strict";
// Source file from duniter: Crypto-currency software to manage libre currency such as Ğ1
// Copyright (C) 2018  Cedric Moreau <cem.moreau@gmail.com>
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Affero General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Affero General Public License for more details.
Object.defineProperty(exports, "__esModule", { value: true });
const WS2PConnection_1 = require("./WS2PConnection");
const keyring_1 = require("../../../lib/common-libs/crypto/keyring");
const constants_1 = require("./constants");
const WS2PStreamer_1 = require("./WS2PStreamer");
const WS2PSingleWriteStream_1 = require("./WS2PSingleWriteStream");
const proxy_1 = require("../../../lib/proxy");
class WS2PClient {
    constructor(connection, streamer) {
        this.connection = connection;
        this.streamer = streamer;
    }
    disableStream() {
        this.streamer.disable();
    }
    static async connectTo(server, fullEndpointAddress, endpointVersion, expectedWS2PUID, messageHandler, expectedPub, allowKey) {
        const k2 = new keyring_1.Key(server.conf.pair.pub, server.conf.pair.sec);
        const myWs2pId = (server.conf.ws2p && server.conf.ws2p.uuid) ? server.conf.ws2p.uuid : "";
        const c = WS2PConnection_1.WS2PConnection.newConnectionToAddress(Math.min(endpointVersion, constants_1.WS2PConstants.WS2P_API_VERSION), fullEndpointAddress, messageHandler, new WS2PConnection_1.WS2PPubkeyLocalAuth(server.conf.currency, k2, myWs2pId, allowKey), new WS2PConnection_1.WS2PPubkeyRemoteAuth(server.conf.currency, k2, allowKey), proxy_1.ProxiesConf.wsProxy(fullEndpointAddress, server.conf.proxiesConf), {
            connectionTimeout: constants_1.WS2PConstants.REQUEST_TIMEOUT,
            requestTimeout: constants_1.WS2PConstants.REQUEST_TIMEOUT
        }, expectedPub, expectedWS2PUID);
        const singleWriteProtection = new WS2PSingleWriteStream_1.WS2PSingleWriteStream();
        const streamer = new WS2PStreamer_1.WS2PStreamer(c);
        c.connected
            .then(() => {
            // Streaming
            server
                .pipe(singleWriteProtection)
                .pipe(streamer);
        })
            .catch(() => {
            server.unpipe(singleWriteProtection);
            singleWriteProtection.unpipe(streamer);
        });
        c.closed.then(() => {
            server.unpipe(singleWriteProtection);
            singleWriteProtection.unpipe(streamer);
        });
        // Connecting
        try {
            await c.connect();
        }
        catch (e) {
            // Immediately close the connection
            c.close();
            throw e;
        }
        return new WS2PClient(c, streamer);
    }
}
exports.WS2PClient = WS2PClient;
//# sourceMappingURL=WS2PClient.js.map