import { WS2PServer } from "./WS2PServer";
import { Server } from "../../../../server";
import { WS2PClient } from "./WS2PClient";
import { WS2PConnection } from "./WS2PConnection";
import { WS2PMessageHandler } from "./impl/WS2PMessageHandler";
export interface WS2PHead {
    message: string;
    sig: string;
    messageV2?: string;
    sigV2?: string;
    step?: number;
}
export interface WS2pHeadCache extends WS2PHead {
    blockstamp: string;
}
export declare class WS2PCluster {
    private server;
    static getFullAddress(host: string, port: number, path?: string | null | undefined): string;
    private ws2pServer;
    private ws2pClients;
    private host;
    private port;
    private syncBlockInterval;
    private syncDocpoolInterval;
    private fifo;
    private maxLevel1Size;
    private messageHandler;
    private banned;
    private ok4reconnect;
    private banned4Sync;
    private blockstampsCache;
    private memberkeysCache;
    private headsCache;
    private newHeads;
    private headsTimeout;
    private reconnectionInteval;
    private constructor();
    getKnownHeads(): Promise<WS2PHead[]>;
    headsReceived(heads: WS2PHead[]): Promise<void>;
    private headReceived;
    private isMemberKey;
    private isConnectedKey;
    private existsBlock;
    static plugOn(server: Server): WS2PCluster;
    maxLevel1Peers: number;
    readonly maxLevel2Peers: number;
    listen(host: string, port: number): Promise<WS2PServer>;
    close(): Promise<void>;
    clientsCount(): number;
    numberOfConnectedPublicNodesWithSameKey(): number;
    servedCount(): number;
    privilegedNodes(): string[];
    connectToRemoteWS(endpointVersion: number, host: string, port: number, path: string, messageHandler: WS2PMessageHandler, expectedPub: string, ws2pEndpointUUID?: string): Promise<WS2PClient>;
    connectToWS2Peers(): Promise<void>;
    private numberOfFreeRooms;
    listenServerFlow(): void;
    private broadcastHead;
    private spreadNewHeads;
    private incrementHeadsStep;
    private sayHeadChangedTo;
    private getApi;
    private countFreeRooms;
    trimServerConnections(): Promise<void>;
    removeLowPriorityConnections(preferedKeys: string[]): Promise<void>;
    keyPriorityLevel(pubkey: string, preferedOrPrivilegedKeys: string[]): Promise<number>;
    private getPreferedNodes;
    protected acceptPubkey(pub: string, isSync: boolean, syncConnectedPubkeys: string[], connectedPubkeys: string[], connectedWS2PUID: string[], getConcurrentConnexionsCount: () => number, maxConcurrentConnexionsSize: number, priorityKeys: string[], priorityKeysOnly: boolean, targetWS2PUID?: string): Promise<boolean>;
    isNewSiblingNode(pub: string, targetWS2PUID: string, connectedWS2PUID: string[]): boolean;
    getLevel1Connections(): Promise<WS2PConnection[]>;
    getLevel2Connections(): Promise<WS2PConnection[]>;
    getAllConnections(): WS2PConnection[];
    startCrawling(waitConnection?: boolean): Promise<void>;
    stopCrawling(): Promise<void>;
    pullBlocks(): Promise<void>;
    private makeApullShot;
    pullDocpool(): Promise<void>;
    getConnectedPubkeys(): string[];
    getConnectedWS2PUID(): string[];
    banConnection(c: WS2PConnection, reason: string): void;
    rememberAndPrepareBanSyncConnection(pub: string): void;
}
