"use strict";
// Source file from duniter: Crypto-currency software to manage libre currency such as Ğ1
// Copyright (C) 2018  Cedric Moreau <cem.moreau@gmail.com>
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Affero General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Affero General Public License for more details.
Object.defineProperty(exports, "__esModule", { value: true });
var WS2P_REQ;
(function (WS2P_REQ) {
    WS2P_REQ[WS2P_REQ["KNOWN_PEERS"] = 0] = "KNOWN_PEERS";
    WS2P_REQ[WS2P_REQ["PEER_DOCUMENT"] = 1] = "PEER_DOCUMENT";
    WS2P_REQ[WS2P_REQ["WOT_REQUIREMENTS_OF_PENDING"] = 2] = "WOT_REQUIREMENTS_OF_PENDING";
    WS2P_REQ[WS2P_REQ["BLOCKS_CHUNK"] = 3] = "BLOCKS_CHUNK";
    WS2P_REQ[WS2P_REQ["BLOCK_BY_NUMBER"] = 4] = "BLOCK_BY_NUMBER";
    WS2P_REQ[WS2P_REQ["CURRENT"] = 5] = "CURRENT";
    WS2P_REQ[WS2P_REQ["MILESTONES_PAGE"] = 6] = "MILESTONES_PAGE";
    WS2P_REQ[WS2P_REQ["MILESTONES"] = 7] = "MILESTONES";
})(WS2P_REQ = exports.WS2P_REQ || (exports.WS2P_REQ = {}));
class WS2PRequester {
    constructor(ws2pc) {
        this.ws2pc = ws2pc;
    }
    static fromConnection(ws2pc) {
        return new WS2PRequester(ws2pc);
    }
    getPeer() {
        return this.query(WS2P_REQ.PEER_DOCUMENT);
    }
    getPeers() {
        return this.query(WS2P_REQ.KNOWN_PEERS);
    }
    getCurrent() {
        return this.query(WS2P_REQ.CURRENT);
    }
    getMilestonesPage() {
        return this.query(WS2P_REQ.MILESTONES_PAGE);
    }
    getMilestones(page) {
        return this.query(WS2P_REQ.MILESTONES, { page });
    }
    getBlock(number) {
        return this.query(WS2P_REQ.BLOCK_BY_NUMBER, { number });
    }
    getBlocks(count, fromNumber) {
        return this.query(WS2P_REQ.BLOCKS_CHUNK, { count, fromNumber });
    }
    getPubkey() {
        return this.ws2pc.pubkey || "########";
    }
    async getRequirementsPending(minCert = 1) {
        return this.query(WS2P_REQ.WOT_REQUIREMENTS_OF_PENDING, { minCert });
    }
    query(req, params = {}) {
        return this.ws2pc.request({
            name: WS2P_REQ[req],
            params: params
        });
    }
    get hostName() {
        return this.ws2pc.hostName;
    }
}
exports.WS2PRequester = WS2PRequester;
//# sourceMappingURL=WS2PRequester.js.map