"use strict";
// Source file from duniter: Crypto-currency software to manage libre currency such as Ğ1
// Copyright (C) 2018  Cedric Moreau <cem.moreau@gmail.com>
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Affero General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Affero General Public License for more details.
Object.defineProperty(exports, "__esModule", { value: true });
const stream = require("stream");
const logger_1 = require("../../../lib/logger");
const CertificationDTO_1 = require("../../../lib/dto/CertificationDTO");
const IdentityDTO_1 = require("../../../lib/dto/IdentityDTO");
const BlockDTO_1 = require("../../../lib/dto/BlockDTO");
const MembershipDTO_1 = require("../../../lib/dto/MembershipDTO");
const TransactionDTO_1 = require("../../../lib/dto/TransactionDTO");
const PeerDTO_1 = require("../../../lib/dto/PeerDTO");
const constants_1 = require("./constants");
const logger = logger_1.NewLogger();
class WS2PSingleWriteStream extends stream.Transform {
    constructor(protectionDuration = 1000 * constants_1.WS2PConstants.SINGLE_RECORD_PROTECTION_IN_SECONDS) {
        super({ objectMode: true });
        this.protectionDuration = protectionDuration;
        this.detections = {};
    }
    getNbProtectionsCurrently() {
        return Object.keys(this.detections).length;
    }
    async _write(obj, enc, done) {
        let documentHash = '';
        let doStream = false;
        try {
            if (obj.joiners) {
                const dto = BlockDTO_1.BlockDTO.fromJSONObject(obj);
                documentHash = dto.getHash();
            }
            else if (obj.pubkey && obj.uid) {
                const dto = IdentityDTO_1.IdentityDTO.fromJSONObject(obj);
                documentHash = dto.getHash();
            }
            else if (obj.idty_uid) {
                const dto = CertificationDTO_1.CertificationDTO.fromJSONObject(obj);
                documentHash = dto.getHash();
            }
            else if (obj.userid) {
                const dto = MembershipDTO_1.MembershipDTO.fromJSONObject(obj);
                documentHash = dto.getHash();
            }
            else if (obj.issuers) {
                const dto = TransactionDTO_1.TransactionDTO.fromJSONObject(obj);
                documentHash = dto.getHash();
            }
            else if (obj.endpoints) {
                const dto = PeerDTO_1.PeerDTO.fromJSONObject(obj);
                documentHash = dto.getHash();
            }
            if (documentHash) {
                if (!this.detections[documentHash]) {
                    doStream = true;
                    this.detections[documentHash] = 1;
                }
                else {
                    this.detections[documentHash]++;
                    logger.warn('WS2P OUT => Document detected %s times: %s', this.detections[documentHash], JSON.stringify(obj));
                }
                setTimeout(() => {
                    delete this.detections[documentHash];
                }, this.protectionDuration);
            }
            if (doStream) {
                this.push(obj);
            }
        }
        catch (e) {
            logger.warn('WS2P >> SingleWrite >>', e);
        }
        done && done();
    }
}
exports.WS2PSingleWriteStream = WS2PSingleWriteStream;
//# sourceMappingURL=WS2PSingleWriteStream.js.map