"use strict";
// Source file from duniter: Crypto-currency software to manage libre currency such as Ğ1
// Copyright (C) 2018  Cedric Moreau <cem.moreau@gmail.com>
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Affero General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Affero General Public License for more details.
Object.defineProperty(exports, "__esModule", { value: true });
const TransactionDTO_1 = require("../lib/dto/TransactionDTO");
const local_rules_1 = require("../lib/rules/local_rules");
const global_rules_1 = require("../lib/rules/global_rules");
const FIFOService_1 = require("./FIFOService");
const errors_1 = require("../lib/common-libs/errors");
const DBTx_1 = require("../lib/db/DBTx");
const constants = require('../lib/constants');
class TransactionService extends FIFOService_1.FIFOService {
    constructor(fifoPromiseHandler) {
        super(fifoPromiseHandler);
    }
    setConfDAL(newConf, newDAL) {
        this.dal = newDAL;
        this.conf = newConf;
        this.logger = require('../lib/logger').NewLogger(this.dal.profile);
    }
    processTx(txObj) {
        const tx = TransactionDTO_1.TransactionDTO.fromJSONObject(txObj, this.conf.currency);
        const hash = tx.getHash();
        return this.pushFIFO(hash, async () => {
            try {
                this.logger.info('⬇ TX %s:%s from %s', tx.output_amount, tx.output_base, tx.issuers);
                const existing = await this.dal.getTxByHash(tx.hash);
                const current = await this.dal.getCurrentBlockOrNull();
                if (!current) {
                    throw Error(errors_1.DataErrors[errors_1.DataErrors.NO_TRANSACTION_POSSIBLE_IF_NOT_CURRENT_BLOCK]);
                }
                if (existing) {
                    throw constants.ERRORS.TX_ALREADY_PROCESSED;
                }
                // Start checks...
                const nextBlockWithFakeTimeVariation = { medianTime: current.medianTime + 1 };
                const dto = TransactionDTO_1.TransactionDTO.fromJSONObject(tx);
                await local_rules_1.LOCAL_RULES_HELPERS.checkSingleTransactionLocally(dto, this.conf);
                await global_rules_1.GLOBAL_RULES_HELPERS.checkTxBlockStamp(tx, this.dal);
                await global_rules_1.GLOBAL_RULES_HELPERS.checkSingleTransaction(dto, nextBlockWithFakeTimeVariation, this.conf, this.dal, this.dal.getTxByHash.bind(this.dal));
                const server_pubkey = this.conf.pair && this.conf.pair.pub;
                if (!(await this.dal.txsDAL.sandbox.acceptNewSandBoxEntry({
                    issuers: tx.issuers,
                    output_base: tx.output_base,
                    output_amount: tx.output_amount
                }, server_pubkey))) {
                    throw constants.ERRORS.SANDBOX_FOR_TRANSACTION_IS_FULL;
                }
                await this.dal.saveTransaction(DBTx_1.DBTx.fromTransactionDTO(tx));
                this.logger.info('✔ TX %s:%s from %s', tx.output_amount, tx.output_base, tx.issuers);
                return tx;
            }
            catch (e) {
                this.logger.info('✘ TX %s:%s from %s', tx.output_amount, tx.output_base, tx.issuers);
                throw e;
            }
        });
    }
}
exports.TransactionService = TransactionService;
//# sourceMappingURL=TransactionsService.js.map