"use strict";
// Source file from duniter: Crypto-currency software to manage libre currency such as Ğ1
// Copyright (C) 2018  Cedric Moreau <cem.moreau@gmail.com>
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Affero General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Affero General Public License for more details.
Object.defineProperty(exports, "__esModule", { value: true });
const test_framework_1 = require("../integration/tools/test-framework");
const constants_1 = require("../../app/lib/common-libs/constants");
describe('BlockchainDAO', () => test_framework_1.writeBasicTestWithConfAnd2Users({
    sigReplay: 3,
    sigPeriod: 0,
    sigValidity: 10,
    udTime0: 1500000000,
}, (test) => {
    before(() => {
        constants_1.CommonConstants.BLOCK_NEW_GENERATED_VERSION = 11;
    });
    const now = 1500000000;
    test('should be able to init with 2 blocks', async (s1, cat, tac) => {
        await cat.createIdentity();
        await tac.createIdentity();
        await cat.cert(tac);
        await tac.cert(cat);
        await cat.join();
        await tac.join();
        await s1.commit({ time: now, version: 10 });
        await s1.commit({ time: now });
    });
    test('tic & toc join', async (s1, cat, tac, toc, tic) => {
        await toc.createIdentity();
        await tic.createIdentity();
        await tac.cert(toc);
        await cat.cert(tic);
        await toc.join();
        await tic.join();
        await s1.commit({ time: now });
        const idtyBlocks = await s1.dal.blockDAL.findWithIdentities();
        test_framework_1.assertEqual(idtyBlocks.length, 2);
        test_framework_1.assertEqual(idtyBlocks[0], 0);
        test_framework_1.assertEqual(idtyBlocks[1], 2);
        const certBlocks = await s1.dal.blockDAL.findWithCertifications();
        test_framework_1.assertEqual(certBlocks.length, 2);
        test_framework_1.assertEqual(certBlocks[0], 0);
        test_framework_1.assertEqual(certBlocks[1], 2);
        const mssBlocks = await s1.dal.blockDAL.findWithJoiners();
        test_framework_1.assertEqual(mssBlocks.length, 2);
        test_framework_1.assertEqual(mssBlocks[0], 0);
        test_framework_1.assertEqual(mssBlocks[1], 2);
        const udBlocks = await s1.dal.blockDAL.findWithUD();
        test_framework_1.assertEqual(udBlocks.length, 1);
        test_framework_1.assertEqual(udBlocks[0], 1);
    });
    test('some cert join', async (s1, cat, tac, toc, tic) => {
        await tic.cert(toc);
        await s1.commit({ time: now });
        const idtyBlocks = await s1.dal.blockDAL.findWithIdentities();
        test_framework_1.assertEqual(idtyBlocks.length, 2);
        test_framework_1.assertEqual(idtyBlocks[0], 0);
        test_framework_1.assertEqual(idtyBlocks[1], 2);
        const certBlocks = await s1.dal.blockDAL.findWithCertifications();
        test_framework_1.assertEqual(certBlocks.length, 3);
        test_framework_1.assertEqual(certBlocks[0], 0);
        test_framework_1.assertEqual(certBlocks[1], 2);
        test_framework_1.assertEqual(certBlocks[2], 3);
        const mssBlocks = await s1.dal.blockDAL.findWithJoiners();
        test_framework_1.assertEqual(mssBlocks.length, 2);
        test_framework_1.assertEqual(mssBlocks[0], 0);
        test_framework_1.assertEqual(mssBlocks[1], 2);
    });
    test('send money', async (s1, cat, tac, toc, tic) => {
        await cat.sendMoney(100, toc);
        await tac.sendMoney(100, toc);
        await s1.commit({ time: now }); // b#4
        await toc.sendMoney(100, cat);
        await s1.commit({ time: now }); // b#5
        const idtyBlocks = await s1.dal.blockDAL.findWithIdentities();
        test_framework_1.assertEqual(idtyBlocks.length, 2);
        test_framework_1.assertEqual(idtyBlocks[0], 0);
        test_framework_1.assertEqual(idtyBlocks[1], 2);
        const certBlocks = await s1.dal.blockDAL.findWithCertifications();
        test_framework_1.assertEqual(certBlocks.length, 3);
        test_framework_1.assertEqual(certBlocks[0], 0);
        test_framework_1.assertEqual(certBlocks[1], 2);
        test_framework_1.assertEqual(certBlocks[2], 3);
        const mssBlocks = await s1.dal.blockDAL.findWithJoiners();
        test_framework_1.assertEqual(mssBlocks.length, 2);
        test_framework_1.assertEqual(mssBlocks[0], 0);
        test_framework_1.assertEqual(mssBlocks[1], 2);
        const txBlocks = await s1.dal.blockDAL.findWithTXs();
        test_framework_1.assertEqual(txBlocks.length, 2);
        test_framework_1.assertEqual(txBlocks[0], 4);
        test_framework_1.assertEqual(txBlocks[1], 5);
    });
    test('revert block containing a cert', async (s1) => {
        await s1.revert();
        await s1.revert();
        await s1.revert();
        const idtyBlocks = await s1.dal.blockDAL.findWithIdentities();
        test_framework_1.assertEqual(idtyBlocks.length, 2);
        test_framework_1.assertEqual(idtyBlocks[0], 0);
        test_framework_1.assertEqual(idtyBlocks[1], 2);
        const certBlocks = await s1.dal.blockDAL.findWithCertifications();
        test_framework_1.assertEqual(certBlocks.length, 2);
        test_framework_1.assertEqual(certBlocks[0], 0);
        test_framework_1.assertEqual(certBlocks[1], 2);
        const mssBlocks = await s1.dal.blockDAL.findWithJoiners();
        test_framework_1.assertEqual(mssBlocks.length, 2);
        test_framework_1.assertEqual(mssBlocks[0], 0);
        test_framework_1.assertEqual(mssBlocks[1], 2);
    });
    test('revert block containing toc and tic', async (s1) => {
        await s1.revert();
        const idtyBlocks = await s1.dal.blockDAL.findWithIdentities();
        test_framework_1.assertEqual(idtyBlocks.length, 1);
        test_framework_1.assertEqual(idtyBlocks[0], 0);
        const certBlocks = await s1.dal.blockDAL.findWithCertifications();
        test_framework_1.assertEqual(certBlocks.length, 1);
        test_framework_1.assertEqual(certBlocks[0], 0);
        const mssBlocks = await s1.dal.blockDAL.findWithJoiners();
        test_framework_1.assertEqual(mssBlocks.length, 1);
        test_framework_1.assertEqual(mssBlocks[0], 0);
    });
    test('revert all', async (s1) => {
        await s1.revert();
        await s1.revert();
        test_framework_1.assertEqual((await s1.dal.blockDAL.findWithIdentities()).length, 0);
        test_framework_1.assertEqual((await s1.dal.blockDAL.findWithCertifications()).length, 0);
        test_framework_1.assertEqual((await s1.dal.blockDAL.findWithJoiners()).length, 0);
    });
    after(() => {
        constants_1.CommonConstants.BLOCK_NEW_GENERATED_VERSION = 10;
    });
}));
//# sourceMappingURL=block-dao-test.js.map