"use strict";
// Source file from duniter: Crypto-currency software to manage libre currency such as Ğ1
// Copyright (C) 2018  Cedric Moreau <cem.moreau@gmail.com>
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Affero General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Affero General Public License for more details.
Object.defineProperty(exports, "__esModule", { value: true });
const index_1 = require("../../../app/lib/common-libs/parsers/index");
const indexer_1 = require("../../../app/lib/indexer");
const local_rules_1 = require("../../../app/lib/rules/local_rules");
const index_2 = require("../../../app/lib/rules/index");
const blocks_data_1 = require("../../data/blocks-data");
const BlockDTO_1 = require("../../../app/lib/dto/BlockDTO");
const should = require('should');
const conf = {
    sigQty: 1,
    powZeroMin: 1,
    powPeriod: 18,
    incDateMin: 10,
    avgGenTime: 60,
    medianTimeBlocks: 20,
    dt: 100,
    ud0: 100,
    c: 0.1
};
describe("Block local coherence", function () {
    it('a valid block should be well formatted', test(index_2.ALIAS.ALL_LOCAL_BUT_POW_AND_SIGNATURE, blocks_data_1.BLOCK_TEST_DATA.VALID_ROOT));
    describe("should be rejected", function () {
        it('if wrong signature block', test(local_rules_1.LOCAL_RULES_FUNCTIONS.checkBlockSignature, blocks_data_1.BLOCK_TEST_DATA.WRONG_SIGNATURE, 'Block\'s signature must match'));
        it('if root block does not have Parameters', test(local_rules_1.LOCAL_RULES_FUNCTIONS.checkParameters, blocks_data_1.BLOCK_TEST_DATA.ROOT_WITHOUT_PARAMETERS, 'Parameters must be provided for root block'));
        it('if proof-of-work does not match PoWMin field', test(local_rules_1.LOCAL_RULES_FUNCTIONS.checkProofOfWork, blocks_data_1.BLOCK_TEST_DATA.WRONG_PROOF_OF_WORK, 'Not a proof-of-work'));
        it('if non-root has Parameters', test(local_rules_1.LOCAL_RULES_FUNCTIONS.checkParameters, blocks_data_1.BLOCK_TEST_DATA.NON_ROOT_WITH_PARAMETERS, 'Parameters must not be provided for non-root block'));
        it('if root block has PreviousHash', test(local_rules_1.LOCAL_RULES_FUNCTIONS.checkPreviousHash, blocks_data_1.BLOCK_TEST_DATA.ROOT_WITH_PREVIOUS_HASH, 'PreviousHash must not be provided for root block'));
        it('if root block has PreviousIssuer', test(local_rules_1.LOCAL_RULES_FUNCTIONS.checkPreviousIssuer, blocks_data_1.BLOCK_TEST_DATA.ROOT_WITH_PREVIOUS_ISSUER, 'PreviousIssuer must not be provided for root block'));
        it('if non-root block does not have PreviousHash', test(local_rules_1.LOCAL_RULES_FUNCTIONS.checkPreviousHash, blocks_data_1.BLOCK_TEST_DATA.NON_ROOT_WITHOUT_PREVIOUS_HASH, 'PreviousHash must be provided for non-root block'));
        it('if non-root block does not have PreviousIssuer', test(local_rules_1.LOCAL_RULES_FUNCTIONS.checkPreviousIssuer, blocks_data_1.BLOCK_TEST_DATA.NON_ROOT_WITHOUT_PREVIOUS_ISSUER, 'PreviousIssuer must be provided for non-root block'));
        it('a V2 block with Dividend must have UnitBase field', test(local_rules_1.LOCAL_RULES_FUNCTIONS.checkUnitBase, blocks_data_1.BLOCK_TEST_DATA.UD_BLOCK_WIHTOUT_BASE, 'Document has unkown fields or wrong line ending format'));
        it('a V3 root block must have UnitBase field', test(local_rules_1.LOCAL_RULES_FUNCTIONS.checkUnitBase, blocks_data_1.BLOCK_TEST_DATA.V3_ROOT_BLOCK_NOBASE, 'Document has unkown fields or wrong line ending format'));
        it('a V3 root block must have UnitBase field equal 0', test(local_rules_1.LOCAL_RULES_FUNCTIONS.checkUnitBase, blocks_data_1.BLOCK_TEST_DATA.V3_ROOT_BLOCK_POSITIVE_BASE, 'UnitBase must equal 0 for root block'));
        it('a block with wrong date (in past)', test(local_rules_1.LOCAL_RULES_FUNCTIONS.checkBlockTimes, blocks_data_1.BLOCK_TEST_DATA.WRONG_DATE_LOWER, 'A block must have its Time between MedianTime and MedianTime + 1440'));
        it('a block with wrong date (in future, but too far)', test(local_rules_1.LOCAL_RULES_FUNCTIONS.checkBlockTimes, blocks_data_1.BLOCK_TEST_DATA.WRONG_DATE_HIGHER_BUT_TOO_HIGH, 'A block must have its Time between MedianTime and MedianTime + 1440'));
        it('a root block with different time & medianTime should fail', test(local_rules_1.LOCAL_RULES_FUNCTIONS.checkBlockTimes, blocks_data_1.BLOCK_TEST_DATA.WRONG_ROOT_TIMES, 'Root block must have Time equal MedianTime'));
        it('a block with good date', test(local_rules_1.LOCAL_RULES_FUNCTIONS.checkBlockTimes, blocks_data_1.BLOCK_TEST_DATA.GOOD_DATE_HIGHER));
        it('Block cannot contain wrongly signed identities', test(local_rules_1.LOCAL_RULES_FUNCTIONS.checkIdentitiesSignature, blocks_data_1.BLOCK_TEST_DATA.WRONGLY_SIGNED_IDENTITIES, 'Identity\'s signature must match'));
        it('block with colliding uids in identities', test(local_rules_1.LOCAL_RULES_FUNCTIONS.checkIdentitiesUserIDConflict, blocks_data_1.BLOCK_TEST_DATA.COLLIDING_UIDS, 'Block must not contain twice same identity uid'));
        it('a block with colliding pubkeys in identities', test(local_rules_1.LOCAL_RULES_FUNCTIONS.checkIdentitiesPubkeyConflict, blocks_data_1.BLOCK_TEST_DATA.COLLIDING_PUBKEYS, 'Block must not contain twice same identity pubkey'));
        it('a block with identities not matchin joins', test(local_rules_1.LOCAL_RULES_FUNCTIONS.checkIdentitiesMatchJoin, blocks_data_1.BLOCK_TEST_DATA.WRONG_IDTY_MATCH_JOINS, 'Each identity must match a newcomer line with same userid and certts'));
        it('Block cannot contain wrongly signed join', test(local_rules_1.LOCAL_RULES_FUNCTIONS.checkMembershipsSignature, blocks_data_1.BLOCK_TEST_DATA.WRONGLY_SIGNED_JOIN, 'Membership\'s signature must match'));
        it('Block cannot contain wrongly signed active', test(local_rules_1.LOCAL_RULES_FUNCTIONS.checkMembershipsSignature, blocks_data_1.BLOCK_TEST_DATA.WRONGLY_SIGNED_ACTIVE, 'Membership\'s signature must match'));
        it('Block cannot contain wrongly signed leave', test(local_rules_1.LOCAL_RULES_FUNCTIONS.checkMembershipsSignature, blocks_data_1.BLOCK_TEST_DATA.WRONGLY_SIGNED_LEAVE, 'Membership\'s signature must match'));
        it('Block cannot contain a same pubkey more than once in joiners', test(local_rules_1.LOCAL_RULES_FUNCTIONS.checkPubkeyUnicity, blocks_data_1.BLOCK_TEST_DATA.MULTIPLE_JOINERS, 'Block cannot contain a same pubkey more than once in joiners, actives, leavers and excluded'));
        it('Block cannot contain a same pubkey more than once in actives', test(local_rules_1.LOCAL_RULES_FUNCTIONS.checkPubkeyUnicity, blocks_data_1.BLOCK_TEST_DATA.MULTIPLE_ACTIVES, 'Block cannot contain a same pubkey more than once in joiners, actives, leavers and excluded'));
        it('Block cannot contain a same pubkey more than once in leavers', test(local_rules_1.LOCAL_RULES_FUNCTIONS.checkPubkeyUnicity, blocks_data_1.BLOCK_TEST_DATA.MULTIPLE_LEAVES, 'Block cannot contain a same pubkey more than once in joiners, actives, leavers and excluded'));
        it('Block cannot contain a same pubkey more than once in excluded', test(local_rules_1.LOCAL_RULES_FUNCTIONS.checkPubkeyUnicity, blocks_data_1.BLOCK_TEST_DATA.MULTIPLE_EXCLUDED, 'Block cannot contain a same pubkey more than once in joiners, actives, leavers and excluded'));
        it('Block cannot contain a same pubkey more than once in joiners, actives, leavers and excluded', test(local_rules_1.LOCAL_RULES_FUNCTIONS.checkPubkeyUnicity, blocks_data_1.BLOCK_TEST_DATA.MULTIPLE_OVER_ALL, 'Block cannot contain a same pubkey more than once in joiners, actives, leavers and excluded'));
        it('Block cannot have revoked key in joiners,actives,leavers', test(local_rules_1.LOCAL_RULES_FUNCTIONS.checkMembershipUnicity, blocks_data_1.BLOCK_TEST_DATA.REVOKED_WITH_MEMBERSHIPS, 'Unicity constraint PUBLIC_KEY on MINDEX is not respected'));
        it('Block cannot have revoked key duplicates', test(local_rules_1.LOCAL_RULES_FUNCTIONS.checkRevokedUnicity, blocks_data_1.BLOCK_TEST_DATA.REVOKED_WITH_DUPLICATES, 'A single revocation per member is allowed'));
        it('Block revoked keys must be in excluded', test(local_rules_1.LOCAL_RULES_FUNCTIONS.checkRevokedAreExcluded, blocks_data_1.BLOCK_TEST_DATA.REVOKED_NOT_IN_EXCLUDED, 'A revoked member must be excluded'));
        it('Block cannot contain 2 certifications from same issuer', test(local_rules_1.LOCAL_RULES_FUNCTIONS.checkCertificationOneByIssuer, blocks_data_1.BLOCK_TEST_DATA.MULTIPLE_CERTIFICATIONS_FROM_SAME_ISSUER, 'Block cannot contain two certifications from same issuer'));
        it('Block cannot contain identical certifications', test(local_rules_1.LOCAL_RULES_FUNCTIONS.checkCertificationUnicity, blocks_data_1.BLOCK_TEST_DATA.IDENTICAL_CERTIFICATIONS, 'Block cannot contain identical certifications (A -> B)'));
        it('Block cannot contain certifications concerning a leaver', test(local_rules_1.LOCAL_RULES_FUNCTIONS.checkCertificationIsntForLeaverOrExcluded, blocks_data_1.BLOCK_TEST_DATA.LEAVER_WITH_CERTIFICATIONS, 'Block cannot contain certifications concerning leavers or excluded members'));
        it('Block cannot contain certifications concerning an excluded member', test(local_rules_1.LOCAL_RULES_FUNCTIONS.checkCertificationIsntForLeaverOrExcluded, blocks_data_1.BLOCK_TEST_DATA.EXCLUDED_WITH_CERTIFICATIONS, 'Block cannot contain certifications concerning leavers or excluded members'));
        it('Block cannot contain transactions without issuers (1)', test(local_rules_1.LOCAL_RULES_FUNCTIONS.checkTxIssuers, blocks_data_1.BLOCK_TEST_DATA.TRANSACTION_WITHOUT_ISSUERS, 'A transaction must have at least 1 issuer'));
        it('Block cannot contain transactions without issuers (2)', test(local_rules_1.LOCAL_RULES_FUNCTIONS.checkTxSources, blocks_data_1.BLOCK_TEST_DATA.TRANSACTION_WITHOUT_SOURCES, 'A transaction must have at least 1 source'));
        it('Block cannot contain transactions without issuers (3)', test(local_rules_1.LOCAL_RULES_FUNCTIONS.checkTxRecipients, blocks_data_1.BLOCK_TEST_DATA.TRANSACTION_WITHOUT_RECIPIENT, 'A transaction must have at least 1 recipient'));
        it('Block cannot contain transactions with identical sources in one transaction', test(local_rules_1.LOCAL_RULES_FUNCTIONS.checkTxSources, blocks_data_1.BLOCK_TEST_DATA.TRANSACTION_WITH_DUPLICATED_SOURCE_SINGLE_TX, 'It cannot exist 2 identical sources for transactions inside a given block'));
        it('Block cannot contain transactions with identical sources in a pack of transactions', test(local_rules_1.LOCAL_RULES_FUNCTIONS.checkTxSources, blocks_data_1.BLOCK_TEST_DATA.TRANSACTION_WITH_DUPLICATED_SOURCE_MULTIPLE_TX, 'It cannot exist 2 identical sources for transactions inside a given block'));
        it('Block cannot contain transactions with empty output conditions', test(local_rules_1.LOCAL_RULES_FUNCTIONS.checkTxRecipients, blocks_data_1.BLOCK_TEST_DATA.TRANSACTION_WITH_EMPTY_TX_CONDITIONS, 'Empty conditions are forbidden'));
        it('Block cannot contain transactions with wrong total', test(local_rules_1.LOCAL_RULES_FUNCTIONS.checkTxAmounts, blocks_data_1.BLOCK_TEST_DATA.TRANSACTION_WRONG_TOTAL, 'Transaction inputs sum must equal outputs sum'));
        it('Block cannot contain transactions with wrong base transformation', test(local_rules_1.LOCAL_RULES_FUNCTIONS.checkTxAmounts, blocks_data_1.BLOCK_TEST_DATA.TRANSACTION_WRONG_TRANSFORM, 'Transaction output base amount does not equal previous base deltas'));
        it('Block cannot contain transactions with unexisting lower base in sources', test(local_rules_1.LOCAL_RULES_FUNCTIONS.checkTxAmounts, blocks_data_1.BLOCK_TEST_DATA.TRANSACTION_WRONG_TRANSFORM_LOW_BASE, 'Transaction output base amount does not equal previous base deltas'));
        it('Block cannot contain transactions with more than 100 lines', test(local_rules_1.LOCAL_RULES_FUNCTIONS.checkTxLen, blocks_data_1.BLOCK_TEST_DATA.TRANSACTION_TOO_LONG, 'A transaction has a maximum size of 100 lines'));
        it('Block cannot contain transactions with a too large output', test(local_rules_1.LOCAL_RULES_FUNCTIONS.checkTxLen, blocks_data_1.BLOCK_TEST_DATA.OUTPUT_TOO_LONG, 'A transaction output has a maximum size of 2000 characters'));
        it('Block cannot contain transactions with a too large unlock', test(local_rules_1.LOCAL_RULES_FUNCTIONS.checkTxLen, blocks_data_1.BLOCK_TEST_DATA.UNLOCK_TOO_LONG, 'A transaction unlock has a maximum size of 2000 characters'));
        it('Block cannot be refused with a good V3 transaction', test(local_rules_1.LOCAL_RULES_FUNCTIONS.checkTxAmounts, blocks_data_1.BLOCK_TEST_DATA.TRANSACTION_V3_GOOD_AMOUNTS));
        it('Block cannot contain transactions with wrong signatures', test(local_rules_1.LOCAL_RULES_FUNCTIONS.checkTxSignature, blocks_data_1.BLOCK_TEST_DATA.TRANSACTION_WITH_WRONG_SIGNATURES, 'Signature from a transaction must match'));
    });
});
function test(rule, raw, expectedMessage) {
    return async () => {
        try {
            let obj = index_1.parsers.parseBlock.syncWrite(raw);
            let block = BlockDTO_1.BlockDTO.fromJSONObject(obj);
            let index = indexer_1.Indexer.localIndex(block, conf);
            await rule(block, conf, index); // conf parameter is not always used
            if (expectedMessage) {
                throw 'Test should have thrown an error';
            }
        }
        catch (e) {
            if (!expectedMessage) {
                console.error(e.stack || e);
            }
            if (e.uerr) {
                // This is a controlled error
                e.uerr.message.should.equal(expectedMessage);
            }
            else if (e) {
                // This is a controlled error
                e.message.should.equal(expectedMessage);
            }
            else {
                // throw Error(e)
                // Display non wrapped errors (wrapped error is an error in constants.js)
                // console.error(e.stack || e);
            }
        }
    };
}
//# sourceMappingURL=block-local.js.map