"use strict";
// Source file from duniter: Crypto-currency software to manage libre currency such as Ğ1
// Copyright (C) 2018  Cedric Moreau <cem.moreau@gmail.com>
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Affero General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Affero General Public License for more details.
Object.defineProperty(exports, "__esModule", { value: true });
const toolbox_1 = require("../tools/toolbox");
const TestUser_1 = require("../tools/TestUser");
const underscore_1 = require("../../../app/lib/common-libs/underscore");
const commonConf = {
    ipv4: '127.0.0.1',
    currency: 'bb',
    httpLogs: true,
    forksize: 3,
    sigQty: 1
};
let s1, cat, toc;
describe("Revert root", function () {
    before(async () => {
        s1 = toolbox_1.NewTestingServer(underscore_1.Underscore.extend({
            pair: {
                pub: 'HgTTJLAQ5sqfknMq7yLPZbehtuLSsKj9CxWN7k8QvYJd',
                sec: '51w4fEShBk1jCMauWu4mLpmDVfHksKmWcygpxriqCEZizbtERA6de4STKRkQBpxmMUwsKXRjSzuQ8ECwmqN1u2DP'
            },
            rootoffset: 10,
            sigQty: 1, dt: 1, ud0: 120
        }, commonConf));
        cat = new TestUser_1.TestUser('cat', { pub: 'HgTTJLAQ5sqfknMq7yLPZbehtuLSsKj9CxWN7k8QvYJd', sec: '51w4fEShBk1jCMauWu4mLpmDVfHksKmWcygpxriqCEZizbtERA6de4STKRkQBpxmMUwsKXRjSzuQ8ECwmqN1u2DP' }, { server: s1 });
        toc = new TestUser_1.TestUser('toc', { pub: 'DKpQPUL4ckzXYdnDRvCRKAm1gNvSdmAXnTrJZ7LvM5Qo', sec: '64EYRvdPpTfLGGmaX5nijLXRqWXaVz8r1Z1GtaahXwVSJGQRn7tqkxLb288zwSYzELMEG5ZhXSBYSxsTsz1m9y8F' }, { server: s1 });
        await s1.initDalBmaConnections();
        await cat.createIdentity();
        await toc.createIdentity();
        await toc.cert(cat);
        await cat.cert(toc);
        await cat.join();
        await toc.join();
        await s1.commit();
    });
    it('/block/0 should exist', () => s1.expectJSON('/blockchain/block/0', {
        number: 0
    }));
    it('/wot/cat should exist', () => s1.expectThat('/wot/lookup/cat', (res) => {
        res.should.have.property('results').length(1);
        res.results[0].should.have.property('uids').length(1);
        res.results[0].uids[0].should.have.property('uid').equal('cat');
        res.results[0].uids[0].should.have.property('others').length(1);
    }));
    it('reverting should erase everything', async () => {
        await s1.revert();
        await s1.expectError('/blockchain/current', 404, 'No current block');
        await s1.expectError('/blockchain/block/0', 404, 'Block not found');
        await s1.expectError('/wot/lookup/cat', 404, 'No matching identity'); // Revert completely removes the identity
    });
    after(() => {
        return s1.closeCluster();
    });
});
//# sourceMappingURL=branches_revert.js.map