"use strict";
// Source file from duniter: Crypto-currency software to manage libre currency such as Ğ1
// Copyright (C) 2018  Cedric Moreau <cem.moreau@gmail.com>
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Affero General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Affero General Public License for more details.
Object.defineProperty(exports, "__esModule", { value: true });
const toolbox_1 = require("../tools/toolbox");
const TestUser_1 = require("../tools/TestUser");
const index_1 = require("../../../app/modules/bma/index");
const underscore_1 = require("../../../app/lib/common-libs/underscore");
const shutdown_engine_1 = require("../tools/shutdown-engine");
const http_expect_1 = require("../tools/http-expect");
const rp = require('request-promise');
const MEMORY_MODE = true;
const commonConf = {
    ipv4: '127.0.0.1',
    currency: 'bb'
};
let s1, cat, tic1, tic2;
describe("Lookup identity grouping", () => {
    before(async () => {
        s1 = toolbox_1.NewTestingServer(underscore_1.Underscore.extend({
            name: 'bb12',
            memory: MEMORY_MODE,
            port: '4452',
            pair: {
                pub: 'HgTTJLAQ5sqfknMq7yLPZbehtuLSsKj9CxWN7k8QvYJd',
                sec: '51w4fEShBk1jCMauWu4mLpmDVfHksKmWcygpxriqCEZizbtERA6de4STKRkQBpxmMUwsKXRjSzuQ8ECwmqN1u2DP'
            }
        }, commonConf));
        cat = new TestUser_1.TestUser('cat', { pub: 'HgTTJLAQ5sqfknMq7yLPZbehtuLSsKj9CxWN7k8QvYJd', sec: '51w4fEShBk1jCMauWu4mLpmDVfHksKmWcygpxriqCEZizbtERA6de4STKRkQBpxmMUwsKXRjSzuQ8ECwmqN1u2DP' }, { server: s1 });
        tic1 = new TestUser_1.TestUser('tic1', { pub: 'DNann1Lh55eZMEDXeYt59bzHbA3NJR46DeQYCS2qQdLV', sec: '468Q1XtTq7h84NorZdWBZFJrGkB18CbmbHr9tkp9snt5GiERP7ySs3wM8myLccbAAGejgMRC9rqnXuW3iAfZACm7' }, { server: s1 });
        tic2 = new TestUser_1.TestUser('tic2', { pub: 'DNann1Lh55eZMEDXeYt59bzHbA3NJR46DeQYCS2qQdLV', sec: '468Q1XtTq7h84NorZdWBZFJrGkB18CbmbHr9tkp9snt5GiERP7ySs3wM8myLccbAAGejgMRC9rqnXuW3iAfZACm7' }, { server: s1 });
        // Server initialization
        await s1.initWithDAL().then(index_1.BmaDependency.duniter.methods.bma).then((bmapi) => bmapi.openConnections());
        // cat is publishing its identity, no problem
        await cat.createIdentity();
        // tic1 is publishing its identity
        await tic1.createIdentity();
        // tic2 is publishing its identity, but he has **the same pubkey as tic1**.
        // This is OK on the protocol side, but the lookup should group the 2 identities
        // under the same pubkey
        await tic2.createIdentity();
        await cat.join();
        await tic1.join();
    });
    after(() => {
        return Promise.all([
            shutdown_engine_1.shutDownEngine(s1)
        ]);
    });
    it('cat should have only 1 identity in 1 pubkey', () => http_expect_1.expectAnswer(rp('http://127.0.0.1:4452/wot/lookup/cat', { json: true }), (res) => {
        res.should.have.property('results').length(1);
        // cat pubkey
        res.results[0].should.have.property('pubkey').equal('HgTTJLAQ5sqfknMq7yLPZbehtuLSsKj9CxWN7k8QvYJd');
        // only 1 UID
        res.results[0].should.have.property('uids').length(1);
        // which is cat
        res.results[0].uids[0].should.have.property('uid').equal('cat');
    }));
    it('tic should have only 2 identities in 1 pubkey', () => http_expect_1.expectAnswer(rp('http://127.0.0.1:4452/wot/lookup/tic', { json: true }), (res) => {
        // We want to have only 1 result for the 2 identities
        res.should.have.property('results').length(1);
        // because they share the same pubkey
        res.results[0].should.have.property('pubkey').equal('DNann1Lh55eZMEDXeYt59bzHbA3NJR46DeQYCS2qQdLV');
        // but got 2 UIDs
        res.results[0].should.have.property('uids').length(2);
        // which are tic1
        res.results[0].uids[0].should.have.property('uid').equal('tic1');
        // which are tic2
        res.results[0].uids[1].should.have.property('uid').equal('tic2');
    }));
    it('should exist 2 pending memberships', () => http_expect_1.expectAnswer(rp('http://127.0.0.1:4452/wot/pending', { json: true }), (res) => {
        res.should.have.property('memberships').length(2);
        res.memberships[0].should.have.property('pubkey').equal('DNann1Lh55eZMEDXeYt59bzHbA3NJR46DeQYCS2qQdLV');
        res.memberships[0].should.have.property('uid').equal('tic1');
        res.memberships[0].should.have.property('version').equal(10);
        res.memberships[0].should.have.property('currency').equal('bb');
        res.memberships[0].should.have.property('membership').equal('IN');
        res.memberships[0].should.have.property('blockNumber').equal(0);
        res.memberships[0].should.have.property('blockHash').equal('E3B0C44298FC1C149AFBF4C8996FB92427AE41E4649B934CA495991B7852B855');
        res.memberships[0].should.have.property('written').equal(null);
    }));
});
//# sourceMappingURL=identity-lookup.js.map