"use strict";
// Source file from duniter: Crypto-currency software to manage libre currency such as Ğ1
// Copyright (C) 2018  Cedric Moreau <cem.moreau@gmail.com>
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Affero General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Affero General Public License for more details.
Object.defineProperty(exports, "__esModule", { value: true });
const constants_1 = require("../../../app/modules/prover/lib/constants");
const toolbox_1 = require("../tools/toolbox");
const TestUser_1 = require("../tools/TestUser");
const index_1 = require("../../../app/modules/bma/index");
const index_2 = require("../../../app/modules/prover/index");
const underscore_1 = require("../../../app/lib/common-libs/underscore");
const shutdown_engine_1 = require("../tools/shutdown-engine");
const http_expect_1 = require("../tools/http-expect");
const websocket_1 = require("../../../app/lib/common-libs/websocket");
const PeerDTO_1 = require("../../../app/lib/dto/PeerDTO");
const should = require('should');
const assert = require('assert');
const rp = require('request-promise');
constants_1.ProverConstants.CORES_MAXIMUM_USE_IN_PARALLEL = 1;
let server, server2, cat, toc;
describe("HTTP API", function () {
    const now = 1500000000;
    let commit;
    before(async () => {
        server = toolbox_1.NewTestingServer({
            name: 'bb11',
            ipv4: '127.0.0.1',
            port: '7777',
            currency: 'bb',
            httpLogs: true,
            sigQty: 1,
            dt: 240,
            dtReeval: 240,
            udTime0: now,
            medianTimeBlocks: 1,
            udReevalTime0: now + 20000000,
            pair: {
                pub: 'HgTTJLAQ5sqfknMq7yLPZbehtuLSsKj9CxWN7k8QvYJd',
                sec: '51w4fEShBk1jCMauWu4mLpmDVfHksKmWcygpxriqCEZizbtERA6de4STKRkQBpxmMUwsKXRjSzuQ8ECwmqN1u2DP'
            }
        });
        server2 = toolbox_1.NewTestingServer({
            name: 'bb12',
            ipv4: '127.0.0.1',
            port: '30410',
            currency: 'bb',
            httpLogs: true,
            sigQty: 1,
            dt: 240,
            dtReeval: 240,
            udTime0: now,
            medianTimeBlocks: 1,
            udReevalTime0: now + 20000000,
            pair: {
                pub: 'DKpQPUL4ckzXYdnDRvCRKAm1gNvSdmAXnTrJZ7LvM5Qo',
                sec: '64EYRvdPpTfLGGmaX5nijLXRqWXaVz8r1Z1GtaahXwVSJGQRn7tqkxLb288zwSYzELMEG5ZhXSBYSxsTsz1m9y8F'
            }
        });
        cat = new TestUser_1.TestUser('cat', { pub: 'HgTTJLAQ5sqfknMq7yLPZbehtuLSsKj9CxWN7k8QvYJd', sec: '51w4fEShBk1jCMauWu4mLpmDVfHksKmWcygpxriqCEZizbtERA6de4STKRkQBpxmMUwsKXRjSzuQ8ECwmqN1u2DP' }, { server: server });
        toc = new TestUser_1.TestUser('toc', { pub: 'DKpQPUL4ckzXYdnDRvCRKAm1gNvSdmAXnTrJZ7LvM5Qo', sec: '64EYRvdPpTfLGGmaX5nijLXRqWXaVz8r1Z1GtaahXwVSJGQRn7tqkxLb288zwSYzELMEG5ZhXSBYSxsTsz1m9y8F' }, { server: server });
        commit = makeBlockAndPost(server);
        let s = await server.initWithDAL();
        let bmapi = await index_1.BmaDependency.duniter.methods.bma(s);
        await bmapi.openConnections();
        let s2 = await server2.initWithDAL();
        let bmapi2 = await index_1.BmaDependency.duniter.methods.bma(s2);
        await bmapi2.openConnections();
        await cat.createIdentity();
        await toc.createIdentity();
        await toc.cert(cat);
        await cat.cert(toc);
        await cat.join();
        await toc.join();
        const b0 = await commit({ time: now });
        const b1 = await commit({ time: now + 120 });
        await server2.writeBlock(b0);
        await server2.writeBlock(b1);
        server._server.addEndpointsDefinitions(() => Promise.resolve('SOME_FAKE_ENDPOINT_P1'));
        server2._server.addEndpointsDefinitions(() => Promise.resolve('SOME_FAKE_ENDPOINT_P2'));
        const p1 = await server.PeeringService.generateSelfPeer(server.conf);
        await server2.PeeringService.generateSelfPeer(server2.conf);
        await server2.writePeer(p1);
        server2.writeBlock(await commit({ time: now + 120 * 2 }));
        server2.writeBlock(await commit({ time: now + 120 * 3 }));
        server2.writeBlock(await commit({ time: now + 120 * 4 }));
    });
    after(() => {
        return Promise.all([
            shutdown_engine_1.shutDownEngine(server)
        ]);
    });
    function makeBlockAndPost(theServer) {
        return async function (options) {
            const block = await index_2.ProverDependency.duniter.methods.generateAndProveTheNext(theServer._server, null, null, options);
            const res = await postBlock(theServer)(block);
            return JSON.parse(res);
        };
    }
    describe("/blockchain", function () {
        it('/parameters/ should give the parameters', function () {
            return expectJSON(rp('http://127.0.0.1:7777/blockchain/parameters', { json: true }), {
                udTime0: now,
                udReevalTime0: now + 20000000,
                dt: 240,
                dtReeval: 240
            });
        });
        it('/block/0 should exist', function () {
            return expectJSON(rp('http://127.0.0.1:7777/blockchain/block/0', { json: true }), {
                number: 0,
                dividend: null,
                monetaryMass: 0
            });
        });
        it('/block/1 should exist and have integer Dividend', function () {
            return expectJSON(rp('http://127.0.0.1:7777/blockchain/block/1', { json: true }), {
                number: 1,
                dividend: 100,
                monetaryMass: 200
            });
        });
        it('/block/2 should exist and have NULL Dividend', () => {
            return expectJSON(rp('http://127.0.0.1:7777/blockchain/block/2', { json: true }), {
                number: 2,
                dividend: null,
                monetaryMass: 200
            });
        });
        it('/block/3 should exist and have NULL Dividend', () => {
            return expectJSON(rp('http://127.0.0.1:7777/blockchain/block/3', { json: true }), {
                number: 3,
                dividend: 100,
                monetaryMass: 400
            });
        });
        it('/block/88 should not exist', function () {
            return http_expect_1.expectError(404, rp('http://127.0.0.1:7777/blockchain/block/88'));
        });
        it('/current should exist', function () {
            return expectJSON(rp('http://127.0.0.1:7777/blockchain/current', { json: true }), {
                number: 4
            });
        });
        it('/membership should not accept wrong signature', function () {
            return http_expect_1.expectError(400, 'wrong signature for membership', rp.post('http://127.0.0.1:7777/blockchain/membership', {
                json: {
                    membership: 'Version: 10\n' +
                        'Type: Membership\n' +
                        'Currency: bb\n' +
                        'Issuer: 6upqFiJ66WV6N3bPc8x8y7rXT3syqKRmwnVyunCtEj7o\n' +
                        'Block: 0-E3B0C44298FC1C149AFBF4C8996FB92427AE41E4649B934CA495991B7852B855\n' +
                        'Membership: IN\n' +
                        'UserID: someuid\n' +
                        'CertTS: 0-E3B0C44298FC1C149AFBF4C8996FB92427AE41E4649B934CA495991B7852B855\n' +
                        'cJohoG/qmxm7KwqCB71RXRSIvHu7IcYB1zWE33OpPLGmedH mdPWad32S7G9j9IDpI8QpldalhdT4BUIHlAtCw==\n'
                }
            }));
        });
        it('/membership should not accept wrong signature 2', function () {
            return http_expect_1.expectError(400, 'Document has unkown fields or wrong line ending format', rp.post('http://127.0.0.1:7777/blockchain/membership', {
                json: {
                    membership: 'Version: 2\n' +
                        'Type: Membership\n' +
                        'Currency: bb\n' +
                        'Issuer: 6upqFiJ66WV6N3bPc8x8y7rXT3syqKRmwnVyunCtEj7o\n' +
                        'Block: 0-E3B0C44298FC1C149AFBF4C8996FB92427AE41E4649B934CA495991B7852B855\n' +
                        'UserID: someuid\n' +
                        'CertTS: 1421787800\n' +
                        'cJohoG/qmxm7KwqCB71RXRSIvHu7IcYB1zWE33OpPLGmedH mdPWad32S7G9j9IDpI8QpldalhdT4BUIHlAtCw==\n'
                }
            }));
        });
        it('/membership should not accept wrong signature 3', function () {
            return http_expect_1.expectError(400, 'Document has unkown fields or wrong line ending format', rp.post('http://127.0.0.1:7777/blockchain/membership', {
                json: {
                    membership: 'Version: 2\n' +
                        'Type: Membership\n' +
                        'Currency: bb\n' +
                        'Issuer: 6upqFiJ66WV6N3bPc8x8y7rXT3syqKRmwnVyunCtEj7o\n' +
                        'Block: 0--E3B0C44298FC1C149AFBF4C8996FB92427AE41E4649B934CA495991B7852B855\n' +
                        'Membership: IN\n' +
                        'UserID: someuid\n' +
                        'CertTS: 1421787800\n' +
                        'cJohoG/qmxm7KwqCB71RXRSIvHu7IcYB1zWE33OpPLGmedH mdPWad32S7G9j9IDpI8QpldalhdT4BUIHlAtCw==\n'
                }
            }));
        });
        it('/difficulties should have current block number + 1', function () {
            return http_expect_1.expectAnswer(rp('http://127.0.0.1:7777/blockchain/difficulties', { json: true }), function (res) {
                res.should.have.property('block').equal(5);
                res.should.have.property('levels').have.length(1);
            });
        });
    });
    describe("/ws", function () {
        it('/block should exist', function (done) {
            const client = new websocket_1.WebSocket('ws://127.0.0.1:7777/ws/block');
            client.on('open', function open() {
                client.terminate();
                done();
            });
        });
        it('/block should send a block', function (done) {
            let completed = false;
            const client = new websocket_1.WebSocket('ws://127.0.0.1:7777/ws/block');
            client.once('message', function message(data) {
                const block = JSON.parse(data);
                should(block).have.property('number', 4);
                should(block).have.property('dividend').equal(null);
                should(block).have.property('monetaryMass').equal(400);
                should(block).have.property('monetaryMass').not.equal("400");
                if (!completed) {
                    completed = true;
                    done();
                }
            });
        });
        it('/block (number 5,6,7) should send a block', async () => {
            server2.writeBlock(await commit({ time: now + 120 * 5 }));
            const client = new websocket_1.WebSocket('ws://127.0.0.1:7777/ws/block');
            let resolve5, resolve6, resolve7;
            const p5 = new Promise(res => resolve5 = res);
            const p6 = new Promise(res => resolve6 = res);
            const p7 = new Promise(res => resolve7 = res);
            client.on('message', function message(data) {
                const block = JSON.parse(data);
                if (block.number === 5)
                    resolve5(block);
                if (block.number === 6)
                    resolve6(block);
                if (block.number === 7)
                    resolve7(block);
            });
            server2.writeBlock(await commit({ time: now + 120 * 6 }));
            server2.writeBlock(await commit({ time: now + 120 * 7 }));
            const b5 = await p5;
            should(b5).have.property('number', 5);
            should(b5).have.property('dividend').equal(100);
            should(b5).have.property('monetaryMass').equal(600);
            should(b5).have.property('monetaryMass').not.equal("600");
            const b6 = await p6;
            should(b6).have.property('number', 6);
            should(b6).have.property('dividend').equal(null);
            should(b6).have.property('monetaryMass').equal(600);
            should(b6).have.property('monetaryMass').not.equal("600");
            const b7 = await p7;
            should(b7).have.property('number', 7);
            should(b7).have.property('dividend').equal(100);
            should(b7).have.property('monetaryMass').equal(800);
            should(b7).have.property('monetaryMass').not.equal("800");
        });
        it('/block should answer to pings', function (done) {
            const client = new websocket_1.WebSocket('ws://127.0.0.1:7777/ws/block');
            client.on('pong', function message() {
                client.terminate();
                done();
            });
            client.on('open', function open() {
                client.ping();
            });
        });
        it('/peer (number 5,6,7) should send a peer document', async () => {
            const client = new websocket_1.WebSocket('ws://127.0.0.1:30410/ws/peer');
            let resolve5, resolve6;
            const p5 = new Promise(res => resolve5 = res);
            const p6 = new Promise(res => resolve6 = res);
            server._server.addEndpointsDefinitions(() => Promise.resolve("BASIC_MERKLED_API localhost 7777"));
            client.on('message', function message(data) {
                const peer = JSON.parse(data);
                if (peer.block.match(/2-/)) {
                    server2.PeeringService.generateSelfPeer(server.conf);
                    return resolve5(peer);
                }
                if ((peer.block.match(/1-/) || peer.block.match(/3-/)) && peer.pubkey === 'DKpQPUL4ckzXYdnDRvCRKAm1gNvSdmAXnTrJZ7LvM5Qo') {
                    return resolve6(peer);
                }
            });
            const p1 = await server.PeeringService.generateSelfPeer({
                currency: server.conf.currency
            }, 0);
            await server2._server.writeRawPeer(PeerDTO_1.PeerDTO.fromJSONObject(p1).getRawSigned());
            const b5 = await p5;
            should(b5).have.property('version', 10);
            const b6 = await p6;
            should(b6).have.property('version', 10);
        });
    });
});
async function expectJSON(promise, json) {
    try {
        const resJson = await promise;
        underscore_1.Underscore.keys(json).forEach(function (key) {
            resJson.should.have.property(String(key)).equal(json[key]);
        });
    }
    catch (err) {
        if (err.response) {
            assert.equal(err.response.statusCode, 200);
        }
        else
            throw err;
    }
}
function postBlock(server2) {
    return function (block) {
        return post(server2, '/blockchain/block')({
            block: typeof block == 'string' ? block : block.getRawSigned()
        })
            .then(async (result) => {
            const numberToReach = block.number;
            await new Promise((res) => {
                const interval = setInterval(async () => {
                    const current = await server2.dal.getCurrentBlockOrNull();
                    if (current && current.number == numberToReach) {
                        res();
                        clearInterval(interval);
                    }
                }, 1);
            });
            return result;
        });
    };
}
function post(server2, uri) {
    return function (data) {
        return rp.post('http://' + [server2.conf.ipv4, server2.conf.port].join(':') + uri, {
            form: data
        });
    };
}
//# sourceMappingURL=http-api.js.map