"use strict";
// Source file from duniter: Crypto-currency software to manage libre currency such as Ğ1
// Copyright (C) 2018  Cedric Moreau <cem.moreau@gmail.com>
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Affero General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Affero General Public License for more details.
Object.defineProperty(exports, "__esModule", { value: true });
const keyring_1 = require("../../../app/lib/common-libs/crypto/keyring");
const IdentityDTO_1 = require("../../../app/lib/dto/IdentityDTO");
const constants_1 = require("../../../app/lib/common-libs/constants");
const CertificationDTO_1 = require("../../../app/lib/dto/CertificationDTO");
const MembershipDTO_1 = require("../../../app/lib/dto/MembershipDTO");
const RevocationDTO_1 = require("../../../app/lib/dto/RevocationDTO");
const index_1 = require("../../../app/modules/crawler/index");
const buid_1 = require("../../../app/lib/common-libs/buid");
const index_2 = require("../../../app/lib/common-libs/parsers/index");
const TransactionDTO_1 = require("../../../app/lib/dto/TransactionDTO");
const PeerDTO_1 = require("../../../app/lib/dto/PeerDTO");
const underscore_1 = require("../../../app/lib/common-libs/underscore");
const request = require('request');
class TestUser {
    constructor(uid, options, node) {
        this.uid = uid;
        this.options = options;
        this.node = node;
        this.pub = "";
        this.sec = "";
        this.createdIdentity = "";
        // For sync code
        if (this.options.pub && this.options.sec) {
            this.pub = this.options.pub;
            this.sec = this.options.sec;
        }
    }
    get keypair() {
        return new keyring_1.Key(this.pub, this.sec);
    }
    init(done) {
        if (this.options.pub && this.options.sec) {
            this.pub = this.options.pub;
            this.sec = this.options.sec;
            done();
        }
        else {
            throw 'Not keypair information given for testing user ' + this.uid;
        }
    }
    async createIdentity(useRoot, fromServer) {
        if (!this.pub) {
            this.init(() => { });
        }
        const current = await this.node.server.BlockchainService.current();
        let buid = !useRoot && current ? buid_1.Buid.format.buid(current.number, current.hash) : constants_1.CommonConstants.SPECIAL_BLOCK;
        this.createdIdentity = IdentityDTO_1.IdentityDTO.fromJSONObject({
            buid: buid,
            uid: this.uid,
            issuer: this.pub,
            currency: this.node.server.conf.currency
        }).getRawUnSigned();
        this.createdIdentity += keyring_1.KeyGen(this.pub, this.sec).signSync(this.createdIdentity) + '\n';
        await this.submitIdentity(this.createdIdentity, fromServer);
    }
    submitIdentity(raw, fromServer) {
        return this.doPost('/wot/add', {
            "identity": raw
        }, fromServer);
    }
    getIdentityRaw() {
        return this.createdIdentity;
    }
    async makeCert(user, fromServer, overrideProps) {
        const lookup = await this.lookup(user.pub, fromServer);
        const current = await this.node.server.BlockchainService.current();
        const idty = underscore_1.Underscore.filter(lookup.results[0].uids, uidEntry => uidEntry.uid === user.uid)[0];
        let buid = current ? buid_1.Buid.format.buid(current.number, current.hash) : constants_1.CommonConstants.SPECIAL_BLOCK;
        const cert = {
            "version": constants_1.CommonConstants.DOCUMENTS_VERSION,
            "currency": this.node.server.conf.currency,
            "issuer": this.pub,
            "idty_issuer": user.pub,
            "idty_uid": idty.uid,
            "idty_buid": idty.meta.timestamp,
            "idty_sig": idty.self,
            "buid": buid,
            "sig": ""
        };
        underscore_1.Underscore.extend(cert, overrideProps || {});
        const rawCert = CertificationDTO_1.CertificationDTO.fromJSONObject(cert).getRawUnSigned();
        cert.sig = keyring_1.KeyGen(this.pub, this.sec).signSync(rawCert);
        return CertificationDTO_1.CertificationDTO.fromJSONObject(cert);
    }
    async cert(user, fromServer, toServer) {
        const cert = await this.makeCert(user, fromServer);
        await this.doPost('/wot/certify', {
            "cert": cert.getRawSigned()
        }, toServer);
    }
    async join() {
        return await this.sendMembership("IN");
    }
    async leave() {
        return await this.sendMembership("OUT");
    }
    async makeRevocation(givenLookupIdty, overrideProps) {
        const res = givenLookupIdty || (await this.lookup(this.pub));
        const matchingResult = underscore_1.Underscore.filter(res.results[0].uids, uidEntry => uidEntry.uid === this.uid)[0];
        const idty = {
            uid: matchingResult.uid,
            buid: matchingResult.meta.timestamp,
            sig: matchingResult.self
        };
        const revocation = {
            "currency": this.node.server.conf.currency,
            "issuer": this.pub,
            "uid": idty.uid,
            "sig": idty.sig,
            "buid": idty.buid,
            "revocation": ''
        };
        underscore_1.Underscore.extend(revocation, overrideProps || {});
        const rawRevocation = RevocationDTO_1.RevocationDTO.fromJSONObject(revocation).getRawUnsigned();
        revocation.revocation = keyring_1.KeyGen(this.pub, this.sec).signSync(rawRevocation);
        return RevocationDTO_1.RevocationDTO.fromJSONObject(revocation);
    }
    async revoke(givenLookupIdty) {
        const revocation = await this.makeRevocation(givenLookupIdty);
        return this.post('/wot/revoke', {
            "revocation": revocation.getRaw()
        });
    }
    async makeMembership(type, fromServer, overrideProps) {
        const lookup = await this.lookup(this.pub, fromServer);
        const current = await this.node.server.BlockchainService.current();
        const idty = lookup.results[0].uids[0];
        const block = buid_1.Buid.format.buid(current);
        const join = {
            "version": constants_1.CommonConstants.DOCUMENTS_VERSION,
            "currency": this.node.server.conf.currency,
            "issuer": this.pub,
            "block": block,
            "membership": type,
            "userid": this.uid,
            "certts": idty.meta.timestamp,
            "signature": ""
        };
        underscore_1.Underscore.extend(join, overrideProps || {});
        const rawJoin = MembershipDTO_1.MembershipDTO.fromJSONObject(join).getRaw();
        join.signature = keyring_1.KeyGen(this.pub, this.sec).signSync(rawJoin);
        return MembershipDTO_1.MembershipDTO.fromJSONObject(join);
    }
    async sendMembership(type) {
        const ms = await this.makeMembership(type);
        await this.post('/blockchain/membership', {
            "membership": ms.getRawSigned()
        });
    }
    async sendMoney(amount, recipient, comment) {
        const raw = await this.prepareITX(amount, recipient, comment);
        await this.sendTX(raw);
    }
    async sendTX(rawTX) {
        let http = await this.getContacter();
        return http.processTransaction(rawTX);
    }
    async prepareUTX(previousTX, unlocks, outputs, opts) {
        let obj = index_2.parsers.parseTransaction.syncWrite(previousTX);
        // Unlocks inputs with given "unlocks" strings
        let outputsToConsume = obj.outputs;
        if (opts.theseOutputsStart !== undefined) {
            outputsToConsume = outputsToConsume.slice(opts.theseOutputsStart);
        }
        let inputs = outputsToConsume.map((out, index) => {
            const output = TransactionDTO_1.TransactionDTO.outputStr2Obj(out);
            return {
                src: [output.amount, output.base, 'T', obj.hash, (opts.theseOutputsStart || 0) + index].join(':'),
                unlock: unlocks[index]
            };
        });
        return this.signed(this.prepareTX(inputs, outputs, opts));
    }
    async prepareMTX(previousTX, user2, unlocks, outputs, opts) {
        let obj = index_2.parsers.parseTransaction.syncWrite(previousTX);
        // Unlocks inputs with given "unlocks" strings
        let inputs = obj.outputs.map((out, index) => {
            const output = TransactionDTO_1.TransactionDTO.outputStr2Obj(out);
            return {
                src: [output.amount, output.base, 'T', obj.hash, index].join(':'),
                unlock: unlocks[index]
            };
        });
        opts = opts || {};
        opts.issuers = [this.pub, user2.pub];
        return this.signed(this.prepareTX(inputs, outputs, opts), user2);
    }
    async prepareITX(amount, recipient, comment) {
        let sources = [];
        if (!amount || !recipient) {
            throw 'Amount and recipient are required';
        }
        let http = await this.getContacter();
        let current = await http.getCurrent();
        let version = current && Math.min(constants_1.CommonConstants.LAST_VERSION_FOR_TX, current.version);
        let json = await http.getSources(this.pub);
        let i = 0;
        let cumulated = 0;
        let commonbase = 99999999;
        while (i < json.sources.length) {
            let src = json.sources[i];
            sources.push({
                'type': src.type,
                'amount': src.amount,
                'base': src.base,
                'noffset': src.noffset,
                'identifier': src.identifier
            });
            commonbase = Math.min(commonbase, src.base);
            cumulated += src.amount * Math.pow(10, src.base);
            i++;
        }
        if (cumulated < amount) {
            throw 'You do not have enough coins! (' + cumulated + ' ' + this.node.server.conf.currency + ' left)';
        }
        let sources2 = [];
        let total = 0;
        for (let j = 0; j < sources.length && total < amount; j++) {
            let src = sources[j];
            total += src.amount * Math.pow(10, src.base);
            sources2.push(src);
        }
        let inputSum = 0;
        sources2.forEach((src) => inputSum += src.amount * Math.pow(10, src.base));
        let inputs = sources2.map((src) => {
            return {
                src: [src.amount, src.base].concat([src.type, src.identifier, src.noffset]).join(':'),
                unlock: 'SIG(0)'
            };
        });
        let outputs = [{
                qty: amount,
                base: commonbase,
                lock: 'SIG(' + (typeof recipient === 'string' ? recipient : recipient.pub) + ')'
            }];
        if (inputSum - amount > 0) {
            // Rest back to issuer
            outputs.push({
                qty: inputSum - amount,
                base: commonbase,
                lock: "SIG(" + this.pub + ")"
            });
        }
        let raw = this.prepareTX(inputs, outputs, {
            version: version,
            blockstamp: current && [current.number, current.hash].join('-'),
            comment: comment
        });
        return this.signed(raw);
    }
    signed(raw, user2) {
        let signatures = [keyring_1.KeyGen(this.pub, this.sec).signSync(raw)];
        if (user2) {
            signatures.push(keyring_1.KeyGen(user2.pub, user2.sec).signSync(raw));
        }
        return raw + signatures.join('\n') + '\n';
    }
    makeTX(inputs, outputs, theOptions) {
        const raw = this.prepareTX(inputs, outputs, theOptions);
        return this.signed(raw);
    }
    prepareTX(inputs, outputs, theOptions) {
        let opts = theOptions || {};
        let issuers = opts.issuers || [this.pub];
        let raw = '';
        raw += "Version: " + (opts.version || constants_1.CommonConstants.TRANSACTION_VERSION) + '\n';
        raw += "Type: Transaction\n";
        raw += "Currency: " + (opts.currency || this.node.server.conf.currency) + '\n';
        raw += "Blockstamp: " + opts.blockstamp + '\n';
        raw += "Locktime: " + (opts.locktime || 0) + '\n';
        raw += "Issuers:\n";
        issuers.forEach((issuer) => raw += issuer + '\n');
        raw += "Inputs:\n";
        inputs.forEach(function (input) {
            raw += input.src + '\n';
        });
        raw += "Unlocks:\n";
        inputs.forEach(function (input, index) {
            if (input.unlock) {
                raw += index + ":" + input.unlock + '\n';
            }
        });
        raw += "Outputs:\n";
        outputs.forEach(function (output) {
            raw += [output.qty, output.base, output.lock].join(':') + '\n';
        });
        raw += "Comment: " + (opts.comment || "") + "\n";
        return raw;
    }
    async makePeer(endpoints, overrideProps) {
        const peer = PeerDTO_1.PeerDTO.fromJSONObject({
            currency: this.node.server.conf.currency,
            pubkey: this.pub,
            block: '2-00008DF633FC158F9DB4864ABED696C1AA0FE5D617A7B5F7AB8DE7CA2EFCD4CB',
            endpoints: endpoints
        });
        underscore_1.Underscore.extend(peer, overrideProps || {});
        const rawPeer = PeerDTO_1.PeerDTO.fromJSONObject(peer).getRawUnsigned();
        peer.signature = keyring_1.KeyGen(this.pub, this.sec).signSync(rawPeer);
        return PeerDTO_1.PeerDTO.fromJSONObject(peer);
    }
    async post(uri, data, done) {
        return new Promise((resolve, reject) => {
            var postReq = request.post({
                "uri": 'http://' + [this.node.server.conf.remoteipv4, this.node.server.conf.remoteport].join(':') + uri,
                "timeout": 1000 * 100000
            }, function (err, res, body) {
                err = err || (res.statusCode != 200 && body != 'Already up-to-date' && body) || null;
                if (err) {
                    reject(err);
                }
                else {
                    resolve(res);
                }
                done && done(err, res, body);
            });
            postReq.form(data);
        });
    }
    async doPost(uri, data, toServer) {
        const ip = toServer ? toServer.conf.ipv4 : this.node.server.conf.remoteipv4;
        const port = toServer ? toServer.conf.port : this.node.server.conf.remoteport;
        return new Promise((resolve, reject) => {
            var postReq = request.post({
                "uri": 'http://' + [ip, port].join(':') + uri,
                "timeout": 1000 * 100000
            }, function (err, res, body) {
                err = err || (res.statusCode != 200 && body != 'Already up-to-date' && body) || null;
                err ? reject(err) : resolve(res);
            });
            postReq.form(data);
        });
    }
    async getContacter(fromServer) {
        const that = this;
        return new Promise(function (resolve) {
            let theNode = (fromServer && { server: fromServer }) || that.node;
            resolve(index_1.CrawlerDependency.duniter.methods.contacter(theNode.server.conf.ipv4, theNode.server.conf.port, {
                timeout: 1000 * 100000
            }));
        });
    }
    async lookup(pubkey, fromServer) {
        const node2 = await this.getContacter(fromServer);
        return node2.getLookup(pubkey);
    }
}
exports.TestUser = TestUser;
//# sourceMappingURL=TestUser.js.map