"use strict";
// Source file from duniter: Crypto-currency software to manage libre currency such as Ğ1
// Copyright (C) 2018  Cedric Moreau <cem.moreau@gmail.com>
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Affero General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Affero General Public License for more details.
Object.defineProperty(exports, "__esModule", { value: true });
const underscore_1 = require("../../../app/lib/common-libs/underscore");
const should = require('should');
const assert = require('assert');
async function expectHttpCode(code, message, promise) {
    if (arguments.length == 2) {
        promise = arguments[1];
        message = undefined;
    }
    try {
        const res = await promise;
        assert.equal(res.statusCode, code);
    }
    catch (err) {
        if (err.response) {
            assert.equal(err.response.statusCode, code);
            if (message) {
                assert.equal(err.error || err.cause, message);
            }
        }
        else
            throw err;
    }
}
exports.expectHttpCode = expectHttpCode;
async function expectError(code, message, promise) {
    if (arguments.length == 2) {
        promise = arguments[1];
        message = undefined;
    }
    try {
        const res = await promise;
        assert.equal(res.statusCode, code);
    }
    catch (err) {
        if (err.response) {
            assert.equal(err.response.statusCode, code);
            if (message) {
                let errorObj = typeof err.error == "string" ? JSON.parse(err.error) : err.error;
                assert.equal(errorObj.message, message);
            }
        }
        else
            throw err;
    }
}
exports.expectError = expectError;
async function expectJSON(promise, json) {
    try {
        const resJson = await promise;
        underscore_1.Underscore.keys(json).forEach(function (key) {
            resJson.should.have.property(String(key)).equal(json[key]);
        });
    }
    catch (err) {
        if (err.response) {
            assert.equal(err.response.statusCode, 200);
        }
        else
            throw err;
    }
}
exports.expectJSON = expectJSON;
async function expectAnswer(promise, testFunc) {
    try {
        const res = await promise;
        return testFunc(res);
    }
    catch (err) {
        if (err.response) {
            console.error(err.error);
            assert.equal(err.response.statusCode, 200);
        }
        else
            throw err;
    }
}
exports.expectAnswer = expectAnswer;
//# sourceMappingURL=http-expect.js.map