"use strict";
// Source file from duniter: Crypto-currency software to manage libre currency such as Ğ1
// Copyright (C) 2018  Cedric Moreau <cem.moreau@gmail.com>
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Affero General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Affero General Public License for more details.
Object.defineProperty(exports, "__esModule", { value: true });
const keyring_1 = require("../../../app/lib/common-libs/crypto/keyring");
const toolbox_1 = require("../tools/toolbox");
const WS2PRequester_1 = require("../../../app/modules/ws2p/lib/WS2PRequester");
const BlockDTO_1 = require("../../../app/lib/dto/BlockDTO");
const constants_1 = require("../../../app/modules/ws2p/lib/constants");
const assert = require('assert');
describe('WS2P exchange', () => {
    constants_1.WS2PConstants.CONNEXION_TIMEOUT = 100;
    constants_1.WS2PConstants.REQUEST_TIMEOUT = 100;
    let wss;
    let c1, s1;
    before(async () => {
        const serverPair = new keyring_1.Key('DKpQPUL4ckzXYdnDRvCRKAm1gNvSdmAXnTrJZ7LvM5Qo', '64EYRvdPpTfLGGmaX5nijLXRqWXaVz8r1Z1GtaahXwVSJGQRn7tqkxLb288zwSYzELMEG5ZhXSBYSxsTsz1m9y8F');
        const clientPair = new keyring_1.Key('HgTTJLAQ5sqfknMq7yLPZbehtuLSsKj9CxWN7k8QvYJd', '51w4fEShBk1jCMauWu4mLpmDVfHksKmWcygpxriqCEZizbtERA6de4STKRkQBpxmMUwsKXRjSzuQ8ECwmqN1u2DP');
        const res = await toolbox_1.newWS2PBidirectionnalConnection("gtest", serverPair, clientPair, new (class TestingHandler {
            async handlePushMessage(json) {
            }
            async answerToRequest(json) {
                return BlockDTO_1.BlockDTO.fromJSONObject({ number: 1, hash: 'A' });
            }
        }));
        s1 = res.p1;
        c1 = res.p2;
        wss = res.wss;
    });
    after((done) => {
        wss.close(done);
    });
    it('should accept the connection if everything is OK on both side', async () => {
        const requester1 = WS2PRequester_1.WS2PRequester.fromConnection(c1);
        assert.deepEqual(await requester1.getCurrent(), {
            "actives": [],
            "certifications": [],
            "currency": "",
            "dividend": null,
            "excluded": [],
            "fork": false,
            "hash": "A",
            "identities": [],
            "issuer": "",
            "issuersCount": null,
            "issuersFrame": null,
            "issuersFrameVar": null,
            "joiners": [],
            "leavers": [],
            "medianTime": null,
            "membersCount": null,
            "monetaryMass": 0,
            "nonce": null,
            "number": 1,
            "parameters": "",
            "powMin": null,
            "revoked": [],
            "signature": "",
            "time": null,
            "transactions": [],
            "unitbase": null,
            "version": 10
        });
    });
});
//# sourceMappingURL=ws2p_exchange.js.map