"use strict";
// Source file from duniter: Crypto-currency software to manage libre currency such as Ğ1
// Copyright (C) 2018  Cedric Moreau <cem.moreau@gmail.com>
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Affero General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Affero General Public License for more details.
Object.defineProperty(exports, "__esModule", { value: true });
const toolbox_1 = require("../tools/toolbox");
const WS2PCluster_1 = require("../../../app/modules/ws2p/lib/WS2PCluster");
const constants_1 = require("../../../app/modules/ws2p/lib/constants");
const assert = require('assert');
const should = require('should');
describe("WS2P heads propagation", function () {
    constants_1.WS2PConstants.CONNEXION_TIMEOUT = 100;
    constants_1.WS2PConstants.REQUEST_TIMEOUT = 100;
    const now = 1500000000;
    let s1, s2;
    let cluster1, cluster2;
    let cat, tac;
    const catKeyring = { pub: 'HgTTJLAQ5sqfknMq7yLPZbehtuLSsKj9CxWN7k8QvYJd', sec: '51w4fEShBk1jCMauWu4mLpmDVfHksKmWcygpxriqCEZizbtERA6de4STKRkQBpxmMUwsKXRjSzuQ8ECwmqN1u2DP' };
    const tacKeyring = { pub: '2LvDg21dVXvetTD9GdkPLURavLYEqP3whauvPWX4c2qc', sec: '2HuRLWgKgED1bVio1tdpeXrf7zuUszv1yPHDsDj7kcMC4rVSN9RC58ogjtKNfTbH1eFz7rn38U1PywNs3m6Q7UxE' };
    let b0, b1, b2, b3, b4, portBMA1, portWS1, portWS2;
    before(async () => {
        const t1 = getTestingServer(catKeyring);
        const t2 = getTestingServer(tacKeyring);
        s1 = t1.server;
        s2 = t2.server;
        portWS1 = t1.portWS;
        portWS2 = t2.portWS;
        portBMA1 = t1.portBMA;
        cat = toolbox_1.simpleUser('cat', catKeyring, s1);
        tac = toolbox_1.simpleUser('tac', tacKeyring, s1);
        await s1.initDalBmaConnections();
        await s2.initDalBmaConnections();
        await cat.createIdentity();
        await tac.createIdentity();
        await cat.cert(tac);
        await tac.cert(cat);
        await cat.join();
        await tac.join();
        b0 = await s1.commit({ time: now });
        b1 = await s1.commit({ time: now });
        b2 = await s1.commit({ time: now });
        await s2.writeBlock(b0);
        await s2.writeBlock(b1);
        await s2.writeBlock(b2);
        await s1.waitToHaveBlock(2);
        await s2.waitToHaveBlock(2);
        cluster1 = WS2PCluster_1.WS2PCluster.plugOn(s1._server);
        cluster2 = WS2PCluster_1.WS2PCluster.plugOn(s2._server);
        await s1._server.ws2pCluster.listen('127.0.0.1', portWS1);
        await s2._server.ws2pCluster.listen('127.0.0.1', portWS2);
    });
    after(() => s1._server.ws2pCluster.close());
    it('should have b#2 on s1, s2 and s3', async () => {
        const currentS1 = await s1.BlockchainService.current();
        const currentS2 = await s2.BlockchainService.current();
        assert.equal(currentS1 && currentS1.number, 2);
        assert.equal(currentS2 && currentS2.number, 2);
    });
    it('should be able to have a connected network on s2 start', async () => {
        const p1 = await s1.getPeer();
        assert.deepEqual(p1.endpoints, [
            'BASIC_MERKLED_API 127.0.0.1 ' + portBMA1,
            'WS2P 11111111 127.0.0.1 ' + portWS1
        ]);
        await s2.writePeer(p1);
        await s1._server.ws2pCluster.startCrawling(true);
        await s2._server.ws2pCluster.startCrawling(true);
        await s1.expect('/network/ws2p/info', (res) => {
            assert.equal(res.peers.level1, 0);
            assert.equal(res.peers.level2, 1);
        });
        await s2.expect('/network/ws2p/info', (res) => {
            assert.equal(res.peers.level1, 1);
            assert.equal(res.peers.level2, 0);
        });
    });
    it('should be able to receive HEADs', async () => {
        b3 = s1.commit({ time: now });
        await Promise.all([
            s2.waitToHaveBlock(3),
            s2.waitForHeads(2) // head v2 + head v1
        ]);
        await s1.expect('/network/ws2p/info', (res) => {
            assert.equal(res.peers.level1, 0);
            assert.equal(res.peers.level2, 1);
        });
        await s2.expect('/network/ws2p/info', (res) => {
            assert.equal(res.peers.level1, 1);
            assert.equal(res.peers.level2, 0);
        });
        await s2.expect('/network/ws2p/heads', (res) => {
            res.heads.length.should.be.greaterThan(1);
        });
    });
    function getTestingServer(keyring) {
        const conf1 = toolbox_1.simpleTestingConf(now, keyring);
        const portBMA = toolbox_1.getNewTestingPort();
        const portWS = toolbox_1.getNewTestingPort();
        conf1.host = '127.0.0.1';
        conf1.port = portBMA;
        // A server
        conf1.ws2p = {
            upnp: false,
            uuid: '11111111',
            host: '127.0.0.1',
            port: portWS,
            remotehost: '127.0.0.1',
            remoteport: portWS,
            privilegedNodes: []
        };
        const server = toolbox_1.simpleTestingServer(conf1);
        server._server.addEndpointsDefinitions(async () => 'WS2P 11111111 127.0.0.1 ' + portWS);
        return { server, portWS, portBMA };
    }
});
//# sourceMappingURL=ws2p_heads.js.map